/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Text;

namespace Aran.Core {

    /// <summary>
    /// Represents an operator instance.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class Action {

        #region Constants

        /// <summary>
        /// A constant non-null empty <see cref="Int32"/> array.
        /// </summary>
        private static readonly int[] EmptyIntArray = new int[0];

        /// <summary>
        /// The constant which represents no <see cref="Id">identifier</see>.
        /// </summary>
        public const int NoId = -1;

        /// <summary>
        /// The constant which represents the first <see cref="Id">identifier</see>.
        /// </summary>
        public const int FirstId = 0;

        /// <summary>
        /// The constant which represents no <see cref="Preconditions">preconditions</see>.
        /// </summary>
        public static readonly int[] NoPreconditions = EmptyIntArray;

        /// <summary>
        /// The constant which represents no <see cref="Variables">variables</see>.
        /// </summary>
        public static readonly int[] NoVariables = EmptyIntArray;

        /// <summary>
        /// The constant which represents no <see cref="Precondition">precondition</see>.
        /// </summary>
        public const IActionCondition NoPrecondition = null;

        /// <summary>
        /// The constant which represents no <see cref="Deletions">deletions</see>.
        /// </summary>
        public static readonly int[] NoDeletions = EmptyIntArray;

        /// <summary>
        /// The constant which represents no <see cref="Additions">additions</see>.
        /// </summary>
        public static readonly int[] NoAdditions = EmptyIntArray;

        /// <summary>
        /// The constant which represents no <see cref="Effect">effect</see>.
        /// </summary>
        public const IActionEffect NoEffect = null;

        #endregion

        #region Fields

        /// <summary>
        /// The univocal identifier.
        /// </summary>
        public int Id;

        /// <summary>
        /// <para>The facts which must be contained in the <see cref="State.Facts">state fact list</see> in order
        /// to apply this <see cref="Action">action</see> to a <see cref="State">state</see>.</para>
        /// </summary>
        /// <remarks>
        /// Each element represents the index of a <see cref="State.Facts">state fact list</see> element which
        /// must be set to true.
        /// </remarks>
        public int[] Preconditions;

        /// <summary>
        /// <para>The variables of a variable list which are used by this <see cref="Action">action</see>.</para>
        /// </summary>
        /// <remarks>
        /// Each element represents the index of a variable list element used by this
        /// <see cref="Action">action</see>. The indexes can refer to any variable list, but tipically it is a
        /// <see cref="State.Variables">state variable list</see> or a global one, that is, shared by all
        /// <see cref="State">state</see>s.
        /// </remarks>
        public int[] Variables;

        /// <summary>
        /// The <see cref="IActionCondition">condition</see> which must be true in a
        /// <see cref="State">state</see> in order to apply this <see cref="Action">action</see> to it.
        /// </summary>
        public IActionCondition Precondition;

        /// <summary>
        /// The facts to be deleted from the <see cref="State.Facts">state fact list</see> when applying
        /// this <see cref="Action">action</see> to a <see cref="State">state</see>.
        /// </summary>
        /// <remarks>
        /// Each element represents the index of a <see cref="State.Facts">state fact list</see> element
        /// which will be set to false.
        /// </remarks>
        public int[] Deletions;

        /// <summary>
        /// The facts to be added to the <see cref="State.Facts">state fact list</see> when applying
        /// this <see cref="Action">action</see> to a <see cref="State">state</see>.
        /// </summary>
        /// <remarks>
        /// Each element represents the index of a <see cref="State.Facts">state fact list</see> element which
        /// will be set to true.
        /// </remarks>
        public int[] Additions;

        /// <summary>
        /// The <see cref="IActionEffect">effect</see> to be applied when applying this
        /// <see cref="Action">action</see> to a <see cref="State">state</see>.
        /// </summary>
        public IActionEffect Effect;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Action"/> class.
        /// </summary>
        /// <param name="id">The univocal identifier.</param>
        /// <param name="preconditions">
        /// <para>The facts which must be contained in the <see cref="State.Facts">state fact list</see> in order
        /// to apply this <see cref="Action">action</see> to a <see cref="State">state</see>.</para>
        /// <para>Each element represents the index of a <see cref="State.Facts">state fact list</see> element
        /// which must be set to true.</para>
        /// </param>
        /// <param name="variables">
        /// <para>The variables of a variable list which are used by this <see cref="Action">action</see>.</para>
        /// <para>Each element represents the index of a variable list element used by this
        /// <see cref="Action">action</see>. The indexes can refer to any variable list, but tipically it is a
        /// <see cref="State.Variables">state variable list</see> or a global one, that is, shared by all
        /// <see cref="State">state</see>s.</para>
        /// </param>
        /// <param name="precondition">
        /// The <see cref="IActionCondition">condition</see> which must be true in a
        /// <see cref="State">state</see> in order to apply this <see cref="Action">action</see> to it.
        /// </param>
        /// <param name="deletions">
        /// <para>The facts to be deleted from the <see cref="State.Facts">state fact list</see> when applying
        /// this <see cref="Action">action</see> to a <see cref="State">state</see>.</para>
        /// <para>Each element represents the index of a <see cref="State.Facts">state fact list</see> element
        /// which will be set to false.</para>
        /// </param>
        /// <param name="additions">
        /// <para>The facts to be added to the <see cref="State.Facts">state fact list</see> when applying
        /// this <see cref="Action">action</see> to a <see cref="State">state</see>.</para>
        /// <para>Each element represents the index of a <see cref="State.Facts">state fact list</see> element
        /// which will be set to true.</para>
        /// </param>
        /// <param name="effect">
        /// The <see cref="IActionEffect">effect</see> to be applied when applying this
        /// <see cref="Action">action</see> to a <see cref="State">state</see>.
        /// </param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this constructor.
        /// </remarks>
        public Action(int id, int[] preconditions, int[] variables, IActionCondition precondition,
        int[] deletions, int[] additions, IActionEffect effect) {
            Id = id;
            Preconditions = preconditions;
            Variables = variables;
            Precondition = precondition;
            Deletions = deletions;
            Additions = additions;
            Effect = effect;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Action"/> class.
        /// </summary>
        /// <remarks><para>Initializes the new instance with <see cref="NoId">no identifier</see>,
        /// <see cref="NoPreconditions">no preconditions</see>, <see cref="NoVariables">no variables</see>,
        /// <see cref="NoPrecondition">no precondition</see>, <see cref="NoDeletions">no deletions</see>,
        /// <see cref="NoAdditions">no additions</see> and <see cref="NoEffect">no effect</see>.</para>
        /// </remarks>
        public Action()
            : this(NoId, NoPreconditions, NoVariables, NoPrecondition, NoDeletions, NoAdditions, NoEffect) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether this <see cref="Action">action</see> is applicable to a the specified
        /// <see cref="State">state</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <remarks>
        /// Is applicable to <paramref name="state"/> if has <see cref="NoPrecondition">no precondition</see>
        /// or the <see cref="Precondition">precondition</see>
        /// <see cref="IActionCondition.IsTrueIn">is true in</see> <paramref name="state"/>.
        /// The verification done by ths method is partial. It assumes
        /// <see cref="Preconditions">preconditions</see> have been verified previously by using an
        /// efficient global data structure, because it is a common and best practice.
        /// </remarks>
        /// <returns>
        /// True if is applicable to <paramref name="state"/>; otherwise, false.
        /// </returns>
        public bool IsApplicableTo(State state) {
            const IActionCondition NoPrecondition = null;
            return (Precondition == NoPrecondition) || Precondition.IsTrueIn(state, this);
        }

        /// <summary>
        /// Applies this <see cref="Action">action</see> to a specified <see cref="State">state</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <remarks>
        /// <para>The application is done regardless of whether this <see cref="Action">action</see>
        /// is applicable to <paramref name="state"/> or not. If needed, that check must be done previously
        /// to the call of this method.</para>
        /// </remarks>
        /// <returns>A new <see cref="State">state</see> with <paramref name="state"/> as its
        /// <see cref="State.Parent">parent</see>, this <see cref="Action">action</see> as its
        /// <see cref="State.AppliedAction">applied action</see> and a copy of <paramref name="state"/>
        /// <see cref="State.Facts">fact list</see> and <see cref="State.Variables">variable list</see> but
        /// modified first by the <see cref="Deletions">deletion fact list</see>, then by the
        /// <see cref="Additions">addition fact list</see> and finally by the <see cref="Effect">effect</see>.</returns>
        public State ApplyTo(State state) {
            const int IntSize = 32;
            //creates the successor by copying its parent facts and variables
            State successor = new State(state, this);
            int[] successorFacts = successor.Facts;
            //modifies the successor facts by deleting those contained in the deletion list
            foreach (int deletion in Deletions) {
                successorFacts[deletion / IntSize] &= ~(1 << (deletion % IntSize));
            }
            //modifies the successor facts by adding those contained in the addition list
            foreach (int addition in Additions) {
                successorFacts[addition / IntSize] |= (1 << (addition % IntSize));
            }
            //modifies the successor by applying the effect
            if (Effect != NoEffect) {
                Effect.ApplyTo(successor, this);
            }
            return successor;
        }

        /// <summary>
        /// Appends an index list <see cref="String">string</see> representacion to a
        /// <see cref="StringBuilder">buffer</see>.
        /// </summary>
        /// <param name="buffer">The <see cref="StringBuilder">buffer</see>.</param>
        /// <param name="indexes">The index list.</param>
        private static void AppendFacts(StringBuilder buffer, int[] indexes) {
            bool isFirst = true;
            buffer.Append("[");
            isFirst = true;
            //for each index in the list
            foreach (int index in indexes) {
                if (isFirst) {
                    isFirst = false;
                } else {
                    buffer.Append(", ");
                }
                //appends the index to the buffer
                buffer.Append(index);
            }
            buffer.Append("]");
        }

        /// <summary>
        /// Returns a <see cref="String">string</see> which represents this <see cref="Action">action</see>.
        /// </summary>
        /// <returns>A <see cref="String">string</see> which represents this <see cref="Action">action</see></returns>
        public override string ToString() {
            const IActionCondition NoPrecondition = null;
            const IActionEffect NoEffect = null;
            //appends the identifier to the buffer
            StringBuilder buffer = new StringBuilder("Id: ");
            buffer.Append(Id);
            buffer.AppendLine();
            //appends the fact precondition list to the buffer
            buffer.Append("Preconditions: ");
            AppendFacts(buffer, Preconditions);
            buffer.AppendLine();
            //appends the variable list to the buffer
            buffer.Append("Variables: ");
            AppendFacts(buffer, Variables);
            buffer.AppendLine();
            //appends the fact deletion list to the buffer
            buffer.Append("Deletions: ");
            AppendFacts(buffer, Deletions);
            buffer.AppendLine();
            //appends the fact addition list to the buffer
            buffer.Append("Additions: ");
            AppendFacts(buffer, Additions);
            buffer.AppendLine();
            //appends the precondition to the buffer
            buffer.Append("Precondition: ");
            if (Precondition == NoPrecondition) {
                buffer.Append("None");
            } else {
                buffer.Append(Precondition.ToString(this));
            }
            buffer.AppendLine();
            //appends the effect to the buffer
            buffer.Append("Effect: ");
            if (Effect == NoEffect) {
                buffer.Append("None");
            } else {
                buffer.Append(Effect.ToString(this));
            }
            return buffer.ToString();
        }

        #endregion

    }

}
