/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents an <see cref="Action">action</see> selector.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public abstract class ActionSelector {

        #region Fields

        /// <summary>
        /// The number of operators.
        /// </summary>
        private readonly int _operatorCount;

        /// <summary>
        /// The number of preconditions by operator.
        /// </summary>
        /// <remarks>
        /// For each index, the value represents the number of preconditions of the operator identified
        /// by that index.
        /// </remarks>
        private readonly int[] _preconditionCounts;

        /// <summary>
        /// The number of actions.
        /// </summary>
        private readonly int _actionCount;

        /// <summary>
        /// The number of <see cref="Action">action</see>s by operator.
        /// </summary>
        /// <remarks>
        /// For each index, the value represents the number of <see cref="Action">action</see>s of the operator
        /// identified by that index.
        /// </remarks>
        private readonly int[] _actionCounts;

        /// <summary>
        /// The action activation table.
        /// </summary>
        /// <remarks>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </remarks>
        private readonly int[][][][] _actionActivationTable;

        /// <summary>
        /// The number of facts.
        /// </summary>
        private readonly int _factCount;

        /// <summary>
        /// The action activation row.
        /// </summary>
        /// <remarks>
        /// <para>It is obtained from the <see cref="_actionActivationTable">action activation table</see> by
        /// accumulating all the bit arrays of each operator precondition depending on the
        /// <see cref="State.Facts">state certainty list</see>.</para>
        /// <para>The row contains a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format <c>table[operatorIndex][preconditionIndex]</c>
        /// accesses to the bit array of the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (operator or precondition in operator)
        /// represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active because there is
        /// almost a fact which is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para>The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </remarks>
        private int[][][] _actionActivationRow;

        /// <summary>
        /// The action selection list.
        /// </summary>
        /// <remarks>
        /// <para>It is obtained from the <see cref="_actionActivationRow">action activation row</see> by
        /// accumulating all the activation states of each <see cref="Action">action</see> on each operator
        /// precondition depending on implementation details.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected because all of its
        /// preconditions are true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// </remarks>
        private int[] _actionSelectionList;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ActionSelector"/> class.
        /// </summary>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="preconditionCounts">
        /// <para>The number of preconditions by operator.</para>
        /// <para>For each index, the value represents the number of preconditions of the operator identified by
        /// that index.</para>
        /// </param>
        /// <param name="actionCount">The number of actions.</param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <param name="actionActivationTable">
        /// <para>The action activation table.</para>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed, correction checks
        /// must be done previously to the call of this constructor.
        /// </remarks>
        protected ActionSelector(int operatorCount, int[] preconditionCounts, int actionCount,
        int[] actionCounts, int[][][][] actionActivationTable) {
            _operatorCount = operatorCount;
            _preconditionCounts = preconditionCounts;
            _actionCount = actionCount;
            _actionCounts = actionCounts;
            _actionActivationTable = actionActivationTable;
            _factCount = actionActivationTable.Length;
            CreateActionActivationRow();
            CreateActionSelectionList();
        }

        #endregion

        #region Methods

        /// <summary>
        /// Creates the <see cref="_actionActivationRow">action activation row</see>.
        /// </summary>
        private void CreateActionActivationRow() {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            int actionCount;
            int preconditionCount;
            int actionListLength;
            _actionActivationRow = new int[_operatorCount][][];
            //for each operator
            for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                actionCount = _actionCounts[operatorIndex];
                preconditionCount = _preconditionCounts[operatorIndex];
                _actionActivationRow[operatorIndex] = new int[preconditionCount][];
                //for each precondition
                for (int preconditionIndex = 0; preconditionIndex < preconditionCount; preconditionIndex++) {
                    actionListLength = (actionCount + IntSizeLessOne) / IntSize;
                    //creates the precondition action activation list
                    _actionActivationRow[operatorIndex][preconditionIndex] = new int[actionListLength];
                }
            }
        }

        /// <summary>
        /// Clears the <see cref="_actionActivationRow">action activation row</see>.
        /// </summary>
        private void ClearActionActivationRow() {
            int preconditionCount;
            int[] actionActivationList;
            //for each operator
            for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                preconditionCount = _preconditionCounts[operatorIndex];
                //for each precondition
                for (int preconditionIndex = 0; preconditionIndex < preconditionCount; preconditionIndex++) {
                    actionActivationList = _actionActivationRow[operatorIndex][preconditionIndex];
                    //clears the precondition action activation list
                    Array.Clear(actionActivationList, 0, actionActivationList.Length);
                }
            }
        }

        /// <summary>
        /// Fills the <see cref="_actionActivationRow">action activation row</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see> from which fill the
        /// <see cref="_actionActivationRow">action selection row</see>.</param>
        private void FillActionActivationRow(State state) {
            const int IntSize = 32;
            int[][][] operators;
            int[][] op;
            int preconditionCount;
            int[] activationList;
            int activationListLength;
            int[] facts = state.Facts;
            ClearActionActivationRow();
            //for each fact in state
            for (int fact = 0; fact < _factCount; fact++) {
                //if the fact is true
                if ((facts[fact / IntSize] & (1 << (fact % IntSize))) != 0) {
                    operators = _actionActivationTable[fact];
                    //for each operator in fact
                    for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                        op = operators[operatorIndex];
                        //for each precondition action activation list in operator
                        preconditionCount = op.Length;
                        for (int preconditionIndex = 0; preconditionIndex < preconditionCount;
                        preconditionIndex++) {
                            activationList = op[preconditionIndex];
                            //for each section in action activation list
                            activationListLength = activationList.Length;
                            for (int sectionIndex = 0; sectionIndex < activationListLength; sectionIndex++) {
                                //add activated actions to total activated actions for this operator
                                _actionActivationRow[operatorIndex][preconditionIndex][sectionIndex]
                                    |= activationList[sectionIndex];
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Creates the <see cref="_actionSelectionList">action selection list</see>.
        /// </summary>
        private void CreateActionSelectionList() {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            const int AllOnes = -1;
            //creates the list
            int actionSelectionListLength = (_actionCount + IntSizeLessOne) / IntSize;
            _actionSelectionList = new int[actionSelectionListLength];
            //sets all bits in the list to one
            for (int sectionIndex = 0; sectionIndex < actionSelectionListLength; sectionIndex++) {
                _actionSelectionList[sectionIndex] = AllOnes;
            }
        }

        /// <summary>
        /// Clears the <see cref="_actionSelectionList">action selection list</see>.
        /// </summary>
        private void ClearActionSelectionList() {
            const int AllOnes = -1;
            int actionSelectionListLength = _actionSelectionList.Length;
            //sets all bits in the list to one
            for (int sectionIndex = 0; sectionIndex < actionSelectionListLength; sectionIndex++) {
                _actionSelectionList[sectionIndex] = AllOnes;
            }
        }

        /// <summary>
        /// Fills the action selection list.
        /// </summary>
        /// <param name="actionSelectionList">
        /// <para>The <see cref="Action">action</see> selection list.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected because all of its
        /// preconditions are true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// </param>
        /// <param name="actionActivationRow">
        /// <para>The <see cref="Action">action</see> activation row.</para>
        /// <para>The row contains a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format <c>table[operatorIndex][preconditionIndex]</c>
        /// accesses to the bit array of the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (operator or precondition in operator)
        /// represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active because there is
        /// almost a fact which is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para>The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <remarks><paramref name="actionSelectionList"/> is obtained from <paramref name="actionActivationRow"/>
        /// by accumulating all the activation states of each <see cref="Action">action</see> on each operator
        /// precondition depending on implementation details.</remarks>
        protected abstract void FillActionSelectionList(int[] actionSelectionList, int[][][] actionActivationRow,
        int[] actionCounts);

        /// <summary>
        /// Gets the <see cref="Action">action</see> selection list for a specified <see cref="State">state</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see> from which get the
        /// <see cref="Action">action</see> selection list.</param>
        /// <remarks>
        /// <para>The <see cref="Action">action</see> selection list is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected because all of its
        /// preconditions are true in the <see cref="State.Facts">state certainty list</see>.</para></remarks>
        /// <returns>The <see cref="Action">action</see> selection list for <paramref name="state"/>.</returns>
        /// <remarks>
        /// <para>Assumes the correction of <paramref name="state"/>, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this method.</para>
        /// <para>The action selection list is shared by all the calls to this method, as a performance
        /// improvement. If needed, cloning tasks must be done after calling the method in order to avoid
        /// problems derived from the action selection list sharing.</para>
        /// </remarks>
        public int[] GetActionSelectionList(State state) {
            FillActionActivationRow(state);
            ClearActionSelectionList();
            FillActionSelectionList(_actionSelectionList, _actionActivationRow, _actionCounts);
            return _actionSelectionList;
        }

        #endregion

    }

}
