/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents an <see cref="Action">action</see> selector which accumulates the activation states of each
    /// <see cref="Action">action</see> on each operator by doing a logic and (conjunction) with them.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class ConjunctionActionSelector : ActionSelector {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ConjunctionActionSelector"/> class.
        /// </summary>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="preconditionCounts">
        /// <para>The number of preconditions by operator.</para>
        /// <para>For each index, the value represents the number of preconditions of the operator identified by
        /// that index.</para>
        /// </param>
        /// <param name="actionCount">The number of actions.</param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <param name="actionActivationTable">
        /// <para>The action activation table.</para>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed, correction checks
        /// must be done previously to the call of this constructor.
        /// </remarks>
        public ConjunctionActionSelector(int operatorCount, int[] preconditionCounts, int actionCount,
        int[] actionCounts, int[][][][] actionActivationTable)
            : base(operatorCount, preconditionCounts, actionCount, actionCounts, actionActivationTable) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Fills the action selection list.
        /// </summary>
        /// <param name="actionSelectionList">
        /// <para>The <see cref="Action">action</see> selection list.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected because all of its
        /// preconditions are true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// </param>
        /// <param name="actionActivationRow">
        /// <para>The <see cref="Action">action</see> activation row.</para>
        /// <para>The row contains a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format <c>table[operatorIndex][preconditionIndex]</c>
        /// accesses to the bit array of the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (operator or precondition in operator)
        /// represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active because there is
        /// almost a fact which is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para>The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <remarks><paramref name="actionSelectionList"/> is obtained from <paramref name="actionActivationRow"/>
        /// by accumulating all the activation states of each <see cref="Action">action</see> on each operator
        /// precondition depending on implementation details.</remarks>
        protected override void FillActionSelectionList(int[] actionSelectionList,
        int[][][] actionActivationRow, int[] actionCounts) {
            const int IntSize = 32;
            const int AllOnes = -1;
            const uint UnsignedAllOnes = 0xffffffff;
            int actionCount;
            int remainingActionCount;
            int activationListLength;
            int lastSectionIndex;
            int activationListSection;
            uint unsignedActivationListSection;
            int[][] op;
            int selectionListSectionOffset = 0;
            int selectionListSectionIndex = 0;
            int localSelectionListSectionIndex = 0;
            int selectionListSectionCapacity = 0;
            int operatorCount = actionActivationRow.Length;
            //for each operator in action activation row
            for (int operatorIndex = 0; operatorIndex < operatorCount; operatorIndex++) {
                op = actionActivationRow[operatorIndex];
                actionCount = actionCounts[operatorIndex];
                remainingActionCount = actionCount;
                //for each precondition action activation list in operator
                foreach (int[] activationList in op) {
                    localSelectionListSectionIndex = selectionListSectionIndex;
                    remainingActionCount = actionCount;
                    //for each section in action activation list
                    activationListLength = activationList.Length;
                    lastSectionIndex = activationListLength - 1;
                    for (int sectionIndex = 0; sectionIndex < activationListLength; sectionIndex++) {
                        //intersect activated actions to total activated actions of this operator
                        activationListSection = activationList[sectionIndex];
                        selectionListSectionCapacity = IntSize - selectionListSectionOffset;
                        //HACK: N >> IntSize is N, this is to avoid that effect
                        if (selectionListSectionCapacity != IntSize) {
                            actionSelectionList[localSelectionListSectionIndex] &=
                                (activationListSection << selectionListSectionOffset) |
                                //HACK: negative int shift right fills with ones, this is to avoid that effect
                                ((int)(UnsignedAllOnes >> selectionListSectionCapacity));
                        } else {
                            actionSelectionList[localSelectionListSectionIndex] &=
                                (activationListSection << selectionListSectionOffset);
                        }
                        if ((sectionIndex != lastSectionIndex) ||
                        (remainingActionCount > selectionListSectionCapacity)) {
                            localSelectionListSectionIndex++;
                            //HACK: N >> IntSize is N, this is to avoid that effect
                            if (selectionListSectionCapacity != IntSize) {
                                //HACK: overflow checking prevents negative int casting
                                unchecked {
                                    unsignedActivationListSection = (uint)activationListSection;
                                }
                                actionSelectionList[localSelectionListSectionIndex] &=
                                    //HACK: negative int shift right fills with ones, this is to avoid that effect
                                    ((int)(unsignedActivationListSection >> selectionListSectionCapacity)) |
                                    (AllOnes << selectionListSectionOffset);
                            }
                            if (sectionIndex != lastSectionIndex) {
                                remainingActionCount -= IntSize;
                            }
                        }
                    }
                }
                selectionListSectionIndex = localSelectionListSectionIndex;
                if (remainingActionCount == selectionListSectionCapacity) {
                    selectionListSectionOffset = 0;
                    selectionListSectionIndex++;
                } else if (remainingActionCount > selectionListSectionCapacity) {
                    selectionListSectionOffset = (remainingActionCount - selectionListSectionCapacity);
                } else {
                    selectionListSectionOffset += remainingActionCount;
                }
                //HACK: N << IntSize is N, N << 0 is also N, this is to avoid that effect
                if ((selectionListSectionOffset != IntSize) && (selectionListSectionOffset != 0)) {
                    actionSelectionList[selectionListSectionIndex] |= (AllOnes << selectionListSectionOffset);
                }
            }
        }

        #endregion

    }

}
