/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Core {

    /// <summary>
    /// Represents a <see cref="RelaxedPlanner">relaxed planner</see> fact layer node.
    /// </summary>
    [Serializable]
    internal sealed class FactLayerNode : LayerNode<FactLayerNode> {

        #region Fields

        /// <summary>
        /// The index of the fact in the <see cref="State.Facts">state fact list</see>.
        /// </summary>
        internal int FactIndex;

        /// <summary>
        /// If the fact is a goal.
        /// </summary>
        internal bool IsGoal;

        /// <summary>
        /// True times list.
        /// </summary>
        /// <remarks>
        /// If a time is contained in the list, the fact is true at that time.
        /// </remarks>
        private List<int> _trueTimes;

        /// <summary>
        /// The <see cref="Action">action</see> which was applied in order to achieve the fact.
        /// </summary>
        internal Action AchievingAction;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FactLayerNode"/> class.
        /// </summary>
        /// <param name="factIndex">The index of the fact in the <see cref="State.Facts">state fact list</see>.</param>
        /// <param name="isGoal">If the fact is a goal.</param>
        internal FactLayerNode(int factIndex, bool isGoal) {
            FactIndex = factIndex;
            IsGoal = isGoal;
            _trueTimes = new List<int>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Determines whether the fact is true at the specified time.
        /// </summary>
        /// <param name="time">The time.</param>
        /// <returns>
        /// True if is true at <paramref name="time"/>; otherwise, false.
        /// </returns>
        internal bool IsTrueAtTime(int time) {
            return _trueTimes.Contains(time);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Marks the fact as true at a time and the previous one.
        /// </summary>
        /// <param name="previousTime">The previous time.</param>
        /// <param name="time">The time.</param>
        internal void MarkAsTrue(int previousTime, int time) {
            _trueTimes.Add(previousTime);
            _trueTimes.Add(time);
        }

        /// <summary>
        /// Clears the node.
        /// </summary>
        internal override void Clear() {
            base.Clear();
            _trueTimes.Clear();
            AchievingAction = null;
        }

        #endregion

    }

}
