/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Core {

    /// <summary>
    /// Represents a state <see cref="IEqualityComparer{T}">equality comparer</see> which only takes into
    /// account the state facts.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class FactsOnlyStateEqualityComparer : EqualityComparer<State> {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FactsOnlyStateEqualityComparer"/> class.
        /// </summary>
        public FactsOnlyStateEqualityComparer() {
            //do nothing
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether two <see cref="State">state</see>s are equal.
        /// </summary>
        /// <param name="x">The first <see cref="State">state</see> to compare.</param>
        /// <param name="y">The second <see cref="State">state</see> to compare.</param>
        /// <returns>
        /// True if the specified <see cref="State">state</see>s are equal; otherwise, false.
        /// </returns>
        public override bool Equals(State x, State y) {
            return x.FactsOnlyEquals(y);
        }

        /// <summary>
        /// Serves as a hash function for the specified <see cref="State">state</see> for hashing algorithms and
        /// data structures, such as a hash table.
        /// </summary>
        /// <param name="obj">The <see cref="State">state</see> for which to get a hash code.</param>
        /// <returns>A hash code for the specified <see cref="State">state</see>.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="obj"/> is a null reference.</exception>
        public override int GetHashCode(State obj) {
            if (obj == null) {
                throw new ArgumentNullException("obj");
            }
            return obj.GetFactsOnlyHashCode();
        }

        #endregion

    }

}
