/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents an <see cref="Action">action</see>-dependant <see cref="State">state</see> condition.
    /// </summary>
    /// <remarks>
    /// <para>An <see cref="Action">action</see>-dependant <see cref="State">state</see> condition typically
    /// must be check in run time because of the impossibility of knowing previously its value.</para>
    /// <para>For checking the <see cref="IActionCondition">condition</see>, the information provided by an
    /// <see cref="Action">action</see> is taken into account.</para>
    /// </remarks>
    [CLSCompliant(true)]
    public interface IActionCondition {

        #region Methods

        /// <summary>
        /// Determines whether this <see cref="IActionCondition">condition</see> is true in a specified
        /// <see cref="State">state</see>, taking into account the information provided by an
        /// <see cref="Action">action</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see> in which check this
        /// <see cref="IActionCondition">condition</see>.</param>
        /// <param name="action">The <see cref="Action">action</see> to take into account.</param>
        /// <returns>
        /// <remarks><para>Assumes the correction of both <paramref name="state"/> and
        /// <paramref name="action"/>, as a performance improvement. If needed, correction checks must be
        /// done previously to the call of this method.</para>
        /// <para>It does not modify neither <paramref name="state"/> nor <paramref name="action"/>.</para>
        /// </remarks>
        /// True if this <see cref="IActionCondition">condition</see> is true in <paramref name="state"/>,
        /// taking into account the information provided by <paramref name="action"/>; false otherwise.
        /// </returns>
        bool IsTrueIn(State state, Action action);

        /// <summary>
        /// Returns a <see cref="String">string</see> that represents this
        /// <see cref="IActionCondition">condition</see>, taking into account the information provided by
        /// an <see cref="Action">action</see>.
        /// </summary>
        /// <param name="action">The <see cref="Action">action</see> to take into account.</param>
        /// <returns>A <see cref="String">string</see> that represents this
        /// <see cref="IActionCondition">condition</see>, taking into account the information provided by
        /// <paramref name="action"/>.</returns>
        string ToString(Action action);

        #endregion

    }

}
