/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents an <see cref="Action">action</see>-dependant <see cref="State">state</see> effect.
    /// </summary>
    /// <remarks>
    /// <para>Typically, an <see cref="Action">action</see>-dependant <see cref="State">state</see> effect 
    /// consequences can not be known until run time because of its dependencies with non-precalculated values
    /// which are those which can only be known at run time.</para>
    /// <para>For applying the <see cref="IActionEffect">effect</see>, the information provided by an
    /// <see cref="Action">action</see> is taken into account.</para>
    /// </remarks>
    [CLSCompliant(true)]
    public interface IActionEffect {

        #region Methods

        /// <summary>
        /// Applies this <see cref="IActionEffect">effect</see> to a specified <see cref="State">state</see>,
        /// taking into account the information provided by an <see cref="Action">action</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see> to which apply this
        /// <see cref="IActionEffect">effect</see>.</param>
        /// <param name="action">The <see cref="Action">action</see> to take into account.</param>
        /// <remarks><para>Assumes the correction of both <paramref name="state"/> and
        /// <paramref name="action"/>, as a performance improvement. If needed, correction checks must
        /// be done previously to the call of this method.</para>
        /// <para>It can modify <paramref name="state"/> but not <paramref name="action"/>.</para>
        /// </remarks>
        void ApplyTo(State state, Action action);

        /// <summary>
        /// Returns a <see cref="String">string</see> that represents this
        /// <see cref="IActionEffect">effect</see>, taking into account the information provided by an
        /// <see cref="Action">action</see>.
        /// </summary>
        /// <param name="action">The <see cref="Action">action</see> to take into account.</param>
        /// <returns>A <see cref="String">string</see> that represents this
        /// <see cref="IActionEffect">effect</see>, taking into account the information provided by
        /// <paramref name="action"/>.</returns>
        string ToString(Action action);

        #endregion

    }

}