/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents a planner which returns relaxed plans.
    /// </summary>
    /// <remarks>
    /// The implentation of this planner is based on the explanations of the paper titled
    /// "The FF Planning System: Fast Plan Generation Through Heuristic Search", written by Jrg Hoffmann and
    /// Bernhard Nebel and published by the Journal of Artificial Intelligence Research in 2001.
    /// </remarks>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class RelaxedPlanner {

        #region Fields

        /// <summary>
        /// The number of facts in the <see cref="State.Facts">state fact list</see>.
        /// </summary>
        private readonly int _factCount;

        /// <summary>
        /// The <see cref="FactLayerNode">fact layer node</see>s.
        /// </summary>
        private FactLayerNode[] _factLayerNodes;

        /// <summary>
        /// The last fact layer of the planning graph.
        /// </summary>
        private Layer<FactLayerNode> _lastFactLayer;

        /// <summary>
        /// The goal sets.
        /// </summary>
        private NodeList<FactLayerNode>[] _goalSets;

        /// <summary>
        /// The number of actions.
        /// </summary>
        private readonly int _actionCount;

        /// <summary>
        /// The <see cref="ActionLayerNode">action layer node</see>s.
        /// </summary>
        private ActionLayerNode[] _actionLayerNodes;

        /// <summary>
        /// The first <see cref="ActionLayerNode">action layer node</see> which contains a precondition-less
        /// <see cref="Action">action</see>.
        /// </summary>
        private ActionLayerNode _firstUnpreconditionedActionLayerNode;

        /// <summary>
        /// The last <see cref="ActionLayerNode">action layer node</see> which contains a precondition-less
        /// <see cref="Action">action</see>.
        /// </summary>
        private ActionLayerNode _lastUnpreconditionedActionLayerNode;

        /// <summary>
        /// The last action layer of the planning graph.
        /// </summary>
        private Layer<ActionLayerNode> _lastActionLayer;

        /// <summary>
        /// The number of goals.
        /// </summary>
        private readonly int _goalCount;

        /// <summary>
        /// The number of reached goals.
        /// </summary>
        private int _reachedGoalCount;

        /// <summary>
        /// The number of operators.
        /// </summary>
        private readonly int _operatorCount;

        /// <summary>
        /// The number of <see cref="Action">action</see>s by operator.
        /// </summary>
        /// <remarks>
        /// For each index, the value represents the number of <see cref="Action">action</see>s of the operator
        /// identified by that index.
        /// </remarks>
        private readonly int[] _actionCounts;

        /// <summary>
        /// The action activation table.
        /// </summary>
        /// <remarks>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </remarks>
        private readonly int[][][][] _actionActivationTable;

        /// <summary>
        /// The action selection list.
        /// </summary>
        /// <remarks>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected because all of its
        /// preconditions are true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// </remarks>
        private readonly int[] _actionSelectionList;

        /// <summary>
        /// If a plan was found.
        /// </summary>
        private bool _planFound;

        /// <summary>
        /// The plan action list.
        /// </summary>
        /// <remarks>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the presence state of the corresponding
        /// <see cref="Action">action</see> in the plan. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is present in the plan.</para>
        /// </remarks>
        private readonly int[] _planActionList;

        /// <summary>
        /// The result returned by the planner.
        /// </summary>
        private readonly RelaxedPlannerResult _result;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelaxedPlanner"/> class.
        /// </summary>
        /// <param name="factCount">The number of facts in the <see cref="State.Facts">state fact list</see>.</param>
        /// <param name="actions">
        /// <para>The actions.</para>
        /// <para>Each action identified by <c>actionIndex</c> is at <c>actionIndex</c>.</para>
        /// </param>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <param name="actionActivationTable">
        /// <para>The action activation table.</para>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="goalCount">The number of goals.</param>
        /// <param name="goals">
        /// <para>The bit array which represents the fact goal list.</para>
        /// <para>Each bit represents the goal state of a fact. If the bit has the one value the fact is a goal.
        /// The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed, correction checks
        /// must be done previously to the call of this constructor.
        /// </remarks>
        public RelaxedPlanner(int factCount, Action[] actions, int operatorCount, int[] actionCounts,
        int[][][][] actionActivationTable, int goalCount, int[] goals) {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            _factCount = factCount;
            CreateFactNodes(goals);
            _actionCount = actions.Length;
            CreateActionNodes(actions);
            _operatorCount = operatorCount;
            _goalCount = goalCount;
            _actionCounts = actionCounts;
            _actionActivationTable = actionActivationTable;
            int actionSelectionListCount = (_actionCount + IntSizeLessOne) / IntSize;
            _actionSelectionList = new int[actionSelectionListCount];
            _planActionList = new int[actionSelectionListCount];
            _result = new RelaxedPlannerResult(_planActionList);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Creates the fact nodes.
        /// </summary>
        /// <param name="goals">
        /// <para>The bit array which represents the fact goal list.</para>
        /// <para>Each bit represents the goal state of a fact. If the bit has the one value the fact is a goal.
        /// The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        private void CreateFactNodes(int[] goals) {
            const int IntSize = 32;
            _factLayerNodes = new FactLayerNode[_factCount];
            bool isGoal;
            //for each fact
            for (int factIndex = 0; factIndex < _factCount; factIndex++) {
                //creates the fact node, being goal if contained in the fact goal list
                isGoal = ((goals[factIndex / IntSize] & (1 << (factIndex % IntSize))) != 0);
                _factLayerNodes[factIndex] = new FactLayerNode(factIndex, isGoal);
            }
        }

        /// <summary>
        /// Creates the action nodes.
        /// </summary>
        /// <param name="actions">
        /// <para>The actions.</para>
        /// <para>Each action identified by <c>actionIndex</c> is at <c>actionIndex</c>.</para>
        /// </param>
        private void CreateActionNodes(Action[] actions) {
            const Action UnreachableAction = null;
            const ActionLayerNode NoActionLayerNode = null;
            Action action;
            ActionLayerNode actionLayerNode = NoActionLayerNode;
            _actionLayerNodes = new ActionLayerNode[_actionCount];
            //for each reachable action
            for (int actionId = 0; actionId < _actionCount; actionId++) {
                action = actions[actionId];
                if (action != UnreachableAction) {
                    //if is a precondition-less action
                    if (action.Preconditions.Length == 0) {
                        //creates the action node
                        actionLayerNode = new ActionLayerNode(0, action);
                        //adds the action node to the precondition-less action node list
                        if (_firstUnpreconditionedActionLayerNode == NoActionLayerNode) {
                            _firstUnpreconditionedActionLayerNode = actionLayerNode;
                        } else {
                            actionLayerNode.Next = _lastUnpreconditionedActionLayerNode;
                        }
                        _lastUnpreconditionedActionLayerNode = actionLayerNode;
                    //if has preconditions
                    } else {
                        //creates the action node
                        actionLayerNode = new ActionLayerNode(action);
                    }
                    _actionLayerNodes[actionId] = actionLayerNode;
                }
            }
        }

        /// <summary>
        /// Selects the actions which are activated when a fact is true in the
        /// <see cref="State.Facts">state certainty list</see>..
        /// </summary>
        /// <param name="factIndex">The fact index.</param>
        private void SelectActions(int factIndex) {
            const int IntSize = 32;
            int[][][] operators = _actionActivationTable[factIndex];
            int remainingActionCount;
            int activationListLength;
            int lastSectionIndex;
            int activationListSection;
            uint unsignedActivationListSection;
            int[][] op;
            int selectionListSectionOffset = 0;
            int selectionListSectionIndex = 0;
            int localSelectionListSectionIndex = 0;
            int selectionListSectionCapacity = 0;
            int actionCount = 0;
            Array.Clear(_actionSelectionList, 0, _actionSelectionList.Length);
            //for each operator in fact
            for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                op = operators[operatorIndex];
                actionCount = _actionCounts[operatorIndex];
                remainingActionCount = actionCount;
                //for each precondition action activation list in operator
                int prec = 0;
                foreach (int[] activationList in op) {
                    localSelectionListSectionIndex = selectionListSectionIndex;
                    remainingActionCount = actionCount;
                    //for each section in action activation list
                    activationListLength = activationList.Length;
                    lastSectionIndex = activationListLength - 1;
                    for (int sectionIndex = 0; sectionIndex < activationListLength; sectionIndex++) {
                        //add activated actions to selected actions
                        activationListSection = activationList[sectionIndex];
                        _actionSelectionList[localSelectionListSectionIndex] |=
                            (activationListSection << selectionListSectionOffset);
                        selectionListSectionCapacity = IntSize - selectionListSectionOffset;
                        if ((sectionIndex != lastSectionIndex) ||
                        (remainingActionCount > selectionListSectionCapacity)) {
                            localSelectionListSectionIndex++;
                            //HACK: N >> IntSize is N, this is to avoid that effect
                            if (selectionListSectionCapacity != IntSize) {
                                //HACK: overflow checking prevents negative int casting
                                unchecked {
                                    unsignedActivationListSection = (uint)activationListSection;
                                }
                                _actionSelectionList[localSelectionListSectionIndex] |=
                                        //HACK: negative int shift right fills with ones, this is to avoid that effect
                                        ((int)(unsignedActivationListSection >> selectionListSectionCapacity));
                            }
                            if (sectionIndex != lastSectionIndex) {
                                remainingActionCount -= IntSize;
                            }
                        }
                    }
                    prec++;
                }
                selectionListSectionIndex = localSelectionListSectionIndex;
                if (remainingActionCount == selectionListSectionCapacity) {
                    selectionListSectionOffset = 0;
                    selectionListSectionIndex++;
                } else if (remainingActionCount > selectionListSectionCapacity) {
                    selectionListSectionOffset = (remainingActionCount - selectionListSectionCapacity);
                } else {
                    selectionListSectionOffset += remainingActionCount;
                }
            }
        }

        /// <summary>
        /// Adds the initial fact nodes to the planning graph.
        /// </summary>
        /// <param name="initialState">The initial state.</param>
        private void AddInitialFacts(State initialState) {
            const int IntSize = 32;
            const FactLayerNode NoFactLayerNode = null;
            FactLayerNode factLayerNode;
            int[] facts = initialState.Facts;
            //for each fact in initial state
            for (int factIndex = 0; factIndex < _factCount; factIndex++) {
                if ((facts[factIndex / IntSize] & (1 << (factIndex % IntSize))) != 0) {
                    //add fact node
                    factLayerNode = _factLayerNodes[factIndex];
                    _lastFactLayer.Add(factLayerNode);
                    //if fact is goal, increase the reached goals count
                    if (factLayerNode.IsGoal) {
                        _reachedGoalCount++;
                    }
                }
            }
            _planFound = (_lastFactLayer.FirstNode != NoFactLayerNode);
        }

        /// <summary>
        /// Adds precondition-less action nodes to the planning graph.
        /// </summary>
        private void AddUnpreconditionedActions() {
            const ActionLayerNode NoActionLayerNode = null;
            _lastActionLayer.FirstNode = _firstUnpreconditionedActionLayerNode;
            _lastActionLayer.LastNode = _lastUnpreconditionedActionLayerNode;
            if (_lastUnpreconditionedActionLayerNode != NoActionLayerNode) {
                _lastUnpreconditionedActionLayerNode.Next = NoActionLayerNode;
            }
        }

        /// <summary>
        /// Adds to the planning graph the actions reached when adding a fact to the graph.
        /// </summary>
        /// <param name="factIndex">The fact index.</param>
        private void AddReachedActions(int factIndex) {
            const int IntSize = 32;
            ActionLayerNode actionLayerNode;
            int lastActionLayerIndex = _lastActionLayer.Index;
            SelectActions(factIndex);
            //for each action activated when the fact is true in the state certainty list
            for (int actionId = 0; actionId < _actionCount; actionId++) {
                if ((_actionSelectionList[actionId / IntSize] & (1 << (actionId % IntSize))) != 0) {
                    actionLayerNode = _actionLayerNodes[actionId];
                    //add the action to the planning graph if all of its preconditions have been reached
                    if (++(actionLayerNode.ReachedPreconditionsCount) ==
                    actionLayerNode.Action.Preconditions.Length) {
                        _lastActionLayer.Add(actionLayerNode);
                    }
                    //updates difficulty of reaching the action
                    actionLayerNode.Difficulty += lastActionLayerIndex;
                }
            }
        }

        /// <summary>
        /// Adds to the planning graph the actions reached when adding last layer facts to the graph.
        /// </summary>
        private void AddReachedActions() {
            const FactLayerNode NoFactLayerNode = null;
            FactLayerNode factLayerNode = _lastFactLayer.FirstNode;
            while (factLayerNode != NoFactLayerNode) {
                AddReachedActions(factLayerNode.FactIndex);
                factLayerNode = factLayerNode.Next;
            }
        }

        /// <summary>
        /// Adds both initial fact and action layers.
        /// </summary>
        /// <param name="initialState">The initial state.</param>
        private void AddInitialLayers(State initialState) {
            AddInitialFacts(initialState);
            if (_planFound && (_reachedGoalCount < _goalCount)) {
                AddUnpreconditionedActions();
                AddReachedActions();
            }
        }

        /// <summary>
        /// Initializes both fact and action layers.
        /// </summary>
        /// <param name="initialState">The initial state.</param>
        private void InitializeLayers(State initialState) {
            const ActionLayerNode NoActionLayerNode = null;
            //clears all fact nodes
            foreach (FactLayerNode factLayerNode in _factLayerNodes) {
                factLayerNode.Clear();
            }
            //resets the last fact layer
            _lastFactLayer = new Layer<FactLayerNode>();
            //clears all reachable action nodes
            foreach (ActionLayerNode actionLayerNode in _actionLayerNodes) {
                if (actionLayerNode != NoActionLayerNode) {
                    actionLayerNode.Clear();
                }
            }
            //resets the last action layer
            _lastActionLayer = new Layer<ActionLayerNode>();
            //resets the reached goal count
            _reachedGoalCount = 0;
            //adds both initial facts and action layers
            AddInitialLayers(initialState);
        }

        /// <summary>
        /// Adds the facts reached by an action layer node to the planning graph.
        /// </summary>
        /// <param name="actionLayerNode">The action layer node.</param>
        private void AddReachedFacts(ActionLayerNode actionLayerNode) {
            FactLayerNode factLayerNode;
            Action action = actionLayerNode.Action;
            int lastFactLayerNodeIndex = _lastFactLayer.Index;
            //for each fact added by the action
            foreach (int factIndex in action.Additions) {
                factLayerNode = _factLayerNodes[factIndex];
                //add the fact to the last layer only if not already added to the planning graph
                if (_lastFactLayer.Add(factLayerNode)) {
                    //if the fact is a goal, increment the reached goal count
                    if (factLayerNode.IsGoal) {
                        _reachedGoalCount++;
                    }
                    factLayerNode.AchievingAction = action;
                //if the fact is already added to the last fact layer of the planning graph
                } else if ((factLayerNode.LayerIndex == lastFactLayerNodeIndex) &&
                //but the difficulty of the achieving action is greater than the current one
                (actionLayerNode.Difficulty < _actionLayerNodes[factLayerNode.AchievingAction.Id].Difficulty)) {
                    //the new achieving action is the current
                    factLayerNode.AchievingAction = action;
                }
            }
        }

        /// <summary>
        /// Adds the facts reached by the last layer action nodes to the planning graph.
        /// </summary>
        private void AddReachedFacts() {
            const ActionLayerNode NoActionLayerNode = null;
            ActionLayerNode actionLayerNode = _lastActionLayer.FirstNode;
            while (actionLayerNode != NoActionLayerNode) {
                AddReachedFacts(actionLayerNode);
                actionLayerNode = actionLayerNode.Next;
            }
        }

        /// <summary>
        /// Adds a fact layer to the planning graph.
        /// </summary>
        private void AddFactLayer() {
            const FactLayerNode NoFactLayerNode = null;
            //creates the layer
            Layer<FactLayerNode> newFactLayer = new Layer<FactLayerNode>(_lastFactLayer.Index + 1);
            newFactLayer.Next = _lastFactLayer;
            _lastFactLayer = newFactLayer;
            //populates the layer
            AddReachedFacts();
            _planFound = (_lastFactLayer.FirstNode != NoFactLayerNode);
        }

        /// <summary>
        /// Adds an action layer to the planning graph.
        /// </summary>
        private void AddActionLayer() {
            //creates the layer
            Layer<ActionLayerNode> newActionLayer = new Layer<ActionLayerNode>(_lastActionLayer.Index + 1);
            newActionLayer.Next = _lastActionLayer;
            _lastActionLayer = newActionLayer;
            //populates the layer
            AddReachedActions();
        }

        /// <summary>
        /// Builds the planning graph from a specified initial state..
        /// </summary>
        /// <param name="initialState">The initial state.</param>
        private void BuildPlanningGraph(State initialState) {
            InitializeLayers(initialState);
            if (_planFound && (_reachedGoalCount < _goalCount)) {
                AddFactLayer();
                while (_planFound && (_reachedGoalCount < _goalCount)) {
                    AddActionLayer();
                    AddFactLayer();
                }
            }
        }

        /// <summary>
        /// Groups the goals into sets.
        /// </summary>
        private void GroupGoalsIntoSets() {
            const FactLayerNode NoFactLayerNode = null;
            int layerIndex = _lastFactLayer.Index;
            int layerCount = layerIndex + 1;
            _goalSets = new NodeList<FactLayerNode>[layerCount];
            NodeList<FactLayerNode> goalSet;
            FactLayerNode factLayerNode;
            FactLayerNode goalSetLastNode;
            Layer<FactLayerNode> factLayer = _lastFactLayer;
            //from the last fact layer to the first
            while (layerIndex > 0) {
                //creates the layer goal set
                goalSet = new NodeList<FactLayerNode>();
                factLayerNode = factLayer.FirstNode;
                //adds to the goal set all goal facts
                while (factLayerNode != NoFactLayerNode) {
                    if (factLayerNode.IsGoal) {
                        goalSet.Add(factLayerNode);
                    }
                    factLayerNode = factLayerNode.Next;
                }
                //ensures goal set list finishing
                goalSetLastNode = goalSet.LastNode;
                if (goalSetLastNode != NoFactLayerNode) {
                    goalSetLastNode.Next = NoFactLayerNode;
                }
                //assings the goal set to its layer index
                _goalSets[layerIndex] = goalSet;
                factLayer = factLayer.Next;
                layerIndex--;
            }
        }

        /// <summary>
        /// Extracts the relaxed plan from the planning graph.
        /// </summary>
        private void ExtractPlan() {
            const int IntSize = 32;
            const FactLayerNode NoFactLayerNode = null;
            const RelaxedPlanNode NoPlanNode = null;
            int lastLayerIndex = _lastFactLayer.Index;
            int previousLayerIndex = lastLayerIndex - 1;
            int factLayerIndex;
            int planLength = 0;
            NodeList<FactLayerNode> goalSet;
            FactLayerNode goalSetNode;
            FactLayerNode factLayerNode;
            Action achievingAction;
            int achievingActionId;
            NodeList<RelaxedPlanNode> plan = new NodeList<RelaxedPlanNode>();
            RelaxedPlanNode firstNextSubplanNode = NoPlanNode;
            //from the last fact layer to the first
            for (int layerIndex = lastLayerIndex; layerIndex > 0; layerIndex--) {
                //resets the full plan in order to be used as the current subplan storage
                plan.Clear();
                goalSet = _goalSets[layerIndex];
                goalSetNode = goalSet.FirstNode;
                //for each goal in the layer
                while (goalSetNode != NoFactLayerNode) {
                    //if the goal is not true at this time
                    if (!goalSetNode.IsTrueAtTime(layerIndex)) {
                        achievingAction = goalSetNode.AchievingAction;
                        achievingActionId = achievingAction.Id;
                        //adds the achieving action to the plan
                        _planActionList[achievingActionId / IntSize] |= (1 << (achievingActionId % IntSize));
                        plan.Add(new RelaxedPlanNode(achievingAction));
                        planLength++;
                        //for each fact in achieving action preconditions
                        foreach (int factIndex in achievingAction.Preconditions) {
                            factLayerNode = _factLayerNodes[factIndex];
                            factLayerIndex = factLayerNode.LayerIndex;
                            //if the fact is not an initial one and is not true at previous time
                            if ((factLayerIndex != 0) && !factLayerNode.IsTrueAtTime(previousLayerIndex)) {
                                //adds the fact as goal in its layer
                                _goalSets[factLayerIndex].Add(factLayerNode);
                                factLayerNode.Next = NoFactLayerNode;
                            }
                        }
                        //for each fact in achieving action additions
                        foreach (int factIndex in achievingAction.Additions) {
                            //mark the fact as true at this time and the previous one
                            _factLayerNodes[factIndex].MarkAsTrue(previousLayerIndex, layerIndex);
                        }
                    }
                    goalSetNode = goalSetNode.Next;
                }
                //appends the previous subplan to the full plan
                plan.Add(firstNextSubplanNode);
                //stores the full plan as the previous subplan
                firstNextSubplanNode = plan.FirstNode;
                previousLayerIndex--;
            }
            _result.PlanFound = _planFound;
            _result.FirstPlanNode = plan.FirstNode;
            _result.PlanLength = planLength;
        }

        /// <summary>
        /// Resets the planner.
        /// </summary>
        private void Reset() {
            _lastActionLayer = null;
            _lastFactLayer = null;
            _goalSets = null;
        }

        /// <summary>
        /// Searches for a relaxed plan from a specified initial state.
        /// </summary>
        /// <param name="initialState">The initial state.</param>
        /// <remarks>
        /// <para>Assumes the correction of <paramref name="initialState"/>, as a performance improvement. If
        /// needed, correction checks must be done previously to the call of this method.</para>
        /// <para>The <see cref="RelaxedPlannerResult">search result</see> is shared by all the calls to this
        /// method, as a performance improvement. If needed, cloning tasks must be done after calling the method
        /// in order to avoid problems derived from the <see cref="RelaxedPlannerResult">search result</see> sharing.</para></remarks>
        /// <returns>The <see cref="RelaxedPlannerResult">searh result</see> with the relaxed plan, if found.</returns>
        public RelaxedPlannerResult Plan(State initialState) {
            _result.Clear();
            if (_goalCount > 0) {
                BuildPlanningGraph(initialState);
                GroupGoalsIntoSets();
                ExtractPlan();
                Reset();
            }
            return _result;
        }

        #endregion

    }

}
