/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Core {

    /// <summary>
    /// Represents a <see cref="RelaxedPlanner">relaxed planner</see> result.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class RelaxedPlannerResult {

        #region Fields

        /// <summary>
        /// If a plan was found.
        /// </summary>
        public bool PlanFound;

        /// <summary>
        /// The plan length.
        /// </summary>
        public int PlanLength;

        /// <summary>
        /// The first node of the plan.
        /// </summary>
        public RelaxedPlanNode FirstPlanNode;

        /// <summary>
        /// The plan action list.
        /// </summary>
        /// <remarks>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the presence state of the corresponding
        /// <see cref="Action">action</see> in the plan. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is present in the plan.</para>
        /// </remarks>
        public readonly int[] PlanActionList;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelaxedPlannerResult"/> class.
        /// </summary>
        /// <param name="planActionList">
        /// <para>The plan action list.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the presence state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is present in the plan.</para>
        /// </param>
        internal RelaxedPlannerResult(int[] planActionList) {
            PlanActionList = planActionList;
            PlanFound = true;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Clears this result.
        /// </summary>
        /// <remarks>
        /// Sets <see cref="PlanFound">the plan found state</see> to true,
        /// <see cref="PlanLength">the plan length</see> to zero and
        /// <see cref="FirstPlanNode">the first plan node</see> to <see cref="RelaxedPlanNode.None">none</see>
        /// and clears <see cref="PlanActionList">the plan action list</see>.
        /// </remarks>
        internal void Clear() {
            PlanFound = true;
            PlanLength = 0;
            FirstPlanNode = RelaxedPlanNode.None;
            Array.Clear(PlanActionList, 0, PlanActionList.Length);
        }

        #endregion

    }

}
