/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Text;

namespace Aran.Core {

    /// <summary>
    /// Represents a state.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class State : IEquatable<State> {

        #region Constants
        
        //TODO: set the best value for OverlappingAvoidanceRangeSize based on empirical results
        /// <summary>
        /// The overlapping avoidance range size. 
        /// </summary>
        /// <remarks>
        /// The implementation of the <see cref="GetHashCode()"/> method is a sum of the facts, and maybe
        /// variables, hash codes. The sum is done by applying a multiplier in order to avoid hash codes
        /// overlapping as far as possible. That multiplier is called the overlapping avoidance range size.
        /// </remarks>
        private const int OverlappingAvoidanceRangeSize = 31;

        /// <summary>
        /// The constant which represents no <see cref="Parent">parent</see>.
        /// </summary>
        public const State NoParent = null;

        /// <summary>
        /// The constant which represents no <see cref="AppliedAction">applied action</see>.
        /// </summary>
        public const Action NoAppliedAction = null;

        /// <summary>
        /// The constant which represents no <see cref="Facts">facts</see>.
        /// </summary>
        public static readonly int[] NoFacts = new int[0];

        /// <summary>
        /// The constant which represents no <see cref="Variables">variables</see>.
        /// </summary>
        public static readonly float[] NoVariables = new float[0];

        #endregion

        #region Fields

        /// <summary>
        /// <para>The <see cref="State">state</see> from which was created this one.</para>
        /// <para><see cref="NoParent">No parent</see> if it is an initial <see cref="State">state</see>.</para>
        /// </summary>
        public State Parent;

        /// <summary>
        /// The number of actions applied in order to reach this <see cref="State">state</see> from an initial one.
        /// </summary>
        public int PlanLength;

        /// <summary>
        /// <para>The <see cref="Action">action</see> applied to the <see cref="Parent">parent</see> in order
        /// to create this <see cref="State">state</see>.</para>
        /// <para><see cref="NoAppliedAction">No applied action</see> if it is an initial <see cref="State">state</see>.</para>
        /// </summary>
        public Action AppliedAction;

        /// <summary>
        /// <para>The bit array which represents the fact certainty list.</para>
        /// <para>Each bit represents the certainty of a fact. If the bit has the one value the fact is
        /// true. If the bit has the zero value the fact is false. The
        /// bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </summary>
        public int[] Facts;

        /// <summary>
        /// <para>The values of the variables.</para>
        /// <para>Each element represents the value of a variable.</para>
        /// </summary>
        public float[] Variables;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="State"/> class.
        /// </summary>
        /// <param name="parent">
        /// <para>The <see cref="State">state</see> from which was created this one.</para>
        /// <para><see cref="NoParent">No parent</see> if it is an initial <see cref="State">state</see>.</para>
        /// </param>
        /// <param name="appliedAction">
        /// <para>The <see cref="Action">action</see> applied to the <see cref="Parent">parent</see> in order
        /// to create this <see cref="State">state</see>.</para>
        /// <para><see cref="NoAppliedAction">No applied action</see> if it is an initial <see cref="State">state</see>.</para>
        /// </param>
        /// <param name="facts">
        /// <para>The bit array which represents the fact certainty list.</para>
        /// <para>Each bit represents the certainty of a fact. If the bit has the one value the fact is
        /// true. If the bit has the zero value the fact is false. The
        /// bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="variables">
        /// <para>The values of the variables.</para>
        /// <para>Each element represents the value of a variable.</para>
        /// </param>
        /// <remarks>
        /// <para><see cref="PlanLength">Plan length</see> is initialized to <paramref name="parent"/> plan
        /// length plus one if <paramref name="parent"/> is different from <see cref="NoParent">none</see>.</para>
        /// <para>Assumes the correction of all parameters, as a performance improvement. If needed, correction
        /// checks must be done previously to the call of this constructor.</para>
        /// </remarks>
        public State(State parent, Action appliedAction, int[] facts, float[] variables) {
            SetParent(parent);
            AppliedAction = appliedAction;
            Facts = facts;
            Variables = variables;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="State"/> class.
        /// </summary>
        /// <param name="factCount">The fact count.</param>
        /// <param name="variableCount">The variable count.</param>
        /// <remarks><para>Initializes the instance with <see cref="NoParent">no parent</see>,
        /// <see cref="NoAppliedAction">no applied action</see>, all <see cref="Facts">facts</see>
        /// to false, all <see cref="Variables">variables</see> set to zero and
        /// <see cref="PlanLength">plan length</see> set to zero.</para>
        /// <para>Assumes the correction of both <paramref name="factsCount"/> and
        /// <paramref name="variablesCount"/>, as a performance improvement. If needed, correction checks must
        /// be done previously to the call of this constructor.</para>
        /// </remarks>
        public State(int factCount, int variableCount) {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            Facts = new int[(factCount + IntSizeLessOne) / IntSize];
            Variables = new float[variableCount];
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="State"/> class.
        /// </summary>
        /// <param name="facts">
        /// <para>The bit array which represents the fact certainty list.</para>
        /// <para>Each bit represents the certainty of a fact. If the bit has the one value the fact is
        /// true. If the bit has the zero value the fact is false. The
        /// bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="variableCount">The variable count.</param>
        /// <remarks><para>Initializes the instance with <see cref="NoParent">no parent</see>,
        /// <see cref="NoAppliedAction">no applied action</see>, the <paramref name="facts"/> provided,
        /// all <see cref="Variables">variable values</see> set to zero and
        /// <see cref="PlanLength">plan length</see> set to zero.</para>
        /// <para>Assumes the correction of both <paramref name="facts"/> and <paramref name="variableCount"/>,
        /// as a performance improvement. If needed, correction checks must be done previously to the call of
        /// this constructor.</para>
        /// </remarks>
        public State(int[] facts, int variableCount) {
            Facts = facts;
            Variables = new float[variableCount];
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="State"/> class.
        /// </summary>
        /// <param name="parent">The <see cref="State">state</see> from which it is created.</param>
        /// <param name="appliedAction">The <see cref="Action">action</see> applied to
        /// <paramref name="parent"/> in order to create this <see cref="State">state</see>.</param>
        /// <remarks>
        /// <para>Copies all <see cref="Facts">facts</see> and <see cref="Variables">variables</see>
        /// from <paramref name="parent"/>.</para>
        /// <para><see cref="PlanLength">Plan length</see> is initialized to <paramref name="parent"/> plan
        /// length plus one.</para>
        /// <para>Assumes the correction of both <paramref name="parent"/> and <paramref name="appliedAction"/>,
        /// as a performance improvement. If needed, correction checks must be done previously to the call of
        /// this constructor.</para>
        /// </remarks>
        internal State(State parent, Action appliedAction) {
            SetParent(parent);
            AppliedAction = appliedAction;
            Facts = (int[])Parent.Facts.Clone();
            Variables = (float[])Parent.Variables.Clone();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="State"/> class.
        /// </summary>
        /// <remarks><para>Initializes the new instance with <see cref="NoParent">no parent</see>,
        /// <see cref="NoAppliedAction">no applied action</see>, <see cref="NoFacts">no facts</see>,
        /// <see cref="NoVariables">no variables</see> and <see cref="PlanLength">plan length</see> set to zero.</para>
        /// </remarks>
        public State() {
            Facts = NoFacts;
            Variables = NoVariables;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Sets the parent and updates the plan length.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <remarks>
        /// <para><see cref="PlanLength">Plan length</see> is set to <paramref name="parent"/> plan
        /// length plus one if <paramref name="parent"/> is different from <see cref="NoParent">none</see>.</para>
        /// <para>Assumes the correction of <paramref name="parent"/>, as a performance improvement.
        /// If needed, correction checks must be done previously to the call of this method.</para>
        /// </remarks>
        public void SetParent(State parent) {
            Parent = parent;
            if (parent != NoParent) {
                PlanLength = parent.PlanLength + 1;
            }
        }

        /// <summary>
        /// Indicates whether this <see cref="State">state</see> is equal to another object.
        /// </summary>
        /// <remarks>
        /// <para>Both this <see cref="State">state</see> and <paramref name="obj"/> are equal if
        /// <paramref name="obj"/> is a <see cref="State">state</see> and their
        /// <see cref="Facts">fact certainty lists</see> and their <see cref="Variables">variables</see>
        /// are equal.</para>
        /// <para>Assumes structural equality between both <see cref="State">state</see>s. In other words,
        /// assumes that both <see cref="State">state</see>s have the same number of facts and the same number
        /// of variables and does no checks to verify this condition, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this method.</para>
        /// </remarks>
        /// <param name="obj">The object to be compared with this <see cref="State">state</see>.</param>
        /// <returns>True if this <see cref="State">state</see> is equal to
        /// <paramref name="obj"/>; otherwise, false.
        /// </returns>
        public override bool Equals(Object obj) {
            State other = obj as State;
            return (other != null) && Equals(other);
        }

        /// <summary>
        /// Indicates whether this <see cref="State">state</see> is equal to another <see cref="State">state</see>.
        /// </summary>
        /// <param name="other">The <see cref="State">state</see> to be compared with this.</param>
        /// <remarks>
        /// <para>Both <see cref="State">state</see>s are equal if their <see cref="Facts">fact certainty lists</see>
        /// are equal.</para>
        /// <para>Assumes structural equality between both <see cref="State">state</see>s. In other words,
        /// assumes that both <see cref="State">state</see>s have the same number of facts and does no checks to
        /// verify this condition, as a performance improvement. If needed, correction checks must be done
        /// previously to the call of this method.</para>
        /// </remarks>
        /// <returns>True if this <see cref="State">state</see> is equal to <paramref name="other"/>; otherwise, false.
        /// </returns>
        public bool FactsOnlyEquals(State other) {
            //facts certainty lists equality checking
            int[] thisFacts = Facts;
            int[] otherFacts = other.Facts;
            int factSectionCount = thisFacts.Length;
            for (int factSectionIndex = 0; factSectionIndex < factSectionCount; factSectionIndex++) {
                if (thisFacts[factSectionIndex] != otherFacts[factSectionIndex]) {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Indicates whether this <see cref="State">state</see> is equal to another <see cref="State">state</see>.
        /// </summary>
        /// <param name="other">The <see cref="State">state</see> to be compared with this.</param>
        /// <param name="variables">
        /// <para>The <see cref="State">state</see> variables to be considered.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is considered.</para>
        /// </param>
        /// <remarks>
        /// <para>Both <see cref="State">state</see>s are equal if their <see cref="Facts">fact certainty lists</see>
        /// and some of their <see cref="Variables">variables</see> are equal.</para>
        /// <para>Assumes structural equality between both <see cref="State">state</see>s. In other words,
        /// assumes that both <see cref="State">state</see>s have the same number of facts and the same number of
        /// variables and does no checks to verify this condition, as a performance improvement. Assumes that
        /// the list of variables to be considered has the same length as the number of variables per state and
        /// does no checks to verify this condition, as a performance improvement. If needed, correction checks
        /// must be done previously to the call of this method.</para>
        /// </remarks>
        /// <returns>True if this <see cref="State">state</see> is equal to <paramref name="other"/>; otherwise, false.
        /// </returns>
        public bool SomeVariablesEquals(State other, bool[] variables) {
            if (FactsOnlyEquals(other)) {
                //variable values equality checking
                float[] thisVariables = Variables;
                float[] otherVariables = other.Variables;
                int variableCount = thisVariables.Length;
                for (int variableIndex = 0; variableIndex < variableCount; variableIndex++) {
                    if (variables[variableIndex] &&
                    (thisVariables[variableIndex] != otherVariables[variableIndex])) {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// Indicates whether this <see cref="State">state</see> is equal to another <see cref="State">state</see>.
        /// </summary>
        /// <param name="other">The <see cref="State">state</see> to be compared with this.</param>
        /// <remarks>
        /// <para>Both <see cref="State">state</see>s are equal if their <see cref="Facts">fact certainty lists</see>
        /// and their <see cref="Variables">variables</see> are equal.</para>
        /// <para>Assumes structural equality between both <see cref="State">state</see>s. In other words,
        /// assumes that both <see cref="State">state</see>s have the same number of facts and the same number of
        /// variables and does no checks to verify this condition, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this method.</para>
        /// </remarks>
        /// <returns>True if this <see cref="State">state</see> is equal to <paramref name="other"/>; otherwise, false.
        /// </returns>
        public bool Equals(State other) {
            if (FactsOnlyEquals(other)) {
                //variable values equality checking
                float[] thisVariables = Variables;
                float[] otherVariables = other.Variables;
                int variableCount = thisVariables.Length;
                for (int variableIndex = 0; variableIndex < variableCount; variableIndex++) {
                    if (thisVariables[variableIndex] != otherVariables[variableIndex]) {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// Returns a hash code for this <see cref="State">state</see>.
        /// </summary>
        /// <remarks>
        /// Only facts are taken into account while calculating the hash code. Values returned by this method
        /// may change in the future depending on implementation details.
        /// </remarks>
        /// <returns>
        /// A hash code for this <see cref="State">state</see>.
        /// </returns>
        public int GetFactsOnlyHashCode() {
            //initializes the hash code to zero
            int hashCode = 0;
            //sums facts hash codes to the hash code
            foreach (int factSection in Facts) {
                //HACK: overflow checking prevents int cycling, and we want cycle ints
                unchecked {
                    hashCode = (OverlappingAvoidanceRangeSize * hashCode) + factSection.GetHashCode();
                }
            }
            return hashCode;
        }

        /// <summary>
        /// Returns a hash code for this <see cref="State">state</see>.
        /// </summary>
        /// <param name="variables">
        /// <para>The <see cref="State">state</see> variables to be considered.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is considered.</para>
        /// </param>
        /// <remarks>
        /// Only some variables are taken into account while calculating the hash code. Values returned by this
        /// method may change in the future depending on implementation details.
        /// </remarks>
        /// <returns>
        /// A hash code for this <see cref="State">state</see>.
        /// </returns>
        public int GetSomeVariablesHashCode(bool[] variables) {
            int hashCode = GetFactsOnlyHashCode();
            //sums variables hash codes to the hash code
            int variableCount = Variables.Length;
            for (int variableIndex = 0; variableIndex < variableCount; variableIndex++) {
                if (variables[variableIndex]) {
                    //HACK: overflow checking prevents int cycling, and we want cycle ints
                    unchecked {
                        hashCode = (OverlappingAvoidanceRangeSize * hashCode) + 
                            Variables[variableIndex].GetHashCode();
                    }
                }
            }
            return hashCode;
        }

        /// <summary>
        /// Returns a hash code for this <see cref="State">state</see>.
        /// </summary>
        /// <remarks>
        /// Values returned by this method may change in the future depending on implementation details.
        /// </remarks>
        /// <returns>
        /// A hash code for this <see cref="State">state</see>.
        /// </returns>
        public override int GetHashCode() {
            int hashCode = GetFactsOnlyHashCode();
            //sums variables hash codes to the hash code
            foreach (float value in Variables) {
                //HACK: overflow checking prevents int cycling, and we want cycle ints
                unchecked {
                    hashCode = (OverlappingAvoidanceRangeSize * hashCode) + value.GetHashCode();
                }
            }
            return hashCode;
        }

        /// <summary>
        /// Appends a <see cref="String">string</see> to a <see cref="StringBuilder">buffer</see>
        /// preceding all start of line by a tab-like white space string.
        /// </summary>
        /// <param name="buffer">The <see cref="StringBuilder">buffer</see>.</param>
        /// <param name="str">The string.</param>
        private static void AppendTabbed(StringBuilder buffer, string str) {
            const string Tab = "    ";
            buffer.Append(Tab);
            foreach (char c in str) {
                if (c == '\n') {
                    buffer.AppendLine();
                    buffer.Append(Tab);
                } else {
                    buffer.Append(c);
                }
            }
        }

        /// <summary>
        /// Appends the undetailed <see cref="Parent">parent</see>
        /// <see cref="State.ToString(bool)">string representation</see> to a <see cref="StringBuilder">buffer</see>.
        /// </summary>
        /// <param name="buffer">The buffer.</param>
        private void AppendParent(StringBuilder buffer) {
            buffer.Append("Parent:");
            if (Parent != NoParent) {
                buffer.AppendLine();
                AppendTabbed(buffer, Parent.ToString(false));
            } else {
                buffer.Append(" None");
            }
        }

        /// <summary>
        /// Appends the <see cref="AppliedAction">applied action</see>
        /// <see cref="Action.ToString">string representation</see> to a <see cref="StringBuilder">buffer</see>.
        /// </summary>
        /// <param name="buffer">The <see cref="StringBuilder">buffer</see>.</param>
        private void AppendAppliedAction(StringBuilder buffer) {
            buffer.Append("Applied action:");
            if (AppliedAction != NoAppliedAction) {
                buffer.AppendLine();
                AppendTabbed(buffer, AppliedAction.ToString());
            } else {
                buffer.Append(" None");
            }
        }

        /// <summary>
        /// Appends the <see cref="Facts">fact certainty list</see> <see cref="String">string</see>
        /// representation to a <see cref="StringBuilder">buffer</see>.
        /// </summary>
        /// <param name="buffer">The <see cref="StringBuilder">buffer</see>.</param>
        private void AppendFacts(StringBuilder buffer) {
            const int IntSize = 32;
            bool isFirst = true;
            int factsSectionCount = Facts.Length;
            int factsSection;
            buffer.Append("Facts: [");
            //for each fact
            for (int factsSectionIndex = 0; factsSectionIndex < factsSectionCount; factsSectionIndex++) {
                factsSection = Facts[factsSectionIndex];
                for (int fact = 0; fact < IntSize; fact++) {
                    //if the fact is true
                    if ((factsSection & (1 << (fact % IntSize))) != 0) {
                        if (isFirst) {
                            isFirst = false;
                        } else {
                            buffer.Append(", ");
                        }
                        //appends the fact index to the buffer
                        buffer.Append((factsSectionIndex * IntSize) + fact);
                    }
                }
            }
            buffer.Append("]");
        }

        /// <summary>
        /// Appends the <see cref="Variables">variables</see> <see cref="String">string</see> representation
        /// to a <see cref="StringBuilder">buffer</see>.
        /// </summary>
        /// <param name="buffer">The <see cref="StringBuilder">buffer</see>.</param>
        private void AppendVariables(StringBuilder buffer) {
            bool isFirst = true;
            buffer.Append("Variables: [");
            isFirst = true;
            //for each variable
            foreach (float variable in Variables) {
                if (isFirst) {
                    isFirst = false;
                } else {
                    buffer.Append(", ");
                }
                //appends the variable value to the buffer
                buffer.Append(variable);
            }
            buffer.Append("]");
        }

        /// <summary>
        /// Returns a <see cref="String">string</see> which represents this <see cref="State">state</see>.
        /// </summary>
        /// <param name="detailed">If set to true the <see cref="String">string</see>
        /// representation will include information about the <see cref="Parent">parent</see> and the
        /// <see cref="AppliedAction">applied action</see>.</param>
        /// <returns>A <see cref="String">string</see> which represents this <see cref="State">state</see></returns>
        public string ToString(bool detailed) {
            StringBuilder buffer = new StringBuilder();
            if (detailed) {
                AppendParent(buffer);
                buffer.AppendLine();
                AppendAppliedAction(buffer);
                buffer.AppendLine();
            }
            AppendFacts(buffer);
            buffer.AppendLine();
            AppendVariables(buffer);
            return buffer.ToString();
        }

        /// <summary>
        /// Returns a <see cref="String">string</see> which represents this <see cref="State">state</see>.
        /// </summary>
        /// <remarks>
        /// The <see cref="String">string</see> will not include either information about the
        /// <see cref="Parent">parent</see> or the <see cref="AppliedAction">applied action</see>.
        /// </remarks>
        /// <returns>A <see cref="String">string</see> which represents this <see cref="State">state</see></returns>
        public override string ToString() {
            return ToString(true);
        }

        #endregion

    }

}
