/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Text;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.Globalization;

namespace Aran.Planning.Xml.Command {

    /// <summary>
    /// Represents the resources for this namespace classes.
    /// </summary>
    [Serializable]
    internal sealed class Resources {

        #region Constants

        /// <summary>
        /// The English of United Kingdom culture.
        /// </summary>
        internal static readonly CultureInfo English = CultureInfo.GetCultureInfo("en-GB");

        /// <summary>
        /// The neutral English culture.
        /// </summary>
        internal static readonly CultureInfo NeutralEnglish = new CultureInfo("en");

        #endregion

        #region Fields

        /// <summary>
        /// The underlying resource manager.
        /// </summary>
        private readonly ResourceManager _resources;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Resources"/> class.
        /// </summary>
        internal Resources() {
            try {
                SetCulture(Thread.CurrentThread.CurrentCulture);
            } catch (NotSupportedException) {
                SetCulture(English);
            }
            _resources = new ResourceManager(GetType());
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the tile.
        /// </summary>
        /// <value>The title.</value>
        internal string Title {
            get {
                return _resources.GetString("Title", Culture);
            }
        }

        /// <summary>
        /// Gets the "version" word.
        /// </summary>
        /// <value>The "version" word.</value>
        internal string Version {
            get {
                return _resources.GetString("Version", Culture);
            }
        }

        /// <summary>
        /// Gets the license notice.
        /// </summary>
        /// <value>The license notice.</value>
        internal string LicenseNotice {
            get {
                return _resources.GetString("LicenseNotice", Culture);
            }
        }

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        private static CultureInfo Culture {
            get {
                return CultureInfo.CurrentUICulture;
            }
        }

        /// <summary>
        /// Gets the "null arguments" message.
        /// </summary>
        /// <value>The "null arguments" message.</value>
        internal string NullArguments {
            get {
                return _resources.GetString("NullArguments", Culture);
            }
        }

        /// <summary>
        /// Gets the planner message format.
        /// </summary>
        /// <value>The planner message format.</value>
        private string MessageFormat {
            get {
                return _resources.GetString("MessageFormat", Culture);
            }
        }

        /// <summary>
        /// Gets the "warning" word.
        /// </summary>
        /// <value>The "warning" word.</value>
        private string Warning {
            get {
                return _resources.GetString("Warning", Culture);
            }
        }

        /// <summary>
        /// Gets the "error" word.
        /// </summary>
        /// <value>The "error" word.</value>
        private string Error {
            get {
                return _resources.GetString("Error", Culture);
            }
        }

        /// <summary>
        /// Gets the "parser error" words.
        /// </summary>
        /// <value>The "parser error" words.</value>
        private string ParserError {
            get {
                return _resources.GetString("ParserError", Culture);
            }
        }

        /// <summary>
        /// Gets the "minute" word.
        /// </summary>
        /// <value>The "minute" word.</value>
        private string Minute {
            get {
                return _resources.GetString("Minute", Culture);
            }
        }

        /// <summary>
        /// Gets the "minutes" word.
        /// </summary>
        /// <value>The "minutes" word.</value>
        private string Minutes {
            get {
                return _resources.GetString("Minutes", Culture);
            }
        }

        /// <summary>
        /// Gets the "second" word.
        /// </summary>
        /// <value>The "second" word.</value>
        private string Second {
            get {
                return _resources.GetString("Second", Culture);
            }
        }

        /// <summary>
        /// Gets the "seconds" word.
        /// </summary>
        /// <value>The "seconds" word.</value>
        private string Seconds {
            get {
                return _resources.GetString("Seconds", Culture);
            }
        }

        /// <summary>
        /// Gets the "millisecond" word.
        /// </summary>
        /// <value>The "millisecond" word.</value>
        private string Millisecond {
            get {
                return _resources.GetString("Millisecond", Culture);
            }
        }

        /// <summary>
        /// Gets the "milliseconds" word.
        /// </summary>
        /// <value>The "milliseconds" word.</value>
        private string Milliseconds {
            get {
                return _resources.GetString("Milliseconds", Culture);
            }
        }

        /// <summary>
        /// Gets the time format.
        /// </summary>
        /// <value>The time format.</value>
        private string TimeFormat {
            get {
                return _resources.GetString("TimeFormat", Culture);
            }
        }

        /// <summary>
        /// Gets the time message format.
        /// </summary>
        /// <value>The time message format.</value>
        private string TimeMessageFormat {
            get {
                return _resources.GetString("TimeMessageFormat", Culture);
            }
        }

        /// <summary>
        /// Gets the "translating" word.
        /// </summary>
        /// <value>The "translating" word.</value>
        internal string Translating {
            get {
                return _resources.GetString("Translating", Culture);
            }
        }

        /// <summary>
        /// Gets the "translating time" words.
        /// </summary>
        /// <value>The "translating time" words.</value>
        private string TranslatingTime {
            get {
                return _resources.GetString("TranslatingTime", Culture);
            }
        }

        /// <summary>
        /// Gets the "searching time" words.
        /// </summary>
        /// <value>The "searching time" words.</value>
        private string SearchingTime {
            get {
                return _resources.GetString("SearchingTime", Culture);
            }
        }

        /// <summary>
        /// Gets the "searching" word.
        /// </summary>
        /// <value>The "searching" word.</value>
        internal string Searching {
            get {
                return _resources.GetString("Searching", Culture);
            }
        }

        /// <summary>
        /// Gets the "exporting input schema" message.
        /// </summary>
        /// <value>The "exporting input schema" message.</value>
        internal string ExportingInputSchema {
            get {
                return _resources.GetString("ExportingInputSchema", Culture);
            }
        }

        /// <summary>
        /// Gets the "exporting output schema" message.
        /// </summary>
        /// <value>The "exporting output schema" message.</value>
        internal string ExportingOutputSchema {
            get {
                return _resources.GetString("ExportingOutputSchema", Culture);
            }
        }

        /// <summary>
        /// Gets the "exporting license" message.
        /// </summary>
        /// <value>The "exporting license" message.</value>
        internal string ExportingLicense {
            get {
                return _resources.GetString("ExportingLicense", Culture);
            }
        }

        /// <summary>
        /// Gets the "export done" message.
        /// </summary>
        /// <value>The "export done" message.</value>
        internal string ExportDone {
            get {
                return _resources.GetString("ExportDone", Culture);
            }
        }

        /// <summary>
        /// Gets the "plan found" message.
        /// </summary>
        /// <value>The "plan found" message.</value>
        internal string PlanFound {
            get {
                return _resources.GetString("PlanFound", Culture);
            }
        }

        /// <summary>
        /// Gets the "plan not found" message.
        /// </summary>
        /// <value>The "plan not found" message.</value>
        internal string PlanNotFound {
            get {
                return _resources.GetString("PlanNotFound", Culture);
            }
        }

        /// <summary>
        /// Gets the "unknown algorithm" message.
        /// </summary>
        /// <value>The "unknown algorithm" message.</value>
        internal string UnknownAlgorithm {
            get {
                return _resources.GetString("UnknownAlgorithm", Culture);
            }
        }

        /// <summary>
        /// Gets the "usage" word.
        /// </summary>
        /// <value>The "usage" word.</value>
        internal string Usage {
            get {
                return _resources.GetString("Usage", Culture);
            }
        }

        /// <summary>
        /// Gets the "options" word.
        /// </summary>
        /// <value>The "options" word.</value>
        internal string Options {
            get {
                return _resources.GetString("Options", Culture);
            }
        }

        /// <summary>
        /// Gets the "available options" words.
        /// </summary>
        /// <value>The "available options" words.</value>
        internal string AvailableOptions {
            get {
                return _resources.GetString("AvailableOptions", Culture);
            }
        }

        /// <summary>
        /// Gets the "file" word.
        /// </summary>
        /// <value>The "file" word.</value>
        internal string File {
            get {
                return _resources.GetString("File", Culture);
            }
        }

        /// <summary>
        /// Gets the "length" word.
        /// </summary>
        /// <value>The "length" word.</value>
        internal string Length {
            get {
                return _resources.GetString("Length", Culture);
            }
        }

        /// <summary>
        /// Gets the milliseconds type name.
        /// </summary>
        /// <value>The milliseconds type name.</value>
        internal string MillisecondsType {
            get {
                return _resources.GetString("MillisecondsType", Culture);
            }
        }

        /// <summary>
        /// Gets the "algorithm" word.
        /// </summary>
        /// <value>The "algorithm" word.</value>
        internal string Algorithm {
            get {
                return _resources.GetString("Algorithm", Culture);
            }
        }

        /// <summary>
        /// Gets the "width" word.
        /// </summary>
        /// <value>The "width" word.</value>
        internal string Width {
            get {
                return _resources.GetString("Width", Culture);
            }
        }

        /// <summary>
        /// Gets the "weight" word.
        /// </summary>
        /// <value>The "weight" word.</value>
        internal string Weight {
            get {
                return _resources.GetString("Weight", Culture);
            }
        }

        /// <summary>
        /// Gets the "type" word.
        /// </summary>
        /// <value>The "type" word.</value>
        internal string Type {
            get {
                return _resources.GetString("Type", Culture);
            }
        }

        /// <summary>
        /// Gets the "predicate" word.
        /// </summary>
        /// <value>The "predicate" word.</value>
        internal string Predicate {
            get {
                return _resources.GetString("Predicate", Culture);
            }
        }

        /// <summary>
        /// Gets the "encoding" word.
        /// </summary>
        /// <value>The "encoding" word.</value>
        internal string Encoding {
            get {
                return _resources.GetString("Encoding", Culture);
            }
        }

        /// <summary>
        /// Gets the "language" word.
        /// </summary>
        /// <value>The "language" word.</value>
        internal string Language {
            get {
                return _resources.GetString("Language", Culture);
            }
        }

        /// <summary>
        /// Gets the problem encoding help text.
        /// </summary>
        /// <value>The problem encoding help text.</value>
        internal string ProblemEncodingEnumHelp {
            get {
                return _resources.GetString("ProblemEncodingEnumHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the domain encoding help text.
        /// </summary>
        /// <value>The domain encoding help text.</value>
        internal string DomainEncodingEnumHelp {
            get {
                return _resources.GetString("DomainEncodingEnumHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "XML encoding name" words.
        /// </summary>
        /// <value>The "XML encoding name" words.</value>
        internal string XmlEncodingName {
            get {
                return _resources.GetString("XmlEncodingName", Culture);
            }
        }

        /// <summary>
        /// Gets the custom encoding help text.
        /// </summary>
        /// <value>The custom encoding help text.</value>
        internal string CustomEncodingEnumHelp {
            get {
                return _resources.GetString("CustomEncodingEnumHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "default" word.
        /// </summary>
        /// <value>The "default" word.</value>
        internal string Default {
            get {
                return _resources.GetString("Default", Culture);
            }
        }

        /// <summary>
        /// Gets the short format.
        /// </summary>
        /// <value>The short format.</value>
        internal string ShortFormat {
            get {
                return _resources.GetString("ShortFormat", Culture);
            }
        }

        /// <summary>
        /// Gets the "default first" words.
        /// </summary>
        /// <value>The "default first" words.</value>
        internal string DefaultFirst {
            get {
                return _resources.GetString("DefaultFirst", Culture);
            }
        }

        /// <summary>
        /// Gets the "default second" words.
        /// </summary>
        /// <value>The "default second" words.</value>
        internal string DefaultSecond {
            get {
                return _resources.GetString("DefaultSecond", Culture);
            }
        }

        /// <summary>
        /// Gets the problem option help text.
        /// </summary>
        /// <value>The problem option help text.</value>
        internal string ProblemHelp {
            get {
                return _resources.GetString("ProblemHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the domain option help text.
        /// </summary>
        /// <value>The domain option help text.</value>
        internal string DomainHelp {
            get {
                return _resources.GetString("DomainHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the out option help text.
        /// </summary>
        /// <value>The out option help text.</value>
        internal string OutHelp {
            get {
                return _resources.GetString("OutHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the language option help text.
        /// </summary>
        /// <value>The language option help text.</value>
        internal string LanguageHelp {
            get {
                return _resources.GetString("LanguageHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the overwrite option help text.
        /// </summary>
        /// <value>The overwrite option help text.</value>
        internal string OverwriteHelp {
            get {
                return _resources.GetString("OverwriteHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the maxPlanLength option help text.
        /// </summary>
        /// <value>The maxPlanLength option help text.</value>
        internal string MaxPlanLengthHelp {
            get {
                return _resources.GetString("MaxPlanLengthHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the maxTime option help text.
        /// </summary>
        /// <value>The maxTime option help text.</value>
        internal string MaxTimeHelp {
            get {
                return _resources.GetString("MaxTimeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the algorithm option help text.
        /// </summary>
        /// <value>The algorithm option help text.</value>
        internal string AlgorithmHelp {
            get {
                return _resources.GetString("AlgorithmHelp", Culture) + ":";
            }
        }

        /// <summary>
        /// Gets the algorithm option use help text.
        /// </summary>
        /// <value>The algorithm option use help text.</value>
        internal string AlgorithmUseHelp {
            get {
                return _resources.GetString("AlgorithmUseHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "exit codes" text.
        /// </summary>
        /// <value>The "exit codes" text.</value>
        internal string ExitCodes {
            get {
                return _resources.GetString("ExitCodes", Culture);
            }
        }

        /// <summary>
        /// Gets the "all OK" exit code help text.
        /// </summary>
        /// <value>The "all OK" exit code help text.</value>
        internal string AllOkExitCodeHelp {
            get {
                return _resources.GetString("AllOkExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "plan not found" exit code help text.
        /// </summary>
        /// <value>The "plan not found" exit code help text.</value>
        internal string PlanNotFoundExitCodeHelp {
            get {
                return _resources.GetString("PlanNotFoundExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "invalid arguments" exit code help text.
        /// </summary>
        /// <value>The "invalid arguments" exit code help text.</value>
        internal string InvalidArgumentsExitCodeHelp {
            get {
                return _resources.GetString("InvalidArgumentsExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "translation error" exit code help text.
        /// </summary>
        /// <value>The "translation error" exit code help text.</value>
        internal string TranslationErrorExitCodeHelp {
            get {
                return _resources.GetString("TranslationErrorExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "export error" exit code help text.
        /// </summary>
        /// <value>The "export error" exit code help text.</value>
        internal string ExportErrorExitCodeHelp {
            get {
                return _resources.GetString("ExportErrorExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "internal error" exit code help text.
        /// </summary>
        /// <value>The "internal error" exit code help text.</value>
        internal string InternalErrorExitCodeHelp {
            get {
                return _resources.GetString("InternalErrorExitCodeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the A* algorithm name.
        /// </summary>
        /// <value>The A* algorithm name.</value>
        internal string AStarName {
            get {
                return _resources.GetString("AStarName", Culture);
            }
        }

        /// <summary>
        /// Gets the Weighted A* algorithm name.
        /// </summary>
        /// <value>The Weighted A* algorithm name.</value>
        internal string WeightedAStarName {
            get {
                return _resources.GetString("WeightedAStarName", Culture);
            }
        }

        /// <summary>
        /// Gets the Beam Search algorithm name.
        /// </summary>
        /// <value>The Beam Search algorithm name.</value>
        internal string BeamSearchName {
            get {
                return _resources.GetString("BeamSearchName", Culture);
            }
        }

        /// <summary>
        /// Gets the Dijkstra algorithm name.
        /// </summary>
        /// <value>The Dijkstra algorithm name.</value>
        internal string DijkstraName {
            get {
                return _resources.GetString("DijkstraName", Culture);
            }
        }

        /// <summary>
        /// Gets the Enforced Hill Climbing algorithm name.
        /// </summary>
        /// <value>The Enforced Hill Climbing algorithm name.</value>
        internal string EnforcedHillClimbingName {
            get {
                return _resources.GetString("EnforcedHillClimbingName", Culture);
            }
        }

        /// <summary>
        /// Gets the Hill Climbing algorithm name.
        /// </summary>
        /// <value>The Hill Climbing algorithm name.</value>
        internal string HillClimbingName {
            get {
                return _resources.GetString("HillClimbingName", Culture);
            }
        }

        /// <summary>
        /// Gets the beamWidth option help text.
        /// </summary>
        /// <value>The beamWidth option help text.</value>
        internal string BeamWidthHelp {
            get {
                return _resources.GetString("BeamWidthHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the heuristicWeight option help text.
        /// </summary>
        /// <value>The heuristicWeight option help text.</value>
        internal string HeuristicWeightHelp {
            get {
                return _resources.GetString("HeuristicWeightHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the rootType option help text.
        /// </summary>
        /// <value>The rootType option help text.</value>
        internal string RootTypeHelp {
            get {
                return _resources.GetString("RootTypeHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the noRoot option help text.
        /// </summary>
        /// <value>The noRoot option help text.</value>
        internal string NoRootHelp {
            get {
                return _resources.GetString("NoRootHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the rootInequality option help text.
        /// </summary>
        /// <value>The rootInequality option help text.</value>
        internal string RootInequalityHelp {
            get {
                return _resources.GetString("RootInequalityHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the noRootInequality option help text.
        /// </summary>
        /// <value>The noRootInequality option help text.</value>
        internal string NoRootInequalityHelp {
            get {
                return _resources.GetString("NoRootInequalityHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the inequality option help text.
        /// </summary>
        /// <value>The inequality option help text.</value>
        internal string InequalityHelp {
            get {
                return _resources.GetString("InequalityHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the noWarn option help text.
        /// </summary>
        /// <value>The noWarn option help text.</value>
        internal string NoWarnHelp {
            get {
                return _resources.GetString("NoWarnHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the warnAsError option help text.
        /// </summary>
        /// <value>The warnAsError option help text.</value>
        internal string WarnAsErrorHelp {
            get {
                return _resources.GetString("WarnAsErrorHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the exportInputSchema option help text.
        /// </summary>
        /// <value>The exportInputSchema option help text.</value>
        internal string ExportInputSchemaHelp {
            get {
                return _resources.GetString("ExportInputSchemaHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the exportOutputSchema option help text.
        /// </summary>
        /// <value>The exportOutputSchema option help text.</value>
        internal string ExportOutputSchemaHelp {
            get {
                return _resources.GetString("ExportOutputSchemaHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the exportLicence option help text.
        /// </summary>
        /// <value>The exportLicence option help text.</value>
        internal string ExportLicenseHelp {
            get {
                return _resources.GetString("ExportLicenseHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the noLogo option help text.
        /// </summary>
        /// <value>The noLogo option help text.</value>
        internal string NoLogoHelp {
            get {
                return _resources.GetString("NoLogoHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the verbose option help text.
        /// </summary>
        /// <value>The verbose option help text.</value>
        internal string VerboseHelp {
            get {
                return _resources.GetString("VerboseHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the help option help text.
        /// </summary>
        /// <value>The help option help text.</value>
        internal string HelpHelp {
            get {
                return _resources.GetString("HelpHelp", Culture);
            }
        }

        /// <summary>
        /// Gets the "examples" word.
        /// </summary>
        /// <value>The "examples" word.</value>
        internal string Examples {
            get {
                return _resources.GetString("Examples", Culture);
            }
        }

        /// <summary>
        /// Gets the domain example file name.
        /// </summary>
        /// <value>The domain example file name.</value>
        internal string DomainExample {
            get {
                return _resources.GetString("DomainExample", Culture);
            }
        }

        /// <summary>
        /// Gets the problem example file name.
        /// </summary>
        /// <value>The problem example file name.</value>
        internal string ProblemExample {
            get {
                return _resources.GetString("ProblemExample", Culture);
            }
        }

        /// <summary>
        /// Gets the domain and problem example file name.
        /// </summary>
        /// <value>The domain and problem example file name.</value>
        internal string DomainAndProblemExample {
            get {
                return _resources.GetString("DomainAndProblemExample", Culture);
            }
        }

        /// <summary>
        /// Gets the out example file name.
        /// </summary>
        /// <value>The out example file name.</value>
        internal string OutExample {
            get {
                return _resources.GetString("OutExample", Culture);
            }
        }

#if !DEBUG
        /// <summary>
        /// Gets the "internal error" message.
        /// </summary>
        /// <value>The "internal error" message.</value>
        internal string InternalError {
            get {
                return _resources.GetString("InternalError", Culture);
            }
        }
#endif

        #endregion

        #region Methods

        /// <summary>
        /// Gets the neutral culture of a specified one.
        /// </summary>
        /// <param name="culture">The culture.</param>
        /// <returns>The neutral culture of <paramref name="culture"/>.</returns>
        private static CultureInfo GetNeutral(CultureInfo culture) {
            const char Separator = '-';
            CultureInfo neutralCulture = culture;
            string name = culture.Name;
            int indexOfSeparator = name.IndexOf(Separator);
            if (indexOfSeparator > 0) {
                string neutralName = name.Substring(0, indexOfSeparator);
                neutralCulture = new CultureInfo(neutralName);
            }
            return neutralCulture;
        }

        /// <summary>
        /// Determines whether the specified culture is supported.
        /// </summary>
        /// <param name="culture">The culture.</param>
        /// <remarks>A culture is supported if there is a satellite assembly for that culture or the
        /// corresponding neutral one. Neutral English culture is the default one, so it is supported although
        /// there is no satellite assembly it.</remarks>
        /// <returns>
        /// True if <paramref name="culture"/> is supported; otherwise, false.
        /// </returns>
        private static bool IsSupported(CultureInfo culture) {
            bool isSupported = !culture.IsNeutralCulture;
            if (isSupported) {
                Assembly executingAssembly = Assembly.GetExecutingAssembly();
                try {
                    executingAssembly.GetSatelliteAssembly(culture);
                } catch (FileNotFoundException) {
                    CultureInfo neutralCulture = GetNeutral(culture);
                    try {
                        executingAssembly.GetSatelliteAssembly(neutralCulture);
                    } catch (FileNotFoundException) {
                        isSupported = neutralCulture.Equals(NeutralEnglish);
                    }
                }
            }
            return isSupported;
        }

        /// <summary>
        /// Sets the culture.
        /// </summary>
        /// <param name="culture">The culture.</param>
        /// <exception cref="NotSupportedException">The culture is not supported.</exception>
        internal static void SetCulture(CultureInfo culture) {
            if (!IsSupported(culture)) {
                throw new NotSupportedException();
            }
            Thread currentThread = Thread.CurrentThread;
            currentThread.CurrentCulture = culture;
            currentThread.CurrentUICulture = culture;
        }

        /// <summary>
        /// Gets a parametrized message.
        /// </summary>
        /// <param name="name">The message name.</param>
        /// <param name="arguments">The arguments of the message.</param>
        /// <returns>The parametrized message.</returns>
        private string GetMessage(string name, params object[] arguments) {
            CultureInfo culture = Culture;
            return String.Format(culture, _resources.GetString(name, culture), arguments);
        }

        /// <summary>
        /// Gets a formatted planner message.
        /// </summary>
        /// <param name="type">The message type.</param>
        /// <param name="text">The message text.</param>
        /// <returns></returns>
        private string GetFormattedMessage(string type, string text) {
            return String.Format(Culture, MessageFormat, type, text);
        }

        /// <summary>
        /// Gets a warning planner message.
        /// </summary>
        /// <param name="text">The message text.</param>
        /// <returns>The warning planner message.</returns>
        internal string GetWarningMessage(string text) {
            return GetFormattedMessage(Warning, text);
        }

        /// <summary>
        /// Gets an error planner message.
        /// </summary>
        /// <param name="text">The message text.</param>
        /// <returns>The error planner message.</returns>
        internal string GetErrorMessage(string text) {
            return GetFormattedMessage(Error, text);
        }

        /// <summary>
        /// Gets a parser error planner message.
        /// </summary>
        /// <param name="text">The message text.</param>
        /// <returns>The parser error planner message.</returns>
        internal string GetParserErrorMessage(string text) {
            return GetFormattedMessage(ParserError, text);
        }

        /// <summary>
        /// Transforms a time into text.
        /// </summary>
        /// <param name="time">The time.</param>
        /// <returns>The text corresponding to <paramref name="time"/>.</returns>
        private string GetTime(TimeSpan time) {
            int minutes = time.Minutes;
            int seconds = time.Seconds;
            int milliseconds = time.Milliseconds;
            string minutesString = null;
            string secondsString = null;
            string millisecondsString = null;
            CultureInfo culture = Culture;
            if (minutes == 1) {
                minutesString = String.Format(culture, Minute, minutes);
            } else {
                minutesString = String.Format(culture, Minutes, minutes);
            }
            if (seconds == 1) {
                secondsString = String.Format(culture, Second, seconds);
            } else {
                secondsString = String.Format(culture, Seconds, seconds);
            }
            if (milliseconds == 1) {
                millisecondsString = String.Format(culture, Millisecond, milliseconds);
            } else {
                millisecondsString = String.Format(culture, Milliseconds, milliseconds);
            }
            return String.Format(culture, TimeFormat, minutesString, secondsString, millisecondsString);
        }

        /// <summary>
        /// Gets a time planner message.
        /// </summary>
        /// <param name="type">The message type.</param>
        /// <param name="time">The time.</param>
        /// <returns>The time planner message.</returns>
        private string GetTimeMessage(string type, TimeSpan time) {
            return String.Format(Culture, TimeMessageFormat, type, GetTime(time));
        }

        /// <summary>
        /// Gets a translating time planner message.
        /// </summary>
        /// <param name="time">The time.</param>
        /// <returns>The translating time planner message.</returns>
        internal string GetTranslatingTimeMessage(TimeSpan time) {
            return GetTimeMessage(TranslatingTime, time);
        }

        /// <summary>
        /// Gets a searching time planner message.
        /// </summary>
        /// <param name="time">The time.</param>
        /// <returns>The searching time planner message.</returns>
        internal string GetSearchingTimeMessage(TimeSpan time) {
            return GetTimeMessage(SearchingTime, time);
        }

        /// <summary>
        /// Gets a "null argument" message.
        /// </summary>
        /// <param name="position">The argument position.</param>
        /// <returns>The "null argument" message.</returns>
        internal string GetNullArgument(int position) {
            return GetMessage("NullArgument", position);
        }

        /// <summary>
        /// Gets a "invalid option" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid option" message.</returns>
        internal string GetInvalidOption(string option) {
            return GetMessage("InvalidOption", option);
        }

        /// <summary>
        /// Gets a "no file specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no file specified" message.</returns>
        internal string GetNoFileSpecified(string option) {
            return GetMessage("NoFileSpecified", option);
        }

        /// <summary>
        /// Gets a "no length specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no length specified" message.</returns>
        internal string GetNoLengthSpecified(string option) {
            return GetMessage("NoLengthSpecified", option);
        }

        /// <summary>
        /// Gets a "no type specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no type specified" message.</returns>
        internal string GetNoTypeSpecified(string option) {
            return GetMessage("NoTypeSpecified", option);
        }

        /// <summary>
        /// Gets a "no milliseconds specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no milliseconds specified" message.</returns>
        internal string GetNoMillisecondsSpecified(string option) {
            return GetMessage("NoMillisecondsSpecified", option);
        }

        /// <summary>
        /// Gets a "no algorithm specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no algorithm specified" message.</returns>
        internal string GetNoAlgorithmSpecified(string option) {
            return GetMessage("NoAlgorithmSpecified", option);
        }

        /// <summary>
        /// Gets a "no width specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no width specified" message.</returns>
        internal string GetNoWidthSpecified(string option) {
            return GetMessage("NoWidthSpecified", option);
        }

        /// <summary>
        /// Gets a "no weight specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no weight specified" message.</returns>
        internal string GetNoWeightSpecified(string option) {
            return GetMessage("NoWeightSpecified", option);
        }

        /// <summary>
        /// Gets a "no predicate specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no predicate specified" message.</returns>
        internal string GetNoPredicateSpecified(string option) {
            return GetMessage("NoPredicateSpecified", option);
        }

        /// <summary>
        /// Gets a "no encoding specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no encoding specified" message.</returns>
        internal string GetNoEncodingSpecified(string option) {
            return GetMessage("NoEncodingSpecified", option);
        }

        /// <summary>
        /// Gets a "no language specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "no language specified" message.</returns>
        internal string GetNoLanguageSpecified(string option) {
            return GetMessage("NoLanguageSpecified", option);
        }

        /// <summary>
        /// Gets an "insufficient write permissions" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "insufficient write permissions" message.</returns>
        internal string GetInsufficientWritePermissions(string path) {
            return GetMessage("InsufficientWritePermissions", path);
        }

        /// <summary>
        /// Gets an "invalid path" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "invalid path" message.</returns>
        internal string GetInvalidPath(string path) {
            return GetMessage("InvalidPath", path);
        }

        /// <summary>
        /// Gets a "file already exists" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "file already exists" message.</returns>
        internal string GetFileAlreadyExists(string path) {
            return GetMessage("FileAlreadyExists", path);
        } 

        /// <summary>
        /// Gets a "file can not be written" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "file can not be written" message.</returns>
        internal string GetFileCanNotBeWritten(string path) {
            return GetMessage("FileCanNotBeWritten", path);
        }

        /// <summary>
        /// Gets an "invalid length specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid length specified" message.</returns>
        internal string GetInvalidLengthSpecified(string option) {
            return GetMessage("InvalidLengthSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid milliseconds specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid milliseconds specified" message.</returns>
        internal string GetInvalidMillisecondsSpecified(string option) {
            return GetMessage("InvalidMillisecondsSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid algorithm specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid algorithm specified" message.</returns>
        internal string GetInvalidAlgorithmSpecified(string option) {
            return GetMessage("InvalidAlgorithmSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid width specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid width specified" message.</returns>
        internal string GetInvalidWidthSpecified(string option) {
            return GetMessage("InvalidWidthSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid weight specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid weight specified" message.</returns>
        internal string GetInvalidWeightSpecified(string option) {
            return GetMessage("InvalidWeightSpecified", option);
        }

        /// <summary>
        /// Gets an "unsupported algorithm option" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <returns>The "unsupported algorithm option" message and
        /// <paramref name="algorithmName"/>.</returns>
        internal string GetUnsupportedAlgorithmOption(string option, string algorithmName) {
            return GetMessage("UnsupportedAlgorithmOption", option, algorithmName);
        }

        /// <summary>
        /// Gets an "algorithm option already specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "algorithm option already specified".</returns>
        internal string GetAlgorithmOptionAlreadySpecified(string option) {
            return GetMessage("AlgorithmOptionAlreadySpecified", option);
        }

        /// <summary>
        /// Gets an "option already specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "option already specified".</returns>
        internal string GetOptionAlreadySpecified(string option) {
            return GetMessage("OptionAlreadySpecified", option);
        }

        /// <summary>
        /// Gets an "invalid encoding specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid encoding specified".</returns>
        internal string GetInvalidEncodingSpecified(string option) {
            return GetMessage("InvalidEncodingSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid language specified" message.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <returns>The "invalid language specified".</returns>
        internal string GetInvalidLanguageSpecified(string option) {
            return GetMessage("InvalidLanguageSpecified", option);
        }

        /// <summary>
        /// Gets an "invalid inequality specified" message.
        /// </summary>
        /// <param name="predicate">The predicate name.</param>
        /// <returns>The "invalid inequality specified".</returns>
        internal string GetInequalityAlreadySpecified(string predicate) {
            return GetMessage("InequalityAlreadySpecified", predicate);
        }

        /// <summary>
        /// Gets an "invalid root inequality specified" message.
        /// </summary>
        /// <param name="predicate">The predicate name.</param>
        /// <returns>The "invalid root inequality specified".</returns>
        internal string GetRootInequalityAlreadySpecified(string predicate) {
            return GetMessage("RootInequalityAlreadySpecified", predicate);
        }

        /// <summary>
        /// Gets a file information message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <param name="lineNumber">The file line number.</param>
        /// <param name="linePosition">The file line position.</param>
        /// <returns>The file information message, <paramref name="lineNumber"/>
        /// and <paramref name="linePosition"/>.</returns>
        internal string GetFileInfo(string path, int lineNumber, int linePosition) {
            return GetMessage("FileInfoFormat", path, lineNumber, linePosition);
        }

        /// <summary>
        /// Gets the help tip text.
        /// </summary>
        /// <param name="executableName">The executable name.</param>
        /// <param name="helpOption">The help option name.</param>
        /// <returns>The help tip text.</returns>
        internal string GetShowHelpTip(string executableName, string helpOption) {
            return GetMessage("ShowHelpTip", executableName, helpOption);
        }

        /// <summary>
        /// Gets an "algorithm selected" message.
        /// </summary>
        /// <param name="algorithmDescription">The algorithm description.</param>
        /// <returns>The "algorithm selected" message.</returns>
        internal string GetAlgorithmSelected(string algorithmDescription) {
            return GetMessage("AlgorithmSelected", algorithmDescription);
        }

        /// <summary>
        /// Gets a default value text.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The default value text.</returns>
        internal string GetDefaultValue(string value) {
            const string DefaultValueFormat = "({0}: {1})";
            return String.Format(Culture, DefaultValueFormat, Default, value);
        }

        /// <summary>
        /// Gets a short format text.
        /// </summary>
        /// <param name="format">The short format.</param>
        /// <returns>The short format text.</returns>
        internal string GetShortFormat(string format) {
            const string DefaultShortFormatFormat = "({0}: {1})";
            return String.Format(Culture, DefaultShortFormatFormat, ShortFormat, format);
        }

        /// <summary>
        /// Gets an algorithm help text.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <returns>The algorithm help text.</returns>
        internal string GetAlgorithmEnumHelp(string option, string algorithmName) {
            const string EnumFormat = "- {0}, {1}";
            return String.Format(Culture, EnumFormat, option, GetMessage("AlgorithmEnumHelp", algorithmName));
        }

        /// <summary>
        /// Gets an algorithm help text.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <param name="shortFormat">The option short format.</param>
        /// <returns>The algorithm help text.</returns>
        internal string GetAlgorithmEnumHelp(string option, string algorithmName, string shortFormat) {
            const char Space = ' ';
            return GetAlgorithmEnumHelp(option, algorithmName) + Space + GetShortFormat(shortFormat);
        }

        /// <summary>
        /// Gets the default first algorithm help text.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <param name="shortFormat">The option short format.</param>
        /// <returns>The default first algorithm help text.</returns>
        internal string GetDefaultFirstAlgorithmEnumHelp(string option, string algorithmName, string shortFormat) {
            const string DefaultFormat = "{0} ({1})";
            return String.Format(Culture, DefaultFormat,
                GetAlgorithmEnumHelp(option, algorithmName, shortFormat), DefaultFirst);
        }

        /// <summary>
        /// Gets the default second algorithm help text.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <returns>The default second algorithm help text.</returns>
        internal string GetDefaultSecondAlgorithmEnumHelp(string option, string algorithmName) {
            const string DefaultFormat = "{0} ({1})";
            return String.Format(Culture, DefaultFormat,
                GetAlgorithmEnumHelp(option, algorithmName), DefaultSecond);
        }

        /// <summary>
        /// Gets the algorithm option restriction help text.
        /// </summary>
        /// <param name="algorithmName">The algorithm name.</param>
        /// <returns>The algorithm option restriction help text.</returns>
        internal string GetAlgorithmOptionHelp(string algorithmName) {
            return GetMessage("AlgorithmOptionHelp", algorithmName);
        }

        /// <summary>
        /// Gets the encoding help text.
        /// </summary>
        /// <param name="shortFormat">The option short format.</param>
        /// <returns>The encoding help text.</returns>
        internal string GetEncodingHelp(string shortFormat) {
            string encodingHelp = _resources.GetString("EncodingHelp", Culture);
            return String.Format(Culture, "{0} {1}:", encodingHelp, GetShortFormat(shortFormat));
        }

        /// <summary>
        /// Gets an encoding option value help text.
        /// </summary>
        /// <param name="option">The option value name.</param>
        /// <param name="description">The value description.</param>
        /// <returns>The encoding option value help.</returns>
        internal static string GetEncodingEnumHelp(string option, string description) {
            const string EnumFormat = "- {0}, {1}";
            return String.Format(Culture, EnumFormat, option, description);
        }

        /// <summary>
        /// Gets the default encoding option value help text.
        /// </summary>
        /// <param name="option">The option value name.</param>
        /// <param name="description">The value description.</param>
        /// <returns>The default encoding option value help.</returns>
        internal string GetDefaultEncodingEnumHelp(string option, string description) {
            const string DefaultFormat = "{0} ({1})";
            return String.Format(Culture, DefaultFormat, GetEncodingEnumHelp(option, description), Default);
        }

        /// <summary>
        /// Gets a parametrized option name.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="parameterTypes">The parameter types.</param>
        /// <returns>The parametrized option name.</returns>
        internal static string GetOption(string option, params string[] parameterTypes) {
            StringBuilder buffer = new StringBuilder(option);
            CultureInfo culture = Culture;
            foreach (string type in parameterTypes) {
                buffer.AppendFormat(culture, " <{0}>", type);
            }
            return buffer.ToString();
        }

        #endregion

    }

}
