/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Xml;
using System.Text;
using System.Security;
using System.Reflection;
using System.Globalization;
using System.Collections.Generic;

using Option = Aran.Planning.Xml.Command.XmlPlannerCommandOption;
using ShortOption = Aran.Planning.Xml.Command.XmlPlannerCommandShortOption;
using DefaultValue = Aran.Planning.Xml.Command.XmlPlannerCommandDefaultValue;

namespace Aran.Planning.Xml.Command {

    /// <summary>
    /// Represents a command line planner which uses XML documents to read and write data.
    /// </summary>
    [CLSCompliant(true)]
    public sealed class XmlPlannerCommand {

        /*
         * NOTE: This class is too large. Some code should be placed into other helper classes in future
         * implementations. Help writing and option processing are candidates to be moved out of here.
         */

        #region Constants

        /// <summary>
        /// The string prefix used for text indentation.
        /// </summary>
        private const string MessageTab = "    ";

        /// <summary>
        /// The character used to mark translator error positions.
        /// </summary>
        private const char PositionMark = '^';

        /// <summary>
        /// The current plattform option prefix.
        /// </summary>
        private static char PlattformOptionPrefix;

        /// <summary>
        /// The alternative plattform option prefix.
        /// </summary>
        private static char AlternativePlattformOptionPrefix;

        /// <summary>
        /// The current plattform option format.
        /// </summary>
        private static readonly string PlattformOptionFormat;

        /// <summary>
        /// The current plattform help option short name.
        /// </summary>
        internal static readonly string PlattformHelpShortOption;

        /// <summary>
        /// The alternative plattform help option short name.
        /// </summary>
        private static readonly string AlternativePlattformHelpShortOption;

        #endregion

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The arguments.
        /// </summary>
        private readonly string[] _arguments;

        /// <summary>
        /// The <see cref="XmlPlanner">planner</see>.
        /// </summary>
        private readonly XmlPlanner _planner;

        /// <summary>
        /// If is set up.
        /// </summary>
        private bool _setUp;

        /// <summary>
        /// If the current algorithm is the first added to the <see cref="XmlPlanner.SearchAlgorithms">algorithm sequence</see>.
        /// </summary>
        private bool _isFirstAlgorithm;

        /// <summary>
        /// If help must be shown.
        /// </summary>
        private bool _showHelp;

        /// <summary>
        /// If output file can be overwritten.
        /// </summary>
        private bool _overwrite;

        /// <summary>
        /// If the inequality predicate for the root type has been set.
        /// </summary>
        private bool _rootInequalitySet;

        /// <summary>
        /// If the name of the inequality predicate for the root type.
        /// </summary>
        private string _rootInequality;

        /// <summary>
        /// If no inequality predicate for the root type must be defined.
        /// </summary>
        private bool _noRootInequality;

        /// <summary>
        /// If the information header must not be shown.
        /// </summary>
        private bool _noLogo;

        /// <summary>
        /// If the input XML schema must be exported.
        /// </summary>
        private bool _exportInputSchema;

        /// <summary>
        /// The path of the file in which the input XML schema must be exported.
        /// </summary>
        private string _inputSchemaExportPath;

        /// <summary>
        /// If the output XML schema must be exported.
        /// </summary>
        private bool _exportOutputSchema;

        /// <summary>
        /// The path of the file in which the output XML schema must be exported.
        /// </summary>
        private string _outputSchemaExportPath;

        /// <summary>
        /// If the license must be exported.
        /// </summary>
        private bool _exportLicense;

        /// <summary>
        /// The path of the file in which the license must be exported.
        /// </summary>
        private string _licenseExportPath;

        /// <summary>
        /// If no warning see <see cref="XmlPlanner">planner</see> messages must be shown.
        /// </summary>
        private bool _noWarn;

        /// <summary>
        /// If all warning see <see cref="XmlPlanner">planner</see> messages must be treated as errors.
        /// </summary>
        private bool _warnAsError;

        /// <summary>
        /// If detailed information must be shown.
        /// </summary>
        private bool _verbose;

        /// <summary>
        /// The errors generated while executing.
        /// </summary>
        private readonly IList<string> _errors;

        /// <summary>
        /// The messages returned by the see <see cref="XmlPlanner">planner</see>.
        /// </summary>
        private XmlPlannerMessageCollection _plannerMessages;

        /// <summary>
        /// If the messages returned by the see <see cref="XmlPlanner">planner</see> have been shown.
        /// </summary>
        private bool _plannerMessagesShown;

        /// <summary>
        /// A simple file XML see <see cref="Encoding">encoding</see> cache.
        /// </summary>
        private IDictionary<string, Encoding> _encodingCache;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes the <see cref="XmlPlannerCommand"/> class.
        /// </summary>
        static XmlPlannerCommand() {
            const string ArgumentFormat = "{0}";
            const char WindowsOptionPrefix = '/';
            const char UnixOptionPrefix = '-';
            const string WindowsHelpShortOption = "?";
            const string UnixHelpShortOption = "h";
            if (Environment.OSVersion.Platform == PlatformID.Unix) {
                PlattformOptionPrefix = UnixOptionPrefix;
                AlternativePlattformOptionPrefix = WindowsOptionPrefix;
                PlattformHelpShortOption = UnixOptionPrefix + UnixHelpShortOption;
                AlternativePlattformHelpShortOption = UnixOptionPrefix + WindowsHelpShortOption;
            } else {
                PlattformOptionPrefix = WindowsOptionPrefix;
                AlternativePlattformOptionPrefix = UnixOptionPrefix;
                PlattformHelpShortOption = WindowsOptionPrefix + WindowsHelpShortOption;
                AlternativePlattformHelpShortOption = WindowsOptionPrefix + UnixHelpShortOption;
            }
            PlattformOptionFormat = PlattformOptionPrefix + ArgumentFormat;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerCommand"/> class.
        /// </summary>
        /// <param name="arguments">The arguments.</param>
        /// <exception cref="ArgumentNullException"><paramref name="arguments"/> is a null reference.</exception>
        /// <exception cref="ArgumentException">An <paramref name="arguments"/> element is a null reference.</exception>
        public XmlPlannerCommand(params string[] arguments) {
            _resources = new Resources();
            //check arguments
            if (arguments == null) {
                throw new ArgumentNullException("arguments", _resources.NullArguments);
            }
            int argumentCount = arguments.Length;
            string argument;
            for (int argumentIndex = 0; argumentIndex < argumentCount; argumentIndex++) {
                argument = arguments[argumentIndex];
                if (argument == null) {
                    throw new ArgumentException("arguments", _resources.GetNullArgument(argumentIndex));
                }
            }
            //initialize
            _arguments = arguments;
            _planner = new XmlPlanner();
            _planner.StateChanged += Planner_StateChanged;
            _planner.SearchAlgorithmSelected += Planner_SearchAlgorithmSelected;
            _planner.RootTypeEnabled = true;
            _planner.DomainPath = XmlPlannerCommandDefaultValue.Domain;
            _planner.ProblemPath = XmlPlannerCommandDefaultValue.Problem;
            _planner.OutputPath = XmlPlannerCommandDefaultValue.Out;
            _rootInequality = XmlPlannerCommandDefaultValue.RootInequality;
            XmlPlannerSearchAlgorithmSequence algorithms = _planner.SearchAlgorithms;
            algorithms.ReplaceAllBy(XmlPlannerSearchAlgorithmId.EnforcedHillClimbing);
            algorithms.Add(XmlPlannerSearchAlgorithmId.AStar);
            _errors = new List<string>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the executable name.
        /// </summary>
        /// <value>The executable name.</value>
        private static string ExecutableName {
            get {
                return Environment.GetCommandLineArgs()[0];
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the localized option depending on plattform.
        /// </summary>
        /// <param name="option">The option.</param>
        /// <returns>The localized option.</returns>
        internal static string GetOptionDependingOnPlattform(string option) {
            return String.Format(CultureInfo.InvariantCulture, PlattformOptionFormat, option);
        }

        /// <summary>
        /// Gets the normalized option depending on plattform.
        /// </summary>
        /// <param name="option">The option.</param>
        /// <returns>The normalized option.</returns>
        private static string NormalizeOptionDependingOnPlattform(string option) {
            const int First= 0;
            const int Second = 1;
            const int MinimumOptionLength = 2;
            string normalizedOption = option;
            if ((option.Length >= MinimumOptionLength) && (option[First] == AlternativePlattformOptionPrefix)) {
                normalizedOption = PlattformOptionPrefix + option.Substring(Second);
            }
            if (normalizedOption.Equals(AlternativePlattformHelpShortOption,
            StringComparison.InvariantCultureIgnoreCase)) {
                normalizedOption = PlattformHelpShortOption;
            }
            return normalizedOption;
        }

        /// <summary>
        /// Checks if an option has been already specified.
        /// </summary>
        /// <param name="specifiedOptions">The specified options.</param>
        /// <param name="option">The option.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the option has been already specified.
        /// </remarks>
        private void CheckOptionUnicity(IDictionary<string, string> specifiedOptions, string option) {
            try {
                specifiedOptions.Add(option, option);
            } catch (ArgumentException) {
                _errors.Add(_resources.GetOptionAlreadySpecified(option));
            }
        }

        /// <summary>
        /// Determines whether a specified argument is equal to a specified option.
        /// </summary>
        /// <param name="option">The option.</param>
        /// <param name="argument">The argument.</param>
        /// <returns>
        /// True if <paramref name="argument"/> is equal to <paramref name="option"/>; otherwise, false.
        /// </returns>
        private static bool IsOption(string option, string argument) {
            string normalizedArgument = NormalizeOptionDependingOnPlattform(argument);
            return normalizedArgument.Equals(option, StringComparison.InvariantCultureIgnoreCase);
        }

        /// <summary>
        /// Determines whether a specified argument is equal to a specified option or its short format.
        /// </summary>
        /// <param name="option">The option.</param>
        /// <param name="shortOption">The option short format.</param>
        /// <param name="argument">The argument.</param>
        /// <returns>
        /// True if <paramref name="argument"/> is equal to <paramref name="option"/> or
        /// <paramref name="shortOption"/>; otherwise, false.
        /// </returns>
        private static bool IsOption(string option, string shortOption, string argument) {
            string normalizedArgument = NormalizeOptionDependingOnPlattform(argument);
            return (normalizedArgument.Equals(shortOption, StringComparison.InvariantCultureIgnoreCase) ||
                normalizedArgument.Equals(option, StringComparison.InvariantCultureIgnoreCase));
        }

        /// <summary>
        /// Sets up the domain path.
        /// </summary>
        /// <param name="option">The domain path option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the path is zero-length or no path is
        /// specified.
        /// </remarks>
        private void SetUpDomain(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _planner.DomainPath = path;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the problem path.
        /// </summary>
        /// <param name="option">The problem path option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the path is zero-length or no path is
        /// specified.
        /// </remarks>
        private void SetUpProblem(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _planner.ProblemPath = path;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the output path.
        /// </summary>
        /// <param name="option">The output path option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the path is zero-length or no path is
        /// specified.
        /// </remarks>
        private void SetUpOutput(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _planner.OutputPath = path;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Creates a search algorithm.
        /// </summary>
        /// <param name="option">The search algorithm option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <param name="algorithmName">The algorithm name specified by the search algorithm option value.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the algorithm name is unknown or no
        /// algorithm name is specified.
        /// </remarks>
        /// <returns>The search algorithm identified by <paramref name="algorithmName"/> or a null reference
        /// on error.</returns>
        private XmlPlannerSearchAlgorithm CreateSearchAlgorithm(string option,
        XmlPlannerCommandArgumentManager argumentManager, out string algorithmName) {
            XmlPlannerSearchAlgorithm algorithm = null;
            XmlPlannerSearchAlgorithmId algorithmId = XmlPlannerSearchAlgorithmId.AStar;
            algorithmName = null;
            try {
                algorithmName = argumentManager.ExtractArgument();
                try {
                    algorithmId = XmlPlannerCommandShortAlgorithm.GetAlgorithmId(algorithmName);
                } catch (ArgumentException) {
                    algorithmId = XmlPlannerCommandAlgorithm.GetAlgorithmId(algorithmName);
                }
                algorithm = XmlPlannerSearchAlgorithm.Create(algorithmId);
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidAlgorithmSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoAlgorithmSpecified(option));
            }
            return algorithm;
        }

        /// <summary>
        /// Sets the beam width of a search algorithm.
        /// </summary>
        /// <param name="option">The beam width option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <param name="algorithmName">The search algorithm name.</param>
        /// <param name="algorithm">The search algorithm.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the algorithm does not support the
        /// beam width option, the option value is invalid or no value is specified.
        /// </remarks>
        private void SetUpBeamWidth(string option, XmlPlannerCommandArgumentManager argumentManager,
        string algorithmName, XmlPlannerSearchAlgorithm algorithm) {
            try {
                int beamWidth = Int32.Parse(argumentManager.ExtractArgument(), Resources.English);
                try {
                    algorithm.BeamWidth = beamWidth;
                } catch (InvalidOperationException) {
                    _errors.Add(_resources.GetUnsupportedAlgorithmOption(option, algorithmName));
                }
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidWidthSpecified(option));
            } catch (OverflowException) {
                _errors.Add(_resources.GetInvalidWidthSpecified(option));
            } catch (FormatException) {
                _errors.Add(_resources.GetInvalidWidthSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoWidthSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the estimated remaining cost weight in the total estimated cost formula of a search
        /// algorithm.
        /// </summary>
        /// <param name="option">The heuristic weight option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <param name="algorithmName">The search algorithm name.</param>
        /// <param name="algorithm">The search algorithm.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the algorithm does not support the
        /// heuristic weight option, the option value is invalid or no value is specified.
        /// </remarks>
        private void SetUpHeuristicWeight(string option, XmlPlannerCommandArgumentManager argumentManager,
        string algorithmName, XmlPlannerSearchAlgorithm algorithm) {
            try {
                float heuristicWeight = Single.Parse(argumentManager.ExtractArgument(), Resources.English);
                try {
                    algorithm.HeuristicWeight = heuristicWeight;
                } catch (InvalidOperationException) {
                    _errors.Add(_resources.GetUnsupportedAlgorithmOption(option, algorithmName));
                }
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidWeightSpecified(option));
            } catch (OverflowException) {
                _errors.Add(_resources.GetInvalidWeightSpecified(option));
            } catch (FormatException) {
                _errors.Add(_resources.GetInvalidWeightSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoWeightSpecified(option));
            }
        }

        /// <summary>
        /// Adds a search algorithm to the sequence.
        /// </summary>
        /// <param name="algorithm">The search algorithm.</param>
        private void AddAlgorithm(XmlPlannerSearchAlgorithm algorithm) {
            if (_isFirstAlgorithm) {
                _planner.SearchAlgorithms.ReplaceAllBy(algorithm);
            } else {
                _planner.SearchAlgorithms.Add(algorithm);
            }
            _isFirstAlgorithm = false;
        }

        /// <summary>
        /// Sets up a search algorithm.
        /// </summary>
        /// <param name="option">The search algorithm option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds errors to the <see cref="_errors">error list</see> if any option is duplicated.
        /// </remarks>
        /// <returns>True if no errors found; false otherwise.</returns>
        private bool SetUpAlgorithm(string option, XmlPlannerCommandArgumentManager argumentManager) {
            string argument;
            string algorithmName;
            //create the search algorithm
            XmlPlannerSearchAlgorithm algorithm = CreateSearchAlgorithm(option, argumentManager,
                out algorithmName);
            bool beamWidthSpecified = false;
            bool heuristicWeightSpecified = false;
            bool success = (algorithm != null);
            bool finished = !success;
            //while search algorithm option found
            while (!finished && argumentManager.HasArguments) {
                argument = argumentManager.GetArgument();
                //if is the beam width option, set up it
                if (IsOption(Option.BeamWidth, ShortOption.BeamWidth, argument)) {
                    string suboption = argumentManager.ExtractArgument();
                    if (beamWidthSpecified) {
                        _errors.Add(_resources.GetAlgorithmOptionAlreadySpecified(suboption));
                    } else {
                        SetUpBeamWidth(suboption, argumentManager, algorithmName, algorithm);
                        beamWidthSpecified = true;
                    }
                //if is the heuristic weight option, set up it
                } else if (IsOption(Option.HeuristicWeight, ShortOption.HeuristicWeight, argument)) {
                    string suboption = argumentManager.ExtractArgument();
                    if (heuristicWeightSpecified) {
                        _errors.Add(_resources.GetAlgorithmOptionAlreadySpecified(suboption));
                    } else {
                        SetUpHeuristicWeight(suboption, argumentManager, algorithmName, algorithm);
                        heuristicWeightSpecified = true;
                    }
                } else {
                    finished = true;
                }
            }
            //if no errors found, add the algorithm to the sequence
            if (success) {
                AddAlgorithm(algorithm);
            }
            return success;
        }

        /// <summary>
        /// Sets up an inequality predicate.
        /// </summary>
        /// <param name="predicate">The predicate name.</param>
        /// <param name="type">The type name of the objects being affected by the inequality.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if an inequality with the same predicate
        /// name has been already specified.
        /// </remarks>
        private void SetUpInequality(string predicate, string type) {
            try {
                _planner.Inequalities.Add(predicate, type);
            } catch (ArgumentException) {
                if (!_planner.Inequalities[predicate].Type.Equals(type)) {
                    _errors.Add(_resources.GetInequalityAlreadySpecified(predicate));
                }
            }
        }

        /// <summary>
        /// Sets up an inequality predicate.
        /// </summary>
        /// <param name="option">The inequality predicate option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if an inequality with the same predicate
        /// name has been already specified, no predicate name is specified or no type name is specified.
        /// </remarks>
        private void SetUpInequality(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string predicate = argumentManager.ExtractArgument();
                try {
                    string type = argumentManager.ExtractArgument();
                    SetUpInequality(predicate, type);
                } catch (InvalidOperationException) {
                    _errors.Add(_resources.GetNoTypeSpecified(option));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoPredicateSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the maximum plan length.
        /// </summary>
        /// <param name="option">The maximum plan length option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpMaxPlanLength(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                int maxPlanLength = Int32.Parse(argumentManager.ExtractArgument(), Resources.English);
                if (maxPlanLength >= XmlPlannerMinimumValue.MaxPlanLength) {
                    _planner.MaxPlanLength = maxPlanLength;
                    _planner.MaxPlanLengthEnabled = true;
                } else {
                    _errors.Add(_resources.GetInvalidLengthSpecified(option));
                }
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidLengthSpecified(option));
            } catch (OverflowException) {
                _errors.Add(_resources.GetInvalidLengthSpecified(option));
            } catch (FormatException) {
                _errors.Add(_resources.GetInvalidLengthSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoLengthSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the maximum planning time.
        /// </summary>
        /// <param name="option">The maximum planning time option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpMaxTime(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                long maxTime = Int64.Parse(argumentManager.ExtractArgument(), Resources.English);
                if (maxTime >= XmlPlannerMinimumValue.MaxTime) {
                    _planner.MaxTime = maxTime;
                    _planner.MaxTimeEnabled = true;
                } else {
                    _errors.Add(_resources.GetInvalidMillisecondsSpecified(option));
                }
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidMillisecondsSpecified(option));
            } catch (OverflowException) {
                _errors.Add(_resources.GetInvalidMillisecondsSpecified(option));
            } catch (FormatException) {
                _errors.Add(_resources.GetInvalidMillisecondsSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoMillisecondsSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the root type.
        /// </summary>
        /// <param name="option">The root type option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if no option value is specified.
        /// </remarks>
        private void SetUpRootType(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                _planner.RootType = argumentManager.ExtractArgument();
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoTypeSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the root type predicate inequality.
        /// </summary>
        /// <param name="option">The root type predicate inequality option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if no option value is specified or the
        /// option is already specified.
        /// </remarks>
        private void SetUpRootInequality(string option, XmlPlannerCommandArgumentManager argumentManager) {
            if (!_rootInequalitySet) {
                try {
                    _rootInequality = argumentManager.ExtractArgument();
                    _rootInequalitySet = true;
                } catch (InvalidOperationException) {
                    _errors.Add(_resources.GetNoTypeSpecified(option));
                }
            } else {
                _errors.Add(_resources.GetRootInequalityAlreadySpecified(option));
            }
        }

        /// <summary>
        /// Sets up the predicate inequalities.
        /// </summary>
        private void SetUpInequalities() {
            if (!_noRootInequality && _planner.RootTypeEnabled) {
                SetUpInequality(_rootInequality, _planner.RootType);
            }
            _planner.InequalitiesEnabled = (_planner.Inequalities.Count > 0);
        }

        /// <summary>
        /// Sets up the messages language.
        /// </summary>
        /// <param name="option">The messages language option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpLanguage(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                CultureInfo culture = CultureInfo.GetCultureInfo(argumentManager.ExtractArgument());
                Resources.SetCulture(culture);
            } catch (NotSupportedException) {
                _errors.Add(_resources.GetInvalidLanguageSpecified(option));
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidLanguageSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoLanguageSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the output encoding.
        /// </summary>
        /// <param name="option">The output encoding option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpEncoding(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string encodingName = argumentManager.ExtractArgument();
                if (encodingName.Equals(XmlPlannerCommandEncoding.Problem,
                StringComparison.InvariantCultureIgnoreCase)) {
                    _planner.OutputEncodingSource = XmlPlannerOutputEncodingSource.Problem;
                } else if (encodingName.Equals(XmlPlannerCommandEncoding.Domain,
                StringComparison.InvariantCultureIgnoreCase)) {
                    _planner.OutputEncodingSource = XmlPlannerOutputEncodingSource.Domain;
                } else {
                    _planner.OutputEncoding = Encoding.GetEncoding(encodingName);
                    _planner.OutputEncodingSource = XmlPlannerOutputEncodingSource.Configuration;
                }
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidEncodingSpecified(option));
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoEncodingSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the input XML schema export.
        /// </summary>
        /// <param name="option">The input XML schema export option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpExportInputSchema(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _inputSchemaExportPath = path;
                    _exportInputSchema = true;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the output XML schema export.
        /// </summary>
        /// <param name="option">The output XML schema export option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpExportOutputSchema(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _outputSchemaExportPath = path;
                    _exportOutputSchema = true;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the license export.
        /// </summary>
        /// <param name="option">The license export option name.</param>
        /// <param name="argumentManager">The argument manager.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if the the option value is invalid or no
        /// value is specified.
        /// </remarks>
        private void SetUpExportLicense(string option, XmlPlannerCommandArgumentManager argumentManager) {
            try {
                string path = argumentManager.ExtractArgument();
                if (path.Length > 0) {
                    _licenseExportPath = path;
                    _exportLicense = true;
                } else {
                    _errors.Add(_resources.GetInvalidPath(path));
                }
            } catch (InvalidOperationException) {
                _errors.Add(_resources.GetNoFileSpecified(option));
            }
        }

        /// <summary>
        /// Sets up the command by processing <see cref="_arguments">the arguments</see>.
        /// </summary>
        private void SetUp() {
            if (_arguments.Length > 0) {
                XmlPlannerCommandArgumentManager argumentManager = new XmlPlannerCommandArgumentManager(_arguments);
                IDictionary<string, string> specifiedOptions = new Dictionary<string, string>();
                string option;
                bool finished = false;
                _isFirstAlgorithm = true;
                while (!finished && argumentManager.HasArguments) {
                    option = argumentManager.ExtractArgument();
                    if (IsOption(Option.Domain, ShortOption.Domain, option)) {
                        SetUpDomain(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.Domain);
                    } else if (IsOption(Option.Problem, ShortOption.Problem, option)) {
                        SetUpProblem(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.Problem);
                    } else if (IsOption(Option.Out, ShortOption.Out, option)) {
                        SetUpOutput(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.Out);
                    } else if (IsOption(Option.Algorithm, ShortOption.Algorithm, option)) {
                        finished = !SetUpAlgorithm(option, argumentManager);
                    } else if (IsOption(Option.Inequality, ShortOption.Inequality, option)) {
                        SetUpInequality(option, argumentManager);
                    } else if (IsOption(Option.MaxPlanLength, option)) {
                        SetUpMaxPlanLength(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.MaxPlanLength);
                    } else if (IsOption(Option.MaxTime, option)) {
                        SetUpMaxTime(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.MaxTime);
                    } else if (IsOption(Option.RootType, option)) {
                        SetUpRootType(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.RootType);
                    } else if (IsOption(Option.RootInequality, option)) {
                        SetUpRootInequality(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.RootInequality);
                    } else if (IsOption(Option.Language, ShortOption.Language, option)) {
                        SetUpLanguage(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.Language);
                    } else if (IsOption(Option.Encoding, ShortOption.Encoding, option)) {
                        SetUpEncoding(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.Encoding);
                    } else if (IsOption(Option.ExportInputSchema, option)) {
                        SetUpExportInputSchema(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.ExportInputSchema);
                    } else if (IsOption(Option.ExportOutputSchema, option)) {
                        SetUpExportOutputSchema(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.ExportOutputSchema);
                    } else if (IsOption(Option.ExportLicense, option)) {
                        SetUpExportLicense(option, argumentManager);
                        CheckOptionUnicity(specifiedOptions, Option.ExportLicense);
                    } else if (IsOption(Option.Overwrite, option)) {
                        _overwrite = true;
                        _planner.OverwriteOutput = true;
                    } else if (IsOption(Option.Help, ShortOption.Help, option)) {
                        _showHelp = true;
                    } else if (IsOption(Option.NoRoot, option)) {
                        _planner.RootTypeEnabled = false;
                    } else if (IsOption(Option.NoRootInequality, option)) {
                        _noRootInequality = true;
                    } else if (IsOption(Option.NoLogo, option)) {
                        _noLogo = true;
                    } else if (IsOption(Option.NoWarn, option)) {
                        _noWarn = true;
                    } else if (IsOption(Option.WarnAsError, option)) {
                        _warnAsError = true;
                    } else if (IsOption(Option.Verbose, ShortOption.Verbose, option)) {
                        _verbose = true;
                    } else {
                        _errors.Add(_resources.GetInvalidOption(option));
                    }
                }
            }
            _setUp = true;
        }

        /// <summary>
        /// Writes a text in one or more lines, applying word-wrapping.
        /// </summary>
        /// <param name="lineTab">The prefix to add to any new line.</param>
        /// <param name="text">The text.</param>
        /// <param name="lineWidth">The maximum width each line.</param>
        private static void WriteLines(string lineTab, string text, int lineWidth) {
            const char Space = ' ';
            //split the text in words
            string[] words = text.Split(new char[] { Space });
            if (words.Length > 0) {
                StringBuilder line = new StringBuilder();
                bool isFirtLine = true;
                //for each word
                foreach (string word in words) {
                    if ((line.Length + word.Length) > lineWidth) { //if the current line is full of words
                        if (isFirtLine) {
                            isFirtLine = false;
                        } else {
                            Console.Write(lineTab);
                        }
                        //write the current line
                        Console.WriteLine(line.ToString());
                        line.Length = 0;
                        //append the word to the next line
                        line.Append(word);
                    } else { //if the current line can have more words
                        if (line.Length > 0) {
                            line.Append(Space);
                        }
                        //append the word to the current line
                        line.Append(word);
                    }
                }
                //write the last line
                if (line.Length > 0) {
                    if (!isFirtLine) {
                        Console.Write(lineTab);
                    }
                    Console.WriteLine(line.ToString());
                }
            }
        }

        /// <summary>
        /// Shows the command header, with information like title, version, author and license notice.
        /// </summary>
        private void ShowLogo() {
            const int Unique = 0;
            Assembly assembly = Assembly.GetExecutingAssembly();
            object[] attributes = assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
            AssemblyCopyrightAttribute copyrightAttribute = attributes[Unique] as AssemblyCopyrightAttribute;
            attributes = assembly.GetCustomAttributes(typeof(AssemblyCompanyAttribute), false);
            AssemblyCompanyAttribute companyAttribute = attributes[Unique] as AssemblyCompanyAttribute;
            //write the title
            Console.WriteLine(_resources.Title);
            Version version = assembly.GetName().Version;
            //write the version
            Console.Write(_resources.Version);
            if (version.Revision != 0) {
                Console.WriteLine(" {0}.{1}.{2}.{3}",
                    version.Major, version.Minor, version.Build, version.Revision);
            } else if (version.Build != 0) {
                Console.WriteLine(" {0}.{1}.{2}", version.Major, version.Minor, version.Build);
            } else {
                Console.WriteLine(" {0}.{1}", version.Major, version.Minor);
            }
            //write the author
            Console.WriteLine("{0} {1}", copyrightAttribute.Copyright, companyAttribute.Company);
            Console.WriteLine();
            //write the license notice
            string licenseNotice =
                String.Format(CultureInfo.InvariantCulture, _resources.LicenseNotice, Option.ExportLicense);
            WriteLines(String.Empty, licenseNotice, Console.WindowWidth - Environment.NewLine.Length);
            Console.WriteLine();
        }

        /// <summary>
        /// Shows an option help.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="optionLength">The length of the option name.</param>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="help">The option help text.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <param name="writeNewLine">If write a new line after the option help text.</param>
        private static void ShowOptionHelp(string option, int optionLength, string helpTab, string help,
        int lineWidth, bool writeNewLine) {
            Console.Write(option.PadRight(optionLength));
            WriteLines(helpTab, help, lineWidth);
            if (writeNewLine) {
                Console.WriteLine();
            }
        }

        /// <summary>
        /// Shows an option help.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="optionLength">The length of the option name.</param>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="help">The option help text.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <remarks>Writes a new line after the option help text.</remarks>
        private static void ShowOptionHelp(string option, int optionLength, string helpTab, string help,
        int lineWidth) {
            ShowOptionHelp(option, optionLength, helpTab, help, lineWidth, true);
        }

        /// <summary>
        /// Shows an option help.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="optionLength">The length of the option name.</param>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="help">The option help text.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <param name="shortFormat">The option short name.</param>
        /// <param name="writeNewLine">If write a new line after the option help text.</param>
        private void ShowOptionHelp(string option, int optionLength, string helpTab, string help, int lineWidth,
        string shortFormat, bool writeNewLine) {
            const char Space = ' ';
            Console.Write(option.PadRight(optionLength));
            string fullHelp = help + Space + _resources.GetShortFormat(shortFormat);
            WriteLines(helpTab, fullHelp, lineWidth);
            if (writeNewLine) {
                Console.WriteLine();
            }
        }

        /// <summary>
        /// Shows an option help.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="optionLength">The length of the option name.</param>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="help">The option help text.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <param name="shortFormat">The option short name.</param>
        /// <param name="defaultValue">The option default value.</param>
        /// <param name="writeNewLine">If write a new line after the option help text.</param>
        private void ShowOptionHelp(string option, int optionLength, string helpTab, string help, int lineWidth,
        string shortFormat, string defaultValue, bool writeNewLine) {
            const char Space = ' ';
            Console.Write(option.PadRight(optionLength));
            string fullHelp = help + Space + _resources.GetShortFormat(shortFormat) + Space +
                _resources.GetDefaultValue(defaultValue);
            WriteLines(helpTab, fullHelp, lineWidth);
            if (writeNewLine) {
                Console.WriteLine();
            }
        }

        /// <summary>
        /// Shows an option help.
        /// </summary>
        /// <param name="option">The option name.</param>
        /// <param name="optionLength">The length of the option name.</param>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="help">The option help text.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <param name="defaultValue">The option default value.</param>
        /// <remarks>Writes a new line after the option help text.</remarks>
        private void ShowOptionHelpDefault(string option, int optionLength, string helpTab, string help,
        int lineWidth, string defaultValue) {
            const char Space = ' ';
            Console.Write(option.PadRight(optionLength));
            string fullHelp = help + Space + _resources.GetDefaultValue(defaultValue);
            WriteLines(helpTab, fullHelp, lineWidth);
            Console.WriteLine();
        }

        /// <summary>
        /// Gets the maximum text length in a text array.
        /// </summary>
        /// <param name="texts">The text array.</param>
        /// <returns>The maximum text length in <paramref name="texts"/>.</returns>
        private static int GetMaxLength(string[] texts) {
            int maxLength = 0;
            foreach (string text in texts) {
                maxLength = Math.Max(maxLength, text.Length);
            }
            return maxLength;
        }

        /// <summary>
        /// Writes the algorithm option values help.
        /// </summary>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        private void ShowAlgorithmEnumHelp(string helpTab, int lineWidth) {
            string internalHelpTab = helpTab + "  ";
            Console.Write(helpTab);
            //write the Enforced Hill Climbing option value
            WriteLines(internalHelpTab, _resources.GetDefaultFirstAlgorithmEnumHelp(
                XmlPlannerCommandAlgorithm.EnforcedHillClimbing, _resources.EnforcedHillClimbingName,
                XmlPlannerCommandShortAlgorithm.EnforcedHillClimbing), lineWidth);
            Console.Write(helpTab);
            //write the A* algorithm option value
            WriteLines(internalHelpTab, _resources.GetDefaultSecondAlgorithmEnumHelp(
                XmlPlannerCommandAlgorithm.AStar, _resources.AStarName), lineWidth);
            Console.Write(helpTab);
            //write the Weighted A* algorithm option value
            WriteLines(internalHelpTab, _resources.GetAlgorithmEnumHelp(XmlPlannerCommandAlgorithm.WeightedAStar,
                _resources.WeightedAStarName, XmlPlannerCommandShortAlgorithm.WeightedAStar), lineWidth);
            Console.Write(helpTab);
            //write the Dijkstra algorithm option value
            WriteLines(internalHelpTab, _resources.GetAlgorithmEnumHelp(XmlPlannerCommandAlgorithm.Dijkstra,
                _resources.DijkstraName), lineWidth);
            Console.Write(helpTab);
            //write the Beam Search algorithm option value
            WriteLines(internalHelpTab, _resources.GetAlgorithmEnumHelp(XmlPlannerCommandAlgorithm.BeamSearch,
                _resources.BeamSearchName, XmlPlannerCommandShortAlgorithm.BeamSearch), lineWidth);
            Console.Write(helpTab);
            //write the Hill Climbing algorithm option value
            WriteLines(internalHelpTab, _resources.GetAlgorithmEnumHelp(XmlPlannerCommandAlgorithm.HillClimbing,
                _resources.HillClimbingName, XmlPlannerCommandShortAlgorithm.HillClimbing), lineWidth);
            Console.Write(helpTab);
            WriteLines(helpTab, _resources.AlgorithmUseHelp, lineWidth);
            Console.WriteLine();
        }

        /// <summary>
        /// Shows an algorithm option value help.
        /// </summary>
        /// <param name="helpTab">The prefix to add to any new option value help text line.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        /// <param name="algorithmName">The algorithm name.</param>
        private void ShowAlgorithmOptionHelp(string helpTab, int lineWidth, string algorithmName) {
            Console.Write(helpTab);
            WriteLines(helpTab, _resources.GetAlgorithmOptionHelp(algorithmName), lineWidth);
            Console.WriteLine();
        }

        /// <summary>
        /// Sets the first letter of a text as a capital one.
        /// </summary>
        /// <param name="str">The text.</param>
        /// <returns>A copy of <paramref name="text"/> with the first letter as a capital one.</returns>
        private static string Capitalize(string str) {
            return Char.ToUpper(str[0], CultureInfo.InvariantCulture) + str.Substring(1);
        }

        /// <summary>
        /// Writes the encoding option values help.
        /// </summary>
        /// <param name="helpTab">The prefix to add to any new option help text line.</param>
        /// <param name="lineWidth">The maximum width of each help text line.</param>
        private void ShowEncodingEnumHelp(string helpTab, int lineWidth) {
            string internalHelpTab = helpTab + "  ";
            Console.Write(helpTab);
            //write the problem encoding option value
            WriteLines(internalHelpTab, _resources.GetDefaultEncodingEnumHelp(
                Capitalize(XmlPlannerCommandEncoding.Problem), _resources.ProblemEncodingEnumHelp), lineWidth);
            Console.Write(helpTab);
            //write the domain encoding option value
            WriteLines(internalHelpTab, Resources.GetEncodingEnumHelp(Capitalize(XmlPlannerCommandEncoding.Domain),
                _resources.DomainEncodingEnumHelp), lineWidth);
            Console.Write(helpTab);
            //write the custom encoding option value
            WriteLines(internalHelpTab, Resources.GetEncodingEnumHelp(_resources.XmlEncodingName,
                _resources.CustomEncodingEnumHelp), lineWidth);
            Console.WriteLine();
        }

        /// <summary>
        /// Writes the options help.
        /// </summary>
        private void ShowOptionsHelp() {
            const int SeparatorLength = 2;
            string domain = Resources.GetOption(Option.Domain, _resources.File);
            string problem = Resources.GetOption(Option.Problem, _resources.File);
            string output = Resources.GetOption(Option.Out, _resources.File);
            string maxPlanLength = Resources.GetOption(Option.MaxPlanLength, _resources.Length);
            string maxTime = Resources.GetOption(Option.MaxTime, _resources.MillisecondsType);
            string algorithm = Resources.GetOption(Option.Algorithm, _resources.Algorithm);
            string beamWidth = Resources.GetOption(Option.BeamWidth, _resources.Width);
            string heuristicWeight = Resources.GetOption(Option.HeuristicWeight, _resources.Weight);
            string rootType = Resources.GetOption(Option.RootType, _resources.Type);
            string rootInequality = Resources.GetOption(Option.RootInequality, _resources.Predicate);
            string inequality = Resources.GetOption(Option.Inequality, _resources.Predicate, _resources.Type);
            string encoding = Resources.GetOption(Option.Encoding, _resources.Encoding);
            string exportInputSchema = Resources.GetOption(Option.ExportInputSchema, _resources.File);
            string exportOutputSchema = Resources.GetOption(Option.ExportOutputSchema, _resources.File);
            string exportLicense = Resources.GetOption(Option.ExportLicense, _resources.File);
            string language = Resources.GetOption(Option.Language, _resources.Language);
            string[] allOptions = { domain, problem, output, maxPlanLength, maxTime, algorithm, beamWidth,
                heuristicWeight, rootType, rootInequality, inequality, encoding, exportInputSchema,
                exportOutputSchema, exportLicense, Option.Overwrite, Option.NoRoot, Option.NoRootInequality,
                Option.NoWarn, Option.NoLogo, Option.Verbose, Option.Help };
            int optionLength = GetMaxLength(allOptions) + SeparatorLength;
            int helpLength = Console.WindowWidth - optionLength - Environment.NewLine.Length;
            string helpTab = String.Empty.PadLeft(optionLength);
            //write domain option help
            ShowOptionHelp(domain, optionLength, helpTab, _resources.DomainHelp, helpLength,
                ShortOption.Domain, DefaultValue.Domain, true);
            //write problem option help
            ShowOptionHelp(problem, optionLength, helpTab, _resources.ProblemHelp, helpLength,
                ShortOption.Problem, DefaultValue.Problem, true);
            //write output option help
            ShowOptionHelp(output, optionLength, helpTab, _resources.OutHelp, helpLength,
                ShortOption.Out, DefaultValue.Out, true);
            //write overwrite option help
            ShowOptionHelp(Option.Overwrite, optionLength, helpTab, _resources.OverwriteHelp, helpLength);
            //write maximum plan length option help
            ShowOptionHelp(maxPlanLength, optionLength, helpTab, _resources.MaxPlanLengthHelp, helpLength);
            //write maximum planning time option help
            ShowOptionHelp(maxTime, optionLength, helpTab, _resources.MaxTimeHelp, helpLength);
            //write algorithm option help
            ShowOptionHelp(algorithm, optionLength, helpTab, _resources.AlgorithmHelp, helpLength,
                ShortOption.Algorithm, false);
            ShowAlgorithmEnumHelp(helpTab, helpLength);
            //write beam width option help
            ShowOptionHelp(beamWidth, optionLength, helpTab, _resources.BeamWidthHelp, helpLength,
                ShortOption.BeamWidth, DefaultValue.BeamWidth.ToString(Resources.English), false);
            ShowAlgorithmOptionHelp(helpTab, helpLength, _resources.BeamSearchName);
            //write heuristic weight option help
            ShowOptionHelp(heuristicWeight, optionLength, helpTab, _resources.HeuristicWeightHelp, helpLength,
                ShortOption.HeuristicWeight, DefaultValue.HeuristicWeight.ToString(Resources.English), false);
            ShowAlgorithmOptionHelp(helpTab, helpLength, _resources.WeightedAStarName);
            //write root type option help
            ShowOptionHelpDefault(rootType, optionLength, helpTab, _resources.RootTypeHelp, helpLength,
                DefaultValue.RootType);
            //write root type disabling option help
            ShowOptionHelp(Option.NoRoot, optionLength, helpTab, _resources.NoRootHelp, helpLength);
            ShowOptionHelpDefault(rootInequality, optionLength, helpTab, _resources.RootInequalityHelp,
                helpLength, DefaultValue.RootInequality);
            //write root type inequality disabling option help
            ShowOptionHelp(Option.NoRootInequality, optionLength, helpTab, _resources.NoRootInequalityHelp, helpLength);
            ShowOptionHelp(inequality, optionLength, helpTab, _resources.InequalityHelp, helpLength,
                ShortOption.Inequality, true);
            //write warning disabling option help
            ShowOptionHelp(Option.NoWarn, optionLength, helpTab, _resources.NoWarnHelp, helpLength);
            //write warning as error option help
            ShowOptionHelp(Option.WarnAsError, optionLength, helpTab, _resources.WarnAsErrorHelp, helpLength);
            ShowOptionHelp(encoding, optionLength, helpTab, _resources.GetEncodingHelp(ShortOption.Encoding),
                helpLength, false);
            //write encoding option help
            ShowEncodingEnumHelp(helpTab, helpLength);
            //write export XML input schema option help
            ShowOptionHelp(exportInputSchema, optionLength, helpTab, _resources.ExportInputSchemaHelp, helpLength);
            //write export XML output schema option help
            ShowOptionHelp(exportOutputSchema, optionLength, helpTab, _resources.ExportOutputSchemaHelp, helpLength);
            //write export license option help
            ShowOptionHelp(exportLicense, optionLength, helpTab, _resources.ExportLicenseHelp, helpLength);
            //write language option help
            ShowOptionHelp(language, optionLength, helpTab, _resources.LanguageHelp, helpLength,
                ShortOption.Language, true);
            //write header disabling option help
            ShowOptionHelp(Option.NoLogo, optionLength, helpTab, _resources.NoLogoHelp, helpLength);
            //write verbose option help
            ShowOptionHelp(Option.Verbose, optionLength, helpTab, _resources.VerboseHelp, helpLength, ShortOption.Verbose, true);
            //write help option help
            ShowOptionHelp(Option.Help, optionLength, helpTab, _resources.HelpHelp, helpLength, ShortOption.Help, true);
        }

        /// <summary>
        /// Writes the exit code help.
        /// </summary>
        private void ShowExitCodeHelp() {
            const string Format = "{0,2}: {1}";
            Console.WriteLine(Format, XmlPlannerCommandExitCode.AllOk, _resources.AllOkExitCodeHelp);
            Console.WriteLine(Format, XmlPlannerCommandExitCode.PlanNotFound, _resources.PlanNotFoundExitCodeHelp);
            Console.WriteLine(Format, XmlPlannerCommandExitCode.InvalidArguments, _resources.InvalidArgumentsExitCodeHelp);
            Console.WriteLine(Format, XmlPlannerCommandExitCode.TranslationError, _resources.TranslationErrorExitCodeHelp);
            Console.WriteLine(Format, XmlPlannerCommandExitCode.ExportError, _resources.ExportErrorExitCodeHelp);
            Console.WriteLine(Format, XmlPlannerCommandExitCode.InternalError, _resources.InternalErrorExitCodeHelp);
            Console.WriteLine();
        }

        /// <summary>
        /// Writes the usage.
        /// </summary>
        private void ShowUsage() {
            //write usage header
            Console.WriteLine("{0}: {1} [{2}]", _resources.Usage, ExecutableName, _resources.Options);
            Console.WriteLine();
            //write options
            Console.WriteLine("{0}:", _resources.AvailableOptions);
            Console.WriteLine();
            ShowOptionsHelp();
            //write exit codes
            Console.WriteLine("{0}:", _resources.ExitCodes);
            Console.WriteLine();
            ShowExitCodeHelp();
            //write examples header
            Console.WriteLine("{0}:", _resources.Examples);
            Console.WriteLine();
            //write first example
            Console.WriteLine("{0} {1} {2}.xml {3} {4}.xml {5} {6}", ExecutableName,
                XmlPlannerCommandShortOption.Problem, _resources.DomainAndProblemExample, ShortOption.Domain,
                _resources.DomainAndProblemExample, ShortOption.Encoding, XmlPlannerCommandEncoding.Domain);
            //write second example
            Console.WriteLine("{0} {1} {2} {3} 10 {4} {5} {6} {7} {8} {9}", ExecutableName, XmlPlannerCommandShortOption.Algorithm,
                XmlPlannerCommandAlgorithm.BeamSearch, ShortOption.BeamWidth, ShortOption.Algorithm,
                XmlPlannerCommandAlgorithm.HillClimbing, ShortOption.Encoding, Encoding.UTF8.WebName,
                ShortOption.Language, Resources.English.ToString());
            //write third example
            Console.WriteLine("{0} {1} {2}.xml {3} {4}.xml {5} {6}.xml {7} {8} {9} " + (2.5).ToString(Resources.English),
                ExecutableName, ShortOption.Domain, _resources.DomainExample, XmlPlannerCommandShortOption.Problem,
                _resources.ProblemExample, ShortOption.Out, _resources.OutExample, ShortOption.Algorithm,
                XmlPlannerCommandShortAlgorithm.WeightedAStar, ShortOption.HeuristicWeight);
        }

        /// <summary>
        /// Writes the errors.
        /// </summary>
        /// <param name="exitCode">The exit code to set.</param>
        /// <returns>If there was errors to show.</returns>
        private bool ShowErrors(int exitCode) {
            bool errorsShown = (_errors.Count > 0);
            if (errorsShown) {
                //write the errors
                foreach (string message in _errors) {
                    Console.Error.WriteLine(_resources.GetErrorMessage(message));
                }
                Console.WriteLine();
                //write help tip
                Console.WriteLine(_resources.GetShowHelpTip(ExecutableName, ShortOption.Help));
                Console.Error.WriteLine();
                //set exit code
                Environment.ExitCode = exitCode;
                _errors.Clear();
            }
            return errorsShown;
        }

        /// <summary>
        /// Gets the XML <see cref="Encoding">encoding</see> of a file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The XML <see cref="Encoding">encoding</see> of the file.</returns>
        private Encoding GetEncoding(string filePath) {
            Encoding encoding = null;
            try {
                //tries to get the encoding from teh cache
                encoding = _encodingCache[filePath];
            } catch (KeyNotFoundException) {
                //if the encoding is not cached
                //read it from the file
                Stream file = File.OpenRead(filePath);
                XmlTextReader reader = new XmlTextReader(file);
                try {
                    reader.Read();
                    encoding = reader.Encoding;
                } catch (XmlException) {
                    //do nothing; the file has invalid XML data
                }
                file.Close();
                //add the encoding to the cache
                _encodingCache.Add(filePath, encoding);
            }
            return encoding;
        }

        /// <summary>
        /// Gets a text of a line of a file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="lineNumber">The line number.</param>
        /// <returns>The text of the line.</returns>
        private string GetLine(string filePath, int lineNumber) {
            //open the file
            Encoding encoding = GetEncoding(filePath);
            StreamReader reader = null;
            if (encoding != null) {
                reader = new StreamReader(filePath, encoding);
            } else {
                reader = new StreamReader(filePath);
            }
            //search the line
            for (int i = 1; i < lineNumber; i++) {
                reader.ReadLine();
            }
            string line = reader.ReadLine();
            reader.Close();
            return line;
        }

        /// <summary>
        /// Gets a line position mark.
        /// </summary>
        /// <remarks>A line position mark is a blank line with a mark character at some position.</remarks>
        /// <param name="line">The line to mark.</param>
        /// <param name="linePosition">The line position.</param>
        /// <returns>The line position mark.</returns>
        private static string GetLinePositionMark(string line, int linePosition) {
            StringBuilder linePositionMark = new StringBuilder(line.Length);
            int i = 1;
            char c;
            while (i < linePosition) {
                c = line[i];
                switch (c) {
                    case '\t':
                        linePositionMark.Append(c);
                        break;
                    default:
                        linePositionMark.Append(' ');
                        break;
                }
                i++;
            }
            linePositionMark.Append(PositionMark);
            return linePositionMark.ToString();
        }

        /// <summary>
        /// Writes the message.
        /// </summary>
        /// <param name="output">The output in which write the message.</param>
        /// <param name="message">The message from which get the line information, if any.</param>
        /// <param name="text">The text to write.</param>
        private void ShowMessage(TextWriter output, XmlPlannerMessage message, string text) {
            //write the text
            output.WriteLine(text);
            if (message.HasFileInfo) {
                string filePath = message.FilePath;
                int lineNumber = message.LineNumber;
                int iinePosition = message.LinePosition;
                //write the line information header
                output.WriteLine(_resources.GetFileInfo(filePath, lineNumber, iinePosition));
                string line = GetLine(filePath, lineNumber);
                int lineLength = line.Length;
                int maximumWidth = Console.WindowWidth - MessageTab.Length;
                int offset = 0;
                if (lineLength > maximumWidth) {
                    offset = (lineLength - maximumWidth) / 2;
                    line = line.Substring(offset, maximumWidth);
                }
                string linePositionMark = GetLinePositionMark(line, iinePosition - offset);
                output.Write(MessageTab);
                //write the line
                output.WriteLine(line.Trim());
                output.Write(MessageTab);
                //write the line position mark
                output.WriteLine(linePositionMark.Trim());
            }
        }

        /// <summary>
        /// Writes a warning message.
        /// </summary>
        /// <param name="message">The message.</param>
        private void ShowWarning(XmlPlannerMessage message) {
            ShowMessage(Console.Out, message, _resources.GetWarningMessage(message.Text));
        }

        /// <summary>
        /// Writes an error message.
        /// </summary>
        /// <param name="message">The message.</param>
        private void ShowError(XmlPlannerMessage message) {
            ShowMessage(Console.Error, message, _resources.GetErrorMessage(message.Text));
        }

        /// <summary>
        /// Writes all the planner messages.
        /// </summary>
        /// <returns>If there was messages to show.</returns>
        private bool ShowPlannerMessages() {
            bool messagesShown = false;
            _plannerMessages.Sort();
            _encodingCache = new Dictionary<string, Encoding>();
            //write the messages, if any
            foreach (XmlPlannerMessage message in _plannerMessages) {
                switch (message.Type) {
                    case XmlPlannerMessageType.Warning:
                        if (_warnAsError) {
                            ShowError(message);
                            messagesShown = true;
                        } else if (!_noWarn) {
                            ShowWarning(message);
                            messagesShown = true;
                        }
                        break;
                    case XmlPlannerMessageType.Error:
                        ShowError(message);
                        messagesShown = true;
                        break;
                    case XmlPlannerMessageType.ParserError:
                        Console.Error.WriteLine(_resources.GetParserErrorMessage(message.Text));
                        messagesShown = true;
                        break;
                }
            }
            //if any message shown
            if (messagesShown) {
                Console.WriteLine();
                if (_plannerMessages.TotalErrorCount > 0) {
                    //sets the exit code
                    Environment.ExitCode = XmlPlannerCommandExitCode.TranslationError;
                }
            }
            //cancel the planning when warning detected and warnAsError enabled
            if (_warnAsError && (_plannerMessages.WarningCount > 0)) {
                _planner.Cancel();
            }
            return messagesShown;
        }

        /// <summary>
        /// Opens a file to be written.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if any error occurs.
        /// </remarks>
        internal TextWriter OpenWrite(string path) {
            TextWriter writer = null;
            Stream file = null;
            try {
                if (File.Exists(path)) {
                    if (_overwrite) {
                        File.Delete(path);
                        file = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                    } else {
                        _errors.Add(_resources.GetFileAlreadyExists(path));
                    }
                } else {
                    file = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                }
                if (file != null) {
                    writer = new StreamWriter(file);
                }
            } catch (UnauthorizedAccessException) {
                _errors.Add(_resources.GetInsufficientWritePermissions(path));
            } catch (ArgumentException) {
                _errors.Add(_resources.GetInvalidPath(path));
            } catch (PathTooLongException) {
                _errors.Add(_resources.GetInvalidPath(path));
            } catch (DirectoryNotFoundException) {
                _errors.Add(_resources.GetInvalidPath(path));
            } catch (IOException) {
                _errors.Add(_resources.GetFileCanNotBeWritten(path));
            } catch (SecurityException) {
                _errors.Add(_resources.GetInsufficientWritePermissions(path));
            } catch (NotSupportedException) {
                _errors.Add(_resources.GetInvalidPath(path));
            }
            return writer;
        }

        /// <summary>
        /// Exports a file.
        /// </summary>
        /// <param name="schema">The file.</param>
        /// <param name="path">The file path.</param>
        /// <remarks>
        /// Adds an error to the <see cref="_errors">error list</see> if any error occurs.
        /// </remarks>
        private void ExportFile(Stream schema, string path) {
            TextWriter writer = OpenWrite(path);
            if (writer != null) {
                TextReader reader = new StreamReader(schema);
                string line = reader.ReadLine();
                if (line != null) {
                    writer.Write(line);
                }
                line = reader.ReadLine();
                while (line != null) {
                    writer.WriteLine();
                    writer.Write(line);
                    line = reader.ReadLine();
                }
                writer.Close();
            }
        }

        /// <summary>
        /// Exports the files.
        /// </summary>
        /// <returns>If the export was successfully done.</returns>
        private bool ExportFiles() {
            bool exported = (_exportInputSchema || _exportOutputSchema || _exportLicense);
            if (exported) {
                //export input schema
                if (_exportInputSchema) {
                    if (_verbose) {
                        Console.WriteLine(_resources.ExportingInputSchema);
                    }
                    ExportFile(XmlPlanner.GetInputSchema(), _inputSchemaExportPath);
                }
                //export output schema
                if (_exportOutputSchema) {
                    if (_verbose) {
                        Console.WriteLine(_resources.ExportingOutputSchema);
                    }
                    ExportFile(XmlPlanner.GetOutputSchema(), _outputSchemaExportPath);
                }
                //export license
                if (_exportLicense) {
                    if (_verbose) {
                        Console.WriteLine(_resources.ExportingLicense);
                    }
                    ExportFile(XmlPlanner.GetLicense(), _licenseExportPath);
                }
                //write errors, if any
                if (!ShowErrors(XmlPlannerCommandExitCode.ExportError)) {
                    Console.WriteLine(_resources.ExportDone);
                }
            }
            return exported;
        }

        /// <summary>
        /// Executes the planner.
        /// </summary>
        private void ExecutePlanner() {
            try {
                _planner.Plan();
                Console.WriteLine(_resources.PlanFound);
                Environment.ExitCode = XmlPlannerCommandExitCode.AllOk;
            } catch (XmlPlanNotFoundException) {
                Console.WriteLine(_resources.PlanNotFound);
                Environment.ExitCode = XmlPlannerCommandExitCode.PlanNotFound;
            }
        }

        /// <summary>
        /// Executes the command.
        /// </summary>
        public void Execute() {
            try {
                //set up
                if (!_setUp) {
                    SetUp();
                    SetUpInequalities();
                }
                //show header
                if (!_noLogo) {
                    ShowLogo();
                }
                //show help, if needed
                if (_showHelp) {
                    ShowUsage();
                //show errors, if any, and export XML schemas, if needed
                } else if (!ShowErrors(XmlPlannerCommandExitCode.InvalidArguments) && !ExportFiles()) {
                    //search for a plan
                    ExecutePlanner();
                }
#if DEBUG
            } catch (Exception exception) {
                Console.Error.WriteLine(exception.ToString());
                Environment.ExitCode = XmlPlannerCommandExitCode.InternalError;
#else
            } catch (Exception) {
                Console.Error.WriteLine(_resources.GetErrorMessage(_resources.InternalError));
                Environment.ExitCode = XmlPlannerCommandExitCode.InternalError;
#endif
            }
        }

        /// <summary>
        /// Executes the command.
        /// </summary>
        /// <param name="arguments">The arguments.</param>
        public static void Execute(params string[] arguments) {
            new XmlPlannerCommand(arguments).Execute();
        }

        /// <summary>
        /// Shows the state changinf detailed information.
        /// </summary>
        /// <param name="previousState">The previous state.</param>
        /// <param name="currentState">The current state.</param>
        /// <param name="timeTaken">The time taken by the previous state.</param>
        private void ShowStateChangedDetailedInfo(XmlPlannerState previousState, XmlPlannerState currentState,
        TimeSpan timeTaken) {
            //write translation and searching time
            if (previousState == XmlPlannerState.Translating) {
                Console.WriteLine(_resources.GetTranslatingTimeMessage(timeTaken));
                Console.WriteLine();
            } else if (previousState == XmlPlannerState.Searching) {
                Console.WriteLine(_resources.GetSearchingTimeMessage(timeTaken));
                Console.WriteLine();
            }
            //writes a message indicating what is doing now, only when translating or searching
            if (currentState == XmlPlannerState.Translating) {
                Console.WriteLine(_resources.Translating);
                Console.WriteLine();
            } else if (currentState == XmlPlannerState.Searching) {
                Console.WriteLine(_resources.Searching);
                Console.WriteLine();
            }
        }

        #endregion

        #region Event handlers

        /// <summary>
        /// Handles the planner state changing event.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event data.</param>
        private void Planner_StateChanged(object sender, XmlPlannerStateChangedEventArgs e) {
            XmlPlannerState previousState = e.PreviousState;
            //assign messages the firt time this handler is invoked
            if (previousState == XmlPlannerState.Stopped) {
                _plannerMessages = e.Messages;
            } else if (previousState != XmlPlannerState.Starting) {
                //shows the messages after starting, if any
                if (!_plannerMessagesShown) {
                    _plannerMessagesShown = ShowPlannerMessages();
                }
                //show detailed state chaning information, if needed
                if (_verbose) {
                    ShowStateChangedDetailedInfo(previousState, e.CurrentState, e.TimeTaken);
                }
            }
        }

        /// <summary>
        /// Handles the planner search algorithm selection event.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event data.</param>
        private void Planner_SearchAlgorithmSelected(object sender, XmlPlannerSearchAlgorithmSelectedEventArgs e) {
            if (_verbose) {
                //write the selected algorithm description, if needed
                Console.WriteLine(_resources.GetAlgorithmSelected(e.SearchAlgorithm.Description));
                Console.WriteLine();
            }
        }

        #endregion

    }

}
