/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml.Command {

    /// <summary>
    /// Represents a <see cref="XmlPlannerCommand">command line XML planner</see> search algorithm option value.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public static class XmlPlannerCommandAlgorithm {

        #region Constants

        /// <summary>
        /// The A* algorithm.
        /// </summary>
        public const string AStar = "AStar";

        /// <summary>
        /// The Beam Search algorithm.
        /// </summary>
        public const string BeamSearch = "BeamSearch";

        /// <summary>
        /// The Dijkstra algorithm.
        /// </summary>
        public const string Dijkstra = "Dijkstra";

        /// <summary>
        /// The Enforced Hill Climbing algorithm.
        /// </summary>
        public const string EnforcedHillClimbing = "EnforcedHillClimbing";

        /// <summary>
        /// The Hill Climbing algorithm.
        /// </summary>
        public const string HillClimbing = "HillClimbing";

        /// <summary>
        /// The Weighted A* algorithm.
        /// </summary>
        public const string WeightedAStar = "WeightedAStar";

        #endregion

        #region Methods

        /// <summary>
        /// Gets an algorithm identifier from its name.
        /// </summary>
        /// <param name="name">The algorithm name.</param>
        /// <returns>The algorithm identifier.</returns>
        /// <exception cref="ArgumentException"><paramref name="name"/> is unknown.</exception>
        internal static XmlPlannerSearchAlgorithmId GetAlgorithmId(string name) {
            if (name.Equals(AStar, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.AStar;
            } else if (name.Equals(BeamSearch, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.BeamSearch;
            } else if (name.Equals(Dijkstra, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.Dijkstra;
            } else if (name.Equals(EnforcedHillClimbing, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.EnforcedHillClimbing;
            } else if (name.Equals(HillClimbing, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.HillClimbing;
            } else if (name.Equals(WeightedAStar, StringComparison.InvariantCultureIgnoreCase)) {
                return XmlPlannerSearchAlgorithmId.WeightedAStar;
            } else {
                throw new ArgumentException(new Resources().UnknownAlgorithm, "name");
            }
        }

        /// <summary>
        /// Gets an algorithm name from its identifier.
        /// </summary>
        /// <param name="algorithmId">The algorithm identifier.</param>
        /// <returns>The algorithm name.</returns>
        /// <exception cref="ArgumentException"><paramref name="algorithmId"/> is unknown.</exception>
        internal static string GetAlgorithmName(XmlPlannerSearchAlgorithmId algorithmId) {
            switch (algorithmId) {
                case XmlPlannerSearchAlgorithmId.AStar:
                    return XmlPlannerCommandAlgorithm.AStar;
                case XmlPlannerSearchAlgorithmId.BeamSearch:
                    return XmlPlannerCommandAlgorithm.BeamSearch;
                case XmlPlannerSearchAlgorithmId.EnforcedHillClimbing:
                    return XmlPlannerCommandAlgorithm.EnforcedHillClimbing;
                case XmlPlannerSearchAlgorithmId.HillClimbing:
                    return XmlPlannerCommandAlgorithm.HillClimbing;
                case XmlPlannerSearchAlgorithmId.WeightedAStar:
                    return XmlPlannerCommandAlgorithm.WeightedAStar;
                default:
                    throw new ArgumentException(new Resources().UnknownAlgorithm, "algorithm");
            }
        }

        #endregion

    }

}
