/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml.Command {

    /// <summary>
    /// Represents a <see cref="XmlPlannerCommand">command line XML planner</see> argument manager.
    /// </summary>
    [Serializable]
    internal sealed class XmlPlannerCommandArgumentManager {

        #region Fields

        /// <summary>
        /// The arguments to manage.
        /// </summary>
        private readonly string[] _arguments;

        /// <summary>
        /// The number of arguments.
        /// </summary>
        private readonly int _argumentCount;

        /// <summary>
        /// The index of the current argument.
        /// </summary>
        private int _index;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerCommandArgumentManager"/> class.
        /// </summary>
        /// <param name="arguments">The arguments to manage.</param>
        internal XmlPlannerCommandArgumentManager(string[] arguments) {
            _arguments = arguments;
            _argumentCount = arguments.Length;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether the current argument index is less than the argument count.
        /// </summary>
        /// <value>true if the current argument index is less than the argument count; otherwise, false.</value>
        internal bool HasArguments {
            get {
                return (_index < _argumentCount);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Checks the argument count.
        /// </summary>
        /// <exception cref="InvalidOperationException">The current  argument index is greater than or equal the
        /// argument count.</exception>
        private void CheckArgumentCount() {
            if (_index >= _argumentCount) {
                throw new InvalidOperationException();
            }
        }

        /// <summary>
        /// Gets the current argument.
        /// </summary>
        /// <returns>The current argument.</returns>
        internal string GetArgument() {
            CheckArgumentCount();
            return _arguments[_index];
        }

        /// <summary>
        /// Gets the current argument and increments the argument index.
        /// </summary>
        /// <returns>The current argument.</returns>
        internal string ExtractArgument() {
            CheckArgumentCount();
            return _arguments[_index++];
        }

        #endregion

    }

}
