/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Resources;
using System.Globalization;

using Aran.Translation;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents the resources for this namespace classes.
    /// </summary>
    [Serializable]
    internal sealed class Resources {

        #region Fields

        /// <summary>
        /// The underlying resource manager.
        /// </summary>
        private readonly ResourceManager _resources;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Resources"/> class.
        /// </summary>
        internal Resources() {
            _resources = new ResourceManager(GetType());
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        private static CultureInfo Culture {
            get {
                return CultureInfo.CurrentUICulture;
            }
        }

        /// <summary>
        /// Gets the "null domain path" message.
        /// </summary>
        /// <value>The "null domain path" message.</value>
        internal string NullDomainPath {
            get {
                return _resources.GetString("NullDomainPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "empty domain path" message.
        /// </summary>
        /// <value>The "empty domain path" message.</value>
        internal string EmptyDomainPath {
            get {
                return _resources.GetString("EmptyDomainPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "null problem path" message.
        /// </summary>
        /// <value>The "null problem path" message.</value>
        internal string NullProblemPath {
            get {
                return _resources.GetString("NullProblemPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "empty problem path" message.
        /// </summary>
        /// <value>The "empty problem path" message.</value>
        internal string EmptyProblemPath {
            get {
                return _resources.GetString("EmptyProblemPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "null output path" message.
        /// </summary>
        /// <value>The "null output path" message.</value>
        internal string NullOutputPath {
            get {
                return _resources.GetString("NullOutputPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "null output encoding" message.
        /// </summary>
        /// <value>The "null output encoding" message.</value>
        internal string NullOutputEncoding {
            get {
                return _resources.GetString("NullOutputEncoding", Culture);
            }
        }

        /// <summary>
        /// Gets the "empty output path" message.
        /// </summary>
        /// <value>The "empty output path" message.</value>
        internal string EmptyOutputPath {
            get {
                return _resources.GetString("EmptyOutputPath", Culture);
            }
        }

        /// <summary>
        /// Gets the "null predicate" message.
        /// </summary>
        /// <value>The "null predicate" message.</value>
        internal string NullPredicate {
            get {
                return _resources.GetString("NullPredicate", Culture);
            }
        }

        /// <summary>
        /// Gets the "empty predicate" message.
        /// </summary>
        /// <value>The "empty predicate" message.</value>
        internal string EmptyPredicate {
            get {
                return _resources.GetString("EmptyPredicate", Culture);
            }
        }

        /// <summary>
        /// Gets the "predicate already exists" message.
        /// </summary>
        /// <value>The "predicate already exists" message.</value>
        internal string PredicateAlreadyExists {
            get {
                return _resources.GetString("PredicateAlreadyExists", Culture);
            }
        }

        /// <summary>
        /// Gets the "predicate not found" message.
        /// </summary>
        /// <value>The "predicate not found" message.</value>
        internal string PredicateNotFound {
            get {
                return _resources.GetString("PredicateNotFound", Culture);
            }
        }

        /// <summary>
        /// Gets the "null type" message.
        /// </summary>
        /// <value>The "null type" message.</value>
        internal string NullType {
            get {
                return _resources.GetString("NullType", Culture);
            }
        }

        /// <summary>
        /// Gets the "empty type" message.
        /// </summary>
        /// <value>The "empty type" message.</value>
        internal string EmptyType {
            get {
                return _resources.GetString("EmptyType", Culture);
            }
        }

        /// <summary>
        /// Gets the "no plan found" message.
        /// </summary>
        /// <value>The "no plan found" message.</value>
        internal string NoPlanFound {
            get {
                return _resources.GetString("NoPlanFound", Culture);
            }
        }

        /// <summary>
        /// Gets the "domain mismatch" message.
        /// </summary>
        /// <value>The "domain mismatch" message.</value>
        internal string DomainMismatch {
            get {
                return _resources.GetString("DomainMismatch", Culture);
            }
        }

        /// <summary>
        /// Gets the "unknown search algorithm" message.
        /// </summary>
        /// <value>The "unknown search algorithm" message.</value>
        internal string UnknownSearchAlgorithm {
            get {
                return _resources.GetString("UnknownSearchAlgorithm", Culture);
            }
        }

        /// <summary>
        /// Gets the "beam width" name.
        /// </summary>
        /// <value>The "beam width" name.</value>
        internal string BeamWidth {
            get {
                return _resources.GetString("BeamWidth", Culture);
            }
        }

        /// <summary>
        /// Gets the "heuristic weight" name.
        /// </summary>
        /// <value>The "heuristic weight" name.</value>
        internal string HeuristicWeight {
            get {
                return _resources.GetString("HeuristicWeight", Culture);
            }
        }

        /// <summary>
        /// Gets the "with" word.
        /// </summary>
        /// <value>The "with" word.</value>
        internal string With {
            get {
                return _resources.GetString("With", Culture);
            }
        }

        /// <summary>
        /// Gets the "and" word.
        /// </summary>
        /// <value>The "and" word.</value>
        internal string And {
            get {
                return _resources.GetString("And", Culture);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets a message text from a message code.
        /// </summary>
        /// <param name="code">The message code.</param>
        /// <returns>The message text for <paramref name="code"/>.</returns>
        internal string GetMessage(MessageCode code) {
            return _resources.GetString(code.ToString(), Culture);
        }

        /// <summary>
        /// Gets a message text from a message code and an inequality predicate.
        /// </summary>
        /// <param name="code">The message code.</param>
        /// <param name="inequality">The inequality predicate.</param>
        /// <returns>The message text for <paramref name="code"/> and <paramref name="inequality"/>.</returns>
        internal string GetInequalityMessage(MessageCode code, InequalityPredicate<string, string> inequality) {
            CultureInfo culture = Culture;
            string messageFormat = _resources.GetString("Inequality" + code.ToString(), culture);
            return String.Format(culture, messageFormat, inequality.PredicateId, inequality.TypeId);
        }

        /// <summary>
        /// Gets a parametrized message by name.
        /// </summary>
        /// <param name="name">The message name.</param>
        /// <param name="arguments">The message arguments.</param>
        /// <returns>The parametrized message.</returns>
        private string GetMessage(string name, params object[] arguments) {
            CultureInfo culture = Culture;
            return String.Format(culture, _resources.GetString(name, culture), arguments);
        }

        /// <summary>
        /// Gets the "maximum planning time out of range" message.
        /// </summary>
        /// <param name="minimum">The minimum value maximum planning time can take.</param>
        /// <returns>The "maximum planning time out of range" message.</returns>
        internal string GetMaxTimeOutOfRange(long minimum) {
            return GetMessage("MaxTimeOutOfRange", minimum);
        }

        /// <summary>
        /// Gets the "maximum plan length out of range" message.
        /// </summary>
        /// <param name="minimum">The minimum value maximum plan length can take.</param>
        /// <returns>The "maximum plan length out of range" message.</returns>
        internal string GetMaxPlanLengthOutOfRange(int minimum) {
            return GetMessage("MaxPlanLengthOutOfRange", minimum);
        }

        /// <summary>
        /// Gets the "beam width out of range" message.
        /// </summary>
        /// <param name="minimum">The minimum value beam width can take.</param>
        /// <returns>The "beam width out of range" message.</returns>
        internal string GetBeamWidthOutOfRange(int minimum) {
            return GetMessage("BeamWidthOutOfRange", minimum);
        }

        /// <summary>
        /// Gets the "heuristic weight out of range" message.
        /// </summary>
        /// <param name="minimum">The minimum value heuristic weight can take.</param>
        /// <returns>The "heuristic weight out of range" message.</returns>
        internal string GetHeuristicWeightOutOfRange(float minimum) {
            return GetMessage("HeuristicWeightOutOfRange", minimum);
        }

        /// <summary>
        /// Gets the "insufficient read permissions" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "insufficient read permissions" message.</returns>
        internal string GetInsufficientReadPermissions(string path) {
            return GetMessage("InsufficientReadPermissions", path);
        }

        /// <summary>
        /// Gets the "insufficient write permissions" message.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>The "insufficient write permissions" message.</returns>
        internal string GetInsufficientWritePermissions(string path) {
            return GetMessage("InsufficientWritePermissions", path);
        }

        /// <summary>
        /// Gets the "invalid path" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "invalid path" message.</returns>
        internal string GetInvalidPath(string path) {
            return GetMessage("InvalidPath", path);
        }

        /// <summary>
        /// Gets the "file not found" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "file not found" message.</returns>
        internal string GetFileNotFound(string path) {
            return GetMessage("FileNotFound", path);
        }

        /// <summary>
        /// Gets the "file already exists" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "file already exists" message.</returns>
        internal string GetFileAlreadyExists(string path) {
            return GetMessage("FileAlreadyExists", path);
        }

        /// <summary>
        /// Gets the "file can not be written" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "file can not be written" message.</returns>
        internal string GetFileCanNotBeWritten(string path) {
            return GetMessage("FileCanNotBeWritten", path);
        }

        /// <summary>
        /// Gets the "no domain found" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "no domain found" message.</returns>
        internal string GetNoDomainFound(string path) {
            return GetMessage("NoDomainFound", path);
        }

        /// <summary>
        /// Gets the "no problem found" message.
        /// </summary>
        /// <param name="path">The path of the file.</param>
        /// <returns>The "no problem found" message.</returns>
        internal string GetNoProblemFound(string path) {
            return GetMessage("NoProblemFound", path);
        }

        /// <summary>
        /// Gets the name of a search algorithm.
        /// </summary>
        /// <param name="algorithmId">The algorithm identifier.</param>
        /// <returns>The name of the search algorithm.</returns>
        internal string GetSearchAlgorithmName(XmlPlannerSearchAlgorithmId algorithmId) {
            return _resources.GetString(algorithmId.ToString(), Culture);
        }

        /// <summary>
        /// Gets a "variable = value" message.
        /// </summary>
        /// <param name="variable">The variable.</param>
        /// <param name="value">The value.</param>
        /// <returns>The "variable = value" message.</returns>
        internal string GetVariableEqualValue(string variable, int value) {
            return GetMessage("VariableEqualValue", variable, value);
        }

        /// <summary>
        /// Gets a "variable = value" message.
        /// </summary>
        /// <param name="variable">The variable.</param>
        /// <param name="value">The value.</param>
        /// <returns>The "variable = value" message.</returns>
        internal string GetVariableEqualValue(string variable, float value) {
            return GetMessage("VariableEqualValue", variable, value);
        }

        #endregion

    }

}
