/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Xml;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlDocument">XML document</see> which creates
    /// <see cref="XmlElementWithLineInfo">line information provided element</see>s.
    /// </summary>
    internal sealed class XmlDocumentWithLineInfo : XmlDocument {

        #region Fields

        /// <summary>
        /// The path.
        /// </summary>
        private readonly string _path;

        /// <summary>
        /// If is loaded.
        /// </summary>
        private bool _loaded;

        /// <summary>
        /// The content reader.
        /// </summary>
        private XmlTextReader _reader;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlDocumentWithLineInfo"/> class.
        /// </summary>
        /// <param name="path">The path.</param>
        internal XmlDocumentWithLineInfo(string path) {
            _path = path;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the path.
        /// </summary>
        /// <value>The path.</value>
        internal string Path {
            get {
                return _path;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Loads the document from <see cref="_path">the path</see>.
        /// </summary>
        internal void Load() {
            if (!_loaded) {
                _reader = new XmlTextReader(File.OpenRead(_path));
                Load(_reader);
                _reader.Close();
                _loaded = true;
            }
        }

        /// <summary>
        /// Creates an element with the specified <see cref="XmlNode.Prefix"/>,
        /// <see cref="XmlDocument.LocalName"/>, and namespace URI.
        /// </summary>
        /// <param name="prefix">The prefix of the new element (if any). <see cref="String.Empty"/> and null are
        /// equivalent.</param>
        /// <param name="localName">The local name of the new element.</param>
        /// <param name="namespaceURI">The namespace URI of the new element (if any). <see cref="String.Empty"/>
        /// and null are equivalent.</param>
        /// <returns>
        /// The new <see cref="XmlElement"/>.
        /// </returns>
        public override XmlElement CreateElement(string prefix, string localName, string namespaceURI) {
            return new XmlElementWithLineInfo(prefix, localName, namespaceURI, this,
                _reader.LineNumber, _reader.LinePosition);
        }

        /// <summary>
        /// Gets the <see cref="IXmlLineInfo">line information</see> of an element from its xpath.
        /// </summary>
        /// <param name="xpath">The xpath of the element.</param>
        /// <returns>The <see cref="IXmlLineInfo">line information</see> of the first element identified by
        /// <paramref name="xpath"/>.</returns>
        internal IXmlLineInfo GetLineInfo(string xpath) {
            return SelectSingleNode(xpath) as XmlElementWithLineInfo;
        }

        #endregion

    }

}