/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Xml;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlElement">XML element</see> with line information.
    /// </summary>
    [Serializable]
    internal sealed class XmlElementWithLineInfo : XmlElement, IXmlLineInfo {

        #region Fields

        /// <summary>
        /// The line number.
        /// </summary>
        private readonly int _lineNumber;

        /// <summary>
        /// The line position.
        /// </summary>
        private readonly int _linePosition;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlElementWithLineInfo"/> class.
        /// </summary>
        /// <param name="prefix">The prefix.</param>
        /// <param name="localName">The local name.</param>
        /// <param name="namespaceURI">The namespace URI.</param>
        /// <param name="doc">The document.</param>
        /// <param name="lineNumber">The line number.</param>
        /// <param name="linePosition">The line position.</param>
        internal XmlElementWithLineInfo(string prefix, string localName, string namespaceURI, XmlDocument doc,
        int lineNumber, int linePosition)
            : base(prefix, localName, namespaceURI, doc) {
            _lineNumber = lineNumber;
            _linePosition = linePosition;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the current line number.
        /// </summary>
        /// <value></value>
        /// <returns>The current line number or 0 if no line information is available (for example,
        /// <see cref="IXmlLineInfo.HasLineInfo"/> returns false).</returns>
        public int LineNumber {
            get {
                return _lineNumber;
            }
        }

        /// <summary>
        /// Gets the current line position.
        /// </summary>
        /// <value></value>
        /// <returns>The current line position or 0 if no line information is available (for example,
        /// <see cref="IXmlLineInfo.HasLineInfo"/> returns false).</returns>
        public int LinePosition {
            get {
                return _linePosition;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets a value indicating whether the class can return line information.
        /// </summary>
        /// <returns>
        /// True if <see cref="IXmlLineInfo.LineNumber"/> and <see cref="IXmlLineInfo.LinePosition"/> can be
        /// provided; otherwise, false.
        /// </returns>
        public bool HasLineInfo() {
            return true;
        }

        #endregion

    }

}