/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Text;
using System.Reflection;

using Aran.Translation;
using Aran.Representation.Xml;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a planner which uses XML documents to read and write data.
    /// </summary>
    [CLSCompliant(true)]
    public sealed class XmlPlanner {

        #region Events

        /// <summary>
        /// Occurs when the value of the <see cref="State"/> property changes.
        /// </summary>
        public event EventHandler<XmlPlannerStateChangedEventArgs> StateChanged;

        /// <summary>
        /// Occurs when a <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> is selected from
        /// <see cref="_searchAlgorithms">the sequence</see> to be executed.
        /// </summary>
        public event EventHandler<XmlPlannerSearchAlgorithmSelectedEventArgs> SearchAlgorithmSelected;

        #endregion

        #region Fields

        /// <summary>
        /// The state.
        /// </summary>
        private XmlPlannerState _state;

        /// <summary>
        /// If planning has been canceled.
        /// </summary>
        private bool _canceled;

        /// <summary>
        /// The time in which the current <see cref="State">state</see> was applied.
        /// </summary>
        private DateTime _startTime;

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The domain XML input document path.
        /// </summary>
        private string _domainPath;

        /// <summary>
        /// The problem XML input document path.
        /// </summary>
        private string _problemPath;

        /// <summary>
        /// The plan XML output document path.
        /// </summary>
        private string _outputPath;

        /// <summary>
        /// The <see cref="_outputEncoding">plan XML output document encoding</see> source.
        /// </summary>
        private XmlPlannerOutputEncodingSource _outputEncodingSource;

        /// <summary>
        /// The plan XML output document custom <see cref="Encoding">encoding</see>.
        /// </summary>
        private Encoding _outputEncoding;

        /// <summary>
        /// If the plan XML output document can be overwritten.
        /// </summary>
        private bool _overwriteOutput;

        /// <summary>
        /// If <see cref="_maxTime">the maximum planning time</see> must be considered.
        /// </summary>
        private bool _maxTimeEnabled;

        /// <summary>
        /// <para>The maximum planning time.</para>
        /// <para>The value is specified in milliseconds.</para>
        /// </summary>
        private long _maxTime;

        /// <summary>
        /// If <see cref="_maxPlanLength">the maximum plan length</see> must be considered.
        /// </summary>
        private bool _maxPlanLengthEnabled;

        /// <summary>
        /// The maximum plan length.
        /// </summary>
        private int _maxPlanLength;

        /// <summary>
        /// The sequence of <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s to be executed until
        /// a plan is found.
        /// </summary>
        private XmlPlannerSearchAlgorithmSequence _searchAlgorithms;

        /// <summary>
        /// The <see cref="XmlPlannerInequalitySet">predicate inequalities</see> to be considered when
        /// translating both domain and problem.
        /// </summary>
        private readonly XmlPlannerInequalitySet _inequalities;

        /// <summary>
        /// The translator.
        /// </summary>
        private readonly Translator<string, string, string, string, string, string, string> _translator;

        /// <summary>
        /// The parser.
        /// </summary>
        private readonly XmlParser _parser;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlanner"/> class.
        /// </summary>
        public XmlPlanner() {
            _state = XmlPlannerState.Stopped;
            _resources = new Resources();
            _domainPath = XmlPlannerDefaultValue.DomainPath;
            _problemPath = XmlPlannerDefaultValue.ProblemPath;
            _outputPath = XmlPlannerDefaultValue.OutputPath;
            _outputEncodingSource = XmlPlannerDefaultValue.OutputEncodingSource;
            _outputEncoding = XmlPlannerDefaultValue.OutputEncoding;
            _overwriteOutput = XmlPlannerDefaultValue.OverwriteOutput;
            _maxTime = XmlPlannerDefaultValue.MaxTime;
            _maxPlanLength = XmlPlannerDefaultValue.MaxPlanLength;
            _searchAlgorithms = new XmlPlannerSearchAlgorithmSequence(XmlPlannerDefaultValue.SearchAlgorithm);
            _parser = new XmlParser();
            _parser.AutoCloseStreams = true;
            _translator = new Translator<string, string, string, string, string, string, string>(_parser);
            _translator.InequalityPredicatesEnabled = XmlPlannerDefaultValue.InequalitiesEnabled;
            _translator.RootTypeEnabled = XmlPlannerDefaultValue.RootTypeEnabled;
            _translator.RootTypeId = XmlPlannerDefaultValue.RootType;
            _inequalities = new XmlPlannerInequalitySet(_resources, Translator.InequalityPredicates);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the state.
        /// </summary>
        /// <value>The state.</value>
        public XmlPlannerState State {
            get {
                return _state;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the planning has been canceled.
        /// </summary>
        /// <value>True if planning has been canceled; otherwise, false.</value>
        public bool Canceled {
            get {
                return _canceled;
            }
        }

        /// <summary>
        /// Gets the parser.
        /// </summary>
        /// <value>The parser.</value>
        internal XmlParser Parser {
            get {
                return _parser;
            }
        }

        /// <summary>
        /// Gets the translator.
        /// </summary>
        /// <value>The translator.</value>
        internal Translator<string, string, string, string, string, string, string> Translator {
            get {
                return _translator;
            }
        }

        /// <summary>
        /// Gets the resource manager.
        /// </summary>
        /// <value>The resource manager.</value>
        internal Resources Resources {
            get {
                return _resources;
            }
        }

        /// <summary>
        /// Gets or sets the domain XML input document path.
        /// </summary>
        /// <value>The domain XML input document path.</value>
        /// <exception cref="ArgumentNullException">The value is a null reference.</exception>
        /// <exception cref="ArgumentException">The value is empty.</exception>
        public string DomainPath {
            get {
                return _domainPath;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value", _resources.NullDomainPath);
                }
                if (value.Length == 0) {
                    throw new ArgumentException(_resources.EmptyDomainPath, "value");
                }
                _domainPath = value;
            }
        }

        /// <summary>
        /// Gets or sets the problem XML input document path.
        /// </summary>
        /// <value>The problem XML input document path.</value>
        /// <exception cref="ArgumentNullException">The value is a null reference.</exception>
        /// <exception cref="ArgumentException">The value is empty.</exception>
        public string ProblemPath {
            get {
                return _problemPath;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value", _resources.NullProblemPath);
                }
                if (value.Length == 0) {
                    throw new ArgumentException(_resources.EmptyProblemPath, "value");
                }
                _problemPath = value;
            }
        }

        /// <summary>
        /// Gets or sets the plan XML output document path.
        /// </summary>
        /// <value>The plan XML output document path.</value>
        /// <exception cref="ArgumentNullException">The value is a null reference.</exception>
        /// <exception cref="ArgumentException">The value is empty.</exception>
        public string OutputPath {
            get {
                return _outputPath;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value", _resources.NullOutputPath);
                }
                if (value.Length == 0) {
                    throw new ArgumentException(_resources.EmptyOutputPath, "value");
                }
                _outputPath = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the plan XML output document can be overwritten.
        /// </summary>
        /// <value>True if the plan XML output document can be overwritten; otherwise, false.</value>
        public bool OverwriteOutput {
            get {
                return _overwriteOutput;
            }
            set {
                _overwriteOutput = value;
            }
        }

        /// <summary>
        /// Gets or sets the plan XML output document <see cref="Encoding">encoding</see> source.
        /// </summary>
        /// <value>The plan XML output document <see cref="Encoding">encoding</see> source.</value>
        public XmlPlannerOutputEncodingSource OutputEncodingSource {
            get {
                return _outputEncodingSource;
            }
            set {
                _outputEncodingSource = value;
            }
        }

        /// <summary>
        /// Gets or sets the plan XML output document <see cref="Encoding">encoding</see>.
        /// </summary>
        /// <value>The plan XML output document <see cref="Encoding">encoding</see>.</value>
        /// <exception cref="ArgumentNullException">The value is a null reference.</exception>
        public Encoding OutputEncoding {
            get {
                return _outputEncoding;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value", _resources.NullOutputEncoding);
                }
                _outputEncoding = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether <see cref="MaxTime">the maximum planning time</see> must be
        /// considered.
        /// </summary>
        /// <value>
        /// True if <see cref="MaxTime">the maximum planning time</see> must be considered; otherwise, false.
        /// </value>
        public bool MaxTimeEnabled {
            get {
                return _maxTimeEnabled;
            }
            set {
                _maxTimeEnabled = value;
            }
        }

        /// <summary>
        /// Gets or sets the maximum planning time, in milliseconds.
        /// </summary>
        /// <value>The the maximum planning time, in milliseconds.</value>
        /// <exception cref="ArgumentOutOfRangeException">The value is less than
        /// <see cref="XmlPlannerMinimumValue.MaxTime">the minimum one.</see>.</exception>
        public long MaxTime {
            get {
                return _maxTime;
            }
            set {
                if (value < XmlPlannerMinimumValue.MaxTime) {
                    throw new ArgumentOutOfRangeException("value",
                        _resources.GetMaxTimeOutOfRange(XmlPlannerMinimumValue.MaxTime));
                }
                _maxTime = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether <see cref="MaxPlanLength">the maximum plan length</see> must
        /// be considered.
        /// </summary>
        /// <value>True if <see cref="MaxPlanLength">the maximum plan length</see> must be considered; otherwise,
        /// false.</value>
        public bool MaxPlanLengthEnabled {
            get {
                return _maxPlanLengthEnabled;
            }
            set {
                _maxPlanLengthEnabled = value;
            }
        }

        /// <summary>
        /// Gets or sets the maximum plan length.
        /// </summary>
        /// <value>The the maximum plan length.</value>
        /// <exception cref="ArgumentOutOfRangeException">The value is less than
        /// <see cref="XmlPlannerMinimumValue.MaxPlanLength">the minimum one.</see>.</exception>
        public int MaxPlanLength {
            get {
                return _maxPlanLength;
            }
            set {
                if (value < XmlPlannerMinimumValue.MaxPlanLength) {
                    throw new ArgumentOutOfRangeException("value",
                        _resources.GetMaxPlanLengthOutOfRange(XmlPlannerMinimumValue.MaxPlanLength));
                }
                _maxPlanLength = value;
            }
        }

        /// <summary>
        /// Gets the sequence of <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s to be executed
        /// until a plan is found.
        /// </summary>
        /// <value>The sequence of <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s to be executed
        /// until a plan is found.</value>
        public XmlPlannerSearchAlgorithmSequence SearchAlgorithms {
            get {
                return _searchAlgorithms;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether <see cref="RootType">the root type</see> must be considered.
        /// </summary>
        /// <value>True if <see cref="RootType">the root type</see> must be considered; otherwise, false.</value>
        public bool RootTypeEnabled {
            get {
                return _translator.RootTypeEnabled;
            }
            set {
                _translator.RootTypeEnabled = value;
            }
        }

        /// <summary>
        /// Gets or sets the domain root type.
        /// </summary>
        /// <value>The domain root type.</value>
        /// <remarks>
        /// The domain root type is an special type which is the parent of all types specified in the domain XML
        /// document. For that reason, it must not be defined in that document, although it can be used in it
        /// and also in the problem document.
        /// </remarks>
        /// <exception cref="ArgumentNullException">The value is a null reference.</exception>
        /// <exception cref="ArgumentException">The value is empty.</exception>
        public string RootType {
            get {
                return _translator.RootTypeId;
            }
            set {
                if (value.Length == 0) {
                    throw new ArgumentException(_resources.EmptyType, "value");
                }
                try {
                    _translator.RootTypeId = value;
                } catch (ArgumentNullException) {
                    throw new ArgumentNullException("value", _resources.NullType);
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether
        /// <see cref="XmlPlannerInequalitySet">the predicate inequalities</see> must be considered.
        /// </summary>
        /// <value>True if <see cref="XmlPlannerInequalitySet">the predicate inequalities</see> must be
        /// considered; otherwise, false.</value>
        public bool InequalitiesEnabled {
            get {
                return _translator.InequalityPredicatesEnabled;
            }
            set {
                _translator.InequalityPredicatesEnabled = value;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlPlannerInequalitySet">predicate inequalities</see> to be considered when
        /// translating both domain and problem.
        /// </summary>
        /// <value>The <see cref="XmlPlannerInequalitySet">predicate inequalities</see> to be considered when
        /// translating both domain and problem.</value>
        public XmlPlannerInequalitySet Inequalities {
            get {
                return _inequalities;
            }
        }

        /// <summary>
        /// Gets a value indicating whether domain and problem paths are equal.
        /// </summary>
        /// <value>True if domain and problem paths are equal; otherwise, false.</value>
        internal bool DomainAndProblemPathsAreEqual {
            get {
                return _domainPath.Equals(_problemPath, StringComparison.CurrentCultureIgnoreCase);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Sets the state.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <param name="messages">The messages accumulated until now.</param>
        private void SetState(XmlPlannerState state, XmlPlannerMessageCollection messages) {
            XmlPlannerState previousState = _state;
            _state = state;
            TimeSpan timeTaken = DateTime.Now.Subtract(_startTime);
            _startTime = DateTime.Now;
            if (StateChanged != null) {
                StateChanged(this, new XmlPlannerStateChangedEventArgs(previousState, state, timeTaken, messages));
            }
        }

        /// <summary>
        /// Changes the state to the next one.
        /// </summary>
        /// <param name="messages">The messages accumulated until now.</param>
        internal void SetNextState(XmlPlannerMessageCollection messages) {
            switch (_state) {
                case XmlPlannerState.Stopped:
                    SetState(XmlPlannerState.Starting, messages);
                    break;
                case XmlPlannerState.Starting:
                    SetState(XmlPlannerState.SettingUpTranslator, messages);
                    break;
                case XmlPlannerState.SettingUpTranslator:
                    SetState(XmlPlannerState.Translating, messages);
                    break;
                case XmlPlannerState.Translating:
                    SetState(XmlPlannerState.SettingUpSearcher, messages);
                    break;
                case XmlPlannerState.SettingUpSearcher:
                    SetState(XmlPlannerState.Searching, messages);
                    break;
                case XmlPlannerState.Searching:
                    SetState(XmlPlannerState.SettingUpOutputWriter, messages);
                    break;
                case XmlPlannerState.SettingUpOutputWriter:
                    SetState(XmlPlannerState.WritingOutput, messages);
                    break;
                case XmlPlannerState.WritingOutput:
                    SetState(XmlPlannerState.Stopped, messages);
                    break;
            }
        }

        /// <summary>
        /// Sets the state to <see cref="XmlPlannerState.Stopped">stopped</see> if not already set.
        /// </summary>
        /// <param name="messages">The messages accumulated until now.</param>
        private void SetStopped(XmlPlannerMessageCollection messages) {
            if (_state != XmlPlannerState.Stopped) {
                SetState(XmlPlannerState.Stopped, messages);
            }
        }

        /// <summary>
        /// Notifies the selected <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <param name="algorithm">The algorithm.</param>
        internal void NotifySelectedSearchAlgorithm(XmlPlannerSearchAlgorithm algorithm) {
            if (SearchAlgorithmSelected != null) {
                SearchAlgorithmSelected(this, new XmlPlannerSearchAlgorithmSelectedEventArgs(algorithm));
            }
        }

        /// <summary>
        /// Cancels the planning.
        /// </summary>
        public void Cancel() {
            _canceled = true;
        }

        /// <summary>
        /// Executes the planner.
        /// </summary>
        /// <exception cref="XmlPlanNotFoundException">No plan is found.</exception>
        public void Plan() {
            _canceled = false;
            _startTime = DateTime.Now;
            XmlPlannerMessageManager messages = new XmlPlannerMessageManager(this);
            XmlPlannerMessageCollection messageCollection = messages.Collection;
            SetNextState(messageCollection);
            XmlPlannerFileManager fileManager = new XmlPlannerFileManager(_resources, messages);
            XmlPlannerTranslator translator = new XmlPlannerTranslator(this, fileManager, messages);
            XmlPlannerSearcher searcher = new XmlPlannerSearcher(this, translator, messages);
            XmlPlannerOutputWriter writer = new XmlPlannerOutputWriter(this, fileManager, translator, searcher, messages);
            SetNextState(messageCollection);
            bool success = translator.Translate() && searcher.Search() && writer.Write();
            SetStopped(messageCollection);
            if (!success) {
                throw new XmlPlanNotFoundException(_resources.NoPlanFound, messages.Collection);
            }
        }

        /// <summary>
        /// Gets the input XML schema.
        /// </summary>
        /// <returns>A <see cref="Stream">stream</see> from which read the input XML schema.</returns>
        public static Stream GetInputSchema() {
            return XmlParser.GetInputSchema();
        }

        /// <summary>
        /// Gets the output XML schema.
        /// </summary>
        /// <returns>A <see cref="Stream">stream</see> from which read the output XML schema.</returns>
        public static Stream GetOutputSchema() {
            const string XmlSchemaFileName = "Output.xsd";
            return Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(XmlPlanner), XmlSchemaFileName);
        }

        /// <summary>
        /// Gets the license plain text file.
        /// </summary>
        /// <returns>A <see cref="Stream">stream</see> from which read the license plain text file.</returns>
        public static Stream GetLicense() {
            const string LicenseFileName = "License.txt";
            return Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(XmlPlanner), LicenseFileName);
        }

        #endregion

    }

}
