/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Translation;
using Aran.Searching.Specialized;
using Aran.Searching;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents the Enforced Hill Climbing algorithm.
    /// </summary>
    [Serializable]
    internal sealed class XmlPlannerEnforcedHillClimbingAlgorithm : XmlPlannerSearchAlgorithm {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerEnforcedHillClimbingAlgorithm"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        internal XmlPlannerEnforcedHillClimbingAlgorithm(Resources resources)
            : base(resources) {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="XmlPlannerSearchAlgorithmId">identifier</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerSearchAlgorithmId">identifier</see>.</value>
        public override XmlPlannerSearchAlgorithmId Id {
            get {
                return XmlPlannerSearchAlgorithmId.EnforcedHillClimbing;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Creates the enforced hill climbing algorithm.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The enforced hill climbing algorithm</returns>
        private static ISearchAlgorithm<State> CreateEnforcedHillClimbingAlgorithm(XmlPlanner planner,
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            IInitialStateProvider<State> initialStateProvider = CreateInitialStateProvider(translationResult);
            IGoalStateVerifier<State> goalVerifier = CreateGoalStateVerifier(planner, translationResult);
            IStateCostProvider<State> costCalculator = CreateStateCostProvider(translationResult);
            return new EnforcedHillClimbingAlgorithm(initialStateProvider, goalVerifier, costCalculator,
                translationResult.FactCount, translationResult.Actions, translationResult.OperatorCount,
                translationResult.PreconditionCounts, translationResult.ActionCounts,
                translationResult.ActionActivationTable, translationResult.GoalCount,
                translationResult.GoalFacts, CreateStateEqualityComparer(translationResult));
        }

        /// <summary>
        /// Creates the search algorithm.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The search algorithm.</returns>
        internal override ISearchAlgorithm<State> CreateSearchAlgorithm(XmlPlanner planner,
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            //without goals, the relaxed plan heuristic returns always zero
            //so without goals the algorithm is nullated
            if (translationResult.GoalCount == 0) {
                return new NullAlgorithm<State>();
            }
            return CreateEnforcedHillClimbingAlgorithm(planner, translationResult);
        }

        #endregion

    }

}
