/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Security;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a <see cref="XmlPlanner">planner</see> file manager.
    /// </summary>
    internal sealed class XmlPlannerFileManager {

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The <see cref="XmlPlanner">planner</see> messages.
        /// </summary>
        private readonly XmlPlannerMessageManager _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerFileManager"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        /// <param name="messages">The <see cref="XmlPlanner">planner</see> messages.</param>
        internal XmlPlannerFileManager(Resources resources, XmlPlannerMessageManager messages) {
            _resources = resources;
            _messages = messages;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Opens a file to be read.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <returns>A <see cref="Stream">stream</see> from which read the file content.</returns>
        internal Stream OpenRead(string path) {
            Stream file = null;
            try {
                file = File.OpenRead(path);
            } catch (UnauthorizedAccessException) {
                _messages.AddError(_resources.GetInsufficientReadPermissions(path));
            } catch (ArgumentException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (PathTooLongException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (DirectoryNotFoundException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (FileNotFoundException) {
                _messages.AddError(_resources.GetFileNotFound(path));
            } catch (NotSupportedException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            }
            return file;
        }

        /// <summary>
        /// Opens a file to be written.
        /// </summary>
        /// <param name="path">The file path.</param>
        /// <param name="overwrite">If the file can be overwritten.</param>
        /// <returns>
        /// A <see cref="Stream">stream</see> to which write the file content.
        /// </returns>
        internal Stream OpenWrite(string path, bool overwrite) {
            Stream file = null;
            try {
                if (File.Exists(path)) {
                    if (overwrite) {
                        File.Delete(path);
                        file = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                    } else {
                        _messages.AddError(_resources.GetFileAlreadyExists(path));
                    }
                } else {
                    file = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None);
                }
            } catch (UnauthorizedAccessException) {
                _messages.AddError(_resources.GetInsufficientWritePermissions(path));
            } catch (ArgumentException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (PathTooLongException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (DirectoryNotFoundException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            } catch (IOException) {
                _messages.AddError(_resources.GetFileCanNotBeWritten(path));
            } catch (SecurityException) {
                _messages.AddError(_resources.GetInsufficientWritePermissions(path));
            } catch (NotSupportedException) {
                _messages.AddError(_resources.GetInvalidPath(path));
            }
            return file;
        }

        #endregion

    }

}
