/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

using Aran.Translation;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlPlannerInequality">planner predicate inequality</see> set.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public class XmlPlannerInequalitySet : IEnumerable<XmlPlannerInequality> {

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The internal inequality predicates.
        /// </summary>
        private readonly InequalityPredicateSet<string, string> _predicates;

        /// <summary>
        /// The inequality predicates.
        /// </summary>
        private readonly IDictionary<string, XmlPlannerInequality> _inequalities;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerInequalitySet"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        /// <param name="predicates">The internal inequality predicates.</param>
        internal XmlPlannerInequalitySet(Resources resources, InequalityPredicateSet<string, string> predicates) {
            _resources = resources;
            _predicates = predicates;
            _inequalities = new Dictionary<string, XmlPlannerInequality>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the number of inequality predicates.
        /// </summary>
        /// <value>The number of inequality predicates.</value>
        public int Count {
            get {
                return _inequalities.Count;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlPlannerInequality">inequality predicate</see> with the specified predicate
        /// name.
        /// </summary>
        /// <param name="predicate">The inequality predicate name.</param>
        /// <value>The <see cref="XmlPlannerInequality">inequality predicate</see> with
        /// <paramref name="predicate"/>.</value>
        /// <exception cref="ArgumentNullException"><paramref name="predicate"/> is a null reference.</exception>
        /// <exception cref="ArgumentException">No <see cref="XmlPlannerInequality">inequality predicate</see> is
        /// found with <paramref name="predicate"/> name.</exception>
        public XmlPlannerInequality this[string predicate] {
            get {
                try {
                    return _inequalities[predicate];
                } catch (ArgumentNullException) {
                    throw new ArgumentNullException("predicate", _resources.NullPredicate);
                } catch (KeyNotFoundException) {
                    throw new ArgumentException(_resources.PredicateNotFound, "predicate");
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether contains an <see cref="XmlPlannerInequality">inequality predicate</see> with the
        /// specified predicate name.
        /// </summary>
        /// <param name="predicate">The inequality predicate name.</param>
        /// <returns>
        /// True if contains an <see cref="XmlPlannerInequality">inequality predicate</see> with
        /// <paramref name="predicate"/>; otherwise, false.
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicate"/> is a null reference.</exception>
        public bool Contains(string predicate) {
            try {
                return _inequalities.ContainsKey(predicate);
            } catch (ArgumentNullException) {
                throw new ArgumentNullException("predicate", _resources.NullPredicate);
            }
        }

        /// <summary>
        /// Adds an <see cref="XmlPlannerInequality">inequality predicate</see>.
        /// </summary>
        /// <param name="predicate">The inequality predicate name.</param>
        /// <param name="type">The type name of the objects being affected by the inequality.</param>
        /// <returns>The <see cref="XmlPlannerInequality">inequality predicate</see> added.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicate"/> or <paramref name="type"/>
        /// is a null reference.</exception>
        /// <exception cref="ArgumentException"><paramref name="predicate"/> or <paramref name="type"/> is empty
        /// or an <see cref="XmlPlannerInequality">inequality predicate</see> already exists with the same
        /// <paramref name="predicate"/> name.</exception>
        public XmlPlannerInequality Add(string predicate, string type) {
            if (type == null) {
                throw new ArgumentNullException("type", _resources.NullType);
            }
            if (type.Length == 0) {
                throw new ArgumentException(_resources.EmptyType, "type");
            }
            if (predicate == null) {
                throw new ArgumentNullException("predicate", _resources.NullPredicate);
            }
            if (predicate.Length == 0) {
                throw new ArgumentException(_resources.EmptyPredicate, "predicate");
            }
            try {
                XmlPlannerInequality inequality = new XmlPlannerInequality(_predicates.Add(predicate, type));
                _inequalities.Add(predicate, inequality);
                return inequality;
            } catch (ArgumentException) {
                throw new ArgumentException(_resources.PredicateAlreadyExists, "predicate");
            }
        }

        /// <summary>
        /// Removes an <see cref="XmlPlannerInequality">inequality predicate</see> with the specified predicate
        /// name.
        /// </summary>
        /// <param name="predicate">The predicate name.</param>
        /// <returns>True if the <see cref="XmlPlannerInequality">inequality predicate</see> is removed.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicate"/> is a null reference.</exception>
        public bool Remove(string predicate) {
            try {
                bool removed = _inequalities.Remove(predicate);
                if (removed) {
                    _predicates.Remove(predicate);
                }
                return removed;
            } catch (ArgumentNullException) {
                throw new ArgumentNullException("predicate", _resources.NullPredicate);
            }
        }

        /// <summary>
        /// Clears the set.
        /// </summary>
        public void Clear() {
            _inequalities.Clear();
            _predicates.Clear();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the set.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the set.
        /// </returns>
        public IEnumerator<XmlPlannerInequality> GetEnumerator() {
            return _inequalities.Values.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the set.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the set.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (this as IEnumerable<XmlPlannerInequality>).GetEnumerator();
        }

        #endregion

    }

}
