/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a <see cref="XmlPlanner">planner</see> message.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public abstract class XmlPlannerMessage : IComparable<XmlPlannerMessage> {

        #region Fields

        /// <summary>
        /// The <see cref="XmlPlannerMessageType">message type</see>.
        /// </summary>
        private readonly XmlPlannerMessageType _type;

        /// <summary>
        /// The text.
        /// </summary>
        private readonly string _text;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerMessage"/> class.
        /// </summary>
        /// <param name="type">The <see cref="XmlPlannerMessageType">message type</see>.</param>
        /// <param name="text">The text.</param>
        protected internal XmlPlannerMessage(XmlPlannerMessageType type, string text) {
            _type = type;
            _text = text;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessageType">message type</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerMessageType">message type</see>.</value>
        public XmlPlannerMessageType Type {
            get {
                return _type;
            }
        }

        /// <summary>
        /// Gets the text.
        /// </summary>
        /// <value>The text.</value>
        public string Text {
            get {
                return _text;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this message has file information.
        /// </summary>
        /// <value>true if this message has file information; otherwise, false.</value>
        public abstract bool HasFileInfo { get; }

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessageFileType">file type</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerMessageFileType">file type</see>.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public abstract XmlPlannerMessageFileType FileType { get; }

        /// <summary>
        /// Gets the file path.
        /// </summary>
        /// <value>The file path.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public abstract string FilePath { get; }

        /// <summary>
        /// Gets the line number.
        /// </summary>
        /// <value>The line number.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public abstract int LineNumber { get; }

        /// <summary>
        /// Gets the line position.
        /// </summary>
        /// <value>The line position.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public abstract int LinePosition { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Compares the message with another.
        /// </summary>
        /// <param name="other">A message to compare with this one.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the messages being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This message is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This message is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This message is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="other"/> is a null reference.</exception>
        public virtual int CompareTo(XmlPlannerMessage other) {
            const int BothEqual = 0;
            if (other == null) {
                throw new ArgumentNullException("other");
            }
            //compares the types
            int comparison = ((int)_type - (int)other._type);
            //if both types are equal, compares the texts
            if (comparison == BothEqual) {
                comparison = _text.CompareTo(other._text);
            }
            return comparison;
        }

        #endregion

    }

}
