/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlPlannerMessage">planner message</see> collection.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlPlannerMessageCollection : IEnumerable<XmlPlannerMessage> {

        #region Fields

        /// <summary>
        /// The number of <see cref="XmlPlannerMessageType.Warning">warning</see> messages.
        /// </summary>
        private int _warningCount;

        /// <summary>
        /// The number of <see cref="XmlPlannerMessageType.Error">error</see> messages.
        /// </summary>
        private int _errorCount;

        /// <summary>
        /// The number of <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.
        /// </summary>
        private int _parserErrorCount;

        /// <summary>
        /// The <see cref="XmlPlannerMessage">message</see>s.
        /// </summary>
        private readonly List<XmlPlannerMessage> _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerMessageCollection"/> class.
        /// </summary>
        internal XmlPlannerMessageCollection() {
            _messages = new List<XmlPlannerMessage>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessage">message</see> at the specified index.
        /// </summary>
        /// <param name="index">The index of the <see cref="XmlPlannerMessage">message</see>.</param>
        /// <value>The <see cref="XmlPlannerMessage">message</see> at <paramref name="index"/>.</value>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="index"/> is less than zero or
        /// <paramref name="index"/> is equal to or greater than <see cref="Count"/>.</exception>
        public XmlPlannerMessage this[int index] {
            get {
                return _messages[index];
            }
        }

        /// <summary>
        /// Gets the number of <see cref="XmlPlannerMessageType.Warning">warning</see> messages.
        /// </summary>
        /// <value>The number of <see cref="XmlPlannerMessageType.Warning">warning</see> messages.</value>
        public int WarningCount {
            get {
                return _warningCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="XmlPlannerMessageType.Error">error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="XmlPlannerMessageType.Error">error</see> messages.</value>
        public int ErrorCount {
            get {
                return _errorCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.</value>
        public int ParserErrorCount {
            get {
                return _parserErrorCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="XmlPlannerMessageType.Error">error</see> and
        /// <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="XmlPlannerMessageType.Error">error</see> and
        /// <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.</value>
        public int TotalErrorCount {
            get {
                return _errorCount + _parserErrorCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="XmlPlannerMessageType.Warning">warning</see>,
        /// <see cref="XmlPlannerMessageType.Error">error</see> and
        /// <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="XmlPlannerMessageType.Warning">warning</see>,
        /// <see cref="XmlPlannerMessageType.Error">error</see> and
        /// <see cref="XmlPlannerMessageType.ParserError">parser error</see> messages.</value>
        public int Count {
            get {
                return _warningCount + _errorCount + _parserErrorCount;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds a <see cref="XmlPlannerMessage">message</see>.
        /// </summary>
        /// <param name="message">The <see cref="XmlPlannerMessage">message</see>.</param>
        internal void Add(XmlPlannerMessage message) {
            _messages.Add(message);
            switch (message.Type) {
                case XmlPlannerMessageType.Warning:
                    _warningCount++;
                    break;
                case XmlPlannerMessageType.Error:
                    _errorCount++;
                    break;
                case XmlPlannerMessageType.ParserError:
                    _parserErrorCount++;
                    break;
            }
        }

        /// <summary>
        /// Sorts the <see cref="XmlPlannerMessage">message</see>.
        /// </summary>
        /// <remarks>Uses the <see cref="XmlPlannerMessage.CompareTo(XmlPlannerMessage)">message comparison
        /// method</see> when comparing the <see cref="XmlPlannerMessage">message</see>s.</remarks>
        public void Sort() {
            _messages.Sort();
        }

        /// <summary>
        /// Sorts the messages.
        /// </summary>
        /// <param name="comparison">The <see cref="Comparison{T}">comparison</see> to use when comparing
        /// the <see cref="XmlPlannerMessage">message</see>s.</param>
        /// <exception cref="ArgumentNullException"><paramref name="comparison"/> is a null reference.</exception>
        /// <exception cref="ArgumentException">The implementation of <paramref name="comparison"/> caused an
        /// error during the sort. For example, <paramref name="comparison"/> might not return 0 when comparing
        /// a <see cref="XmlPlannerMessage">message</see> with itself.</exception>
        public void Sort(Comparison<XmlPlannerMessage> comparison) {
            _messages.Sort(comparison);
        }

        /// <summary>
        /// Sorts the messages.
        /// </summary>
        /// <param name="comparer">The <see cref="IComparer{T}">comparer</see> to use when comparing
        /// the <see cref="XmlPlannerMessage">message</see>s.</param>
        /// <exception cref="ArgumentNullException"><paramref name="comparison"/> is a null reference.</exception>
        /// <exception cref="ArgumentException">The implementation of <paramref name="comparer"/> caused an
        /// error during the sort. For example, <paramref name="comparer"/> might not return 0 when comparing
        /// a <see cref="XmlPlannerMessage">message</see> with itself.</exception>
        public void Sort(IComparer<XmlPlannerMessage> comparer) {
            _messages.Sort(comparer);
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<XmlPlannerMessage> GetEnumerator() {
            return _messages.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (_messages as IEnumerable).GetEnumerator();
        }

        #endregion

    }

}