/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Xml;

using Aran.Representation.Xml;
using Aran.Translation;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlPlannerMessage">planner message</see> manager.
    /// </summary>
    /// <remarks>The manager is used to create <see cref="XmlPlannerMessage">planner message</see>s from
    /// <see cref="Message{T}">translator messages</see>s.</remarks>
    internal sealed class XmlPlannerMessageManager {

        #region Fields

        /// <summary>
        /// The <see cref="XmlPlanner">planner</see>.
        /// </summary>
        private readonly XmlPlanner _planner;

        /// <summary>
        /// The <see cref="XmlPlannerMessage">message</see> collection.
        /// </summary>
        private readonly XmlPlannerMessageCollection _messages;

        /// <summary>
        /// The domain XML document.
        /// </summary>
        private readonly XmlDocumentWithLineInfo _domainDocument;

        /// <summary>
        /// The problem XML document.
        /// </summary>
        private readonly XmlDocumentWithLineInfo _problemDocument;

        /// <summary>
        /// The domain root xpath.
        /// </summary>
        private readonly string _domainRootXPath;

        /// <summary>
        /// The problem root xpath.
        /// </summary>
        private readonly string _problemRootXPath;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerMessageManager"/> class.
        /// </summary>
        /// <param name="planner">The <see cref="XmlPlanner">planner</see>.</param>
        internal XmlPlannerMessageManager(XmlPlanner planner) {
            _planner = planner;
            _messages = new XmlPlannerMessageCollection();
            _domainDocument = new XmlDocumentWithLineInfo(_planner.DomainPath);
            if (_planner.DomainAndProblemPathsAreEqual) {
                _problemDocument = _domainDocument;
            } else {
                _problemDocument = new XmlDocumentWithLineInfo(_planner.ProblemPath);
            }
            string rootXPath = XPathManager.Create(XmlElementName.Definition);
            _domainRootXPath = XPathManager.Append(rootXPath, XmlElementName.Domain);
            _problemRootXPath = XPathManager.Append(rootXPath, XmlElementName.Problem);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessage">message</see> collection.
        /// </summary>
        /// <value>The <see cref="XmlPlannerMessage">message</see> collection.</value>
        internal XmlPlannerMessageCollection Collection {
            get {
                return _messages;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds an error.
        /// </summary>
        /// <param name="text">The error text.</param>
        internal void AddError(string text) {
            _messages.Add(new XmlPlannerMessageWithoutFileInfo(XmlPlannerMessageType.Error, text));
        }

        /// <summary>
        /// Translates a <see cref="MessageType">translator message type</see> to a
        /// <see cref="XmlPlannerMessageType">message type</see>.
        /// </summary>
        /// <param name="type">The <see cref="MessageType">translator message type</see> to translate.</param>
        /// <returns>The <see cref="MessageType">translator message type</see> translation.</returns>
        private static XmlPlannerMessageType GetMessageType(MessageType type) {
            switch (type) {
                case MessageType.SemanticWarning:
                    return XmlPlannerMessageType.Warning;
                case MessageType.SemanticError:
                    return XmlPlannerMessageType.Error;
                case MessageType.SyntacticError:
                    return XmlPlannerMessageType.ParserError;
                default:
                    return XmlPlannerMessageType.Error;
            }
        }

        /// <summary>
        /// Appends a position to an xpath.
        /// </summary>
        /// <remarks>XML subelements are appended too, depending on the provided parameters information.</remarks>
        /// <param name="xpath">The xpath.</param>
        /// <param name="position">The position.</param>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        /// <returns></returns>
        private static string AppendPosition<S>(string xpath, int position, S source, MessageCode code) {
            string xpathWithPosition = xpath;
            switch (code) {
                case MessageCode.InexistentObject:
                case MessageCode.ArgumentTypeMismatch:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Object, position);
                    break;
                case MessageCode.InexistentPreconditionFact:
                case MessageCode.FactPreconditionAlreadyPresent:
                case MessageCode.FactPreconditionWillBeAlreadyPresent:
                case MessageCode.FactPreconditionAlreadyAdded:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Preconditions,
                        XmlElementName.Fact, position);
                    break;
                case MessageCode.InexistentDeletionFact:
                case MessageCode.FactDeletionAlreadyPresent:
                case MessageCode.FactDeletionWillBeAlreadyPresent:
                case MessageCode.FactDeletionAlreadyAdded:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Deletions,
                        XmlElementName.Fact, position);
                    break;
                case MessageCode.InexistentAdditionFact:
                case MessageCode.FactAdditionAlreadyPresent:
                case MessageCode.FactAdditionWillBeAlreadyPresent:
                case MessageCode.FactAdditionAlreadyRequired:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Additions,
                        XmlElementName.Fact, position);
                    break;
                case MessageCode.InexistentInitialFact:
                case MessageCode.InitialFactAlreadyPresent:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.InitialFacts,
                        XmlElementName.Fact, position);
                    break;
                case MessageCode.InexistentGoalFact:
                case MessageCode.UnreachableGoal:
                case MessageCode.GoalFactAlreadyPresent:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.GoalFacts,
                        XmlElementName.Fact, position);
                    break;
                case MessageCode.InexistentParameterType:
                    if (source.GetType().Equals(typeof(XmlOperator))) {
                        xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Parameters,
                            XmlElementName.Object, position);
                    } else {
                        xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Parameter, position);
                    }
                    break;
                case MessageCode.ParameterIdAlreadyUsed:
                    if (source.GetType().Equals(typeof(XmlOperator))) {
                        xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Parameters,
                            XmlElementName.Object, position);
                    } else {
                        xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Parameter, position);
                    }
                    break;
                case MessageCode.VariableAlreadyDefined:
                    xpathWithPosition = XPathManager.Append(xpath, XmlElementName.Variables,
                        XmlElementName.Variable, position);
                    break;
            }
            return xpathWithPosition;
        }

        /// <summary>
        /// Adds all the specified <see cref="Message{T}">translator message</see>s to
        /// <see cref="_messages">the messages</see>.
        /// </summary>
        /// <param name="messages">The <see cref="Message{T}">translator message</see>s.</param>
        private void AddAll<S>(MessageCollection<S> messages) {
            Resources resources = _planner.Resources;
            XmlDocumentWithLineInfo document = null;
            XmlPlannerMessageFileType fileType = XmlPlannerMessageFileType.Domain;
            string xpath;
            IXmlLineInfo lineInfo;
            //for each message
            foreach (Message<S> message in messages) {
                //gets the source xpath
                xpath = (message.Source as XmlLocatable).XPath;
                //selects the message file type depending on source xpath start
                if (xpath.StartsWith(_domainRootXPath)) {
                    document = _domainDocument;
                    fileType = XmlPlannerMessageFileType.Domain;
                } else if (xpath.StartsWith(_problemRootXPath)) {
                    document = _problemDocument;
                    fileType = XmlPlannerMessageFileType.Problem;
                }
                //if the message has a position, appends it to the source xpath to get a more accurated one
                if (message.HasPosition) {
                    xpath = AppendPosition(xpath, message.Position, message.Source, message.Code);
                }
                //gets the line information from the source xpath
                lineInfo = document.GetLineInfo(xpath);
                //adds a message with the line information
                _messages.Add(new XmlPlannerMessageWithFileInfo(GetMessageType(message.Type),
                    resources.GetMessage(message.Code), fileType, document.Path, lineInfo.LineNumber,
                    lineInfo.LinePosition));
            }
        }

        /// <summary>
        /// Adds all the specified <see cref="Message{T}">translator message</see>s to
        /// <see cref="_messages">the messages</see>.
        /// </summary>
        /// <param name="messages">The <see cref="Message{T}">translator message</see>s.</param>
        private void AddAll(MessageCollection<InequalityPredicate<string, string>> messages) {
            Resources resources = _planner.Resources;
            foreach (Message<InequalityPredicate<string, string>> message in messages) {
                _messages.Add(new XmlPlannerMessageWithoutFileInfo(XmlPlannerMessageType.Error,
                    resources.GetInequalityMessage(message.Code, message.Source)));
            }
        }

        /// <summary>
        /// Adds all the specified <see cref="Message{T}">translator message</see>s to
        /// <see cref="_messages">the messages</see>.
        /// </summary>
        /// <param name="messages">The <see cref="Message{T}">translator message</see>s.</param>
        internal void AddAll(MessageManager<string, string, string, string, string, string, string> messages) {
            _domainDocument.Load();
            _problemDocument.Load();
            AddAll(messages.DomainMessages);
            AddAll(messages.FactMessages);
            AddAll(messages.FunctionMessages);
            AddAll(messages.MetricMessages);
            AddAll(messages.ObjectMessages);
            AddAll(messages.OperandMessages);
            AddAll(messages.OperationMessages);
            AddAll(messages.OperatorMessages);
            AddAll(messages.PredicateMessages);
            AddAll(messages.ProblemMessages);
            AddAll(messages.TypeMessages);
            AddAll(messages.VariableMessages);
            AddAll(messages.InequalityPredicateMessages);
        }

        /// <summary>
        /// Adds all the specified <see cref="XmlErrorInfo">error</see>s to
        /// <see cref="_messages">the messages</see>.
        /// </summary>
        /// <param name="errors">The <see cref="XmlErrorInfo">error</see>s.</param>
        internal void AddAll(XmlErrorInfoCollection errors) {
            XmlParser parser = _planner.Parser;
            Stream domainStream = parser.DomainStream;
            Stream problemStream = parser.ProblemStream;
            XmlPlannerMessageFileType fileType = XmlPlannerMessageFileType.Domain;
            string filePath = null;
            //for each error
            foreach (XmlErrorInfo error in errors) {
                //gets de file type and path depending on the source stream
                if (error.Stream == domainStream) {
                    fileType = XmlPlannerMessageFileType.Domain;
                    filePath = _planner.DomainPath;
                } else if (error.Stream == problemStream) {
                    fileType = XmlPlannerMessageFileType.Problem;
                    filePath = _planner.ProblemPath;
                }
                //adds a message without file information
                _messages.Add(new XmlPlannerMessageWithFileInfo(XmlPlannerMessageType.Error, error.Message, fileType,
                    filePath, error.LineNumber, error.LinePosition));
            }
        }

        #endregion

    }

}
