/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlPlannerMessage">planner message</see> with file information.
    /// </summary>
    [Serializable]
    internal sealed class XmlPlannerMessageWithFileInfo : XmlPlannerMessage {

        #region Fields

        /// <summary>
        /// The <see cref="XmlPlannerMessageFileType">file type</see>.
        /// </summary>
        private readonly XmlPlannerMessageFileType _fileType;

        /// <summary>
        /// The file path.
        /// </summary>
        private readonly string _filePath;

        /// <summary>
        /// The line number.
        /// </summary>
        private readonly int _lineNumber;

        /// <summary>
        /// The line position.
        /// </summary>
        private readonly int _linePosition;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerMessageWithFileInfo"/> class.
        /// </summary>
        /// <param name="type">The <see cref="XmlPlannerMessageType">message type</see>.</param>
        /// <param name="text">The text.</param>
        /// <param name="fileType">The <see cref="XmlPlannerMessageFileType">file type</see>.</param>
        /// <param name="filePath">The file path.</param>
        /// <param name="lineNumber">The line number.</param>
        /// <param name="linePosition">The line position.</param>
        internal XmlPlannerMessageWithFileInfo(XmlPlannerMessageType type, string text, XmlPlannerMessageFileType fileType,
        string filePath, int lineNumber, int linePosition)
            : base(type, text) {
            _fileType = fileType;
            _filePath = filePath;
            _lineNumber = lineNumber;
            _linePosition = linePosition;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this message has file information.
        /// </summary>
        /// <value>true if this message has file information; otherwise, false.</value>
        public override bool HasFileInfo {
            get {
                return true;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessageFileType">file type</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerMessageFileType">file type</see>.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override XmlPlannerMessageFileType FileType {
            get {
                return _fileType;
            }
        }

        /// <summary>
        /// Gets the file path.
        /// </summary>
        /// <value>The file path.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override string FilePath {
            get {
                return _filePath;
            }
        }

        /// <summary>
        /// Gets the line number.
        /// </summary>
        /// <value>The line number.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override int LineNumber {
            get {
                return _lineNumber;
            }
        }

        /// <summary>
        /// Gets the line position.
        /// </summary>
        /// <value>The line position.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override int LinePosition {
            get {
                return _linePosition;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Compares the message with another, considering the file information.
        /// </summary>
        /// <param name="other">A message to compare with this one.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the messages being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This message is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This message is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This message is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="other"/> is a null reference.</exception>
        private int CompareToWithFileInfo(XmlPlannerMessage other) {
            const int BothEqual = 0;
            if (other == null) {
                throw new ArgumentNullException("other");
            }
            //compares the file types
            int comparison = ((int)_fileType - (int)other.FileType);
            //if both file types are equal
            if (comparison == BothEqual) {
                //compares line numbers
                comparison = _lineNumber - other.LineNumber;
                //if both line numbers are equal
                if (comparison == BothEqual) {
                    //compares line positions
                    comparison = _linePosition - other.LinePosition;
                    //if both line positions are equal, executes the base class comparison
                    if (comparison == BothEqual) {
                        comparison = base.CompareTo(other);
                    }
                }
            }
            return comparison;
        }

        /// <summary>
        /// Compares the message with another.
        /// </summary>
        /// <param name="other">A message to compare with this one.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the messages being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This message is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This message is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This message is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="other"/> is a null reference.</exception>
        public override int CompareTo(XmlPlannerMessage other) {
            const int BothEqual = 0;
            const int LessThanOther = -1;
            if (other == null) {
                throw new ArgumentNullException("other");
            }
            int comparison = BothEqual;
            if (other.HasFileInfo) {
                comparison = CompareToWithFileInfo(other);
            } else {
                //messages without file information are considered to be greater
                comparison = LessThanOther;
            }
            return comparison;
        }

        #endregion

    }

}
