/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents an <see cref="XmlPlannerMessage">planner message</see> without file information.
    /// </summary>
    [Serializable]
    internal sealed class XmlPlannerMessageWithoutFileInfo : XmlPlannerMessage {

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerMessageWithoutFileInfo"/> class.
        /// </summary>
        /// <param name="type">The <see cref="XmlPlannerMessageType">message type</see>.</param>
        /// <param name="text">The text.</param>
        internal XmlPlannerMessageWithoutFileInfo(XmlPlannerMessageType type, string text)
            : base(type, text) {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this message has file information.
        /// </summary>
        /// <value>true if this message has file information; otherwise, false.</value>
        public override bool HasFileInfo {
            get {
                return false;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlPlannerMessageFileType">file type</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerMessageFileType">file type</see>.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override XmlPlannerMessageFileType FileType {
            get {
                throw new InvalidOperationException();
            }
        }

        /// <summary>
        /// Gets the file path.
        /// </summary>
        /// <value>The file path.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override string FilePath {
            get {
                throw new InvalidOperationException();
            }
        }

        /// <summary>
        /// Gets the line number.
        /// </summary>
        /// <value>The line number.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override int LineNumber {
            get {
                throw new InvalidOperationException();
            }
        }

        /// <summary>
        /// Gets the line position.
        /// </summary>
        /// <value>The line position.</value>
        /// <exception cref="InvalidOperationException"><see cref="HasFileInfo"/> returns false.</exception>
        public override int LinePosition {
            get {
                throw new InvalidOperationException();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Compares the message with another.
        /// </summary>
        /// <param name="other">A message to compare with this one.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the messages being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This message is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This message is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This message is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="other"/> is a null reference.</exception>
        public override int CompareTo(XmlPlannerMessage other) {
            const int GreaterThanOther = 1;
            const int BothEqual = 0;
            if (other == null) {
                throw new ArgumentNullException("other");
            }
            int comparison = BothEqual;
            if (other.HasFileInfo) {
                //messages without file information are considered to be greater
                comparison = GreaterThanOther;
            } else {
                comparison = base.CompareTo(other);
            }
            return comparison;
        }

        #endregion

    }

}
