/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Xml;
using System.Text;

using Aran.Core;
using Aran.Representation;
using Aran.Translation;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a <see cref="XmlPlanner">planner</see> output writer.
    /// </summary>
    internal sealed class XmlPlannerOutputWriter {

        #region Fields

        /// <summary>
        /// The planner.
        /// </summary>
        private readonly XmlPlanner _planner;

        /// <summary>
        /// The file manager.
        /// </summary>
        private readonly XmlPlannerFileManager _fileManager;

        /// <summary>
        /// The input traslator.
        /// </summary>
        private readonly XmlPlannerTranslator _translator;

        /// <summary>
        /// The plan searcher.
        /// </summary>
        private readonly XmlPlannerSearcher _searcher;

        /// <summary>
        /// The messages.
        /// </summary>
        private readonly XmlPlannerMessageManager _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerOutputWriter"/> class.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="fileManager">The file manager.</param>
        /// <param name="translator">The input translator.</param>
        /// <param name="searcher">The plan searcher.</param>
        /// <param name="messages">The messages.</param>
        internal XmlPlannerOutputWriter(XmlPlanner planner, XmlPlannerFileManager fileManager,
        XmlPlannerTranslator translator, XmlPlannerSearcher searcher, XmlPlannerMessageManager messages) {
            _planner = planner;
            _fileManager = fileManager;
            _translator = translator;
            _searcher = searcher;
            _messages = messages;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets a plan from a result <see cref="State">state</see>.
        /// </summary>
        /// <param name="resultState">The result <see cref="State">state</see>.</param>
        /// <returns>The plan <see cref="Action">action</see> identifiers.</returns>
        private static int[] GetPlan(State resultState) {
            int planLength = resultState.PlanLength;
            int[] plan = new int[planLength];
            State currentState = resultState;
            //for each action
            for (int actionIndex = (planLength - 1); actionIndex >= 0; actionIndex--) {
                //get the action identifier
                plan[actionIndex] = currentState.AppliedAction.Id;
                currentState = currentState.Parent;
            }
            return plan;
        }

        /// <summary>
        /// Gets the XML <see cref="Encoding">encoding</see> of a file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>The XML <see cref="Encoding">encoding</see> of the file</returns>
        private static Encoding GetEncoding(string filePath) {
            Encoding encoding = null;
            Stream file = File.OpenRead(filePath);
            XmlTextReader reader = new XmlTextReader(file);
            try {
                //read the first node, which contains the encoding
                reader.Read();
                encoding = reader.Encoding;
            } catch (XmlException) {
                //do nothing; the file has invalid XML data
            }
            file.Close();
            return encoding;
        }

        /// <summary>
        /// Gets the output document <see cref="Encoding">encoding</see>, depending on the
        /// <see cref="XmlPlanner">planner</see> configuration.
        /// </summary>
        /// <returns>The output document <see cref="Encoding">encoding</see>.</returns>
        private Encoding GetOutputEncoding() {
            Encoding outputEncoding = null;
            //gets the encoding from the domain file, the problem file or the corresponding planner property
            switch (_planner.OutputEncodingSource) {
                case XmlPlannerOutputEncodingSource.Domain:
                    outputEncoding = GetEncoding(_planner.DomainPath);
                    break;
                case XmlPlannerOutputEncodingSource.Problem:
                    outputEncoding = GetEncoding(_planner.ProblemPath);
                    break;
                case XmlPlannerOutputEncodingSource.Configuration:
                    outputEncoding = _planner.OutputEncoding;
                    break;
            }
            //the default encoding is UTF-8
            if (outputEncoding == null) {
                outputEncoding = Encoding.UTF8;
            }
            return outputEncoding;
        }

        /// <summary>
        /// Writes the output document.
        /// </summary>
        /// <returns>If the output document is successfully written.</returns>
        internal bool Write() {
            bool success = !_planner.Canceled;
            if (success) {
                //open the file
                Stream outputFile = _fileManager.OpenWrite(_planner.OutputPath, _planner.OverwriteOutput);
                success = (outputFile != null) && !_planner.Canceled;
                //if no error found and planning not cancelled
                if (success) {
                    //get plan action identifiers from search result
                    int[] plan = GetPlan(_searcher.Result);
                    InstanceDecoder<string, string> actionDecoder = _translator.Result.ActionDecoder;
                    IInstance<string, string> action;
                    XmlPlannerMessageCollection messages = _messages.Collection;
                    _planner.SetNextState(messages);
                    XmlPlanWriter writer = new XmlPlanWriter(outputFile, GetOutputEncoding());
                    //for each action identifier
                    foreach (int actionId in plan) {
                        //get the action from the identifier
                        action = actionDecoder.GetInstance(actionId);
                        //write the action
                        writer.Write(action);
                    }
                    //close the file
                    writer.Close();
                    _planner.SetNextState(messages);
                    success = !_planner.Canceled;
                } else if (outputFile != null) {
                    //close the file
                    outputFile.Close();
                }
            }
            return success;
        }

        #endregion

    }

}
