/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Text;
using System.Collections.Generic;

using Aran.Core;
using Aran.Translation;
using Aran.Functional;
using Aran.Searching.Specialized;
using Aran.Searching;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a search algorithm.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public abstract class XmlPlannerSearchAlgorithm {

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        internal readonly Resources Resources;

        #endregion

        #region Constants

        /// <summary>
        /// The default beam width.
        /// </summary>
        public const int DefaultBeamWidth = BeamSearchAlgorithmValue.DefaultBeamWidth;

        /// <summary>
        /// The default estimated remaining cost weight in the total estimated cost formula.
        /// </summary>
        public const float DefaultHeuristicWeight = WeightedAStarAlgorithmValue.DefaultHeuristicWeight;

        /// <summary>
        /// The minimum beam width.
        /// </summary>
        public const int MinimumBeamWidth = BeamSearchAlgorithmValue.MinimumBeamWidth;

        /// <summary>
        /// The minimum estimated remaining cost weight in the total estimated cost formula.
        /// </summary>
        public const float MinimumHeuristicWeight = WeightedAStarAlgorithmValue.MinimumHeuristicWeight;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerSearchAlgorithm"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        internal XmlPlannerSearchAlgorithm(Resources resources) {
            Resources = resources;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="XmlPlannerSearchAlgorithmId">identifier</see>.
        /// </summary>
        /// <value>The <see cref="XmlPlannerSearchAlgorithmId">identifier</see>.</value>
        public abstract XmlPlannerSearchAlgorithmId Id { get; }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name {
            get {
                return Resources.GetSearchAlgorithmName(Id);
            }
        }

        /// <summary>
        /// Gets the description.
        /// </summary>
        /// <value>The description.</value>
        public string Description {
            get {
                const char Space = ' ';
                StringBuilder buffer = new StringBuilder(Name);
                bool isFirst = true;
                string with = Resources.With;
                string and = Resources.And;
                if (AcceptsBeamWidth) {
                    buffer.Append(Space);
                    buffer.Append(with);
                    buffer.Append(Space);
                    buffer.Append(Resources.GetVariableEqualValue(Resources.BeamWidth, BeamWidth));
                    isFirst = false;
                }
                if (AcceptsHeuristicWeight) {
                    buffer.Append(Space);
                    if (isFirst) {
                        buffer.Append(with);
                    } else {
                        buffer.Append(and);
                    }
                    buffer.Append(Space);
                    buffer.Append(Resources.GetVariableEqualValue(Resources.HeuristicWeight, HeuristicWeight));
                }
                return buffer.ToString();
            }
        }

        /// <summary>
        /// Gets a value indicating whether <see cref="BeamWidth">the beam width</see> can be set.
        /// </summary>
        /// <value>True if <see cref="BeamWidth">the beam width</see> can be set; otherwise, false.</value>
        public virtual bool AcceptsBeamWidth {
            get {
                return false;
            }
        }

        /// <summary>
        /// Gets or sets the beam width.
        /// </summary>
        /// <value>The beam width.</value>
        /// <exception cref="ArgumentOutOfRangeException">The value is less than
        /// <see cref="MinimumBeamWidth">the minimum one.</see>.</exception>
        /// <exception cref="InvalidOperationException"><see cref="AcceptsBeamWidth"/> returns false.</exception>
        public virtual int BeamWidth {
            get {
                throw new InvalidOperationException();
            }
            set {
                throw new InvalidOperationException();
            }
        }

        /// <summary>
        /// Gets a value indicating whether <see cref="HeuristicWeight">the heuristic weight</see> can be set.
        /// </summary>
        /// <value>True if <see cref="HeuristicWeight">the heuristic weight</see> can be set; otherwise, false.</value>
        public virtual bool AcceptsHeuristicWeight {
            get {
                return false;
            }
        }

        /// <summary>
        /// Gets or sets the heuristic weight.
        /// </summary>
        /// <value>The heuristic weight.</value>
        /// <exception cref="ArgumentOutOfRangeException">The value is less than
        /// <see cref="MinimumHeuristicWeight">the minimum one.</see>.</exception>
        /// <exception cref="InvalidOperationException"><see cref="AcceptsHeuristicWeight"/> returns false.</exception>
        public virtual float HeuristicWeight {
            get {
                throw new InvalidOperationException();
            }
            set {
                throw new InvalidOperationException();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Clones this <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <returns>A <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> equal to this one.</returns>
        public virtual XmlPlannerSearchAlgorithm Clone() {
            return (MemberwiseClone() as XmlPlannerSearchAlgorithm);
        }

        /// <summary>
        /// Creates a <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <param name="algorithmId">The <see cref="XmlPlannerSearchAlgorithmId">search algorithm identifier</see>.</param>
        /// <returns>The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> identified by
        /// <paramref name="algorithmId"/>, with default settings.</returns>
        public static XmlPlannerSearchAlgorithm Create(XmlPlannerSearchAlgorithmId algorithmId) {
            Resources resources = new Resources();
            switch (algorithmId) {
                case XmlPlannerSearchAlgorithmId.AStar:
                    return new XmlPlannerAStarAlgorithm(resources);
                case XmlPlannerSearchAlgorithmId.BeamSearch:
                    return new XmlPlannerBeamSearchAlgorithm(resources);
                case XmlPlannerSearchAlgorithmId.Dijkstra:
                    return new XmlPlannerDijkstraAlgorithm(resources);
                case XmlPlannerSearchAlgorithmId.EnforcedHillClimbing:
                    return new XmlPlannerEnforcedHillClimbingAlgorithm(resources);
                case XmlPlannerSearchAlgorithmId.HillClimbing:
                    return new XmlPlannerHillClimbingAlgorithm(resources);
                case XmlPlannerSearchAlgorithmId.WeightedAStar:
                    return new XmlPlannerWeightedAStarAlgorithm(resources);
                default:
                    throw new ArgumentException(resources.UnknownSearchAlgorithm, "algorithmId");
            }
        }

        /// <summary>
        /// Creates the break condition.
        /// </summary>
        /// <remarks>The break condition finishes the planning although the current plan does not satisfy the goals.</remarks>
        /// <param name="planner">The planner.</param>
        /// <returns>The break condition.</returns>
        private static ICondition CreateBreakCondition(XmlPlanner planner) {
            ICondition breakCondition = GoalStateVerifier.NoBreakCondition;
            ICondition planLengthCondition = null;
            ICondition timeCondition = null;
            if (planner.MaxPlanLengthEnabled) {
                planLengthCondition = new PlanLengthCondition(planner.MaxPlanLength);
            }
            if (planner.MaxTimeEnabled) {
                timeCondition = new DateTimeCondition(DateTime.Now.AddMilliseconds(planner.MaxTime));
            }
            //the condition can be a maximum planning time condition, a maximum plan-length condition or both
            if (planLengthCondition != null) {
                if (timeCondition != null) {
                    breakCondition = new MultipleConditionDisjunction(timeCondition, planLengthCondition);
                } else {
                    breakCondition = planLengthCondition;
                }
            } else if (timeCondition != null) {
                breakCondition = timeCondition;
            }
            return breakCondition;
        }

        /// <summary>
        /// Creates the successors generator.
        /// </summary>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The successors generato</returns>
        protected internal static IStateSuccessorsProvider<State> CreateSuccessorsGenerator(
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            ActionSelector actionSelector = new ConjunctionActionSelector(
                translationResult.OperatorCount, translationResult.PreconditionCounts,
                translationResult.ActionCount, translationResult.ActionCounts,
                translationResult.ActionActivationTable);
            IStateSuccessorsProvider<State> successorsGenerator = new StateSuccessorsProvider(
                 translationResult.Actions, actionSelector);
            return successorsGenerator;
        }

        /// <summary>
        /// Creates the heuristic calculator.
        /// </summary>
        /// <param name="translationResult">The translation result.</param>
        /// <param name="costCalculator">The cost calculator.</param>
        /// <returns>The heuristic calculator.</returns>
        protected internal static IStateHeuristicCostProvider<State> CreateHeuristicCalculator(
        TranslationResult<string, string, string, string, string, string, string> translationResult,
        IStateCostProvider<State> costCalculator) {
            IStateHeuristicCostProvider<State> heuristicCalculator = null;
            if (translationResult.GoalCount > 0) {
                RelaxedPlanner relaxedPlanner = new RelaxedPlanner(translationResult.FactCount,
                    translationResult.Actions, translationResult.OperatorCount,
                    translationResult.ActionCounts, translationResult.ActionActivationTable,
                    translationResult.GoalCount, translationResult.GoalFacts);
                heuristicCalculator = new RelaxedPlanHeuristic(relaxedPlanner, costCalculator);
            } else {
                heuristicCalculator = new NullHeuristic<State>();
            }
            return heuristicCalculator;
        }

        /// <summary>
        /// Creates the initial state provider.
        /// </summary>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The initial state provider</returns>
        protected internal static IInitialStateProvider<State> CreateInitialStateProvider(
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            return new ConstantInitialStateProvider<State>(translationResult.InitialState);
        }

        /// <summary>
        /// Creates the goal state verifier.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The goal state verifier.</returns>
        protected internal static IGoalStateVerifier<State> CreateGoalStateVerifier(XmlPlanner planner,
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            return new GoalStateVerifier(CreateBreakCondition(planner), translationResult.GoalFacts,
                translationResult.FunctionalGoal);
        }

        /// <summary>
        /// Creates the state cost provider.
        /// </summary>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The state cost provider.</returns>
        protected internal static IStateCostProvider<State> CreateStateCostProvider(
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            return new FunctionalStateCostProvider(translationResult.Metric);
        }

        /// <summary>
        /// Creates the state equality comparer.
        /// </summary>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The state equality comparer.</returns>
        protected internal static EqualityComparer<State> CreateStateEqualityComparer(
        TranslationResult<string, string, string, string, string, string, string> translationResult) {
            int conditionalVariableCount = translationResult.ConditionalVariableCount;
            if (conditionalVariableCount > 0) {
                if (conditionalVariableCount == translationResult.VariableCount) {
                    return EqualityComparer<State>.Default;
                } else {
                    return new SomeVariablesStateEqualityComparer(translationResult.ConditionalVariables);
                }
            } else {
                return new FactsOnlyStateEqualityComparer();
            }
        }

        /// <summary>
        /// Creates the search algorithm.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="translationResult">The translation result.</param>
        /// <returns>The search algorithm.</returns>
        internal abstract ISearchAlgorithm<State> CreateSearchAlgorithm(XmlPlanner planner,
        TranslationResult<string, string, string, string, string, string, string> translationResult);

        #endregion

    }

}
