/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a sequence of one or more <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class XmlPlannerSearchAlgorithmSequence : IEnumerable<XmlPlannerSearchAlgorithm> {

        #region Fields

        /// <summary>
        /// The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s.
        /// </summary>
        private readonly IList<XmlPlannerSearchAlgorithm> _algorithms;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerSearchAlgorithmSequence"/> class.
        /// </summary>
        /// <param name="algorithm">The first <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.</param>
        internal XmlPlannerSearchAlgorithmSequence(XmlPlannerSearchAlgorithm algorithm) {
            _algorithms = new List<XmlPlannerSearchAlgorithm>();
            _algorithms.Add(algorithm);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> at the specified index.
        /// </summary>
        /// <param name="index">The index of the <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.</param>
        /// <value>The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> at <paramref name="index"/>.</value>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="index"/> is less than zero or
        /// <paramref name="index"/> is equal to or greater than <see cref="Count"/>.</exception>
        public XmlPlannerSearchAlgorithm this[int index] {
            get {
                return _algorithms[index];
            }
        }

        /// <summary>
        /// The number of <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s.
        /// </summary>
        public int Count {
            get {
                return _algorithms.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Replaces all <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s by a specified one.
        /// </summary>
        /// <param name="algorithmId">The identifier of the <see cref="XmlPlannerSearchAlgorithm">search
        /// algorithm</see> to set in place of all existing ones.</param>
        /// <returns>The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> set in place of all
        /// existing ones.</returns>
        /// <exception cref="ArgumentException"><paramref name="algorithmId"/> is unknown.</exception>
        public XmlPlannerSearchAlgorithm ReplaceAllBy(XmlPlannerSearchAlgorithmId algorithmId) {
            try {
                XmlPlannerSearchAlgorithm algorithm = XmlPlannerSearchAlgorithm.Create(algorithmId);
                _algorithms.Clear();
                _algorithms.Add(algorithm);
                return algorithm;
            } catch (ArgumentException exception) {
                throw new ArgumentException(exception.Message, "algorithmId");
            }
        }

        /// <summary>
        /// Replaces all <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>s by a specified one.
        /// </summary>
        /// <param name="algorithm">The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> to set in
        /// place of all existing ones.</param>
        /// <exception cref="ArgumentNullException"><paramref name="algorithm"/> is a null reference.</exception>
        public void ReplaceAllBy(XmlPlannerSearchAlgorithm algorithm) {
            if (algorithm == null) {
                throw new ArgumentNullException("algorithm");
            }
            _algorithms.Clear();
            _algorithms.Add(algorithm);
        }

        /// <summary>
        /// Adds a <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <param name="algorithmId">The identifier of the <see cref="XmlPlannerSearchAlgorithm">search
        /// algorithm</see>.</param>
        /// <returns>The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see> added.</returns>
        /// <exception cref="ArgumentException"><paramref name="algorithmId"/> is unknown.</exception>
        public XmlPlannerSearchAlgorithm Add(XmlPlannerSearchAlgorithmId algorithmId) {
            try {
                XmlPlannerSearchAlgorithm algorithm = XmlPlannerSearchAlgorithm.Create(algorithmId);
                _algorithms.Add(algorithm);
                return algorithm;
            } catch (ArgumentException exception) {
                throw new ArgumentException(exception.Message, "algorithmId");
            }
        }

        /// <summary>
        /// Adds a <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <param name="algorithm">The <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="algorithm"/> is a null reference.</exception>
        public void Add(XmlPlannerSearchAlgorithm algorithm) {
            if (algorithm == null) {
                throw new ArgumentNullException("algorithm");
            }
            _algorithms.Add(algorithm);
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the sequence.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the sequence.
        /// </returns>
        public IEnumerator<XmlPlannerSearchAlgorithm> GetEnumerator() {
            return _algorithms.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the sequence.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the sequence.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (_algorithms as IEnumerable).GetEnumerator();
        }

        #endregion

    }

}
