/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Translation;
using Aran.Functional;
using Aran.Searching.Specialized;
using Aran.Searching;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a <see cref="XmlPlanner">planner</see> search executer.
    /// </summary>
    internal sealed class XmlPlannerSearcher {

        #region Fields

        /// <summary>
        /// The planner.
        /// </summary>
        private readonly XmlPlanner _planner;

        /// <summary>
        /// The input translator.
        /// </summary>
        private readonly XmlPlannerTranslator _translator;

        /// <summary>
        /// The messages.
        /// </summary>
        private readonly XmlPlannerMessageManager _messages;

        /// <summary>
        /// The search result.
        /// </summary>
        private State _result;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerSearcher"/> class.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="translator">The input translator.</param>
        /// <param name="messages">The messages.</param>
        internal XmlPlannerSearcher(XmlPlanner planner, XmlPlannerTranslator translator, XmlPlannerMessageManager messages) {
            _planner = planner;
            _translator = translator;
            _messages = messages;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the search result.
        /// </summary>
        /// <value>The search result.</value>
        public State Result {
            get {
                return _result;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Selects the <see cref="XmlPlannerSearchAlgorithm">search algorithm</see>.
        /// </summary>
        /// <param name="algorithm">The algorithm.</param>
        /// <returns>The internal <see cref="ISearchAlgorithm{T}">search algorithm</see> created by
        /// <paramref name="algorithm"/>.</returns>
        private ISearchAlgorithm<State> SelectSearchAlgorithm(XmlPlannerSearchAlgorithm algorithm) {
            _planner.NotifySelectedSearchAlgorithm(algorithm);
            return algorithm.CreateSearchAlgorithm(_planner, _translator.Result);
        }

        /// <summary>
        /// Searches for a plan.
        /// </summary>
        /// <returns>True if a plan is found; false otherwise.</returns>
        internal bool Search() {
            bool success = !_planner.Canceled;
            if (success) {
                XmlPlannerMessageCollection messages = _messages.Collection;
                XmlPlannerSearchAlgorithmSequence searchAlgorithms = _planner.SearchAlgorithms;
                ISearchAlgorithm<State> searchAlgorithm;
                int searchAlgorithmIndex = 0;
                int searchAlgorithmCount = searchAlgorithms.Count;
                success = false;
                _planner.SetNextState(messages);
                //until plan found, no more algorithms found or planning canceled
                while (!_planner.Canceled && !success && (searchAlgorithmIndex < searchAlgorithmCount)) {
                    try {
                        //select the next algorithm
                        searchAlgorithm = SelectSearchAlgorithm(searchAlgorithms[searchAlgorithmIndex]);
                        //search for a plan
                        _result = searchAlgorithm.Search();
                        success = true;
                    } catch (ResultNotFoundException) {
                        //do nothing
                    }
                    searchAlgorithmIndex++;
                }
                _planner.SetNextState(messages);
            }
            return success;
        }

        #endregion

    }

}
