/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents the arguments of an event which occurs when the value of
    /// <see cref="XmlPlanner.State">the planner state</see> changes.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class XmlPlannerStateChangedEventArgs : EventArgs {

        #region Fields

        /// <summary>
        /// The previous <see cref="XmlPlanner.State">planner state</see>.
        /// </summary>
        private readonly XmlPlannerState _previousState;

        /// <summary>
        /// The current <see cref="XmlPlanner.State">planner state</see>.
        /// </summary>
        private readonly XmlPlannerState _currentState;

        /// <summary>
        /// The time taken by <see cref="_previousState">the previous planner state</see>.
        /// </summary>
        private readonly TimeSpan _timeTaken;

        /// <summary>
        /// The <see cref="XmlPlanner">planner</see> messages.
        /// </summary>
        private readonly XmlPlannerMessageCollection _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerStateChangedEventArgs"/> class.
        /// </summary>
        /// <param name="previousState">The previous <see cref="XmlPlanner.State">planner state</see>.</param>
        /// <param name="currentState">The current <see cref="XmlPlanner.State">planner state</see>.</param>
        /// <param name="timeTaken">The time taken by the previous <see cref="XmlPlanner.State">planner state</see>.</param>
        /// <param name="messages">The <see cref="XmlPlanner">planner</see> messages.</param>
        internal XmlPlannerStateChangedEventArgs(XmlPlannerState previousState, XmlPlannerState currentState,
        TimeSpan timeTaken, XmlPlannerMessageCollection messages) {
            _previousState = previousState;
            _currentState = currentState;
            _timeTaken = timeTaken;
            _messages = messages;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the previous <see cref="XmlPlanner.State">planner state</see>.
        /// </summary>
        /// <value>The previous <see cref="XmlPlanner.State">planner state</see>.</value>
        public XmlPlannerState PreviousState {
            get {
                return _previousState;
            }
        }

        /// <summary>
        /// Gets the current <see cref="XmlPlanner.State">planner state</see>.
        /// </summary>
        /// <value>The current <see cref="XmlPlanner.State">planner state</see>.</value>
        public XmlPlannerState CurrentState {
            get {
                return _currentState;
            }
        }

        /// <summary>
        /// Gets the time taken by <see cref="PreviousState">the previous planner state</see>.
        /// </summary>
        /// <value>The time taken by <see cref="PreviousState">the previous planner state</see>.</value>
        public TimeSpan TimeTaken {
            get {
                return _timeTaken;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlPlanner">planner</see> messages.
        /// </summary>
        /// <value>The <see cref="XmlPlanner">planner</see> messages.</value>
        public XmlPlannerMessageCollection Messages {
            get {
                return _messages;
            }
        }

        #endregion

    }

}
