/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;

using Aran.Representation;
using Aran.Representation.Xml;
using Aran.Translation;

namespace Aran.Planning.Xml {

    /// <summary>
    /// Represents a <see cref="XmlPlanner">planner</see> input translator.
    /// </summary>
    /// <remarks>
    /// The translator creates an internal representation of both domain and problem defined in input documents.
    /// The internal representation is used by the search algorithms to search for a plan.
    /// </remarks>
    internal sealed class XmlPlannerTranslator {

        #region Fields

        /// <summary>
        /// The planner.
        /// </summary>
        private readonly XmlPlanner _planner;

        /// <summary>
        /// The file manager.
        /// </summary>
        private readonly XmlPlannerFileManager _fileManager;

        /// <summary>
        /// The messages.
        /// </summary>
        private readonly XmlPlannerMessageManager _messages;

        /// <summary>
        /// The domain input stream.
        /// </summary>
        private Stream _domainStream;

        /// <summary>
        /// The problem input stream.
        /// </summary>
        private Stream _problemStream;

        /// <summary>
        /// The translation result.
        /// </summary>
        private TranslationResult<string, string, string, string, string, string, string> _result;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlPlannerTranslator"/> class.
        /// </summary>
        /// <param name="planner">The planner.</param>
        /// <param name="fileManager">The file manager.</param>
        /// <param name="messages">The messages.</param>
        internal XmlPlannerTranslator(XmlPlanner planner, XmlPlannerFileManager fileManager,
        XmlPlannerMessageManager messages) {
            _planner = planner;
            _fileManager = fileManager;
            _messages = messages;
        }

        #endregion

        #region Properties

        public TranslationResult<string, string, string, string, string, string, string> Result {
            get {
                return _result;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Opens the domain and problem files.
        /// </summary>
        /// <returns>True if both files are successfully opened; false otherwise.</returns>
        private bool OpenFiles() {
            bool success = !_planner.Canceled;
            if (success) {
                string domainPath = _planner.DomainPath;
                string problemPath = _planner.ProblemPath;
                _domainStream = _fileManager.OpenRead(domainPath);
                //if both files are the same, only open one of them
                if (_planner.DomainAndProblemPathsAreEqual) {
                    _problemStream = _domainStream;
                } else {
                    _problemStream = _fileManager.OpenRead(problemPath);
                }
                success = (_domainStream != null) && (_problemStream != null) && !_planner.Canceled;
                if (success) {
                    XmlParser parser = _planner.Parser;
                    parser.DomainStream = _domainStream;
                    parser.ProblemStream = _problemStream;
                }
            }
            return success;
        }

        /// <summary>
        /// Creates an internal representation of both domain and problem defined in input documents.
        /// </summary>
        /// <returns>True if translation is successfully done; false otherwise.</returns>
        internal bool Translate() {
            bool success = OpenFiles() && !_planner.Canceled;
            if (success) {
                success = false;
                XmlPlannerMessageCollection messages = _messages.Collection;
                _planner.SetNextState(messages);
                try {
                    _result = _planner.Translator.Translate();
                    _messages.AddAll(_result.Messages);
                    _planner.SetNextState(messages);
                    success = !_planner.Canceled;
                } catch (GenerationException<string, string, string, string, string, string, string> exception) {
                    _messages.AddAll(exception.Messages);
                    _planner.SetNextState(messages);
                    //the exceptions with inner exception are unexpected ones
                    if (exception.InnerException != null) {
                        throw;
                    }
                } catch (XmlDomainNotFoundException) {
                    _planner.SetNextState(messages);
                    _messages.AddError(_planner.Resources.GetNoDomainFound(_planner.DomainPath));
                } catch (XmlProblemNotFoundException) {
                    _planner.SetNextState(messages);
                    _messages.AddError(_planner.Resources.GetNoProblemFound(_planner.ProblemPath));
                } catch (XmlDomainMismatchException) {
                    _planner.SetNextState(messages);
                    _messages.AddError(_planner.Resources.DomainMismatch);
                } catch (XmlErrorsFoundException exception) {
                    _messages.AddAll(exception.Errors);
                    _planner.SetNextState(messages);
                } catch (RepresentationException) {
                    _planner.SetNextState(messages);
                    throw;
                }
            }
            return success;
        }

        #endregion

    }

}
