/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace Aran.Representation.Xml.Internal {

    /// <summary>
    /// <para>This class supports the parser infrastructure and is not intended to be used directly from your code.</para>
    /// <para>Represents a problem defined within an XML input document.</para>
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    [XmlType(TypeName = XmlTypeName.Problem)]
    public class InternalProblem {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The name.
        /// </summary>
        [XmlAttribute(AttributeName = XmlAttributeName.Name, Form = XmlSchemaForm.Unqualified, DataType = XmlDataType.String)]
        public string Name;

        /// <summary>
        /// The domain name.
        /// </summary>
        [XmlAttribute(AttributeName = XmlAttributeName.Domain, Form = XmlSchemaForm.Unqualified, DataType = XmlDataType.String)]
        public string Domain;

        /// <summary>
        /// The objects.
        /// </summary>
        [XmlElement(Type = typeof(InternalObjectDeclarationList), ElementName = XmlElementName.Objects, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalObjectDeclarationList Objects;

        /// <summary>
        /// The facts.
        /// </summary>
        [XmlElement(Type = typeof(InternalFactDeclarationList), ElementName = XmlElementName.Facts, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalFactDeclarationList Facts;

        /// <summary>
        /// The variables.
        /// </summary>
        [XmlElement(Type = typeof(InternalVariableDeclarationList), ElementName = XmlElementName.Variables, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalVariableDeclarationList Variables;

        /// <summary>
        /// The initial facts.
        /// </summary>
        [XmlElement(Type = typeof(InternalFactList), ElementName = XmlElementName.InitialFacts, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalFactList InitialFacts;

        /// <summary>
        /// The functional initializations.
        /// </summary>
        [XmlElement(Type = typeof(InternalOperationList), ElementName = XmlElementName.FunctionalInitializations, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalOperationList FunctionalInitializations;

        /// <summary>
        /// The goal facts.
        /// </summary>
        [XmlElement(Type = typeof(InternalFactList), ElementName = XmlElementName.GoalFacts, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalFactList GoalFacts;

        /// <summary>
        /// The functional goals.
        /// </summary>
        [XmlElement(Type = typeof(InternalOperationList), ElementName = XmlElementName.FunctionalGoals, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalOperationList FunctionalGoals;

        /// <summary>
        /// The metric.
        /// </summary>
        [XmlElement(Type = typeof(InternalMetric), ElementName = XmlElementName.Metric, IsNullable = false, Form = XmlSchemaForm.Unqualified)]
        public InternalMetric Metric;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InternalProblem"/> class.
        /// </summary>
        public InternalProblem() {
            Objects = new InternalObjectDeclarationList();
            Facts = new InternalFactDeclarationList();
            Variables = new InternalVariableDeclarationList();
            InitialFacts = new InternalFactList();
            FunctionalInitializations = new InternalOperationList();
            GoalFacts = new InternalFactList();
            FunctionalGoals = new InternalOperationList();
        }

        #endregion

    }
	
}
