/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a domain defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlDomain : XmlLocatable,
        IDomain<string, string, string, string, string, string, string> {

        #region Fields

        /// <summary>
        /// The internal domain representation.
        /// </summary>
        private readonly InternalDomain _domain;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlDomain"/> class.
        /// </summary>
        /// <param name="domain">The internal domain representation.</param>
        internal XmlDomain(InternalDomain domain)
            : base(XPathManager.Append(XPathManager.Create(XmlElementName.Definition),
            XmlElementName.Domain, XmlAttributeName.Name, domain.Name)) {
            _domain = domain;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        public string Id {
            get {
                return _domain.Name;
            }
        }

        /// <summary>
        /// Gets the number of types.
        /// </summary>
        /// <value>The number of types.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int TypeCount {
            get {
                return _domain.Types.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of predicates.
        /// </summary>
        /// <value>The number of predicates.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int PredicateCount {
            get {
                return _domain.Predicates.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of functions.
        /// </summary>
        /// <value>The number of functions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FunctionCount {
            get {
                return _domain.Functions.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of operators.
        /// </summary>
        /// <value>The number of operators.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int OperatorCount {
            get {
                return _domain.Operators.Elements.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the types.
        /// </summary>
        /// <returns>The types.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IType<string>> GetTypes() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Types);
            int position = 1;
            foreach (InternalTypeDeclaration type in _domain.Types.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Type, position);
                yield return new XmlType(xpath, type);
                position++;
            }
        }

        /// <summary>
        /// Gets the predicate signatures.
        /// </summary>
        /// <returns>The predicate signatures.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<ISignature<string, string, string>> GetPredicates() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Predicates);
            int position = 1;
            foreach (InternalSignature predicate in _domain.Predicates.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Predicate, position);
                yield return new XmlSignature(xpath, predicate);
                position++;
            }
        }

        /// <summary>
        /// Gets the function signatures.
        /// </summary>
        /// <returns>The function signatures.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<ISignature<string, string, string>> GetFunctions() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Functions);
            int position = 1;
            foreach (InternalSignature function in _domain.Functions.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Function, position);
                yield return new XmlSignature(xpath, function);
                position++;
            }
        }

        /// <summary>
        /// Gets the operators.
        /// </summary>
        /// <returns>The operators.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperator<string, string, string, string, string, string, string>> GetOperators() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Operators);
            int position = 1;
            foreach (InternalOperator op in _domain.Operators.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Operator, position);
                yield return new XmlOperator(xpath, op);
                position++;
            }
        }

        #endregion

    }

}
