/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents information about an error occurred while parsing an XML document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlErrorInfo {

        #region Fields

        /// <summary>
        /// The stream which were being read when the error occurred.
        /// </summary>
        private readonly Stream _stream;

        /// <summary>
        /// The line number in which the error occurred.
        /// </summary>
        private readonly int _lineNumber;

        /// <summary>
        /// The line position in which the error occurred.
        /// </summary>
        private readonly int _linePosition;

        /// <summary>
        /// The error message.
        /// </summary>
        private readonly string _message;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlErrorInfo"/> class.
        /// </summary>
        /// <param name="stream">The stream which were being read when the error occurred.</param>
        /// <param name="lineNumber">The line number in which the error occurred.</param>
        /// <param name="linePosition">The line position in which the error occurred.</param>
        /// <param name="message">The error message.</param>
        internal XmlErrorInfo(Stream stream, int lineNumber, int linePosition, string message) {
            _stream = stream;
            _lineNumber = lineNumber;
            _linePosition = linePosition;
            _message = message;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the stream which were being read when the error occurred.
        /// </summary>
        /// <value>The stream which were being read when the error occurred.</value>
        public Stream Stream {
            get {
                return _stream;
            }
        }

        /// <summary>
        /// Gets the line number in which the error occurred.
        /// </summary>
        /// <value>The line number in which the error occurred.</value>
        public int LineNumber {
            get {
                return _lineNumber;
            }
        }

        /// <summary>
        /// Gets the line position in which the error occurred.
        /// </summary>
        /// <value>The line position in which the error occurred.</value>
        public int LinePosition {
            get {
                return _linePosition;
            }
        }

        /// <summary>
        /// Gets the error message.
        /// </summary>
        /// <value>The error message.</value>
        public string Message {
            get {
                return _message;
            }
        }

        #endregion

    }

}
