/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a collection of informations about errors occurred while parsing an XML document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlErrorInfoCollection : IEnumerable<XmlErrorInfo> {

        #region Fields

        /// <summary>
        /// The underying information collection.
        /// </summary>
        private readonly IList<XmlErrorInfo> _errors;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlErrorInfoCollection"/> class.
        /// </summary>
        internal XmlErrorInfoCollection() {
            _errors = new List<XmlErrorInfo>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the number of errors.
        /// </summary>
        /// <value>The number of errors.</value>
        public int Count {
            get {
                return _errors.Count;
            }
        }

        /// <summary>
        /// Gets the <see cref="XmlErrorInfo">error</see> at the specified index.
        /// </summary>
        /// <param name="index">The index of the <see cref="XmlErrorInfo">error</see>.</param>
        /// <value>The <see cref="XmlErrorInfo">error</see> at <paramref name="index"/>.</value>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="index"/> is less than zero or
        /// <paramref name="index"/> is equal to or greater than <see cref="Count"/>.</exception>
        public XmlErrorInfo this[int index] {
            get {
                return _errors[index];
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds an information of an error.
        /// </summary>
        /// <param name="stream">The stream which were being read when the error occurred.</param>
        /// <param name="lineNumber">The line number in which the error occurred.</param>
        /// <param name="linePosition">The line position in which the error occurred.</param>
        /// <param name="message">The error message.</param>
        internal void Add(Stream stream, int lineNumber, int linePosition, string message) {
            _errors.Add(new XmlErrorInfo(stream, lineNumber, linePosition, message));
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<XmlErrorInfo> GetEnumerator() {
            return _errors.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (_errors as IEnumerable).GetEnumerator();
        }

        #endregion

    }

}
