/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Security.Permissions;
using System.Runtime.Serialization;

namespace Aran.Representation.Xml {

	/// <summary>
	/// Represents an error occurred when parsing errors are found.
	/// </summary>
	[CLSCompliant(true)]
	[Serializable]
    public sealed class XmlErrorsFoundException : RepresentationException {

        #region Constants

        /// <summary>
        /// The errors name into the streaming context.
        /// </summary>
        private const string ErrorsName = "errors";

        #endregion

        #region Fields

        /// <summary>
        /// The errors reported by the <see cref="XmlParser">parser</see>.
        /// </summary>
        private readonly XmlErrorInfoCollection _errors;

        #endregion

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="XmlErrorsFoundException">XmlErrorsFoundException</see> class.
		/// </summary>
        /// <param name="message">The message that describes the error.</param>
        /// <param name="errors">The errors reported by the <see cref="XmlParser">parser</see>.</param>
        /// <remarks>
		/// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance using
        /// the <paramref name="message"/> parameter.</para>
		/// <para>The following table shows the initial property values for an instance of
		/// <see cref="XmlErrorsFoundException">XmlErrorsFoundException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
        ///       <description>A null reference.</description>
		///    </item>
        ///    <item>
        ///       <term>Message</term>
        ///       <description>The error message string.</description>
        ///    </item>
        ///    <item>
        ///       <term>Errors</term>
        ///       <description>The errors reported by the <see cref="XmlParser">parser</see>.</description>
        ///    </item>
		/// </list>
		/// </remarks>
        internal XmlErrorsFoundException(string message, XmlErrorInfoCollection errors)
            : base(message) {
            _errors = errors;
		}

#if !XBOX360

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlErrorsFoundException">XmlErrorsFoundException</see> class
        /// with serialized data.
        /// </summary>
        /// <param name="info">The serialized object data about the exception being thrown.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        /// <remarks>
        /// <para>This constructor is called during deserialization to reconstitute the exception object
        /// transmitted over a stream.</para>
        /// <para>This constructor is not available for the Xbox 360.</para>
        /// </remarks>
        /// <exception cref="ArgumentNullException">The <paramref name="info"/> parameter is a null reference.</exception>
        /// <exception cref="SerializationException">The class name is a null reference or
        /// <see cref="Exception.HResult">HResult</see> is zero (0).</exception>
        private XmlErrorsFoundException(SerializationInfo info, StreamingContext context)
            : base(info, context) {
            object errors = info.GetValue(ErrorsName, typeof(XmlErrorInfoCollection));
            _errors = (XmlErrorInfoCollection)errors;
        }

#endif

		#endregion

        #region Properties

        /// <summary>
        /// Gets the errors reported by the <see cref="XmlParser">parser</see>.
        /// </summary>
        /// <value>The errors reported by the <see cref="XmlParser">parser</see>.</value>
        public XmlErrorInfoCollection Errors {
            get {
                return _errors;
            }
        }

        #endregion

        #region Methods

#if !XBOX360

        /// <summary>
        /// When overridden in a derived class, sets the <see cref="SerializationInfo"/> with information about
        /// the exception.
        /// </summary>
        /// <param name="info">The <see cref="SerializationInfo"/> that holds the serialized object data about
        /// the exception being thrown.</param>
        /// <param name="context">The <see cref="StreamingContext"/> that contains contextual information about
        /// the source or destination.</param>
        /// <remarks>This method is not available for the Xbox 360.</remarks>
        /// <exception cref="ArgumentNullException">The info parameter is a null reference.</exception>
        /// <PermissionSet><IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Read="*AllFiles*" PathDiscovery="*AllFiles*"/>
        /// <IPermission class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Flags="SerializationFormatter"/>
        /// </PermissionSet>
        [SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter=true)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context) {
            base.GetObjectData(info, context);
            info.AddValue(ErrorsName, _errors);
        }

#endif

        #endregion

    }

}