/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a fact defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlFactDeclaration : XmlLocatable, IIdentifiedInstance<string, string, string> {

        #region Fields

        /// <summary>
        /// The internal fact representation.
        /// </summary>
        private readonly InternalFactDeclaration _fact;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlFactDeclaration"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="fact">The internal fact representation.</param>
        internal XmlFactDeclaration(string xpath, InternalFactDeclaration fact)
            : base(xpath) {
            _fact = fact;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string Id {
            get {
                return _fact.Name;
            }
        }

        /// <summary>
        /// Gets the signature identifier.
        /// </summary>
        /// <value>The signature identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string SignatureId {
            get {
                return _fact.Predicate;
            }
        }

        /// <summary>
        /// Gets the number of arguments.
        /// </summary>
        /// <value>The number of arguments.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ArgumentCount {
            get {
                return _fact.Arguments.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the argument identifiers.
        /// </summary>
        /// <returns>The argument identifiers.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetArgumentIds() {
            foreach (InternalNamed argument in _fact.Arguments) {
                yield return argument.Name;
            }
        }

        #endregion

    }

}
