/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents an operand defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlOperand : XmlLocatable, IOperand<string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal operand representacion.
        /// </summary>
        private readonly InternalOperand _operand;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlOperand"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="operand">The internal operand representacion.</param>
        internal XmlOperand(string xpath, InternalOperand operand)
            : base(xpath) {
            _operand = operand;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <value>The type.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public OperandType Type {
            get {
                return _operand.Type;
            }
        }

        /// <summary>
        /// Gets the variable identifier.
        /// </summary>
        /// <value>The variable identifier.</value>
        /// <exception cref="InvalidOperationException">the <see cref="Type">type</see> is not
        /// <see cref="OperandType.Variable">variable</see>.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string VariableId {
            get {
                if (_operand.Variable == null) {
                    throw new InvalidOperationException();
                }
                return _operand.Variable.Name;
            }
        }

        /// <summary>
        /// Gets the constant value.
        /// </summary>
        /// <value>The constant value.</value>
        /// <exception cref="InvalidOperationException">the <see cref="Type">type</see> is not
        /// <see cref="OperandType.Constant">variable</see>.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public float ConstantValue {
            get {
                if (_operand.Constant == null) {
                    throw new InvalidOperationException();
                }
                return _operand.Constant.Value;
            }
        }

        /// <summary>
        /// Gets the operation.
        /// </summary>
        /// <value>The operation.</value>
        /// <exception cref="InvalidOperationException">the <see cref="Type">type</see> is not
        /// <see cref="OperandType.Operation">variable</see>.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IOperation<string> Operation {
            get {
                if (_operand.Operation == null) {
                    throw new InvalidOperationException();
                }
                return new XmlOperation(XPath, _operand.Operation);
            }
        }

        #endregion

    }

}
