/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents an operation defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlOperation : XmlLocatable, IOperation<string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal operation representacion.
        /// </summary>
        private readonly InternalOperation _operation;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlOperation"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="operation">The internal operation representacion.</param>
        internal XmlOperation(string xpath, InternalOperation operation)
            : base(xpath) {
            _operation = operation;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <value>The type.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public OperationType Type {
            get {
                return XmlOperationTypeConverter.FromXml(_operation.Symbol);
            }
        }

        /// <summary>
        /// Gets the number of operands.
        /// </summary>
        /// <value>The number of operands.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int OperandCount {
            get {
                return _operation.Operands.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the operands.
        /// </summary>
        /// <returns>The operands.</returns>
        /// <remarks>
        /// <para>The operands are returned from left to right. For example, the returned operands for
        /// <c>a = b + c</c> are, in this order, <c>a</c>, <c>b</c>, and <c>c</c>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperand<string>> GetOperands() {
            string xpath = null;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Operand);
            int constantPosition = 1;
            int operationPosition = 1;
            int variablePosition = 1;
            foreach (InternalOperand operand in _operation.Operands) {
                switch (operand.Type) {
                    case OperandType.Constant:
                        xpath = XPathManager.Append(baseXPath, XmlElementName.Constant, constantPosition);
                        constantPosition++;
                        break;
                    case OperandType.Operation:
                        xpath = XPathManager.Append(baseXPath, XmlElementName.Operation, operationPosition);
                        operationPosition++;
                        break;
                    case OperandType.Variable:
                        xpath = XPathManager.Append(baseXPath, XmlElementName.Variable, variablePosition);
                        variablePosition++;
                        break;
                }
                yield return new XmlOperand(xpath, operand);
            }
        }

        #endregion

    }

}
