/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents an operator defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlOperator : XmlLocatable,
        IOperator<string, string, string, string, string, string, string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal operator representacion.
        /// </summary>
        private readonly InternalOperator _operator;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlOperator"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="op">The internal operator representacion.</param>
        internal XmlOperator(string xpath, InternalOperator op)
            : base(xpath) {
            _operator = op;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string Id {
            get {
                return _operator.Name;
            }
        }

        /// <summary>
        /// Gets the number of parameters.
        /// </summary>
        /// <value>The number of parameters.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ParameterCount {
            get {
                return _operator.Parameters.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of facts.
        /// </summary>
        /// <value>The number of facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FactCount {
            get {
                return _operator.Facts.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of variables.
        /// </summary>
        /// <value>The number of variables.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int VariableCount {
            get {
                return _operator.Variables.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of precondition facts.
        /// </summary>
        /// <value>The number of precondition facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int PreconditionCount {
            get {
                return _operator.Preconditions.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of functional preconditions.
        /// </summary>
        /// <value>The number of functional preconditions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FunctionalPreconditionCount {
            get {
                return _operator.FunctionalPreconditions.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of fact deletions.
        /// </summary>
        /// <value>The number of fact deletions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int DeletionCount {
            get {
                return _operator.Deletions.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of fact additions.
        /// </summary>
        /// <value>The number of fact additions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int AdditionCount {
            get {
                return _operator.Additions.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of functional effects.
        /// </summary>
        /// <value>The number of functional effects.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FunctionalEffectCount {
            get {
                return _operator.FunctionalEffects.Elements.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <returns>The parameters.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IObject<string, string>> GetParameters() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Parameters);
            int position = 1;
            foreach (InternalObjectDeclaration parameter in _operator.Parameters.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Object, position);
                yield return new XmlObject(xpath, parameter);
                position++;
            }
        }

        /// <summary>
        /// Gets the facts.
        /// </summary>
        /// <returns>The facts.</returns>
        /// <remarks>
        /// <para>Each fact is represented as a predicate
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified instance</see> and must refer
        /// to a predicate returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetPredicates">the domain
        /// corresponding method</see>.</para>
        /// <para>All fact argument identifiers must refer to parameters returned by
        /// <see cref="GetParameters">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IIdentifiedInstance<string, string, string>> GetFacts() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Facts);
            int position = 1;
            foreach (InternalFactDeclaration fact in _operator.Facts.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Fact, position);
                yield return new XmlFactDeclaration(xpath, fact);
                position++;
            }
        }

        /// <summary>
        /// Gets the variables.
        /// </summary>
        /// <returns>The variables.</returns>
        /// <remarks>
        /// <para>Each variable is represented as a function
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified instance</see> and must refer
        /// to a function returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetFunctions">the domain
        /// corresponding method</see>.</para>
        /// <para>All variable argument identifiers must refer to parameters returned by
        /// <see cref="GetParameters">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IIdentifiedInstance<string, string, string>> GetVariables() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Variables);
            int position = 1;
            foreach (InternalVariableDeclaration variable in _operator.Variables.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Variable, position);
                yield return new XmlVariableDeclaration(xpath, variable);
                position++;
            }
        }

        /// <summary>
        /// Gets the precondition fact identifiers.
        /// </summary>
        /// <returns>The precondition fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetPreconditionIds() {
            foreach (InternalNamed precondition in _operator.Preconditions.Elements) {
                yield return precondition.Name;
            }
        }

        /// <summary>
        /// Gets the functional preconditions.
        /// </summary>
        /// <returns>The functional preconditions.</returns>
        /// <remarks>
        /// <para>All funcional precondition variable identifiers must refer to variables returned by
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperation<string>> GetFunctionalPreconditions() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.FunctionalPreconditions);
            int position = 1;
            foreach (InternalOperation functionalPrecondition in _operator.FunctionalPreconditions.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Operation, position);
                yield return new XmlOperation(xpath, functionalPrecondition);
                position++;
            }
        }

        /// <summary>
        /// Gets the fact deletion identifiers.
        /// </summary>
        /// <returns>The fact deletion identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetDeletionIds() {
            foreach (InternalNamed deletion in _operator.Deletions.Elements) {
                yield return deletion.Name;
            }
        }

        /// <summary>
        /// Gets the fact addition identifiers.
        /// </summary>
        /// <returns>The fact addition identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetAdditionIds() {
            foreach (InternalNamed addition in _operator.Additions.Elements) {
                yield return addition.Name;
            }
        }

        /// <summary>
        /// Gets the functional effects.
        /// </summary>
        /// <returns>The functional effects.</returns>
        /// <remarks>
        /// <para>All funcional effect variable identifiers must refer to variables returned by
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperation<string>> GetFunctionalEffects() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.FunctionalEffects);
            int position = 1;
            foreach (InternalOperation functionalEffect in _operator.FunctionalEffects.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Operation, position);
                yield return new XmlOperation(xpath, functionalEffect);
                position++;
            }
        }

        #endregion

    }

}
