/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Reflection;
using System.Text;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents an <see cref="Object">object</see> capable of providing a knowledge representation from XML documents.
    /// </summary>
    [CLSCompliant(true)]
    public sealed class XmlParser : IKnowledgeProvider<string, string, string, string, string, string, string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Constants

        /// <summary>
        /// The default option for <see cref="AutoCloseDomainStream"/>.
        /// </summary>
        public const bool DefaultAutoCloseDomainStream = true;

        /// <summary>
        /// The default option for <see cref="AutoCloseProblemStream"/>.
        /// </summary>
        public const bool DefaultAutoCloseProblemStream = true;

        /// <summary>
        /// The default option for <see cref="AutoCloseStreams"/>.
        /// </summary>
        public const bool DefaultAutoCloseStreams =
            DefaultAutoCloseDomainStream && DefaultAutoCloseProblemStream;

        /// <summary>
        /// The default option for <see cref="EnsureDomain"/>.
        /// </summary>
        public const bool DefaultEnsureDomain = true;

        /// <summary>
        /// The default option for <see cref="EnsureProblem"/>.
        /// </summary>
        public const bool DefaultEnsureProblem = true;

        #endregion

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The domain stream.
        /// </summary>
        private Stream _domainStream;

        /// <summary>
        /// If the domain stream will be closed after parsing.
        /// </summary>
        private bool _autoCloseDomainStream;

        /// <summary>
        /// The problem stream.
        /// </summary>
        private Stream _problemStream;

        /// <summary>
        /// If the problem stream will be closed after parsing.
        /// </summary>
        private bool _autoCloseProblemStream;

        /// <summary>
        /// The XML serializer.
        /// </summary>
        private readonly XmlSerializer _serializer;

        /// <summary>
        /// The parsing error collection.
        /// </summary>
        private XmlErrorInfoCollection _errors;

        /// <summary>
        /// If domain missing is considered an error.
        /// </summary>
        private bool _ensureDomain;

        /// <summary>
        /// If problem missing is considered an error.
        /// </summary>
        private bool _ensureProblem;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlParser"/> class.
        /// </summary>
        public XmlParser() {
            _resources = new Resources();
            _autoCloseDomainStream = DefaultAutoCloseDomainStream;
            _autoCloseProblemStream = DefaultAutoCloseProblemStream;
            _serializer = new XmlSerializer(typeof(InternalDefinition));
            _ensureDomain = DefaultEnsureDomain;
            _ensureProblem = DefaultEnsureProblem;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the domain stream.
        /// </summary>
        /// <value>The domain stream.</value>
        /// <exception cref="ArgumentNullException"><paramref name="value"/> is a null reference.</exception>
        public Stream DomainStream {
            get {
                return _domainStream;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                }
                _domainStream = value;
            }
        }

        /// <summary>
        /// Gets or sets if the domain stream must be closed after parsing.
        /// </summary>
        /// <value>True if the domain stream must be closed after parsing; otherwise, false.</value>
        public bool AutoCloseDomainStream {
            get {
                return _autoCloseDomainStream;
            }
            set {
                _autoCloseDomainStream = value;
            }
        }

        /// <summary>
        /// Gets or sets the problem stream.
        /// </summary>
        /// <value>The problem stream.</value>
        /// <exception cref="ArgumentNullException"><paramref name="value"/> is a null reference.</exception>
        public Stream ProblemStream {
            get {
                return _problemStream;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                }
                _problemStream = value;
            }
        }

        /// <summary>
        /// Gets or sets if the problem stream must be closed after parsing.
        /// </summary>
        /// <value>True if the problem stream must be closed after parsing; otherwise, false.</value>
        public bool AutoCloseProblemStream {
            get {
                return _autoCloseProblemStream;
            }
            set {
                _autoCloseProblemStream = value;
            }
        }

        /// <summary>
        /// Gets or sets if both domain and problem streams must be closed after parsing.
        /// </summary>
        /// <value>True if both domain and problem streams must be closed after parsing; otherwise, false.</value>
        public bool AutoCloseStreams {
            get {
                return _autoCloseDomainStream && _autoCloseProblemStream;
            }
            set {
                _autoCloseDomainStream = value;
                _autoCloseProblemStream = value;
            }
        }

        /// <summary>
        /// Gets or sets if domain missing is considered an error.
        /// </summary>
        /// <value>True if domain missing is considered an error; otherwise, false.</value>
        public bool EnsureDomain {
            get {
                return _ensureDomain;
            }
            set {
                _ensureDomain = value;
            }
        }

        /// <summary>
        /// Gets or sets if problem missing is considered an error.
        /// </summary>
        /// <value>True if problem missing is considered an error; otherwise, false.</value>
        public bool EnsureProblem {
            get {
                return _ensureProblem;
            }
            set {
                _ensureProblem = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the input schema.
        /// </summary>
        /// <returns>An <see cref="Stream">stream</see> from which read the input schema.</returns>
        public static Stream GetInputSchema() {
            const string XmlSchemaFileName = "Input.xsd";
            return Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(XmlParser), XmlSchemaFileName);
        }

        /// <summary>
        /// Removes the line information from a message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <returns>The message, without the line information.</returns>
        private static string RemoveLineInfo(string message) {
            const char Point = '.';
            string messageWithoutLineInfo = message;
            string[] sentences = message.Split(new char[] { Point });
            int sentencesCount = sentences.Length;
            if (sentencesCount > 1) {
                //the line information is within the last sentence of the message
                string sentenceWithLineInfo = sentences[sentencesCount - 1];
                StringBuilder messageBuffer = new StringBuilder(message.Length - sentenceWithLineInfo.Length - 1);
                //removes the last sentence of the message
                for (int sentenceIndex = 0; sentenceIndex < (sentencesCount - 2); sentenceIndex++) {
                    messageBuffer.Append(sentences[sentenceIndex]);
                    messageBuffer.Append(Point);
                }
                messageWithoutLineInfo = messageBuffer.ToString();
            }
            return messageWithoutLineInfo;
        }

        /// <summary>
        /// Manages an exception occurred while parsing an XML document.
        /// </summary>
        /// <param name="stream">The stream which were being read when the exception occurred.</param>
        /// <param name="exception">The exception.</param>
        private void ManageException(Stream stream, InvalidOperationException exception) {
            XmlException syntacticException = exception.InnerException as XmlException;
            string message;
            if (syntacticException != null) {
                int lineNumber = syntacticException.LineNumber;
                int linePosition = syntacticException.LinePosition;
                //only syntactic exceptions have line information in their message
                message = RemoveLineInfo(syntacticException.Message);
                _errors.Add(stream, lineNumber, linePosition, message);
            } else {
                XmlSchemaValidationException semanticException =
                    exception.InnerException as XmlSchemaValidationException;
                if (semanticException != null) {
                    message = semanticException.Message;
                    _errors.Add(stream, semanticException.LineNumber, semanticException.LinePosition, message);
                } else {
                    throw new RepresentationException(exception);
                }
            }
        }

        /// <summary>
        /// Reads a definition from an XML document.
        /// </summary>
        /// <param name="stream">The stream from which read the XML document.</param>
        /// <returns>The definition.</returns>
        private InternalDefinition ReadDefinition(Stream stream) {
            XmlReaderSettings settings = new XmlReaderSettings();
            //the document is validated via schema
            settings.ValidationType = ValidationType.Schema;
            settings.Schemas.Add(null, new XmlTextReader(GetInputSchema()));
            XmlReader reader = XmlReader.Create(new XmlTextReader(stream), settings);
            InternalDefinition definition = null;
            try {
                definition = _serializer.Deserialize(reader) as InternalDefinition;
            } catch (InvalidOperationException exception) {
                ManageException(stream, exception);
            } catch (Exception exception) {
                //all unknown exceptions are wrapped
                throw new RepresentationException(exception);
            }
            return definition;
        }

        /// <summary>
        /// Closes the streams.
        /// </summary>
        private void CloseStreams() {
            if (_autoCloseDomainStream) {
                _domainStream.Close();
            }
            if ((_problemStream != _domainStream) && _autoCloseProblemStream) {
                _problemStream.Close();
            }
        }

        /// <summary>
        /// Sets the parsing error collection to a null reference.
        /// </summary>
        /// <returns>The parsing error collection.</returns>
        private XmlErrorInfoCollection DisposeErrors() {
            XmlErrorInfoCollection errors = _errors;
            _errors = null;
            return errors;
        }

        /// <summary>
        /// Gets the knowledge representation.
        /// </summary>
        /// <returns>The knowledge representation.</returns>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <exception cref="XmlDomainStreamNotSetException">The domain stream is not set.</exception>
        /// <exception cref="XmlProblemStreamNotSetException">The problem stream is not set.</exception>
        /// <exception cref="XmlErrorsFoundException">Any error occurs while parsing.</exception>
        /// <exception cref="XmlDomainNotFoundException"><see cref="EnsureDomain"/> is true and no domain is
        /// defined into the domain stream document.</exception>
        /// <exception cref="XmlProblemNotFoundException"><see cref="EnsureProblem"/> is true and no problem is
        /// defined into the problem stream document.</exception>
        /// <exception cref="XmlDomainMismatchException">The problem references a domain different from the
        /// provided one.</exception>
        /// <exception cref="RepresentationException">Any other error occurs.</exception>
        public IKnowledge<string, string, string, string, string, string, string> GetKnowledge() {
            if (_domainStream == null) {
                throw new XmlDomainStreamNotSetException(_resources.NoDomainStreamSet);
            }
            if (_problemStream == null) {
                throw new XmlProblemStreamNotSetException(_resources.NoProblemStreamSet);
            }
            _errors = new XmlErrorInfoCollection();
            InternalDefinition domainDefinition = ReadDefinition(_domainStream);
            InternalDefinition problemDefinition = domainDefinition;
            //if both streams are the same, does not read again the stream
            if (_problemStream != _domainStream) {
                problemDefinition = ReadDefinition(_problemStream);
            }
            CloseStreams();
            if (_errors.Count > 0) {
                throw new XmlErrorsFoundException(_resources.ParsingErrorsFound, DisposeErrors());
            }
            DisposeErrors();
            InternalDomain domain = domainDefinition.Domain;
            if ((domain == null) && _ensureDomain) {
                throw new XmlDomainNotFoundException(_resources.NoDomainFound);
            }
            InternalProblem problem = problemDefinition.Problem;
            if ((problem == null) && _ensureProblem) {
                throw new XmlProblemNotFoundException(_resources.NoProblemFound);
            }
            if ((domain == null) && (problem == null) && !problem.Domain.Equals(domain.Name)) {
                throw new XmlDomainMismatchException(_resources.DomainMismatchFound);
            }
            return new XmlKnowledge(new XmlDomain(domain), new XmlProblem(problem));
        }

        #endregion

    }

}
