/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a problem defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlProblem : XmlLocatable,
        IProblem<string, string, string, string, string, string, string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal problem representacion.
        /// </summary>
        private readonly InternalProblem _problem;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlProblem"/> class.
        /// </summary>
        /// <param name="problem">The internal problem representacion.</param>
        internal XmlProblem(InternalProblem problem)
            : base(XPathManager.Append(XPathManager.Create(XmlElementName.Definition),
            XmlElementName.Problem, XmlAttributeName.Name, problem.Name)) {
            _problem = problem;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        public string Id {
            get {
                return _problem.Name;
            }
        }

        /// <summary>
        /// Gets the number of objects.
        /// </summary>
        /// <value>The number of objects.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ObjectCount {
            get {
                return _problem.Objects.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of facts.
        /// </summary>
        /// <value>The number of facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FactCount {
            get {
                return _problem.Facts.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of variables.
        /// </summary>
        /// <value>The number of variables.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int VariableCount {
            get {
                return _problem.Variables.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of initial facts.
        /// </summary>
        /// <value>The number of initial facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int InitialFactCount {
            get {
                return _problem.InitialFacts.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of functional initializations.
        /// </summary>
        /// <value>The number of functional initializations.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FunctionalInitializationCount {
            get {
                return _problem.FunctionalInitializations.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of goal facts.
        /// </summary>
        /// <value>The number of goal facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int GoalFactCount {
            get {
                return _problem.GoalFacts.Elements.Count;
            }
        }

        /// <summary>
        /// Gets the number of functional goals.
        /// </summary>
        /// <value>The number of functional goals.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int FunctionalGoalCount {
            get {
                return _problem.FunctionalGoals.Elements.Count;
            }
        }

        /// <summary>
        /// Determines if a metric has been defined.
        /// </summary>
        /// <value>True if a metric has been defined; otherwise, false.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public bool HasMetric {
            get {
                return (_problem.Metric != null);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the objects.
        /// </summary>
        /// <returns>The objects.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IObject<string, string>> GetObjects() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Objects);
            int position = 1;
            foreach (InternalObjectDeclaration obj in _problem.Objects.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Object, position);
                yield return new XmlObject(xpath, obj);
                position++;
            }
        }

        /// <summary>
        /// Gets the facts.
        /// </summary>
        /// <returns>The facts.</returns>
        /// <remarks>
        /// <para>Each fact is represented as a predicate
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified instance</see> and must refer
        /// to a predicate returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetPredicates">the domain
        /// corresponding method</see>.</para>
        /// <para>All fact argument identifiers must refer to objects returned by <see cref="GetObjects">the
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IIdentifiedInstance<string, string, string>> GetFacts() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Facts);
            int position = 1;
            foreach (InternalFactDeclaration fact in _problem.Facts.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Fact, position);
                yield return new XmlFactDeclaration(xpath, fact);
                position++;
            }
        }

        /// <summary>
        /// Gets the variables.
        /// </summary>
        /// <returns>The variables.</returns>
        /// <remarks>
        /// <para>Each variable is represented as a function
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified instance</see> and must refer
        /// to a function returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetFunctions">the domain
        /// corresponding method</see>.</para>
        /// <para>All variable argument identifiers must refer to objects returned by <see cref="GetObjects">the
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IIdentifiedInstance<string, string, string>> GetVariables() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.Variables);
            int position = 1;
            foreach (InternalVariableDeclaration variable in _problem.Variables.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Variable, position);
                yield return new XmlVariableDeclaration(xpath, variable);
                position++;
            }
        }

        /// <summary>
        /// Gets the initial fact identifiers.
        /// </summary>
        /// <returns>The initial fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetInitialFactIds() {
            foreach (InternalNamed fact in _problem.InitialFacts.Elements) {
                yield return fact.Name;
            }
        }

        /// <summary>
        /// Gets the funcional initializations.
        /// </summary>
        /// <returns>The funcional initializations.</returns>
        /// <remarks>
        /// <para>All funcional initialization variable identifiers must refer to variables returned by
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperation<string>> GetFunctionalInitializations() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.FunctionalInitializations);
            int position = 1;
            foreach (InternalOperation proceduralInitialization in _problem.FunctionalInitializations.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Operation, position);
                yield return new XmlOperation(xpath, proceduralInitialization);
                position++;
            }
        }

        /// <summary>
        /// Gets the goal fact identifiers.
        /// </summary>
        /// <returns>The goal fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetGoalFactIds() {
            foreach (InternalNamed goal in _problem.GoalFacts.Elements) {
                yield return goal.Name;
            }
        }

        /// <summary>
        /// Gets the functional goals.
        /// </summary>
        /// <returns>The functional goals.</returns>
        /// <remarks>
        /// <para>All functional goal variable identifiers must refer to variables returned by
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IOperation<string>> GetFunctionalGoals() {
            string xpath;
            string baseXPath = XPathManager.Append(XPath, XmlElementName.FunctionalGoals);
            int position = 1;
            foreach (InternalOperation proceduralGoal in _problem.FunctionalGoals.Elements) {
                xpath = XPathManager.Append(baseXPath, XmlElementName.Operation, position);
                yield return new XmlOperation(xpath, proceduralGoal);
                position++;
            }
        }

        /// <summary>
        /// Gets the metric.
        /// </summary>
        /// <returns>The metric.</returns>
        /// <remarks>
        /// A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending
        /// on implementation details, while managing the returned result components.</remarks>
        /// <exception cref="InvalidOperationException">No metric is defined.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IMetric<string> GetMetric() {
            if (_problem.Metric == null) {
                throw new InvalidOperationException();
            }
            return new XmlMetric(XPathManager.Append(XPath, XmlElementName.Metric), _problem.Metric);
        }

        #endregion

    }

}
