/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a signature defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlSignature : XmlLocatable, ISignature<string, string, string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal signature representacion.
        /// </summary>
        private readonly InternalSignature _signature;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlSignature"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="signature">The internal signature representacion.</param>
        internal XmlSignature(string xpath, InternalSignature signature)
            : base(xpath) {
            _signature = signature;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string Id {
            get {
                return _signature.Name;
            }
        }

        /// <summary>
        /// Gets the number of parameters.
        /// </summary>
        /// <value>The number of parameters.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ParameterCount {
            get {
                return _signature.Parameters.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <returns>The parameters.</returns>
        /// <remarks>
        /// <para>Each parameter is represented as an <see cref="IObject{TType, TObj}">object</see> and
        /// must refer to a type returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetTypes">the domain
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<IObject<string, string>> GetParameters() {
            string xpath;
            int position = 1;
            foreach (InternalObjectDeclaration parameter in _signature.Parameters) {
                xpath = XPathManager.Append(XPath, XmlElementName.Parameter, position);
                yield return new XmlObject(xpath, parameter);
                position++;
            }
        }

        #endregion

    }

}
