/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a type defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlType : XmlLocatable, IType<string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal type representacion.
        /// </summary>
        private readonly InternalTypeDeclaration _type;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlType"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="type">The internal type representacion.</param>
        internal XmlType(string xpath, InternalTypeDeclaration type)
            : base(xpath) {
            _type = type;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string Id {
            get {
                return _type.Name;
            }
        }

        /// <summary>
        /// Determines if has no parent.
        /// </summary>
        /// <value>True if is root; otherwise, false.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public bool IsRoot {
            get {
                return (_type.Parent == null);
            }
        }

        /// <summary>
        /// Gets the parent identifier.
        /// </summary>
        /// <value>The parent identifier.</value>
        /// <exception cref="InvalidOperationException">No metric is defined.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string ParentId {
            get {
                if (_type.Parent == null) {
                    throw new InvalidOperationException();
                }
                return _type.Parent;
            }
        }

        #endregion

    }

}
