/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation.Xml.Internal;

namespace Aran.Representation.Xml {

    /// <summary>
    /// Represents a variable defined within an XML input document.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class XmlVariableDeclaration : XmlLocatable, IIdentifiedInstance<string, string, string> {

        /*
         * NOTE: This type and all of the same namespace have been designed for being functional, but not
         * efficient, neither in memory consumption nor in time taken. So, in that way, the current
         * implementation of this parser is highly improvable. For example, a future implementation may use its
         * own parsing routines, instead of using the .NET built-in XML serialization ones.
         */

        #region Fields

        /// <summary>
        /// The internal variable representacion.
        /// </summary>
        private readonly InternalVariableDeclaration _variable;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlVariableDeclaration"/> class.
        /// </summary>
        /// <param name="xpath">The xpath.</param>
        /// <param name="variable">The internal variable representacion.</param>
        internal XmlVariableDeclaration(string xpath, InternalVariableDeclaration variable)
            : base(xpath) {
            _variable = variable;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <value>The identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string Id {
            get {
                return _variable.Name;
            }
        }

        /// <summary>
        /// Gets the signature identifier.
        /// </summary>
        /// <value>The signature identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public string SignatureId {
            get {
                return _variable.Function;
            }
        }

        /// <summary>
        /// Gets the number of arguments.
        /// </summary>
        /// <value>The number of arguments.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ArgumentCount {
            get {
                return _variable.Arguments.Count;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the argument identifiers.
        /// </summary>
        /// <returns>The argument identifiers.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<string> GetArgumentIds() {
            foreach (InternalNamed argument in _variable.Arguments) {
                yield return argument.Name;
            }
        }

        #endregion

    }

}
