/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents a domain.
    /// </summary>
    /// <remarks>
    /// The domain may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.
    /// </remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface IDomain<TType, TPred, TFunc, TOp, TObj, TFact, TVar> {

        #region Properties

        /// <summary>
        /// Gets the number of types.
        /// </summary>
        /// <value>The number of types.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int TypeCount { get; }

        /// <summary>
        /// Gets the number of predicates.
        /// </summary>
        /// <value>The number of predicates.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int PredicateCount { get; }

        /// <summary>
        /// Gets the number of functions.
        /// </summary>
        /// <value>The number of functions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FunctionCount { get; }

        /// <summary>
        /// Gets the number of operators.
        /// </summary>
        /// <value>The number of operators.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int OperatorCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the types.
        /// </summary>
        /// <returns>The types.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IType<TType>> GetTypes();

        /// <summary>
        /// Gets the predicate signatures.
        /// </summary>
        /// <returns>The predicate signatures.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<ISignature<TType, TPred, TObj>> GetPredicates();

        /// <summary>
        /// Gets the function signatures.
        /// </summary>
        /// <returns>The function signatures.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<ISignature<TType, TFunc, TObj>> GetFunctions();

        /// <summary>
        /// Gets the operators.
        /// </summary>
        /// <returns>The operators.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> GetOperators();

        #endregion

    }

}
