/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents a signature instance.
    /// </summary>
    /// <remarks>
    /// <para>A <see cref="ISignature{TType, TSig, TObj}">signature</see> instance is a list of objects
    /// which refer to a <see cref="ISignature{TType, TSig, TObj}">signature</see> and has the same length
    /// of the <see cref="ISignature{TType, TSig, TObj}">signature</see> parameter list. Also, each object
    /// type must be the same of the corresponding parameter type.</para>
    /// <para>The instance may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.</para>
    /// </remarks>
    /// <typeparam name="TSig">The type of the signature identifier.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface IInstance<TSig, TObj> {

        #region Properties

        /// <summary>
        /// Gets the signature identifier.
        /// </summary>
        /// <value>The signature identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        TSig SignatureId { get; }

        /// <summary>
        /// Gets the number of arguments.
        /// </summary>
        /// <value>The number of arguments.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int ArgumentCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the argument identifiers.
        /// </summary>
        /// <returns>The argument identifiers.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TObj> GetArgumentIds();

        #endregion

    }

}
