/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents an operation.
    /// </summary>
    /// <remarks>
    /// The operation may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.
    /// </remarks>
    /// <typeparam name="T">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface IOperation<T> {

        #region Properties

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <value>The type.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        OperationType Type { get; }

        /// <summary>
        /// Gets the number of operands.
        /// </summary>
        /// <value>The number of operands.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int OperandCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the operands.
        /// </summary>
        /// <returns>The operands.</returns>
        /// <remarks>
        /// <para>The operands are returned from left to right. For example, the returned operands for
        /// <c>a = b + c</c> are, in this order, <c>a</c>, <c>b</c>, and <c>c</c>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperand<T>> GetOperands();

        #endregion

    }

}
