/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents an operator.
    /// </summary>
    /// <remarks>
    /// The operator may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.
    /// </remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>
        : IIdentified<TOp> {

        #region Properties

        /// <summary>
        /// Gets the number of parameters.
        /// </summary>
        /// <value>The number of parameters.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int ParameterCount { get; }

        /// <summary>
        /// Gets the number of facts.
        /// </summary>
        /// <value>The number of facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FactCount { get; }

        /// <summary>
        /// Gets the number of variables.
        /// </summary>
        /// <value>The number of variables.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int VariableCount { get; }

        /// <summary>
        /// Gets the number of precondition facts.
        /// </summary>
        /// <value>The number of precondition facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int PreconditionCount { get; }

        /// <summary>
        /// Gets the number of functional preconditions.
        /// </summary>
        /// <value>The number of functional preconditions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FunctionalPreconditionCount { get; }

        /// <summary>
        /// Gets the number of fact deletions.
        /// </summary>
        /// <value>The number of fact deletions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int DeletionCount { get; }

        /// <summary>
        /// Gets the number of fact additions.
        /// </summary>
        /// <value>The number of fact additions.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int AdditionCount { get; }

        /// <summary>
        /// Gets the number of functional effects.
        /// </summary>
        /// <value>The number of functional effects.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FunctionalEffectCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <returns>The parameters.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IObject<TType, TObj>> GetParameters();

        /// <summary>
        /// Gets the facts.
        /// </summary>
        /// <returns>The facts.</returns>
        /// <remarks>
        /// <para>Each fact is represented as a predicate
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified signature instance</see> and must refer
        /// to a predicate returned by 
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetPredicates">the domain
        /// corresponding method</see>.</para>
        /// <para>All fact argument identifiers must refer to parameters returned by
        /// <see cref="GetParameters">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IIdentifiedInstance<TPred, TObj, TFact>> GetFacts();

        /// <summary>
        /// Gets the variables.
        /// </summary>
        /// <returns>The variables.</returns>
        /// <remarks>
        /// <para>Each variable is represented as a function
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified signature instance</see> and must refer
        /// to a function returned by 
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetFunctions">the domain
        /// corresponding method</see>.</para>
        /// <para>All variable argument identifiers must refer to parameters returned by
        /// <see cref="GetParameters">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IIdentifiedInstance<TFunc, TObj, TVar>> GetVariables();

        /// <summary>
        /// Gets the precondition fact identifiers.
        /// </summary>
        /// <returns>The precondition fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TFact> GetPreconditionIds();

        /// <summary>
        /// Gets the functional preconditions.
        /// </summary>
        /// <returns>The functional preconditions.</returns>
        /// <remarks>
        /// <para>All funcional precondition variable identifiers must refer to variables returned by 
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperation<TVar>> GetFunctionalPreconditions();

        /// <summary>
        /// Gets the fact deletion identifiers.
        /// </summary>
        /// <returns>The fact deletion identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TFact> GetDeletionIds();

        /// <summary>
        /// Gets the fact addition identifiers.
        /// </summary>
        /// <returns>The fact addition identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TFact> GetAdditionIds();

        /// <summary>
        /// Gets the functional effects.
        /// </summary>
        /// <returns>The functional effects.</returns>
        /// <remarks>
        /// <para>All funcional effect variable identifiers must refer to variables returned by 
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperation<TVar>> GetFunctionalEffects();

        #endregion

    }

}
