/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents a problem.
    /// </summary>
    /// <remarks>
    /// The problem may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.
    /// </remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface IProblem<TType, TPred, TFunc, TOp, TObj, TFact, TVar> {

        #region Properties

        /// <summary>
        /// Gets the number of objects.
        /// </summary>
        /// <value>The number of objects.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int ObjectCount { get; }

        /// <summary>
        /// Gets the number of facts.
        /// </summary>
        /// <value>The number of facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FactCount { get; }

        /// <summary>
        /// Gets the number of variables.
        /// </summary>
        /// <value>The number of variables.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int VariableCount { get; }

        /// <summary>
        /// Gets the number of initial facts.
        /// </summary>
        /// <value>The number of initial facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int InitialFactCount { get; }

        /// <summary>
        /// Gets the number of functional initializations.
        /// </summary>
        /// <value>The number of functional initializations.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FunctionalInitializationCount { get; }

        /// <summary>
        /// Gets the number of goal facts.
        /// </summary>
        /// <value>The number of goal facts.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int GoalFactCount { get; }

        /// <summary>
        /// Gets the number of functional goals.
        /// </summary>
        /// <value>The number of functional goals.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int FunctionalGoalCount { get; }

        /// <summary>
        /// Determines if a metric has been defined.
        /// </summary>
        /// <value>True if a metric has been defined; otherwise, false.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        bool HasMetric { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the objects.
        /// </summary>
        /// <returns>The objects.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IObject<TType, TObj>> GetObjects();

        /// <summary>
        /// Gets the facts.
        /// </summary>
        /// <returns>The facts.</returns>
        /// <remarks>
        /// <para>Each fact is represented as a predicate
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified signature instance</see> and must refer
        /// to a predicate returned by 
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetPredicates">the domain
        /// corresponding method</see>.</para>
        /// <para>All fact argument identifiers must refer to objects returned by <see cref="GetObjects">the
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IIdentifiedInstance<TPred, TObj, TFact>> GetFacts();

        /// <summary>
        /// Gets the variables.
        /// </summary>
        /// <returns>The variables.</returns>
        /// <remarks>
        /// <para>Each variable is represented as a function
        /// <see cref="IIdentifiedInstance{TSig, TObj, TIns}">identified signature instance</see> and must refer
        /// to a function returned by 
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetFunctions">the domain
        /// corresponding method</see>.</para>
        /// <para>All variable argument identifiers must refer to objects returned by <see cref="GetObjects">the
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IIdentifiedInstance<TFunc, TObj, TVar>> GetVariables();

        /// <summary>
        /// Gets the initial fact identifiers.
        /// </summary>
        /// <returns>The initial fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TFact> GetInitialFactIds();

        /// <summary>
        /// Gets the funcional initializations.
        /// </summary>
        /// <returns>The funcional initializations.</returns>
        /// <remarks>
        /// <para>All funcional initialization variable identifiers must refer to variables returned by 
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperation<TVar>> GetFunctionalInitializations();

        /// <summary>
        /// Gets the goal fact identifiers.
        /// </summary>
        /// <returns>The goal fact identifiers.</returns>
        /// <remarks>
        /// <para>All identifiers must refer to facts returned by <see cref="GetFacts">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<TFact> GetGoalFactIds();

        /// <summary>
        /// Gets the functional goals.
        /// </summary>
        /// <returns>The functional goals.</returns>
        /// <remarks>
        /// <para>All functional goal variable identifiers must refer to variables returned by 
        /// <see cref="GetVariables">the corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IOperation<TVar>> GetFunctionalGoals();

        /// <summary>
        /// Gets the metric.
        /// </summary>
        /// <returns>The metric.</returns>
        /// <remarks>
        /// A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending
        /// on implementation details, while managing the returned result components.</remarks>
        /// <exception cref="InvalidOperationException">No metric is defined.</exception>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IMetric<TVar> GetMetric();

        #endregion

    }

}
