/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Representation {

    /// <summary>
    /// Represents a signature.
    /// </summary>
    /// <remarks>
    /// <para>A signature acts as a template for an instantiable element so that all element
    /// <see cref="IInstance{TSig, TObj}">instance</see>s must fulfil it.</para>
    /// <para>The signature may not be full-retrieved. More on-demand retrieving tasks could be done, depending on
    /// implementation details, while managing its components.</para>
    /// </remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TSig">The type of the signature identifier.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    [CLSCompliant(true)]
    public interface ISignature<TType, TSig, TObj> : IIdentified<TSig> {

        #region Properties

        /// <summary>
        /// Gets the number of parameters.
        /// </summary>
        /// <value>The number of parameters.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        int ParameterCount { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <returns>The parameters.</returns>
        /// <remarks>
        /// <para>Each parameter is represented as an <see cref="IObject{TType, TObj}">object</see> and
        /// must refer to a type returned by
        /// <see cref="IDomain{TType, TPred, TFunc, TOp, TObj, TFact, TVar}.GetTypes">the domain
        /// corresponding method</see>.</para>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        IEnumerable<IObject<TType, TObj>> GetParameters();

        #endregion

    }

}
