/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

using Aran.Core;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents an <see cref="IEnumerator{T}">enumerator</see> of <see cref="State">state</see>s
    /// generated by applying <see cref="Action">action</see>s over a specified one.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class ActionGeneratedStateEnumerator : IEnumerator<State> {

        #region Fields

        /// <summary>
        /// The <see cref="State">state</see> over which apply the <see cref="Action">action</see>s in order
        /// to generate the new ones.
        /// </summary>
        private readonly State _state;

        /// <summary>
        /// The <see cref="Action">action</see>s.
        /// </summary>
        private readonly Action[] _actions;

        /// <summary>
        /// The number of <see cref="Action">action</see>s.
        /// </summary>
        private readonly int _actionCount;

        /// <summary>
        /// The list which specifies what <see cref="Action">action</see>s are applied to
        /// <see cref="_state">the state</see>.
        /// </summary>
        /// <remarks>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected and, therefore,
        /// applied to <see cref="_state">the state</see> in order to generate a new one.</para>
        /// </remarks>
        private readonly int[] _actionSelectionList;

        /// <summary>
        /// The current <see cref="Action.Id">action identifier</see>.
        /// </summary>
        private int _currentActionId;

        /// <summary>
        /// The current <see cref="State">state</see>.
        /// </summary>
        private State _currentState;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ActionGeneratedStateEnumerator"/> class.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see> over which apply the
        /// <see cref="Action">action</see>s in order to generate the new ones.</param>
        /// <param name="actions">The <see cref="Action">action</see>s.</param>
        /// <param name="actionSelectionList">
        /// <para>The list which specifies what <see cref="Action">action</see>s are applied to
        /// <see cref="_state">the state</see>.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the selection state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is selected and, therefore,
        /// applied to <see cref="_state">the state</see> in order to generate a new one.</para>
        /// </param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this constructor.
        /// </remarks>
        public ActionGeneratedStateEnumerator(State state, Action[] actions, int[] actionSelectionList) {
            _state = state;
            _actions = actions;
            _actionCount = actions.Length;
            _actionSelectionList = actionSelectionList;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="State">state</see> in the collection at the current position of the
        /// <see cref="IEnumerator{T}">enumerator</see>.
        /// </summary>
        /// <returns>The <see cref="State">state</see> in the collection at the current position of the
        /// <see cref="IEnumerator{T}">enumerator</see>.</returns>
        public State Current {
            get {
                const State NoState = null;
                if (_currentState == NoState) {
                    throw new InvalidOperationException();
                }
                return _currentState;
            }
        }

        /// <summary>
        /// Gets the <see cref="State">state</see> in the collection at the current position of the
        /// <see cref="IEnumerator">enumerator</see>.
        /// </summary>
        /// <returns>The <see cref="State">state</see> in the collection at the current position of the
        /// <see cref="IEnumerator">enumerator</see>.</returns>
        object IEnumerator.Current {
            get {
                const State NoState = null;
                if (_currentState == NoState) {
                    throw new InvalidOperationException();
                }
                return _currentState;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Advances the <see cref="IEnumerator">enumerator</see> to the next <see cref="State">state</see> of
        /// the collection.
        /// </summary>
        /// <returns>
        /// True if the <see cref="IEnumerator">enumerator</see> was successfully advanced to the next
        /// <see cref="State">state</see>; false if the <see cref="IEnumerator">enumerator</see> has passed the
        /// end of the collection.
        /// </returns>
        /// <exception cref="InvalidOperationException">The collection was modified after the
        /// <see cref="IEnumerator">enumerator</see> was created. </exception>
        public bool MoveNext() {
            const int IntSize = 32;
            Action currentAction;
            //for each action
            while (true) {
                if (_currentActionId >= _actionCount) {
                    _currentState = null;
                    return false;
                }
                //if the current action is applicable to the state
                if ((_actionSelectionList[_currentActionId / IntSize] &
                (1 << (_currentActionId % IntSize))) != 0) {
                    currentAction = _actions[_currentActionId];
                    if ((currentAction != null) && currentAction.IsApplicableTo(_state)) {
                        //the current state is the result of applying the current action to the state
                        _currentState = currentAction.ApplyTo(_state);
                        _currentActionId++;
                        return true;
                    }
                }
                _currentActionId++;
            }
        }

        /// <summary>
        /// Sets the <see cref="IEnumerator">enumerator</see> to its initial position, which is before the first
        /// <see cref="State">state</see> in the collection.
        /// </summary>
        /// <exception cref="InvalidOperationException">The collection was modified after the
        /// <see cref="IEnumerator">enumerator</see> was created.</exception>
        public void Reset() {
            _currentActionId = Action.FirstId;
            _currentState = null;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged
        /// resources.
        /// </summary>
        public void Dispose() {
            //do nothing
        }

        #endregion

    }

}
