/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Core;
using Aran.Searching;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents the Enforced Hill Climbing algorithm.
    /// </summary>
    /// <remarks>
    /// The implentation of this algorithm is based on the explanations of the paper titled
    /// "The FF Planning System: Fast Plan Generation Through Heuristic Search", written by Jrg Hoffmann and
    /// Bernhard Nebel and published by the Journal of Artificial Intelligence Research in 2001.
    /// </remarks>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class EnforcedHillClimbingAlgorithm : ISearchAlgorithm<State> {

        #region Fields

        /// <summary>
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </summary>
        private readonly IInitialStateProvider<State> _initialStateProvider;

        /// <summary>
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </summary>
        private readonly IGoalStateVerifier<State> _goalVerifier;

        /// <summary>
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </summary>
        private readonly IStateCostProvider<State> _costCalculator;

        /// <summary>
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </summary>
        private readonly RelaxedPlanHeuristic _heuristicCalculator;

        /// <summary>
        /// The <see cref="Action">action</see> selector.
        /// </summary>
        private readonly ActionSelector _actionSelector;

        /// <summary>
        /// The visited states.
        /// </summary>
        private readonly IDictionary<State, State> _visitedStates;

        /// <summary>
        /// The <see cref="Action">action</see>s.
        /// </summary>
        private readonly Action[] _actions;

        /// <summary>
        /// The number of <see cref="Action">action</see>s.
        /// </summary>
        private readonly int _actionCount;

        /// <summary>
        /// The goal <see cref="State.Facts">facts</see>.
        /// </summary>
        private readonly int[] _goals;

        /// <summary>
        /// The number of goals.
        /// </summary>
        private readonly int _goalCount;

        /// <summary>
        /// The length of <see cref="_goals"/>.
        /// </summary>
        private readonly int _goalsLength;

        /// <summary>
        /// The relaxed plan action list.
        /// </summary>
        /// <remarks>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the presence state of the corresponding
        /// <see cref="Action">action</see> in the relaxed plan. If the bit at <c>actionIndex</c> has the one
        /// value, the <see cref="Action">action</see> identified by <c>actionIndex</c> is present in the plan.</para>
        /// </remarks>
        private int[] _relaxedPlanActionList;

        /// <summary>
        /// If the "helpful actions" heuristic must be used.
        /// </summary>
        private bool _useHelpfulActionsOnly;

        /// <summary>
        /// The first <see cref="EnforcedHillClimbingNode">node</see> of the open list.
        /// </summary>
        private EnforcedHillClimbingNode _openListFirstNode;
        
        /// <summary>
        /// The estimated cost of reaching a goal state from the current one.
        /// </summary>
        private float _estimatedRemainingCost;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="EnforcedHillClimbingAlgorithm"/> class.
        /// </summary>
        /// <param name="initialStateProvider">The <see cref="IInitialStateProvider{T}">initial state provider</see>.</param>
        /// <param name="goalVerifier">The <see cref="IInitialStateProvider{T}">initial state provider</see>.</param>
        /// <param name="costCalculator">The <see cref="IStateCostProvider{T}">cost calculator</see>.</param>
        /// <param name="factCount"><see cref="State.Facts">the state fact certainty list</see> length.</param>
        /// <param name="actions">The <see cref="Action">action</see>s.</param>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="preconditionCounts">
        /// <para>The number of preconditions by operator.</para>
        /// <para>For each index, the value represents the number of preconditions of the operator identified by
        /// that index.</para>
        /// </param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <param name="actionActivationTable">
        /// <para>The action activation table.</para>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="goalCount">The number of goals.</param>
        /// <param name="goals">The goal <see cref="State.Facts">facts</see>.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this constructor.
        /// </remarks>
        public EnforcedHillClimbingAlgorithm(IInitialStateProvider<State> initialStateProvider,
        IGoalStateVerifier<State> goalVerifier, IStateCostProvider<State> costCalculator, int factCount,
        Action[] actions, int operatorCount, int[] preconditionCounts, int[] actionCounts,
        int[][][][] actionActivationTable, int goalCount, int[] goals, IEqualityComparer<State> equalityComparer) {
            _initialStateProvider = initialStateProvider;
            _goalVerifier = goalVerifier;
            _costCalculator = costCalculator;
            RelaxedPlanner relaxedPlanner = new RelaxedPlanner(factCount, actions, operatorCount, actionCounts,
                actionActivationTable, goalCount, goals);
            _heuristicCalculator = new RelaxedPlanHeuristic(relaxedPlanner, _costCalculator);
            _actions = actions;
            _actionCount = actions.Length;
            _actionSelector = new ConjunctionActionSelector(operatorCount, preconditionCounts,
                _actionCount, actionCounts, actionActivationTable);
            _visitedStates = new Dictionary<State, State>(equalityComparer);
            _goals = goals;
            _goalCount = goalCount;
            _goalsLength = _goals.Length;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="EnforcedHillClimbingAlgorithm"/> class.
        /// </summary>
        /// <param name="initialStateProvider">The <see cref="IInitialStateProvider{T}">initial state provider</see>.</param>
        /// <param name="goalVerifier">The <see cref="IInitialStateProvider{T}">initial state provider</see>.</param>
        /// <param name="costCalculator">The <see cref="IStateCostProvider{T}">cost calculator</see>.</param>
        /// <param name="factCount"><see cref="State.Facts">the state fact certainty list</see> length.</param>
        /// <param name="actions">The <see cref="Action">action</see>s.</param>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="preconditionCounts">
        /// <para>The number of preconditions by operator.</para>
        /// <para>For each index, the value represents the number of preconditions of the operator identified by
        /// that index.</para>
        /// </param>
        /// <param name="actionCounts">
        /// <para>The number of <see cref="Action">action</see>s by operator.</para>
        /// <para>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</para>
        /// </param>
        /// <param name="actionActivationTable">
        /// <para>The action activation table.</para>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </param>
        /// <param name="goalCount">The number of goals.</param>
        /// <param name="goals">The goal <see cref="State.Facts">facts</see>.</param>
        /// <remarks>
        /// Assumes the correction of all parameters, as a performance improvement. If needed,
        /// correction checks must be done previously to the call of this constructor. Uses the
        /// <see cref="EqualityComparer{T}.Default">default state equality comparer</see>.
        /// </remarks>
        public EnforcedHillClimbingAlgorithm(IInitialStateProvider<State> initialStateProvider,
        IGoalStateVerifier<State> goalVerifier, IStateCostProvider<State> costCalculator, int factCount,
        Action[] actions, int operatorCount, int[] preconditionCounts, int[] actionCounts,
        int[][][][] actionActivationTable, int goalCount, int[] goals)
            : this(initialStateProvider, goalVerifier, costCalculator, factCount, actions, operatorCount,
            preconditionCounts, actionCounts, actionActivationTable, goalCount, goals,
            EqualityComparer<State>.Default) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds a <see cref="State">state</see> to the visited ones.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <returns>True if <paramref name="state"/> is added (that is, if no already visited); false otherwise.</returns>
        private bool AddToVisited(State state) {
            bool added = true;
            try {
                _visitedStates.Add(state, state);
            } catch (ArgumentException) {
                added = false;
            }
            return added;
        }

        /// <summary>
        /// Extracts the goal identifiers of a <see cref="State">state</see>.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <returns></returns>
        private int[] ExtractGoalIds(State state) {
            const int EmptyGoalSection = 0;
            const int IntSize = 32;
            List<int> goalIdList = new List<int>();
            int[] stateFacts = state.Facts;
            int goalSection;
            int goalId = 0;
            //for each goal
            for (int sectionIndex = 0; sectionIndex < _goalsLength; sectionIndex++) {
                goalSection = stateFacts[sectionIndex] & _goals[sectionIndex];
                if (goalSection != EmptyGoalSection) {
                    for (int goalIndex = 0; goalIndex < IntSize; goalIndex++) {
                        //if the goal is true
                        if ((goalSection & (1 << goalIndex)) != EmptyGoalSection) {
                            //adds the goal identifier to the list
                            goalIdList.Add(goalId);
                        }
                        goalId++;
                    }
                } else {
                    goalId += IntSize;
                }
            }
            int[] goalIds = goalIdList.ToArray();
            //the goal identifier list is sorted before returned
            Array.Sort(goalIds);
            return goalIds;
        }

        /// <summary>
        /// Checks if an <see cref="Action">action</see> deletes some of the specified goals.
        /// </summary>
        /// <param name="action">The <see cref="Action">action</see>.</param>
        /// <param name="goalIds">The goal identifiers.</param>
        /// <returns></returns>
        private static bool Deletes(Action action, int[] goalIds) {
            int[] deletionIds = action.Deletions;
            //for each deletion in action
            foreach (int deletionId in deletionIds) {
                //if the deletion corresponds to a goal, the action deletes some of the specified ones
                if (Array.BinarySearch(goalIds, deletionId) > 0) {
                    return true;
                }
	        }
            return false;
        }

        /// <summary>
        /// Adds a <see cref="State">state</see> to the open list.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <param name="firstOpenListNode">The first <see cref="EnforcedHillClimbingNode">node</see> of the
        /// open list.</param>
        /// <param name="lastOpenListNode">The last <see cref="EnforcedHillClimbingNode">node</see> of the
        /// open list.</param>
        private void AddToOpenList(State state, ref EnforcedHillClimbingNode firstOpenListNode,
        ref EnforcedHillClimbingNode lastOpenListNode) {
            const EnforcedHillClimbingNode NoNode = null;
            //if the state has not been visited, adds it to the visited ones and the open list
            if (AddToVisited(state)) {
                EnforcedHillClimbingNode successorNode = new EnforcedHillClimbingNode(state);
                if (firstOpenListNode == NoNode) {
                    firstOpenListNode = successorNode;
                } else {
                    lastOpenListNode.Next = successorNode;
                }
                lastOpenListNode = successorNode;
            }
        }

        /// <summary>
        /// Adds the successors of a <see cref="State">state</see> to the open list.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <param name="useAddedGoalDeletion">If the "added goal deletion" heuristic must be used.</param>
        /// <returns>If some successor of <paramref name="state"/> is added to the open list.</returns>
        private bool AddSuccessorsToOpenList(State state, bool useAddedGoalDeletion) {
            const int EmptyActionSection = 0;
            const int IntSize = 32;
            const EnforcedHillClimbingNode NoNode = null;
            EnforcedHillClimbingNode firstOpenListNode = NoNode;
            EnforcedHillClimbingNode lastOpenListNode = NoNode;
            int[] goalIds = ExtractGoalIds(state);
            bool hasGoals = (goalIds.Length > 0);
            int actionId = 0;
            int relaxedPlanActionListLength = _relaxedPlanActionList.Length;
            int actionSection;
            int actionIndex;
            Action action;
            int[] actionSelectionList = _actionSelector.GetActionSelectionList(state);
            //for each action of the relaxed plan
            for (int sectionIndex = 0; sectionIndex < relaxedPlanActionListLength; sectionIndex++) {
                //if "helpful actions" heuristic is used, only "helpful actions" derived states are generated
                if (_useHelpfulActionsOnly) {
                    actionSection = actionSelectionList[sectionIndex] & _relaxedPlanActionList[sectionIndex];
                } else {
                    actionSection = actionSelectionList[sectionIndex];
                }
                if (actionSection != EmptyActionSection) {
                    actionIndex = 0;
                    while ((actionIndex < IntSize) && (actionId < _actionCount)) {
                        if ((actionSection & (1 << actionIndex)) != EmptyActionSection) {
                            action = _actions[actionId];
                            //if "added goal deletion" heuristic is used and any goal is deleted by the action,
                            //no successor is generated for the state
                            if (useAddedGoalDeletion && hasGoals && Deletes(action, goalIds)) {
                                return false;
                            }
                            //adds the successor to the local open list
                            AddToOpenList(action.ApplyTo(state), ref firstOpenListNode, ref lastOpenListNode);
                        }
                        actionId++;
                        actionIndex++;
                    }
                } else {
                    actionId += IntSize;
                }
            }
            //adds the local open list to the full open list
            if (_openListFirstNode == NoNode) {
                _openListFirstNode = firstOpenListNode;
            } else {
                _openListFirstNode.Next = firstOpenListNode;
            }
            return true;
        }

        /// <summary>
        /// Extracts the first <see cref="EnforcedHillClimbingNode">node</see> of the open list.
        /// </summary>
        /// <returns>The first <see cref="EnforcedHillClimbingNode">node</see> of the open list</returns>
        private State ExtractFirstFromOpenList() {
            const State NoState = null;
            const EnforcedHillClimbingNode NoNode = null;
            State first = NoState;
            if (_openListFirstNode != NoNode) {
                first = _openListFirstNode.State;
                _openListFirstNode = _openListFirstNode.Next;
            }
            return first;
        }

        /// <summary>
        /// Searches a successor of a <see cref="State">state</see> which is better than it.
        /// </summary>
        /// <param name="state">The <see cref="State">state</see>.</param>
        /// <returns></returns>
        private State SearchForBetterState(State state) {
            const State NoState = null;
            const EnforcedHillClimbingNode NoNode = null;
            float successorEstimatedRemainingCost;
            State successor = NoState;
            _openListFirstNode = NoNode;
            if (AddSuccessorsToOpenList(state, false)) {
                successor = ExtractFirstFromOpenList();
                //for each successor in the open list
                while (successor != NoState) {
                    successorEstimatedRemainingCost =
                        _heuristicCalculator.GetHeuristicCost(successor, out _relaxedPlanActionList);
                    if (successorEstimatedRemainingCost < _estimatedRemainingCost) {
                        //if the successor is better than state, returns it
                        _estimatedRemainingCost = successorEstimatedRemainingCost;
                        return successor;
                    } else if (!Single.IsPositiveInfinity(successorEstimatedRemainingCost)) {
                        //if the successor is worst than state, adds its successors to the open list
                        AddSuccessorsToOpenList(successor, true);
                    }
                    successor = ExtractFirstFromOpenList();
                }
            }
            return NoState;
        }

        /// <summary>
        /// Executes the algorithm.
        /// </summary>
        /// <returns>The result of the search.</returns>
        /// <exception cref="ResultNotFoundException">No result is found.</exception>
        public State Search() {
            const State NoState = null;
            //If no goals exist, a result is never found because all relaxed plans are zero-length
            if (_goalCount == 0) {
                throw new ResultNotFoundException();
            }
            State currentState = _initialStateProvider.GetInitialState();
            State successorState;
            _visitedStates.Clear();
            //adds the state to the visited ones
            _visitedStates.Add(currentState, currentState);
            try {
                //while not goal found
                if (!_goalVerifier.IsGoal(currentState)) {
                    _estimatedRemainingCost =
                        _heuristicCalculator.GetHeuristicCost(currentState, out _relaxedPlanActionList);
                    do {
                        _useHelpfulActionsOnly = true;
                        //searches for a better state using only the "helpful actions"
                        successorState = SearchForBetterState(currentState);
                        //if no better state is found
                        if (successorState == NoState) {
                            _useHelpfulActionsOnly = false;
                            //searches for a better state using all the actions
                            successorState = SearchForBetterState(currentState);
                        }
                        //if no better state is found, no result is found
                        if (successorState == NoState) {
                            throw new ResultNotFoundException();
                        }
                        //the better state is now the current one
                        currentState = successorState;
                    } while (!_goalVerifier.IsGoal(currentState));
                }
            } catch (ResultNotFoundException) {
                throw;
            } catch (OutOfMemoryException exception) {
                //any exception causes a fruitless search
                throw new ResultNotFoundException(exception);
            }
            return currentState;
        }

        #endregion

    }

}
