/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Searching;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents a <see cref="IStateCostProvider{T}">cost provider</see> which calculates the cost of
    /// reaching a <see cref="State">state</see> from the initial one by applying a
    /// <see cref="IFunction">function</see> over it.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class FunctionalStateCostProvider : IStateCostProvider<State> {

        #region Fields

        /// <summary>
        /// The <see cref="IFunction">function</see> to apply over a <see cref="State">state</see> in order to
        /// get the cost of reaching it from the initial one.
        /// </summary>
        private readonly IFunction _function;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FunctionalStateCostProvider"/> class.
        /// </summary>
        /// <param name="function">The <see cref="IFunction">function</see> to apply over a
        /// <see cref="State">state</see> in order to get the cost of reaching it from the initial one.</param>
        /// <remarks>Assumes the correction of <paramref name="function"/>, as a performance improvement.
        /// If needed, correction checks must be done previously to the call of this constructor.
        /// </remarks>
        public FunctionalStateCostProvider(IFunction function) {
            _function = function;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the cost of reaching the specified state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see> from the initial one.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>
        /// The cost of reaching <paramref name="state"/> from the initial state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see>.
        /// </returns>
        public float GetCost(State state) {
            return _function.GetResultIn(state);
        }

        #endregion

    }

}
