/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Searching;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents a <see cref="IGoalStateVerifier{T}">goal state verifier</see> which considers a
    /// <see cref="State">state</see> as a goal when a break <see cref="ICondition">condition</see> is true or
    /// all goal facts are true in <see cref="State.Facts">the state fact certainty list</see> and a goal
    /// <see cref="ICondition">condition</see> is also true.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class GoalStateVerifier : IGoalStateVerifier<State> {

        #region Constants

        /// <summary>
        /// The constant which represents no break <see cref="ICondition">condition</see>.
        /// </summary>
        public const ICondition NoBreakCondition = null;

        /// <summary>
        /// The constant which represents no goal <see cref="State.Facts">facts</see>.
        /// </summary>
        public static readonly int[] NoGoalFacts = new int[0];

        /// <summary>
        /// The constant which represents no goal <see cref="ICondition">condition</see>.
        /// </summary>
        public const ICondition NoGoalCondition = null;

        #endregion

        #region Fields

        /// <summary>
        /// The break <see cref="ICondition">condition</see>.
        /// </summary>
        private readonly ICondition _breakCondition;

        /// <summary>
        /// The goal <see cref="State.Facts">facts</see>.
        /// </summary>
        private readonly int[] _goalFacts;

        /// <summary>
        /// The goal <see cref="ICondition">condition</see>.
        /// </summary>
        private readonly ICondition _goalCondition;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="GoalStateVerifier"/> class.
        /// </summary>
        /// <param name="breakCondition">The break <see cref="ICondition">condition</see>.</param>
        /// <param name="goalFacts">The goal <see cref="State.Facts">facts</see>.</param>
        /// <param name="goalCondition">The goal <see cref="ICondition">condition</see>.</param>
        /// <remarks>
        /// Assumes that <paramref name="goalFacts"/> length is less than or equal to
        /// <see cref="State.Facts">the state fact certainty list</see> length and does no checks to verify this
        /// condition, as a performance improvement. If needed, correction checks must be done previously to
        /// the call of this constructor.
        /// </remarks>
        public GoalStateVerifier(ICondition breakCondition, int[] goalFacts, ICondition goalCondition) {
            _breakCondition = breakCondition;
            _goalFacts = goalFacts;
            _goalCondition = goalCondition;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GoalStateVerifier"/> class.
        /// </summary>
        /// <param name="goalFacts">The goal <see cref="State.Facts">facts</see>.</param>
        /// <param name="goalCondition">The goal <see cref="ICondition">condition</see>.</param>
        /// <remarks>
        /// <para>
        /// <see cref="NoBreakCondition">No break condition</see> is set.
        /// </para>
        /// <para>
        /// Assumes that <paramref name="goalFacts"/> length is less than or equal to
        /// <see cref="State.Facts">the state fact certainty list</see> length and does no checks to verify this
        /// condition, as a performance improvement. If needed, correction checks must be done previously to
        /// the call of this constructor.
        /// </para>
        /// </remarks>
        public GoalStateVerifier(int[] goalFacts, ICondition goalCondition)
            : this(NoBreakCondition, goalFacts, goalCondition) {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GoalStateVerifier"/> class.
        /// </summary>
        /// <param name="breakCondition">The break <see cref="ICondition">condition</see>.</param>
        /// <param name="goalFacts">The goal <see cref="State.Facts">facts</see>.</param>
        /// <remarks>
        /// <para>
        /// <see cref="NoGoalCondition">No goal condition</see> is set.
        /// </para>
        /// <para>
        /// Assumes that <paramref name="goalFacts"/> length is less than or equal to
        /// <see cref="State.Facts">the state fact certainty list</see> length and does no checks to verify this
        /// condition, as a performance improvement. If needed, correction checks must be done previously to
        /// the call of this constructor.
        /// </para>
        /// </remarks>
        public GoalStateVerifier(ICondition breakCondition, int[] goalFacts)
            : this(breakCondition, goalFacts, NoGoalCondition) {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GoalStateVerifier"/> class.
        /// </summary>
        /// <param name="goalFacts">The goal <see cref="State.Facts">facts</see>.</param>
        /// <remarks>
        /// <para>
        /// <see cref="NoBreakCondition">No break condition</see> and
        /// <see cref="NoGoalCondition">no goal condition</see> are set.
        /// </para>
        /// <para>
        /// Assumes that <paramref name="goalFacts"/> length is less than or equal to
        /// <see cref="State.Facts">the state fact certainty list</see> length and does no checks to verify this
        /// condition, as a performance improvement. If needed, correction checks must be done previously to
        /// the call of this constructor.
        /// </para>
        /// </remarks>
        public GoalStateVerifier(int[] goalFacts)
            : this(NoBreakCondition, goalFacts, NoGoalCondition) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether the specified state is a goal for the
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see>.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>
        /// True if <paramref name="state"/> is a goal for the
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see>; otherwise, false.
        /// </returns>
        public bool IsGoal(State state) {
            //break condition checking
            if ((_breakCondition != NoBreakCondition) && _breakCondition.IsTrueIn(state)) {
                return true;
            }
            //fact list intersecion checking
            int[] stateFacts = state.Facts;
            int sectionIndex = 0;
            foreach (int goalFactsSection in _goalFacts) {
                if ((goalFactsSection & stateFacts[sectionIndex]) != goalFactsSection) {
                    return false;
                }
                sectionIndex++;
            }
            //goal condition checking
            return (_goalCondition == NoGoalCondition) || _goalCondition.IsTrueIn(state);
        }

        #endregion

    }

}
