/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Searching;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents an <see cref="IStateHeuristicCostProvider{T}">heuristic cost provider</see> which
    /// returns the cost of the plan provided by the <see cref="RelaxedPlanner">relaxed planner</see> for
    /// reaching the specified <see cref="State">state</see>.
    /// </summary>
    /// <remarks>
    /// The implentation of this heuristic is based on the explanations of the paper titled
    /// "The FF Planning System: Fast Plan Generation Through Heuristic Search", written by Jrg Hoffmann and
    /// Bernhard Nebel and published by the Journal of Artificial Intelligence Research in 2001.
    /// </remarks>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class RelaxedPlanHeuristic : IStateHeuristicCostProvider<State> {

        #region Fields

        /// <summary>
        /// The <see cref="RelaxedPlanner">relaxed planner</see>.
        /// </summary>
        private readonly RelaxedPlanner _relaxedPlanner;

        /// <summary>
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </summary>
        private readonly IStateCostProvider<State> _costCalculator;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelaxedPlanHeuristic"/> class.
        /// </summary>
        /// <param name="relaxedPlanner">The <see cref="RelaxedPlanner">relaxed planner</see>.</param>
        /// <param name="costCalculator">The <see cref="IStateCostProvider{T}">cost calculator</see>.</param>
        /// <remarks>
        /// Assumes the correction of both <paramref name="relaxedPlanner"/> and
        /// <paramref name="costCalculator"/>, as a performance improvement. If needed, correction checks must
        /// be done previously to the call of this constructor.
        /// </remarks>
        public RelaxedPlanHeuristic(RelaxedPlanner relaxedPlanner, IStateCostProvider<State> costCalculator) {
            _relaxedPlanner = relaxedPlanner;
            _costCalculator = costCalculator;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the estimated cost of reaching a goal state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see> from the specified one.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <param name="planActionList">
        /// <para>The plan action list.</para>
        /// <para>It is a bit array with the same format used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// <para>Each bit of the array represents the presence state of the corresponding
        /// <see cref="Action">action</see> in the plan. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is present in the plan.</para>
        /// <para>It is shared by all the calls to this method, as a performance improvement. If needed, cloning
        /// tasks must be done after calling the method in order to avoid problems derived from the sharing.</para>
        /// </param>
        /// <remarks>Assumes the correction of <paramref name="state"/>, as a performance improvement. If
        /// needed, correction checks must be done previously to the call of this method.</remarks>
        /// <returns>
        /// The estimated cost of reaching a goal state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see> from <paramref name="state"/>.
        /// </returns>
        public float GetHeuristicCost(State state, out int[] planActionList) {
            const IActionEffect NoEffect = null;
            //gets the relaxed plan
            RelaxedPlannerResult planningResult = _relaxedPlanner.Plan(state);
            planActionList = planningResult.PlanActionList;
            //if no plan is found, the heuristic is infinity
            if (!planningResult.PlanFound) {
                return Single.PositiveInfinity;
            }
            //if a zero-length plan is found, the heuristic is zero
            if (planningResult.PlanLength == 0) {
                return 0;
            }
            //else, calculates the cost of the current state
            float currentCost = _costCalculator.GetCost(state);
            //calculates the final state by applying the relaxed plan to the current one
            State resultState = new State(state.Parent, State.NoAppliedAction, State.NoFacts,
                (float[])state.Variables.Clone());
            resultState.PlanLength = state.PlanLength + planningResult.PlanLength;
            RelaxedPlanNode planNode = planningResult.FirstPlanNode;
            Action action;
            IActionEffect effect;
            while (planNode != RelaxedPlanNode.None) {
                action = planNode.Action;
                effect = action.Effect;
                if (effect != NoEffect) {
                    effect.ApplyTo(resultState, action);
                }
                planNode = planNode.Next;
            }
            //the heuristic is the cost difference between the current state and the final one
            return _costCalculator.GetCost(resultState) - currentCost;
        }

        /// <summary>
        /// Gets the estimated cost of reaching a goal state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see> from the specified one.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>
        /// The estimated cost of reaching a goal state of a
        /// <see cref="ISearchAlgorithm{T}">search algorithm</see> from <paramref name="state"/>.
        /// </returns>
        public float GetHeuristicCost(State state) {
            int[] planActionList;
            return GetHeuristicCost(state, out planActionList);
        }

        #endregion

    }

}
