/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Core;
using Aran.Searching;

namespace Aran.Searching.Specialized {

    /// <summary>
    /// Represents a <see cref="IStateSuccessorsProvider{T}">successors provider</see> which provides
    /// the successors of a <see cref="State">state</see> by applying the <see cref="Action">action</see>s
    /// selected by an <see cref="ActionSelector">action selector</see>.
    /// </summary>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class StateSuccessorsProvider : IStateSuccessorsProvider<State> {

        #region Fields

        /// <summary>
        /// The <see cref="Action">action</see>s.
        /// </summary>
        private readonly Action[] _actions;

        /// <summary>
        /// The <see cref="ActionSelector">action selector</see>.
        /// </summary>
        private readonly ActionSelector _actionSelector;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="StateSuccessorsProvider"/> class.
        /// </summary>
        /// <param name="actions">The <see cref="Action">action</see>s.</param>
        /// <param name="actionSelector">The <see cref="ActionSelector">action selector</see>.</param>
        /// <remarks>
        /// Assumes the correction of both <paramref name="actions"/> and <paramref name="actionSelector"/>,
        /// as a performance improvement. If needed, correction checks must be done previously to the call of
        /// this constructor.
        /// </remarks>
        public StateSuccessorsProvider(Action[] actions, ActionSelector actionSelector) {
            _actions = actions;
            _actionSelector = actionSelector;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the successors of a specified state of a <see cref="ISearchAlgorithm{T}">search algorithm</see>.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>
        /// The successors of <paramref name="state"/>.
        /// </returns>
        public IEnumerable<State> GetSuccessors(State state) {
            int[] actionSelectionList = _actionSelector.GetActionSelectionList(state);
            return new ActionGeneratedStateEnumerable(state, _actions, actionSelectionList);
        }

        #endregion

    }

}
