/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents the Beam Search algorithm.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class BeamSearchAlgorithm<T> : ISearchAlgorithm<T> {

        #region Fields

        /// <summary>
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </summary>
        private readonly IInitialStateProvider<T> _initialStateProvider;

        /// <summary>
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </summary>
        private readonly IGoalStateVerifier<T> _goalVerifier;

        /// <summary>
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </summary>
        private readonly IStateSuccessorsProvider<T> _successorsGenerator;

        /// <summary>
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </summary>
        private readonly IStateHeuristicCostProvider<T> _heuristicCalculator;

        /// <summary>
        /// The state <see cref="IEqualityComparer{T}">equality comparer</see>.
        /// </summary>
        private readonly IEqualityComparer<T> _equalityComparer;

        /// <summary>
        /// The beam width.
        /// </summary>
        private int _beamWidth;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="beamWidth">The beam width.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="heuristicCalculator"/> or <paramref name="equalityComparer"/> is a null reference.</exception>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="beamWidth"/> is less than
        /// <see cref="BeamSearchAlgorithmValue.MinimumBeamWidth">the minimum one</see>.</exception>
        public BeamSearchAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator, int beamWidth,
        IEqualityComparer<T> equalityComparer) {
            if (initialStateProvider == null) {
                throw new ArgumentNullException("initialStateProvider");
            }
            if (goalVerifier == null) {
                throw new ArgumentNullException("goalVerifier");
            }
            if (successorsGenerator == null) {
                throw new ArgumentNullException("successorsGenerator");
            }
            if (heuristicCalculator == null) {
                throw new ArgumentNullException("heuristicCalculator");
            }
            if (beamWidth < BeamSearchAlgorithmValue.MinimumBeamWidth) {
                throw new ArgumentOutOfRangeException("beamWidth");
            }
            if (equalityComparer == null) {
                throw new ArgumentNullException("equalityComparer");
            }
            _initialStateProvider = initialStateProvider;
            _goalVerifier = goalVerifier;
            _successorsGenerator = successorsGenerator;
            _heuristicCalculator = heuristicCalculator;
            _beamWidth = beamWidth;
            _equalityComparer = equalityComparer;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="beamWidth">The beam width.</param>
        /// <remarks>
        /// Uses the <see cref="EqualityComparer{T}.Default">default state equality comparer</see>.
        /// </remarks>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/> or
        /// <paramref name="heuristicCalculator"/> is a null reference.</exception>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="beamWidth"/> is less than
        /// <see cref="BeamSearchAlgorithmValue.MinimumBeamWidth">the minimum one</see>.</exception>
        public BeamSearchAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator, int beamWidth)
            : this(initialStateProvider, goalVerifier, successorsGenerator, heuristicCalculator,
            beamWidth, EqualityComparer<T>.Default) {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <remarks>
        /// <para>The beam width is set to
        /// <see cref="BeamSearchAlgorithmValue.DefaultBeamWidth">the default one</see>.</para>
        /// <para>Uses the <see cref="EqualityComparer{T}.Default">default state equality comparer</see>.</para>
        /// </remarks>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/> or
        /// <paramref name="heuristicCalculator"/> is a null reference.</exception>
        public BeamSearchAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator)
            : this(initialStateProvider, goalVerifier, successorsGenerator, heuristicCalculator,
            BeamSearchAlgorithmValue.DefaultBeamWidth, EqualityComparer<T>.Default) {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <remarks>
        /// The beam width is set to
        /// <see cref="BeamSearchAlgorithmValue.DefaultBeamWidth">the default one</see>.
        /// </remarks>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="heuristicCalculator"/> or <paramref name="equalityComparer"/> is a null reference.</exception>
        public BeamSearchAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator, IEqualityComparer<T> equalityComparer)
            : this(initialStateProvider, goalVerifier, successorsGenerator, heuristicCalculator,
            BeamSearchAlgorithmValue.DefaultBeamWidth, equalityComparer) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Executes the algorithm.
        /// </summary>
        /// <returns>The result of the search.</returns>
        /// <exception cref="ResultNotFoundException">No result is found.</exception>
        public T Search() {
            T state = _initialStateProvider.GetInitialState();
            //if the initial state is a goal, it is the result
            if (_goalVerifier.IsGoal(state)) {
                return state;
            }
            IDictionary<T, T> visitedStates = new Dictionary<T, T>(_equalityComparer);
            List<BeamSearchNode<T>> openList = new List<BeamSearchNode<T>>();
            List<BeamSearchNode<T>> newOpenList = new List<BeamSearchNode<T>>();
            List<BeamSearchNode<T>> auxList;
            int openListCount;
            //adds the initial state to the open list
            openList.Add(new BeamSearchNode<T>(state));
            //marks the initial state as visited
            visitedStates.Add(state, state);
            try {
                //while open list has states
                while (openList.Count > 0) {
                    //substitutes all open list states by their successors
                    foreach (BeamSearchNode<T> stateNode in openList) {
                        state = stateNode.State;
                        //generates all successors for the current state
                        foreach (T successor in _successorsGenerator.GetSuccessors(state)) {
                            if (!visitedStates.ContainsKey(successor)) {
                                //marks current as visited
                                visitedStates.Add(successor, successor);
                                //if any not already visited successor is a goal, it is the result
                                if (_goalVerifier.IsGoal(successor)) {
                                    return successor;
                                }
                                newOpenList.Add(new BeamSearchNode<T>(successor,
                                    _heuristicCalculator.GetHeuristicCost(successor)));
                            }
                        }
                    }
                    openList.Clear();
                    //filters the open list in order to get the b best ones, where b is the beam width
                    newOpenList.Sort();
                    openListCount = newOpenList.Count;
                    if (openListCount > _beamWidth) {
                        newOpenList.RemoveRange(_beamWidth, openListCount - _beamWidth);
                    }
                    auxList = newOpenList;
                    newOpenList = openList;
                    openList = auxList;
                }
            } catch (OutOfMemoryException exception) {
                //any exception causes a fruitless search
                throw new ResultNotFoundException(exception);
            }
            //if no evaluated state is a goal, no result is found
            throw new ResultNotFoundException();
        }

        #endregion

    }

}
