/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Searching {

    /// <summary>
    /// Represents a <see cref="BeamSearchAlgorithm{T}">Beam Search algorithm</see> node.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [Serializable]
    internal sealed class BeamSearchNode<T> : IComparable<BeamSearchNode<T>> {

        #region Fields

        /// <summary>
        /// The state contained by the node.
        /// </summary>
        internal readonly T State;

        /// <summary>
        /// The estimated cost of reaching a goal state from <see cref="State"/>.
        /// </summary>
        private readonly float _estimatedRemainingCost;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchNode{T}"/> class.
        /// </summary>
        /// <param name="state">The state.</param>
        internal BeamSearchNode(T state) {
            State = state;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BeamSearchNode{T}"/> class.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <param name="estimatedRemainingCost">The estimated cost of reaching a goal state from
        /// <paramref name="state"/>.</param>
        internal BeamSearchNode(T state, float estimatedRemainingCost) {
            State = state;
            _estimatedRemainingCost = estimatedRemainingCost;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Compares this node with another.
        /// </summary>
        /// <param name="other">A node to compare with this.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the nodes being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This node is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This node is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This node is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        public int CompareTo(BeamSearchNode<T> other) {
            float estimatedRemainingCostDifference = _estimatedRemainingCost - other._estimatedRemainingCost;
            if (estimatedRemainingCostDifference > 0) {
                return 1;
            }else if (estimatedRemainingCostDifference < 0) {
                return -1;
            }
            return 0;
        }

        #endregion

    }

}
