/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents the Best First algorithm.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    /// <typeparam name="N">The type of the nodes.</typeparam>
    [Serializable]
    internal class BestFirstAlgorithm<T, N> : ISearchAlgorithm<T>
        where T : IEquatable<T>
        where N : BestFirstNode<T, N>, new() {

        #region Fields

        /// <summary>
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </summary>
        private readonly IInitialStateProvider<T> _initialStateProvider;

        /// <summary>
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </summary>
        private readonly IGoalStateVerifier<T> _goalVerifier;

        /// <summary>
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </summary>
        private readonly IStateSuccessorsProvider<T> _successorsGenerator;

        /// <summary>
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </summary>
        private readonly IStateCostProvider<T> _costCalculator;

        /// <summary>
        /// The <see cref="IStateParentSetter{T}">parent setter</see>.
        /// </summary>
        private readonly IStateParentSetter<T> _parentSetter;

        /// <summary>
        /// The state <see cref="IEqualityComparer{T}">equality comparer</see>.
        /// </summary>
        private readonly IEqualityComparer<T> _equalityComparer;

        /// <summary>
        /// A node in open list which is equal to the last visited.
        /// </summary>
        protected N OpenListNodeSucessor;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BestFirstAlgorithm{T, N}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="costCalculator">
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </param>
        /// <param name="parentSetter">The <see cref="IStateParentSetter{T}">parent setter</see>.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="costCalculator"/>, <paramref name="parentSetter"/> or
        /// <paramref name="equalityComparer"/> is a null reference.</exception>
        internal BestFirstAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateCostProvider<T> costCalculator, IStateParentSetter<T> parentSetter,
        IEqualityComparer<T> equalityComparer) {
            if (initialStateProvider == null) {
                throw new ArgumentNullException("initialStateProvider");
            }
            if (goalVerifier == null) {
                throw new ArgumentNullException("goalVerifier");
            }
            if (successorsGenerator == null) {
                throw new ArgumentNullException("successorsGenerator");
            }
            if (costCalculator == null) {
                throw new ArgumentNullException("costCalculator");
            }
            if (parentSetter == null) {
                throw new ArgumentNullException("parentSetter");
            }
            if (equalityComparer == null) {
                throw new ArgumentNullException("equalityComparer");
            }
            _initialStateProvider = initialStateProvider;
            _goalVerifier = goalVerifier;
            _successorsGenerator = successorsGenerator;
            _costCalculator = costCalculator;
            _parentSetter = parentSetter;
            _equalityComparer = equalityComparer;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Creates a new node.
        /// </summary>
        /// <param name="state">The state of the node.</param>
        /// <returns>The new node.</returns>
        private N CreateNode(T state) {
            N node = new N();
            node.State = state;
            node.EqualityComparer = _equalityComparer;
            return node;
        }

        /// <summary>
        /// Sets the cost of a node.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="cost">The cost.</param>
        protected virtual void SetCost(N node, float cost) {
            node.Cost = cost;
        }

        /// <summary>
        /// Updates the cost of a node.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="cost">The cost.</param>
        protected virtual void UpdateCost(N node, float cost) {
            node.Cost = cost;
        }

        /// <summary>
        /// Executes the algorithm.
        /// </summary>
        /// <returns>The result of the search.</returns>
        /// <exception cref="ResultNotFoundException">No result is found.</exception>
        public T Search() {
            N nodeSuccessor;
            IEnumerable<T> stateSuccessors;
            float nodeSuccessorCost;
            IDictionary<T, T> closedList = new Dictionary<T, T>(_equalityComparer);
            BestFirstQueue<T, N> openList = new BestFirstQueue<T, N>(_equalityComparer);
            IDictionary<T, N> openListNodes = openList.Nodes;
            _parentSetter.Reset();
            T currentState = _initialStateProvider.GetInitialState();
            N currentNode = CreateNode(currentState);
            openList.Enqueue(currentNode);
            //while there are states to visit
            while (openList.Count > 0) {
                currentNode = openList.Dequeue();
                //the new current state is the best so far
                currentState = currentNode.State;
                //if the current state is the goal state
                if (_goalVerifier.IsGoal(currentState)) {
                    //returns the current state as result
                    return currentState;
                }
                //if the current state is not the goal state
                //moves the current state to closed list
                closedList.Add(currentState, currentState);
                stateSuccessors = _successorsGenerator.GetSuccessors(currentState);
                //for each current state successor
                foreach (T stateSuccessor in stateSuccessors) {
                    if (!closedList.ContainsKey(stateSuccessor)) { //if the successor has not been visited
                        nodeSuccessorCost = _costCalculator.GetCost(stateSuccessor);
                        if (openListNodes.TryGetValue(stateSuccessor, out OpenListNodeSucessor)) { //if open list contains the successor
                            //if successor is better than the one contained in open list
                            if (nodeSuccessorCost < OpenListNodeSucessor.Cost) {
                                //assigns the current state as the parent of the open list successor
                                _parentSetter.SetParent(OpenListNodeSucessor.State, currentState);
                                //recalculates the open list successor costs
                                UpdateCost(OpenListNodeSucessor, nodeSuccessorCost);
                                openList.Update(OpenListNodeSucessor);
                            }
                        } else { //if open list not contains the successor
                            nodeSuccessor = CreateNode(stateSuccessor);
                            //calculates the successor costs
                            SetCost(nodeSuccessor, nodeSuccessorCost);
                            openList.Enqueue(nodeSuccessor);
                        }
                    }
                }
            }
            //if no state is a goal, no result is found
            throw new ResultNotFoundException();
        }

        #endregion

    }

}
