/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents a <see cref="BestFirstAlgorithm{T, N}">Best First algorithm</see> node.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    /// <typeparam name="N">The type of the nodes.</typeparam>
    [Serializable]
    internal abstract class BestFirstNode<T, N> : IEquatable<N>, IComparable<N>
        where N : BestFirstNode<T, N>  {

        #region Fields

        /// <summary>
        /// The state contained by the node.
        /// </summary>
        internal T State;

        /// <summary>
        /// The state <see cref="System.Collections.Generic.IEqualityComparer{T}">equality comparer</see>.
        /// </summary>
        internal IEqualityComparer<T> EqualityComparer;

        /// <summary>
        /// The cost of reaching <see cref="State"/> from the initial state.
        /// </summary>
        internal float Cost;

        /// <summary>
        /// The current index of the node in the <see cref="BestFirstQueue{T, N}">open list</see>.
        /// </summary>
        internal int Index;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BestFirstNode{T, N}"/> class.
        /// </summary>
        protected internal BestFirstNode() {
            //do nothing
        }

        #endregion

        #region Methods

        /// <summary>
        /// Indicates whether this node is equal to another.
        /// </summary>
        /// <param name="other">A node to compare with this.</param>
        /// <returns>
        /// True if this node is equal to <paramref name="other"/>; otherwise, false.
        /// </returns>
        public bool Equals(N other) {
            return EqualityComparer.Equals(State, other.State);
        }

        /// <summary>
        /// Gets the cost difference from this node to other.
        /// </summary>
        /// <param name="other">The other node.</param>
        /// <returns>The cost difference from this node to <paramref name="other"/></returns>
        protected abstract float GetCostDifference(N other);

        /// <summary>
        /// Compares this node with another.
        /// </summary>
        /// <param name="other">A node to compare with this.</param>
        /// <returns>
        /// <para>A 32-bit signed integer that indicates the relative order of the nodes being compared.
        /// The return value has the following meanings:</para>
        /// <list type="table">
        /// 	<listheader>
        /// 		<term>Value</term>
        /// 		<description>Meaning</description>
        /// 	</listheader>
        /// 	<item>
        /// 		<term>Less than zero</term>
        /// 		<description>This node is less than <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Zero</term>
        /// 		<description>This node is equal to <paramref name="other"/>.</description>
        /// 	</item>
        /// 	<item>
        /// 		<term>Greater than zero</term>
        /// 		<description>This node is greater than <paramref name="other"/>.</description>
        /// 	</item>
        /// </list>
        /// </returns>
        public int CompareTo(N other) {
            float costDifference = GetCostDifference(other);
            if (costDifference > 0) {
                return 1;
            }else if (costDifference < 0) {
                return -1;
            }
            return 0;
        }

        #endregion

    }

}
