/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents a <see cref="BestFirstAlgorithm{T, N}">Best First algorithm</see> open list. 
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    /// <typeparam name="N">The type of the nodes.</typeparam>
    [Serializable]
    internal sealed class BestFirstQueue<T, N>
        where N : BestFirstNode<T, N> {

        #region Fields

        /// <summary>
        /// The heap of nodes.
        /// </summary>
        private N[] _heap;

        /// <summary>
        /// The nodes hashed by state.
        /// </summary>
        internal IDictionary<T, N> Nodes;

        /// <summary>
        /// The number of nodes in open list.
        /// </summary>
        internal int Count;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BestFirstQueue{T, N}"/> class.
        /// </summary>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        internal BestFirstQueue(IEqualityComparer<T> equalityComparer) {
            const int DefaultCapacity = 10;
            _heap = new N[DefaultCapacity];
            Nodes = new Dictionary<T, N>(equalityComparer);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Heapifies up the <see cref="_heap">node array</see> from a specified node.
        /// </summary>
        /// <param name="node">The node.</param>
        private void HeapifyUpFrom(N node) {
            const int FirstIndex = 0;
            N otherNode;
            int otherNodeIndex;
            int nodeIndex = node.Index;
            bool sorted = (nodeIndex == FirstIndex);
            while (!sorted) {
                otherNodeIndex = (nodeIndex - 1) / 2;
                otherNode = _heap[otherNodeIndex];
                sorted = (node.CompareTo(otherNode) >= 0);
                if (!sorted) {
                    _heap[nodeIndex] = otherNode;
                    otherNode.Index = nodeIndex;
                    _heap[otherNodeIndex] = node;
                    nodeIndex = otherNodeIndex;
                    sorted = (nodeIndex == FirstIndex);
                }
            }
            node.Index = nodeIndex;
        }

        /// <summary>
        /// Heapifies down the <see cref="_heap">node array</see> from a specified node.
        /// </summary>
        /// <param name="node">The node.</param>
        private void HeapifyDownFrom(N node) {
            N minorNode;
            N parentNode;
            int childIndexOffset;
            int leftChildIndex;
            int rightChildIndex;
            int parentIndex;
            int minorNodeIndex = node.Index;
            bool sorted = false;
            do {
                parentIndex = minorNodeIndex;
                childIndexOffset = (parentIndex * 2);
                leftChildIndex = childIndexOffset + 1;
                rightChildIndex = childIndexOffset + 2;
                if ((leftChildIndex < Count) &&
                (_heap[minorNodeIndex].CompareTo(_heap[leftChildIndex]) > 0)) {
                    minorNodeIndex = leftChildIndex;
                }
                if ((rightChildIndex < Count) &&
                (_heap[minorNodeIndex].CompareTo(_heap[rightChildIndex]) > 0)) {
                    minorNodeIndex = rightChildIndex;
                }
                sorted = (minorNodeIndex == parentIndex);
                if (!sorted) {
                    minorNode = _heap[minorNodeIndex];
                    parentNode = _heap[parentIndex];
                    _heap[minorNodeIndex] = parentNode;
                    parentNode.Index = minorNodeIndex;
                    _heap[parentIndex] = minorNode;
                    minorNode.Index = parentIndex;
                }
            } while (!sorted);
        }

        /// <summary>
        /// Updates the specified node.
        /// </summary>
        /// <param name="node">The node.</param>
        internal void Update(N node) {
            int previousIndex = node.Index;
            HeapifyUpFrom(node);
            if (node.Index >= previousIndex) {
                HeapifyDownFrom(node);
            }
        }

        /// <summary>
        /// Enqueues the specified node.
        /// </summary>
        /// <param name="node">The node.</param>
        internal void Enqueue(N node) {
            node.Index = Count;
            int nodesLength = _heap.Length;
            if (Count == nodesLength) {
                N[] newHeap = new N[nodesLength * 2];
                Array.Copy(_heap, 0, newHeap, 0, nodesLength);
                _heap = newHeap;
            }
            _heap[Count++] = node;
            Nodes.Add(node.State, node);
            HeapifyUpFrom(node);
        }

        /// <summary>
        /// Dequeues the root node.
        /// </summary>
        /// <returns>The root node</returns>
        internal N Dequeue() {
            const int FirstIndex = 0;
            N firstNode = _heap[FirstIndex];
            Nodes.Remove(firstNode.State);
            Count--;
            N newFirstNode = _heap[Count];
            _heap[FirstIndex] = newFirstNode;
            newFirstNode.Index = FirstIndex;
            _heap[Count] = default(N);
            HeapifyDownFrom(newFirstNode);
            return firstNode;
        }

        #endregion

    }

}
