/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents the Dijkstra algorithm.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class DijkstraAlgorithm<T> : ISearchAlgorithm<T>
        where T : IEquatable<T> {

        #region Fields

        /// <summary>
        /// The internal algorithm implementation.
        /// </summary>
        private readonly BestFirstAlgorithm<T, DijkstraNode<T>> _internalAlgorithm;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DijkstraAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="costCalculator">
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </param>
        /// <param name="parentSetter">The <see cref="IStateParentSetter{T}">parent setter</see>.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="costCalculator"/>, <paramref name="parentSetter"/> or
        /// <paramref name="equalityComparer"/> is a null reference.</exception>
        public DijkstraAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateCostProvider<T> costCalculator, IStateParentSetter<T> parentSetter,
        IEqualityComparer<T> equalityComparer) {
            try {
                _internalAlgorithm = new BestFirstAlgorithm<T, DijkstraNode<T>>(initialStateProvider,
                    goalVerifier, successorsGenerator, costCalculator, parentSetter, equalityComparer);
            } catch (ArgumentNullException exception) {
                //HACK: the exception is rethrown in order to hide internal implementation stack details
                throw exception;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DijkstraAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="costCalculator">
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </param>
        /// <param name="parentSetter">The <see cref="IStateParentSetter{T}">parent setter</see>.</param>
        /// <remarks>
        /// Uses the <see cref="EqualityComparer{T}.Default">default state equality comparer</see>.
        /// </remarks>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="costCalculator"/> or <paramref name="parentSetter"/> is a null reference.</exception>
        public DijkstraAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateCostProvider<T> costCalculator, IStateParentSetter<T> parentSetter)
            : this(initialStateProvider, goalVerifier, successorsGenerator, costCalculator, parentSetter,
            EqualityComparer<T>.Default) {
            //do nothing
        }

        #endregion

        #region Methods

        /// <summary>
        /// Executes the algorithm.
        /// </summary>
        /// <returns>The result of the search.</returns>
        /// <exception cref="ResultNotFoundException">No result is found.</exception>
        public T Search() {
            try {
                return _internalAlgorithm.Search();
            } catch (ResultNotFoundException exception) {
                //HACK: the exception is rethrown in order to hide internal implementation stack details
                throw exception;
            } catch (OutOfMemoryException exception) {
                //any exception causes a fruitless search
                throw new ResultNotFoundException(exception);
            }
        }

        #endregion

    }

}
