/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents the Hill Climbing algorithm.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class HillClimbingAlgorithm<T> : ISearchAlgorithm<T> {

        #region Fields

        /// <summary>
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </summary>
        private readonly IInitialStateProvider<T> _initialStateProvider;

        /// <summary>
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </summary>
        private readonly IGoalStateVerifier<T> _goalVerifier;

        /// <summary>
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </summary>
        private readonly IStateSuccessorsProvider<T> _successorsGenerator;

        /// <summary>
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </summary>
        private readonly IStateHeuristicCostProvider<T> _heuristicCalculator;

        /// <summary>
        /// The state <see cref="IEqualityComparer{T}">equality comparer</see>.
        /// </summary>
        private readonly IEqualityComparer<T> _equalityComparer;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="HillClimbingAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="heuristicCalculator"/> or <paramref name="equalityComparer"/> is a null reference.</exception>
        public HillClimbingAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator, IEqualityComparer<T> equalityComparer) {
            if (initialStateProvider == null) {
                throw new ArgumentNullException("initialStateProvider");
            }
            if (goalVerifier == null) {
                throw new ArgumentNullException("goalVerifier");
            }
            if (successorsGenerator == null) {
                throw new ArgumentNullException("successorsGenerator");
            }
            if (heuristicCalculator == null) {
                throw new ArgumentNullException("heuristicCalculator");
            }
            if (equalityComparer == null) {
                throw new ArgumentNullException("equalityComparer");
            }
            _initialStateProvider = initialStateProvider;
            _goalVerifier = goalVerifier;
            _successorsGenerator = successorsGenerator;
            _heuristicCalculator = heuristicCalculator;
            _equalityComparer = equalityComparer;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="HillClimbingAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <remarks>
        /// Uses the <see cref="EqualityComparer{T}.Default">default state equality comparer</see>.
        /// </remarks>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/> or
        /// <paramref name="heuristicCalculator"/> is a null reference.</exception>
        public HillClimbingAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateHeuristicCostProvider<T> heuristicCalculator)
            : this(initialStateProvider, goalVerifier, successorsGenerator, heuristicCalculator,
            EqualityComparer<T>.Default) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Executes the algorithm.
        /// </summary>
        /// <returns>The result of the search.</returns>
        /// <exception cref="ResultNotFoundException">No result is found.</exception>
        public T Search() {
            T state = _initialStateProvider.GetInitialState();
            IDictionary<T, T> visitedStates = new Dictionary<T, T>(_equalityComparer);
            float bestEvaluation = Single.PositiveInfinity;
            float evaluation;
            bool successorFound = true;
            try {
                //while not result found
                while (successorFound) {
                    //marks current state as visited
                    visitedStates.Add(state, state);
                    successorFound = false;
                    //generates all successors for the current state
                    foreach (T successor in _successorsGenerator.GetSuccessors(state)) {
                        //if any successor is a goal, it is the result
                        if (_goalVerifier.IsGoal(successor)) {
                            return successor;
                        }
                        //the next current state is the best not already visited successor
                        if (!visitedStates.ContainsKey(successor)) {
                            evaluation = _heuristicCalculator.GetHeuristicCost(successor);
                            if (!successorFound || (evaluation < bestEvaluation)) {
                                bestEvaluation = evaluation;
                                state = successor;
                                successorFound = true;
                            }
                        }
                    }
                }
            } catch (OutOfMemoryException exception) {
                //any exception causes a fruitless search
                throw new ResultNotFoundException(exception);
            }
            //if no evaluated state is a goal, no result is found
            throw new ResultNotFoundException();
        }

        #endregion

    }

}
