/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents an A* algorithm variation.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [Serializable]
    internal abstract class InternalAStarAlgorithm<T> : BestFirstAlgorithm<T, AStarNode<T>>
        where T : IEquatable<T> {

        #region Fields

        /// <summary>
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>
        /// </summary>
        protected readonly IStateHeuristicCostProvider<T> HeuristicCalculator;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InternalAStarAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="costCalculator">
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="parentSetter">The <see cref="IStateParentSetter{T}">parent setter</see>.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="costCalculator"/>, <paramref name="heuristicCalculator"/>,
        /// <paramref name="parentSetter"/> or <paramref name="equalityComparer"/> is a null reference.</exception>
        protected internal InternalAStarAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateCostProvider<T> costCalculator, IStateHeuristicCostProvider<T> heuristicCalculator,
        IStateParentSetter<T> parentSetter, IEqualityComparer<T> equalityComparer)
            : base(initialStateProvider, goalVerifier, successorsGenerator, costCalculator, parentSetter,
            equalityComparer) {
            if (heuristicCalculator == null) {
                throw new ArgumentNullException("heuristicCalculator");
            }
            HeuristicCalculator = heuristicCalculator;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the estimated cost of reaching a goal state from a specified one.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>The estimated cost of reaching a goal state from <paramref name="state"/></returns>
        protected abstract float GetEstimatedRemainingCost(T state);

        /// <summary>
        /// Sets the cost of a node.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="cost">The cost.</param>
        protected override void SetCost(AStarNode<T> node, float cost) {
            node.Cost = cost;
            float estimatedRemainingCost = GetEstimatedRemainingCost(node.State);
            node.EstimatedRemainingCost = estimatedRemainingCost;
            node.EstimatedTotalCost = cost + estimatedRemainingCost;
        }

        /// <summary>
        /// Updates the cost of a node.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="cost">The cost.</param>
        protected override void UpdateCost(AStarNode<T> node, float cost) {
            node.Cost = cost;
            node.EstimatedTotalCost = cost + OpenListNodeSucessor.EstimatedRemainingCost;
        }

        #endregion

    }

}
