/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Searching {

    /// <summary>
    /// Represents the weighted A* algorithm variation.
    /// </summary>
    /// <typeparam name="T">The type of the states.</typeparam>
    [Serializable]
    internal sealed class InternalWeightedAStarAlgorithm<T> : InternalAStarAlgorithm<T>
        where T : IEquatable<T> {

        #region Fields

        private float _heuristicWeight;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InternalWeightedAStarAlgorithm{T}"/> class.
        /// </summary>
        /// <param name="initialStateProvider">
        /// The <see cref="IInitialStateProvider{T}">initial state provider</see>.
        /// </param>
        /// <param name="goalVerifier">
        /// The <see cref="IGoalStateVerifier{T}">goal verifier</see>.
        /// </param>
        /// <param name="successorsGenerator">
        /// The <see cref="IStateSuccessorsProvider{T}">successors generator</see>.
        /// </param>
        /// <param name="costCalculator">
        /// The <see cref="IStateCostProvider{T}">cost calculator</see>.
        /// </param>
        /// <param name="heuristicCalculator">
        /// The <see cref="IStateHeuristicCostProvider{T}">heuristic calculator</see>.
        /// </param>
        /// <param name="parentSetter">The <see cref="IStateParentSetter{T}">parent setter</see>.</param>
        /// <param name="heuristicWeight">The heuristic weight.</param>
        /// <param name="equalityComparer">The state <see cref="IEqualityComparer{T}">equality comparer</see>.</param>
        /// <exception cref="ArgumentNullException"><paramref name="initialStateProvider"/>,
        /// <paramref name="goalVerifier"/>, <paramref name="successorsGenerator"/>,
        /// <paramref name="costCalculator"/>, <paramref name="heuristicCalculator"/>,
        /// <paramref name="parentSetter"/> or <paramref name="equalityComparer"/> is a null reference.</exception>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="heuristicWeight"/> is less than
        /// <see cref="WeightedAStarAlgorithmValue.MinimumHeuristicWeight">the minimum one</see>.</exception>
        internal InternalWeightedAStarAlgorithm(IInitialStateProvider<T> initialStateProvider,
        IGoalStateVerifier<T> goalVerifier, IStateSuccessorsProvider<T> successorsGenerator,
        IStateCostProvider<T> costCalculator, IStateHeuristicCostProvider<T> heuristicCalculator,
        IStateParentSetter<T> parentSetter, float heuristicWeight, IEqualityComparer<T> equalityComparer)
            : base(initialStateProvider, goalVerifier, successorsGenerator, costCalculator,
            heuristicCalculator, parentSetter, equalityComparer) {
            if (heuristicWeight < WeightedAStarAlgorithmValue.MinimumHeuristicWeight) {
                throw new ArgumentOutOfRangeException("heuristicWeight");
            }
            _heuristicWeight = heuristicWeight;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the estimated cost of reaching a goal state from a specified one.
        /// </summary>
        /// <param name="state">The state.</param>
        /// <returns>
        /// The estimated cost of reaching a goal state from <paramref name="state"/>
        /// </returns>
        protected override float GetEstimatedRemainingCost(T state) {
            return (_heuristicWeight * HeuristicCalculator.GetHeuristicCost(state));
        }

        #endregion

    }

}
