/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Runtime.Serialization;

namespace Aran.Searching {

	/// <summary>
	/// Represents an error occurred when no result is found by a
    /// <see cref="ISearchAlgorithm{T}">search algorithm</see>.
	/// </summary>
	[CLSCompliant(true)]
	[Serializable]
	public class ResultNotFoundException : Exception {

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="ResultNotFoundException">ResultNotFoundException</see>
        /// class.
		/// </summary>
		/// <remarks>
		/// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance to a
        /// system-supplied message that describes the error and takes into account the current system culture.</para>
		/// <para>The following table shows the initial property values for an instance of
		/// <see cref="ResultNotFoundException">ResultNotFoundException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
		///       <description>a null reference</description>
		///    </item>
		///    <item>
		///       <term>Message</term>
		///       <description>A system-supplied localized description.</description>
		///    </item>
		/// </list>
		/// </remarks>
		public ResultNotFoundException()
			: this(null as Exception) {
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="ResultNotFoundException">ResultNotFoundException</see> class
		/// with a specified error message. 
		/// </summary>
		/// <param name="message">The error message that explains the reason for the exception.</param>
		/// <remarks>
		/// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance using the
		/// <paramref name="message"/> parameter.</para>
		/// <para>The following table shows the initial property values for an instance of
		/// <see cref="ResultNotFoundException">ResultNotFoundException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
		///       <description>a null reference</description>
		///    </item>
		///    <item>
		///       <term>Message</term>
		///       <description>The error message string.</description>
		///    </item>
		/// </list>
		/// </remarks>
		public ResultNotFoundException(string message)
			: base(message, null) {
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="ResultNotFoundException">ResultNotFoundException</see> class
		/// with a specified error message and a reference to the inner exception that is the cause of this
		/// exception.
		/// </summary>
		/// <param name="message">The error message that explains the reason for the exception.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or
		/// a null reference if no inner exception is specified.</param>
		/// <remarks>
		/// An exception that is thrown as a direct result of a previous exception should include a reference to
		/// the previous exception in the <see cref="Exception.InnerException">InnerException</see> property.
		/// </remarks>
		public ResultNotFoundException(string message, Exception innerException)
			: base(message, innerException) {
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="ResultNotFoundException">ResultNotFoundException</see>
        /// class with a reference to the inner exception that is the cause of this exception.
		/// </summary>
		/// <param name="innerException">The exception that is the cause of the current exception, or
		/// a null reference if no inner exception is specified.</param>
		/// <remarks>
        /// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance to a
        /// system-supplied message that describes the error and takes into account the current system culture.</para>
		/// <para>The following table shows the initial property values for an instance of
		/// <see cref="ResultNotFoundException">ResultNotFoundException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
		///       <description>The inner exception.</description>
		///    </item>
		///    <item>
		///       <term>Message</term>
        ///       <description>A system-supplied localized description.</description>
		///    </item>
		/// </list>
		/// </remarks>
		public ResultNotFoundException(Exception innerException)
			: base(new Resources().NoResultFound, innerException) {
		}

#if !XBOX360

        /// <summary>
        /// Initializes a new instance of the <see cref="ResultNotFoundException">ResultNotFoundException</see>
        /// class with serialized data.
        /// </summary>
        /// <param name="info">The serialized object data about the exception being thrown.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        /// <remarks>
        /// <para>This constructor is called during deserialization to reconstitute the exception object
        /// transmitted over a stream.</para>
        /// <para>This constructor is not available for the Xbox 360.</para>
        /// </remarks>
        /// <exception cref="ArgumentNullException">The <paramref name="info"/> parameter is a null reference.</exception>
        /// <exception cref="SerializationException">The class name is a null reference or
        /// <see cref="Exception.HResult">HResult</see> is zero (0).</exception>
        protected ResultNotFoundException(SerializationInfo info, StreamingContext context)
            : base(info, context) {
        }

#endif

		#endregion

	}

}