/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="Instance{T}">instance</see> argument.
    /// </summary>
    /// <typeparam name="T">The type of the object identifier.</typeparam>
    [Serializable]
    internal sealed class Argument<T> : Usable {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The <see cref="Aran.Translation.TypeInfo{T}">type information</see>.
        /// </summary>
        internal readonly TypeInfo<T> TypeInfo;

        /// <summary>
        /// The current value.
        /// </summary>
        internal Object Value;

        /// <summary>
        /// The number values.
        /// </summary>
        private int _valueCount;

        /// <summary>
        /// The current value index.
        /// </summary>
        /// <remarks>It can take a value from zero to <see cref="_valueCount">number of values</see> - 1.</remarks>
        private int _valueIndex;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Argument{T}"/> class.
        /// </summary>
        /// <param name="typeInfo">The <see cref="Aran.Translation.TypeInfo{T}">type information</see>.</param>
        internal Argument(TypeInfo<T> typeInfo) {
            TypeInfo = typeInfo;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Argument{T}"/> class.
        /// </summary>
        /// <param name="typeInfo">The <see cref="Aran.Translation.TypeInfo{T}">type information</see>.</param>
        /// <param name="value">The initial value.</param>
        internal Argument(TypeInfo<T> typeInfo, Object value)
            : this(typeInfo) {
            Value = value;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Initializes the value to the first one.
        /// </summary>
        internal void InitValue() {
            Value = TypeInfo.Objects[_valueIndex];
            _valueCount = TypeInfo.Objects.Length;
        }

        /// <summary>
        /// Increases the value.
        /// </summary>
        /// <returns>True if the value has been cicled and, therefore, is the first one; false otherwise.</returns>
        internal bool IncreaseValue() {
            bool cicled = false;
            _valueIndex++;
            if (_valueIndex == _valueCount) {
                _valueIndex = 0;
                cicled = true;
            }
            Value = TypeInfo.Objects[_valueIndex];
            return cicled;
        }

        #endregion

    }

}
