/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents a fact, that is, a predicate <see cref="Signature{T}">signature</see> instance.
    /// </summary>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal sealed class Fact<T> : Instance<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The predicate information.
        /// </summary>
        internal readonly PredicateInfo<T> PredicateInfo;

        /// <summary>
        /// If is an operator precondition.
        /// </summary>
        private bool _isPrecondition;

        /// <summary>
        /// If is an operator addition.
        /// </summary>
        private bool _isAddition;

        /// <summary>
        /// If is an operator deletion.
        /// </summary>
        private bool _isDeletion;

        /// <summary>
        /// If is an initial state problem fact.
        /// </summary>
        private bool _isInitial;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Fact{T}"/> class.
        /// </summary>
        /// <param name="predicateInfo">The predicate information.</param>
        /// <param name="arguments">The argument list.</param>
        internal Fact(PredicateInfo<T> predicateInfo, params Argument<T>[] arguments)
            : base(predicateInfo.Signature, arguments, predicateInfo) {
            PredicateInfo = predicateInfo;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this fact is an initial state problem one.
        /// </summary>
        /// <value>true if this fact is an initial state problem one; otherwise, false.</value>
        internal bool IsInitial {
            get {
                return PredicateInfo.IsInitial(this);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds a <see cref="FactAction">fact action</see>.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The fact container (problem or operator) related translation messages.</param>
        /// <param name="source">The fact container (problem or operator).</param>
        /// <param name="position">The fact position into the container.</param>
        /// <param name="action">The fact action.</param>
        internal void AddAction<S>(MessageCollection<S> messages, S source, int position, FactAction action) {
            if (!Used) {
                Used = true; //this fact is now used
                UseInfo.Used = true; //the fact predicate is now used
                foreach (Argument<T> argument in Arguments) {
                    argument.Used = true; //the fact argument is now used
                }
            }
            switch (action) {
                case FactAction.Precondition:
                    //a precodition can not be present for two times
                    if (_isPrecondition) {
                        messages.AddSemanticError(source, MessageCode.FactPreconditionAlreadyPresent, position);
                    }
                    //a precodition can not be also an addition
                    if (_isAddition) {
                        messages.AddSemanticError(source, MessageCode.FactPreconditionAlreadyAdded, position);
                    }
                    _isPrecondition = !_isPrecondition && !_isAddition;
                    break;
                case FactAction.Addition:
                    //an addition can not be also a precodition
                    if (_isPrecondition) {
                        messages.AddSemanticError(source, MessageCode.FactAdditionAlreadyRequired, position);
                    }
                    //an addition can not be present for two times
                    if (_isAddition) {
                        messages.AddSemanticError(source, MessageCode.FactAdditionAlreadyPresent, position);
                    }
                    if (!_isPrecondition && !_isAddition) {
                        _isAddition = true;
                        //updates the number of additions of the same predicate
                        PredicateInfo.AdditionCount++;
                    }
                    break;
                case FactAction.Deletion:
                    //a deletion can not be present for two times
                    if (_isDeletion) {
                        messages.AddSemanticError(source, MessageCode.FactDeletionAlreadyPresent, position);
                    }
                    //a deletion can not be also an addition
                    if (_isAddition) {
                        messages.AddSemanticError(source, MessageCode.FactDeletionAlreadyAdded, position);
                    }
                    if (!_isDeletion && !_isAddition) {
                        _isDeletion = true;
                        //updates the number of deletions of the same predicate
                        PredicateInfo.DeletionCount++;
                    }
                    break;
                case FactAction.Initialization:
                    if (_isInitial) {
                        //an initial fact can not be present for two times
                        messages.AddSemanticError(source, MessageCode.InitialFactAlreadyPresent, position);
                    } else {
                        _isInitial = true;
                        //updates the initial fact collection of the same predicate
                        PredicateInfo.InitialFacts.Add(this);
                    }
                    break;
            }
        }

        #endregion

    }

}