/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Security.Permissions;
using System.Runtime.Serialization;

namespace Aran.Translation {

	/// <summary>
    /// Represents an error occurred while generating the
    /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">translation result</see>.
    /// </summary>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
	[CLSCompliant(true)]
	[Serializable]
	public sealed class GenerationException<TType, TPred, TFunc, TOp, TObj, TFact, TVar> : Exception {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constants

        /// <summary>
        /// The messages serialization name.
        /// </summary>
        private const string MessagesName = "messages";

        #endregion

        #region Fields

        /// <summary>
        /// The messages reported by the
        /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see> generator.
        /// </summary>
        private readonly MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">GenerationException</see> class.
		/// </summary>
        /// <remarks>
        /// <param name="message">The message that describes the error.</param>
        /// <param name="messages">The messages reported by the
        /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see> generator.</param>
        /// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance using
        /// the <paramref name="message"/> parameter.</para>
		/// <para>The following table shows the initial property values for an instance of
        /// <see cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">GenerationException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
		///       <description>A null reference.</description>
		///    </item>
		///    <item>
		///       <term>Message</term>
        ///       <description>The error message string.</description>
        ///    </item>
        ///    <item>
        ///       <term>Messages</term>
        ///       <description>The messages reported by the
        ///       <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see> generator.</description>
        ///    </item>
		/// </list>
		/// </remarks>
        internal GenerationException(string message,
        MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> messages)
            : this(message, null, messages) {
		}

		/// <summary>
        /// Initializes a new instance of the
        /// <see cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">GenerationException</see> class
		/// with a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
		/// <param name="innerException">The exception that is the cause of the current exception, or
        /// a null reference if no inner exception is specified.</param>
        /// <param name="messages">The messages reported by the
        /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see> generator.</param>
		/// <remarks>
        /// <para>Initializes the <see cref="Exception.Message">Message</see> property of the new instance using
        /// the <paramref name="message"/> parameter.</para>
		/// <para>The following table shows the initial property values for an instance of
        /// <see cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">GenerationException</see>.</para>
		/// <list type="table">
		///    <listheader>
		///       <term>Property</term>
		///       <term>Value</term>
		///    </listheader>
		///    <item>
		///       <term>InnerException</term>
		///       <description>The inner exception.</description>
		///    </item>
		///    <item>
		///       <term>Message</term>
        ///       <description>The error message string.</description>
        ///    </item>
        ///    <item>
        ///       <term>Messages</term>
        ///       <description>The messages reported by the
        ///       <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see> generator.</description>
        ///    </item>
		/// </list>
		/// </remarks>
		internal GenerationException(string message, Exception innerException,
        MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> messages)
            : base(message, innerException) {
            _messages = messages;
        }

#if !XBOX360

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">GenerationException</see> class
        /// with serialized data.
        /// </summary>
        /// <param name="info">The serialized object data about the exception being thrown.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        /// <remarks>
        /// <para>This constructor is called during deserialization to reconstitute the exception object
        /// transmitted over a stream.</para>
        /// <para>This constructor is not available for the Xbox 360.</para>
        /// </remarks>
        /// <exception cref="ArgumentNullException">The <paramref name="info"/> parameter is a null reference.</exception>
        /// <exception cref="SerializationException">The class name is a null reference or
        /// <see cref="Exception.HResult">HResult</see> is zero (0).</exception>
        private GenerationException(SerializationInfo info, StreamingContext context)
            : base(info, context) {
            Type messagesType = typeof(MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar>);
            object messages = info.GetValue(MessagesName, messagesType);
            _messages = (MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar>)messages;
        }

#endif

        #endregion

        #region Properties

        /// <summary>
        /// Gets the messages reported by the
        /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">result</see>
        /// generator.
        /// </summary>
        public MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> Messages {
            get {
                return _messages;
            }
        }

        #endregion

        #region Methods

#if !XBOX360

        /// <summary>
        /// When overridden in a derived class, sets the <see cref="SerializationInfo"/> with information
        /// about the exception.
        /// </summary>
        /// <param name="info">The <see cref="SerializationInfo"/> that holds the serialized object data about
        /// the exception being thrown.</param>
        /// <param name="context">The <see cref="StreamingContext"/> that contains contextual information about
        /// the source or destination.</param>
        /// <remarks>This method is not available for the Xbox 360.</remarks>
        /// <exception cref="ArgumentNullException">The info parameter is a null reference. </exception>
        /// <PermissionSet>
        ///     <IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Read="*AllFiles*" PathDiscovery="*AllFiles*"/>
        ///     <IPermission class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Flags="SerializationFormatter"/>
        /// </PermissionSet>
        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context) {
            base.GetObjectData(info, context);
            info.AddValue(MessagesName, _messages);
        }

#endif

        #endregion

    }

}