/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// <para>Represents a predicate inequality.</para>
    /// <para>A predicate inequality is a special predicate which causes
    /// <see cref="Translator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">the translator</see> to
    /// automatically define them, without being defined by the knowledge provider, and generate a fact for each
    /// couple of objects of the type defined in the inequality if both objects not are the same.</para>
    /// </summary>
    /// <typeparam name="TType">The type of the type identifier.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifier.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public class InequalityPredicate<TType, TPred> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The predicate identifier.
        /// </summary>
        private readonly TPred _predicateId;

        /// <summary>
        /// The type identifier of the objects being affected by the inequality.
        /// </summary>
        private readonly TType _typeId;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InequalityPredicate{TType, TPred}"/> class.
        /// </summary>
        /// <param name="predicateId">The predicate identifier.</param>
        /// <param name="typeId">The type identifier of the objects being affected by the inequality.</param>
        internal InequalityPredicate(TPred predicateId, TType typeId) {
            _predicateId = predicateId;
            _typeId = typeId;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the predicate identifier.
        /// </summary>
        /// <value>The predicate identifier.</value>
        public TPred PredicateId {
            get {
                return _predicateId;
            }
        }

        /// <summary>
        /// Gets the type identifier of the objects being affected by the inequality.
        /// </summary>
        /// <value>The type identifier of the objects being affected by the inequality.</value>
        public TType TypeId {
            get {
                return _typeId;
            }
        }

        #endregion

    }

}
