/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="InequalityPredicate{TType, TPred}">planner predicate inequality</see> set.
    /// </summary>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public class InequalityPredicateSet<TType, TPred> : IEnumerable<InequalityPredicate<TType, TPred>> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The inequality predicates.
        /// </summary>
        private readonly IDictionary<TPred, InequalityPredicate<TType, TPred>> _predicates;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InequalityPredicateSet{TType, TPred}"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        internal InequalityPredicateSet(Resources resources) {
            _resources = resources;
            _predicates = new Dictionary<TPred, InequalityPredicate<TType, TPred>>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the number of inequality predicates.
        /// </summary>
        /// <value>The number of inequality predicates.</value>
        public int Count {
            get {
                return _predicates.Count;
            }
        }

        /// <summary>
        /// Gets the <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see> with the
        /// specified predicate identifier.
        /// </summary>
        /// <param name="predicateId">The inequality predicate identifier.</param>
        /// <value>The <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see> with
        /// <paramref name="predicateId"/>.</value>
        /// <exception cref="ArgumentNullException"><paramref name="predicateId"/> is a null reference.</exception>
        /// <exception cref="ArgumentException">No <see cref="InequalityPredicate{TSig, TObj}">inequality
        /// predicate</see> is found with <paramref name="predicateId"/> identifier.</exception>
        public InequalityPredicate<TType, TPred> this[TPred predicateId] {
            get {
                try {
                    return _predicates[predicateId];
                } catch (ArgumentNullException) {
                    throw new ArgumentNullException("predicateId");
                } catch (KeyNotFoundException) {
                    throw new ArgumentException(_resources.PredicateNotFound, "predicateId");
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether contains an <see cref="InequalityPredicate{TSig, TObj}">inequality
        /// predicate</see> with the specified predicate identifier.
        /// </summary>
        /// <param name="predicateId">The inequality predicate identifier.</param>
        /// <returns>
        /// True if contains an <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see>
        /// with <paramref name="predicateId"/>; otherwise, false.
        /// </returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicateId"/> is a null reference.</exception>
        public void Contains(TPred predicateId) {
            try {
                _predicates.ContainsKey(predicateId);
            } catch (ArgumentNullException) {
                throw new ArgumentNullException("predicateId");
            }
        }
        /// <summary>
        /// Adds an <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see>.
        /// </summary>
        /// <param name="predicateId">The inequality predicate identifier.</param>
        /// <param name="typeId">The type identifier of the objects being affected by the inequality.</param>
        /// <returns>The <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see> added.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicateId"/> or <paramref name="typeId"/>
        /// is a null reference.</exception>
        /// <exception cref="ArgumentException">An <see cref="InequalityPredicate{TSig, TObj}">inequality
        /// predicate</see> already exists with the same <paramref name="predicateId"/> identifier.</exception>
        public InequalityPredicate<TType, TPred> Add(TPred predicateId, TType typeId) {
            if (typeId == null) {
                throw new ArgumentNullException("typeId");
            }
            try {
                InequalityPredicate<TType, TPred> predicate =
                    new InequalityPredicate<TType, TPred>(predicateId, typeId);
                _predicates.Add(predicateId, predicate);
                return predicate;
            } catch (ArgumentNullException) {
                throw new ArgumentNullException("predicateId");
            } catch (ArgumentException) {
                throw new ArgumentException(_resources.PredicateAlreadyAdded, "predicateId");
            }
        }

        /// <summary>
        /// Removes an <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see> with the
        /// specified predicate identifier.
        /// </summary>
        /// <param name="predicateId">The predicate identifier.</param>
        /// <returns>True if the <see cref="InequalityPredicate{TSig, TObj}">inequality predicate</see>
        /// is removed.</returns>
        /// <exception cref="ArgumentNullException"><paramref name="predicateId"/> is a null reference.</exception>
        public bool Remove(TPred predicateId) {
            try {
                return _predicates.Remove(predicateId);
            } catch (ArgumentNullException) {
                throw new ArgumentNullException("predicateId");
            }
        }

        /// <summary>
        /// Clears the set.
        /// </summary>
        public void Clear() {
            _predicates.Clear();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the set.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the set.
        /// </returns>
        public IEnumerator<InequalityPredicate<TType, TPred>> GetEnumerator() {
            return _predicates.Values.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the set.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the set.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (this as IEnumerable<InequalityPredicate<TType, TPred>>).GetEnumerator();
        }

        #endregion

    }

}
