/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Aran.Translation.Signature{T}">signature</see> instance.
    /// </summary>
    /// <remarks>
    /// A signature instance is a list of objects which refer to a signature and has the same length of the
    /// signature parameter list. Also, each object type must be the same of the corresponding parameter type.
    /// </remarks>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal abstract class Instance<T> : Usable {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The signature.
        /// </summary>
        internal readonly Signature<T> Signature;

        /// <summary>
        /// The identifier.
        /// </summary>
        internal int Id;

        /// <summary>
        /// The argument list.
        /// </summary>
        internal readonly Argument<T>[] Arguments;

        /// <summary>
        /// The information about the signature usage.
        /// </summary>
        /// <remarks>
        /// <para>It is shared by all instances of the same <see cref="Signature">the signature</see>.</para>
        /// <para>It is separated from <see cref="Signature">the signature</see> because it is only used on
        /// translation time. <see cref="Signature">the signature</see> is also used for encoding/decoding tasks
        /// which do not use this information and, therefore, it must not be part of
        /// <see cref="Signature">the signature</see>.</para>
        /// </remarks>
        protected readonly Usable UseInfo;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Instance{T}"/> class.
        /// </summary>
        /// <param name="signature">The signature.</param>
        /// <param name="arguments">The argument list.</param>
        /// <param name="useInfo">The information about the signature usage.</param>
        protected internal Instance(Signature<T> signature, Argument<T>[] arguments, Usable useInfo) {
            Signature = signature;
            Arguments = arguments;
            UseInfo = useInfo;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Calculates the identifier.
        /// </summary>
        internal void CalculateId() {
            Id = Signature.GetInstanceId(Arguments);
        }

        #endregion

    }

}
