/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="IInstance{TSig, TObj}">instance</see> decoder.
    /// </summary>
    /// <remarks>An <see cref="IInstance{TSig, TObj}">instance</see> decoder can create
    /// <see cref="IInstance{TSig, TObj}">instance</see>s from a number which identifies them. That is,
    /// decodes the <see cref="IInstance{TSig, TObj}">instance</see> identifier in order to get the
    /// <see cref="ISignature{TType, TSig, TObj}">signature</see> identifier and the argument
    /// <see cref="IObject{TType, TObj}">object</see> identifiers. Then, creates the
    /// <see cref="IInstance{TSig, TObj}">instance</see>.</remarks>
    /// <typeparam name="TSig">The type of the signature identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class InstanceDecoder<TSig, TObj> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The signature entries.
        /// </summary>
        /// <remarks>The signature is at position <c>i</c>, where <c>i</c> is the position at
        /// <see cref="_signatureMaxInstanceIds">the signature maximum instance identifier list</see> which
        /// contains the maximum instance identifier for the signature.</remarks>
        private readonly SignatureEntry<TSig, TObj>[] _signatureEntries;

        /// <summary>
        /// The maximum instance identifier for each signature.
        /// </summary>
        /// <remarks>
        /// <para>The <c>i-th</c> position contains the maximum instance identifier for the <c>i-th</c>
        /// signature in the <see cref="_signatureEntries">signature entries list</see>.</para>
        /// <para>Values are ordered from minimum to maximum, starting at zero position.</para>
        /// </remarks>
        private readonly int[] _signatureMaxInstanceIds;

        /// <summary>
        /// The total number of instances for all signatures.
        /// </summary>
        /// <remarks>Is the sum of the number of instances for each signature.</remarks>
        private readonly int _instanceCount;

        /// <summary>
        /// The types.
        /// </summary>
        /// <remarks>The <c>i-th</c> position contains the type with the <c>i</c> identifier.</remarks>
        private readonly Type<TObj>[] _types;

        /// <summary>
        /// The instance creator.
        /// </summary>
        private IInstanceCreator<TSig, TObj> _instanceCreator;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InstanceDecoder{TSig, TObj}"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        /// <param name="signatureEntries">
        /// <para>The signature entries.</para>
        /// <para>The signature is at position <c>i</c>, where <c>i</c> is the position at
        /// the signature maximum instance identifier list which contains the maximum instance identifier for
        /// the signature.</para>
        /// </param>
        /// <param name="signatureMaxInstanceIds">
        /// <para>The maximum instance identifier for each signature.</para>
        /// <para>The <c>i-th</c> position contains the maximum instance identifier for the <c>i-th</c>
        /// signature in the signature entries list.</para>
        /// <para>Values are ordered from minimum to maximum, starting at zero position.</para>
        /// </param>
        /// <param name="instanceCount">
        /// <para>The total number of instances for all signatures.</para>
        /// <para>Is the sum of the number of instances for each signature.</para>
        /// </param>
        /// <param name="types">
        /// <para>The types.</para>
        /// <para>The <c>i-th</c> position contains the type with the <c>i</c> identifier.</para>
        /// </param>
        internal InstanceDecoder(Resources resources, SignatureEntry<TSig, TObj>[] signatureEntries,
        int[] signatureMaxInstanceIds, int instanceCount, Type<TObj>[] types) {
            _resources = resources;
            _signatureEntries = signatureEntries;
            _signatureMaxInstanceIds = signatureMaxInstanceIds;
            _instanceCount = instanceCount;
            _types = types;
            _instanceCreator = new InstanceSynthetizer<TSig, TObj>();
        }

        #endregion

        #region Methods

        /// <summary>
        /// Sets the <see cref="IInstanceCreator{TSig, TObj}">instance creator</see>.
        /// </summary>
        /// <param name="instanceCreator">The instance creator.</param>
        /// <exception cref="ArgumentNullException"><paramref name="instanceCreator"/> is a null reference.</exception>
        public void SetInstanceCreator(IInstanceCreator<TSig, TObj> instanceCreator) {
            if (instanceCreator == null) {
                throw new ArgumentNullException("instanceCreator");
            }
            _instanceCreator = instanceCreator;
        }

        /// <summary>
        /// Gets an <see cref="IInstance{TSig, TObj}">instance</see> from its identifier.
        /// </summary>
        /// <param name="id">The identifier.</param>
        /// <returns>The <see cref="IInstance{TSig, TObj}">instance</see> which corresponds to
        /// <paramref name="id"/>.</returns>
        /// <exception cref="ArgumentException">No <see cref="IInstance{TSig, TObj}">instance</see>
        /// corresponds to <paramref name="id"/></exception>.
        public IInstance<TSig, TObj> GetInstance(int id) {
            //the identificer must be contained in the global identifier range
            if ((id < 0) || (id >= _instanceCount)) {
                throw new ArgumentException(_resources.UnknownIdentifier, "id");
            }
            //decodes the signature
            int signatureIndex = Array.BinarySearch<int>(_signatureMaxInstanceIds, id);
            //if the index is greater than or equal to zero, the decoded identifier is the maximum instance
            //identifier for its signature, and the signature is at that index in _signatureEntries.
            if (signatureIndex < 0) {
                //if Array.BinarySearch does not found an item, it returns the index where the item should be,
                //but negated. That index is, in fact, the index of the inmediately superior identifier
                //in the array for de decoded identifier which is, in this case, the maximum instance identifier
                //for the signature of the decoded identifier. So, the signature is at not(index).
                signatureIndex = ~signatureIndex;
            }
            //gets the signature
            SignatureEntry<TSig, TObj> signatureEntry = _signatureEntries[signatureIndex];
            //decodes the arguments and creates the signature
            return _instanceCreator.CreateInstance(signatureEntry.Id,
                signatureEntry.Signature.GetInstanceArgumentIds(id, _types));
        }

        #endregion

    }

}
