/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="IInstance{TSig, TObj}">instance</see> encoder.
    /// </summary>
    /// <remarks>An <see cref="IInstance{TSig, TObj}">instance</see> encoder can calculate a number which
    /// identifies an <see cref="IInstance{TSig, TObj}">instance</see> from its information. That is,
    /// encodes its see <see cref="ISignature{TType, TSig, TObj}">signature</see> identifier and argument
    /// <see cref="IObject{TType, TObj}">object</see> identifiers in order to get its identifying number.</remarks>
    /// <typeparam name="TSig">The type of the signature identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class InstanceEncoder<TSig, TObj> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The type of the signatures of the instances to encode.
        /// </summary>
        /// <remarks>All instance signatures must be of this type.</remarks>
        private readonly SignatureType _signatureType;

        /// <summary>
        /// The signatures, hashed by identifier.
        /// </summary>
        private readonly IDictionary<TSig, Signature<TObj>> _signatures;

        /// <summary>
        /// The objects, hashed by identifier.
        /// </summary>
        private readonly IDictionary<TObj, Object> _objects;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InstanceEncoder{TSig, TObj}"/> class.
        /// </summary>
        /// <param name="resources">The resources manager.</param>
        /// <param name="signatureType">
        /// <para>The type of the signature.</para>
        /// <para>All instance signatures must be of this type.</para>
        /// </param>
        /// <param name="signatures">The signatures, hashed by identifier.</param>
        /// <param name="objects">The objects, hashed by identifier.</param>
        internal InstanceEncoder(Resources resources, SignatureType signatureType,
        IDictionary<TSig, Signature<TObj>> signatures, IDictionary<TObj, Object> objects) {
            _resources = resources;
            _signatureType = signatureType;
            _signatures = signatures;
            _objects = objects;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the <see cref="IEnumerator{TObj}">enumerator</see> of the argument identifiers of an
        /// instance.
        /// </summary>
        /// <param name="argumentCount">The number of arguments.</param>
        /// <param name="requiredArgumentCount">The required number of arguments for the instance signature.</param>
        /// <param name="instance">The instance.</param>
        /// <returns><paramref name="instance"/> argument identifiers.</returns>
        /// <exception cref="ArgumentException"><paramref name="argumentCount"/> is different from
        /// <paramref name="requiredArgumentCount"/> or the <see cref="IEnumerator{T}">enumerator</see>
        /// of the argument identifiers of <paramref name="instance"/> is a null reference.</exception>
        private IEnumerator<TObj> GetAndCheckArgumentIds(int argumentCount, int requiredArgumentCount,
        IInstance<TSig, TObj> instance) {
            if (argumentCount != requiredArgumentCount) {
                throw new ArgumentException(_resources.InvalidArgumentCount, "instance");
            }
            IEnumerable<TObj> instanceArguments = instance.GetArgumentIds();
            if (instanceArguments == null) {
                throw new ArgumentException(_resources.NullArguments, "instance");
            }
            return instanceArguments.GetEnumerator();
        }

        /// <summary>
        /// Gets the identifier of (and from) an <see cref="IInstance{TSig, TObj}">instance</see>.
        /// </summary>
        /// <param name="instance">The instance.</param>
        /// <returns><paramref name="instance"/> identifier.</returns>
        /// <exception cref="ArgumentException">The number of arguments of <paramref name="instance"/> is
        /// different from the number of parameters of its
        /// <see cref="ISignature{TType, TSig, TObj}">signature</see>, the
        /// <see cref="IEnumerator{T}">enumerator</see> of the argument identifiers of
        /// <paramref name="instance"/> is a null reference or some argument type is different
        /// from the corresponding parameter type.</exception>
        /// <exception cref="ArgumentNullException"><paramref name="instance"/> is a null reference.</exception>
        public int GetId(IInstance<TSig, TObj> instance) {
            if (instance == null) {
                throw new ArgumentNullException("instance");
            }
            try {
                Signature<TObj> signature = _signatures[instance.SignatureId];
                int argumentCount = instance.ArgumentCount;
                return signature.GetInstanceId(argumentCount,
                    GetAndCheckArgumentIds(argumentCount, signature.ParameterCount, instance), _objects);
            } catch (KeyNotFoundException) {
                throw new ArgumentException(_resources.GetUnknownSignature(_signatureType), "instance");
            }
        }

        #endregion

    }

}
