/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Translator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">translator</see>
    /// message.
    /// </summary>
    /// <typeparam name="T">The type of the source.</typeparam>
    [Serializable]
    [CLSCompliant(true)]
    public abstract class Message<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The source.
        /// </summary>
        private readonly T _source;

        /// <summary>
        /// The type.
        /// </summary>
        private readonly MessageType _type;

        /// <summary>
        /// The code.
        /// </summary>
        private readonly MessageCode _code;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Message{T}"/> class.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="type">The type.</param>
        /// <param name="code">The code.</param>
        protected internal Message(T source, MessageType type, MessageCode code) {
            _source = source;
            _type = type;
            _code = code;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the source.
        /// </summary>
        /// <value>The source.</value>
        public T Source {
            get {
                return _source;
            }
        }

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <value>The type.</value>
        public MessageType Type {
            get {
                return _type;
            }
        }

        /// <summary>
        /// Gets the code.
        /// </summary>
        /// <value>The code.</value>
        public MessageCode Code {
            get {
                return _code;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this message has position.
        /// </summary>
        /// <value>True if this message has position; otherwise, false.</value>
        public abstract bool HasPosition { get; }

        /// <summary>
        /// Gets the zero-based position.
        /// </summary>
        /// <value>The zero-based position.</value>
        /// <exception cref="InvalidOperationException">The message has no position.</exception>
        public abstract int Position { get; }

        #endregion

    }

}