/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Message{T}">translator message</see> code.
    /// </summary>
    [Serializable]
    [CLSCompliant(true)]
    public enum MessageCode {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// <para>The argument is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        ArgumentNotUsed,

        /// <summary>
        /// <para>The argument count is not the same as the parameter count.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        ArgumentCountMismatch,

        /// <summary>
        /// <para>The argument type is not the same as the parameter type.</para>
        /// <para>The position of the argument is provided.</para>
        /// </summary>
        ArgumentTypeMismatch,

        /// <summary>
        /// <para>The fact is already present in the addition list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactAdditionAlreadyPresent,

        /// <summary>
        /// <para>The fact is already present in the precondition list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactAdditionAlreadyRequired,

        /// <summary>
        /// <para>The fact will be already present in the addition list under certain combination of arguments.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactAdditionWillBeAlreadyPresent,

        /// <summary>
        /// <para>The fact is already present in the addition list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactDeletionAlreadyAdded,

        /// <summary>
        /// <para>The fact is already present in the deletion list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactDeletionAlreadyPresent,

        /// <summary>
        /// <para>The fact will be already present in the deletion list under certain combination of arguments.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactDeletionWillBeAlreadyPresent,

        /// <summary>
        /// <para>The identifier of the fact is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        FactIdAlreadyUsed,

        /// <summary>
        /// <para>The fact is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        FactNotUsed,

        /// <summary>
        /// <para>The fact is already added.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactPreconditionAlreadyAdded,

        /// <summary>
        /// <para>The fact is already present in the precondition list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactPreconditionAlreadyPresent,

        /// <summary>
        /// <para>The fact will be already present in the precondition list under certain combination of arguments.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        FactPreconditionWillBeAlreadyPresent,

        /// <summary>
        /// <para>The identifier of the function is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        FunctionIdAlreadyUsed,

        /// <summary>
        /// <para>The function is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        FunctionNotUsed,

        /// <summary>
        /// <para>The fact is already present in the goal list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        GoalFactAlreadyPresent,

        /// <summary>
        /// <para>The number of additions is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentAdditionFactCount,

        /// <summary>
        /// <para>The number of arguments is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentArgumentCount,

        /// <summary>
        /// <para>The number of deletions is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentDeletionFactCount,

        /// <summary>
        /// <para>The number of effects is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentEffectOperationCount,

        /// <summary>
        /// <para>The number of facts is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentFactCount,

        /// <summary>
        /// <para>The number of functions is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentFunctionCount,

        /// <summary>
        /// <para>The number of goals is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentGoalFactCount,

        /// <summary>
        /// <para></para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentGoalOperationCount,

        /// <summary>
        /// <para>The number of functional goals is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentInitialFactCount,

        /// <summary>
        /// <para>The number of functional initializations is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentInitializationOperationCount,

        /// <summary>
        /// <para>The number of objects is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentObjectCount,

        /// <summary>
        /// <para>The number of operands is not coherent</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentOperandCount,

        /// <summary>
        /// <para>The number of operations is not coherent</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentOperationCount,

        /// <summary>
        /// <para>The number of operator is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentOperatorCount,

        /// <summary>
        /// <para>The number of parameters is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentParameterCount,

        /// <summary>
        /// <para>The number of preconditions is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentPreconditionFactCount,

        /// <summary>
        /// <para>The number of functional preconditions is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentPreconditionOperationCount,

        /// <summary>
        /// <para>The number of predicates is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentPredicateCount,

        /// <summary>
        /// <para>The number of types is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentTypeCount,

        /// <summary>
        /// <para>The number of variables is not coherent.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        IncoherentVariableCount,

        /// <summary>
        /// <para>The addition does not exist.</para>
        /// <para>The position of the addition is provided.</para>
        /// </summary>
        InexistentAdditionFact,

        /// <summary>
        /// <para>The deletion does not exist.</para>
        /// <para>The position of the deletion is provided.</para>
        /// </summary>
        InexistentDeletionFact,

        /// <summary>
        /// <para>The fact does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentFact,

        /// <summary>
        /// <para>The function does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentFunction,

        /// <summary>
        /// <para>The goal does not exist.</para>
        /// <para>The position of the goal is provided.</para>
        /// </summary>
        InexistentGoalFact,

        /// <summary>
        /// <para>The initial fact does not exist.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        InexistentInitialFact,

        /// <summary>
        /// <para>The object does not exist.</para>
        /// <para>The position of the object is provided.</para>
        /// </summary>
        InexistentObject,

        /// <summary>
        /// <para>The object type does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentObjectType,

        /// <summary>
        /// <para>The parameter type does not exist.</para>
        /// <para>The position of the parameter is provided.</para>
        /// </summary>
        InexistentParameterType,

        /// <summary>
        /// <para>The parent does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentParentType,

        /// <summary>
        /// <para>The precondition does not exist.</para>
        /// <para>The position of the precondition is provided.</para>
        /// </summary>
        InexistentPreconditionFact,

        /// <summary>
        /// <para>The predicate does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentPredicate,

        /// <summary>
        /// <para>The variable does not exist.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InexistentVariable,

        /// <summary>
        /// <para>The fact is already present in the initial fact list.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        InitialFactAlreadyPresent,

        /// <summary>
        /// <para>The number of additions is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidAdditionFactCount,

        /// <summary>
        /// <para>The number of arguments is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidArgumentCount,

        /// <summary>
        /// <para>The number of deletions is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidDeletionFactCount,

        /// <summary>
        /// <para>The number of effects is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidEffectOperationCount,

        /// <summary>
        /// <para>The number of facts is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidFactCount,

        /// <summary>
        /// <para>The number of functions is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidFunctionCount,

        /// <summary>
        /// <para>The number of goals is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidGoalFactCount,

        /// <summary>
        /// <para>The number of functional goals is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidGoalOperationCount,

        /// <summary>
        /// <para>The number of initial facts is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidInitialFactCount,

        /// <summary>
        /// <para>The number of functional initializations is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidInitializationOperationCount,

        /// <summary>
        /// <para>The left operand type is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidLeftOperandType,

        /// <summary>
        /// <para>The metric target must be a variable or function.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidMetricTargetType,

        /// <summary>
        /// <para>The metric type is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidMetricType,

        /// <summary>
        /// <para>The number of objects is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidObjectCount,

        /// <summary>
        /// <para>The number of operands is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidOperandCount,

        /// <summary>
        /// <para>The operand type is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidOperandType,

        /// <summary>
        /// <para>The number of operations is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidOperationCount,

        /// <summary>
        /// <para>The number of operators is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidOperatorCount,

        /// <summary>
        /// <para>The number of parameters is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidParameterCount,

        /// <summary>
        /// <para>The number of preconditions is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidPreconditionFactCount,

        /// <summary>
        /// <para>The number of functional preconditions is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidPreconditionOperationCount,

        /// <summary>
        /// <para>The number of predicates is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidPredicateCount,

        /// <summary>
        /// <para>The right operand type is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidRightOperandType,

        /// <summary>
        /// <para>The number of types is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidTypeCount,

        /// <summary>
        /// <para>The number of variables is not valid.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        InvalidVariableCount,

        /// <summary>
        /// <para>The left operand must be a constant.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        LeftOperandMustBeConstant,

        /// <summary>
        /// <para>The right operand must be variable.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        LeftOperandMustBeVariable,

        /// <summary>
        /// <para>The left operand should not be the value 1, because has no effect.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        LeftOperandShouldNotBeOne,

        /// <summary>
        /// <para>The left operand should not be the value 0, because has no effect.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        LeftOperandShouldNotBeZero,

        /// <summary>
        /// <para>The additions are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullAdditionFacts,

        /// <summary>
        /// <para>The additions enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullAdditionFactsEnumerator,

        /// <summary>
        /// <para>The argument identifiers are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullArgumentIds,

        /// <summary>
        /// <para>The argument identifiers enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullArgumentIdsEnumerator,

        /// <summary>
        /// <para>The deletions are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullDeletionFacts,

        /// <summary>
        /// <para>The deletions enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullDeletionFactsEnumerator,

        /// <summary>
        /// <para>The domain is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullDomain,

        /// <summary>
        /// <para>The effect at some position is a null reference.</para>
        /// <para>The position of the effect is provided.</para>
        /// </summary>
        NullEffectOperation,

        /// <summary>
        /// <para>The effects are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullEffectOperations,

        /// <summary>
        /// <para>The effects enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullEffectOperationsEnumerator,

        /// <summary>
        /// <para>The fact at some position is a null reference.</para>
        /// <para>The position of the fact is provided.</para>
        /// </summary>
        NullFact,

        /// <summary>
        /// <para>The facts are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullFacts,

        /// <summary>
        /// <para>The facts enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullFactsEnumerator,

        /// <summary>
        /// <para>The function at some position is a null reference.</para>
        /// <para>The position of the function is provided.</para>
        /// </summary>
        NullFunction,

        /// <summary>
        /// <para>The functions are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullFunctions,

        /// <summary>
        /// <para>The functions enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullFunctionsEnumerator,

        /// <summary>
        /// <para>The goals are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullGoalFacts,

        /// <summary>
        /// <para>The goals enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullGoalFactsEnumerator,

        /// <summary>
        /// <para>The goal at some position is a null reference.</para>
        /// <para>The position of the goal is provided.</para>
        /// </summary>
        NullGoalOperation,

        /// <summary>
        /// <para>The functional goals are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullGoalOperations,

        /// <summary>
        /// <para>The functional goals enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullGoalOperationsEnumerator,

        /// <summary>
        /// <para>The initial facts are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullInitialFacts,

        /// <summary>
        /// <para>The initial facts enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullInitialFactsEnumerator,

        /// <summary>
        /// <para>The functional initialization at some position is a null reference.</para>
        /// <para>The position of the functional initialization is provided.</para>
        /// </summary>
        NullInitializationOperation,

        /// <summary>
        /// <para>The functional initializations are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullInitializationOperations,

        /// <summary>
        /// <para>The functional initializations enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullInitializationOperationsEnumerator,

        /// <summary>
        /// <para>The knowledge representation is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullKnowledge,

        /// <summary>
        /// <para>The metric is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullMetric,

        /// <summary>
        /// <para>The metric target is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullMetricTarget,

        /// <summary>
        /// <para>The object at some position is a null reference.</para>
        /// <para>The position of the object is provided.</para>
        /// </summary>
        NullObject,

        /// <summary>
        /// <para>The objects are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullObjects,

        /// <summary>
        /// <para>The objects enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullObjectsEnumerator,

        /// <summary>
        /// <para>The operand at some position is a null reference.</para>
        /// <para>The position of the operand is provided.</para>
        /// </summary>
        NullOperand,

        /// <summary>
        /// <para>The operands are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperands,

        /// <summary>
        /// <para>The operands enumerator is a null reference</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperandsEnumerator,

        /// <summary>
        /// <para>The operation at some position is a null reference.</para>
        /// <para>The position of the operation is provided.</para>
        /// </summary>
        NullOperation,

        /// <summary>
        /// <para>The operations are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperations,

        /// <summary>
        /// <para>The operations enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperationsEnumerator,

        /// <summary>
        /// <para>The operator at some position is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperator,

        /// <summary>
        /// <para>The operators are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperators,

        /// <summary>
        /// <para>The operators enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullOperatorsEnumerator,

        /// <summary>
        /// <para>The parameter at some position is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullParameter,

        /// <summary>
        /// <para>The parameters are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullParameters,

        /// <summary>
        /// <para>The parameters enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullParametersEnumerator,

        /// <summary>
        /// <para>The preconditions are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPreconditionFacts,

        /// <summary>
        /// <para>The preconditions enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPreconditionFactsEnumerator,

        /// <summary>
        /// <para>The functional precondition at some position is a null reference.</para>
        /// <para>The position of the functional precondition is provided.</para>
        /// </summary>
        NullPreconditionOperation,

        /// <summary>
        /// <para>The functional preconditions are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPreconditionOperations,

        /// <summary>
        /// <para>The functional preconditions enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPreconditionOperationsEnumerator,

        /// <summary>
        /// <para>The predicate at some position is a null reference.</para>
        /// <para>The position of the predicate is provided.</para>
        /// </summary>
        NullPredicate,

        /// <summary>
        /// <para>The predicates are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPredicates,

        /// <summary>
        /// <para>The predicates enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullPredicatesEnumerator,

        /// <summary>
        /// <para>The problem is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullProblem,

        /// <summary>
        /// <para>The type at some position is a null reference.</para>
        /// <para>The position of the type is provided.</para>
        /// </summary>
        NullType,

        /// <summary>
        /// <para>The types are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullTypes,

        /// <summary>
        /// <para>The types enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullTypesEnumerator,

        /// <summary>
        /// <para>The variable at some position is a null reference.</para>
        /// <para>The position of the variable is provided.</para>
        /// </summary>
        NullVariable,

        /// <summary>
        /// <para>The variables are a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullVariables,

        /// <summary>
        /// <para>The variables enumerator is a null reference.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        NullVariablesEnumerator,

        /// <summary>
        /// <para>The identifier of the object is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        ObjectIdAlreadyUsed,

        /// <summary>
        /// <para>The object is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        ObjectNotUsed,

        /// <summary>
        /// <para>The operation must be a condition.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        OperationIsNotCondition,

        /// <summary>
        /// <para>The operation must be an effect.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        OperationIsNotEffect,

        /// <summary>
        /// <para>The operation must be a function.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        OperationIsNotFunction,

        /// <summary>
        /// <para>The identifier of the operator is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        OperatorIdAlreadyUsed,

        /// <summary>
        /// <para>The operator has no effects.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        OperatorWithNoEffects,

        /// <summary>
        /// <para>The identifier of the parameter is already used.</para>
        /// <para>The position of the parameter is provided.</para>
        /// </summary>
        ParameterIdAlreadyUsed,

        /// <summary>
        /// <para>The identifier of the predicate is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        PredicateIdAlreadyUsed,

        /// <summary>
        /// <para>The predicate is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        PredicateNotUsed,

        /// <summary>
        /// <para>The right operand must be constant.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        RightOperandMustBeConstant,

        /// <summary>
        /// <para>The right operand must not be constant.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        RightOperandMustNotBeConstant,

        /// <summary>
        /// <para>The right operand must not be the value 0.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        RightOperandMustNotBeZero,

        /// <summary>
        /// <para>The right operand should not be the value 1, because has no effect.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        RightOperandShouldNotBeOne,

        /// <summary>
        /// <para>The right operand should not be the value 0, because has no effect.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        RightOperandShouldNotBeZero,

        /// <summary>
        /// <para>The identifier of the type is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        TypeIdAlreadyUsed,

        /// <summary>
        /// <para>The operator will not be executed because there are no arguments to call it.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        UnexecutableOperator,

        /// <summary>
        /// <para>The goal will not be reached because no executable operator adds it.</para>
        /// <para>The position of the goal is provided.</para>
        /// </summary>
        UnreachableGoal,

        /// <summary>
        /// <para>The variable is already defined.</para>
        /// <para>The position of the variable is provided.</para>
        /// </summary>
        VariableAlreadyDefined,

        /// <summary>
        /// <para>The identifier of the variable is already used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        VariableIdAlreadyUsed,

        /// <summary>
        /// <para>The variable is not used.</para>
        /// <para>No position is provided.</para>
        /// </summary>
        VariableNotUsed
	
        #endregion
	
    }

}