/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections;
using System.Collections.Generic;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Message{T}">translator message</see> collection.
    /// </summary>
    /// <typeparam name="T">The type of the sources.</typeparam>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class MessageCollection<T> : IEnumerable<Message<T>> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The global counters.
        /// </summary>
        /// <remarks>
        /// They are shared by all message collections.
        /// </remarks>
        private readonly MessageCounters _counters;

        /// <summary>
        /// The number of semantic warnings.
        /// </summary>
        private int _semanticWarningCount;

        /// <summary>
        /// The number of semantic errors.
        /// </summary>
        private int _semanticErrorCount;

        /// <summary>
        /// The number of syntactic errors.
        /// </summary>
        private int _syntacticErrorCount;

        /// <summary>
        /// The messages.
        /// </summary>
        private readonly IList<Message<T>> _messages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MessageCollection{T}"/> class.
        /// </summary>
        /// <param name="counters">
        /// <para>The global counters.</para>
        /// <para>They are shared by all message collections.</para>
        /// </param>
        internal MessageCollection(MessageCounters counters) {
            _counters = counters;
            _messages = new List<Message<T>>();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the <see cref="Message{T}">message</see> at the specified index.
        /// </summary>
        /// <param name="index">The index of the <see cref="Message{T}">message</see>.</param>
        /// <value>The <see cref="Message{T}">message</see> at <paramref name="index"/>.</value>
        /// <exception cref="ArgumentOutOfRangeException"><paramref name="index"/> is less than zero or
        /// <paramref name="index"/> is equal to or greater than <see cref="Count"/>.</exception>
        public Message<T> this[int index] {
            get {
                return _messages[index];
            }
        }

        /// <summary>
        /// Gets the number of messages.
        /// </summary>
        /// <value>The number of messages..</value>
        public int Count {
            get {
                return _messages.Count;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticWarning">semantic warning</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticWarning">semantic warning</see> messages.</value>
        public int SemanticWarningCount {
            get {
                return _semanticWarningCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticError">semantic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticError">semantic error</see> messages.</value>
        public int SemanticErrorCount {
            get {
                return _semanticErrorCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SyntacticError">syntatic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SyntacticError">syntatic error</see> messages.</value>
        public int SyntacticErrorCount {
            get {
                return _syntacticErrorCount;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticError">semantic error</see> and
        /// <see cref="MessageType.SyntacticError">syntatic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticError">semantic error</see> and
        /// <see cref="MessageType.SyntacticError">syntatic error</see> messages.</value>
        public int TotalErrorCount {
            get {
                return _semanticErrorCount + _syntacticErrorCount;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds a semantic warning message.
        /// </summary>
        /// <param name="message">The message.</param>
        internal void AddSemanticWarning(Message<T> message) {
            _messages.Add(message);
            _semanticWarningCount++;
            _counters.SemanticWarnings++;
        }

        /// <summary>
        /// Adds a semantic warning message without position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The mesage code.</param>
        internal void AddSemanticWarning(T source, MessageCode code) {
            AddSemanticWarning(new MessageWithoutPosition<T>(source, MessageType.SemanticWarning, code));
        }

        /// <summary>
        /// Adds a semantic warning message with position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The mesage code.</param>
        /// <param name="position">The message position.</param>
        internal void AddSemanticWarning(T source, MessageCode code, int position) {
            AddSemanticWarning(new MessageWithPosition<T>(source, MessageType.SemanticWarning, code, position));
        }

        /// <summary>
        /// Adds a semantic error message.
        /// </summary>
        /// <param name="message">The message.</param>
        private void AddSemanticError(Message<T> message) {
            _messages.Add(message);
            _semanticErrorCount++;
            _counters.SemanticErrors++;
        }

        /// <summary>
        /// Adds a semantic error message without position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        internal void AddSemanticError(T source, MessageCode code) {
            AddSemanticError(new MessageWithoutPosition<T>(source, MessageType.SemanticError, code));
        }

        /// <summary>
        /// Adds a semantic error message with position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The vcode.</param>
        /// <param name="position">The message position.</param>
        internal void AddSemanticError(T source, MessageCode code, int position) {
            AddSemanticError(new MessageWithPosition<T>(source, MessageType.SemanticError, code, position));
        }

        /// <summary>
        /// Adds a semantic error message with position and interrupts the translation.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        /// <param name="position">The message position.</param>
        internal void AddSemanticFatalError(T source, MessageCode code, int position) {
            AddSemanticError(source, code, position);
            _counters.InterruptTranslation();
        }

        /// <summary>
        /// Adds a semantic error message without position and interrupts the translation.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        internal void AddSemanticFatalError(T source, MessageCode code) {
            AddSemanticError(source, code);
            _counters.InterruptTranslation();
        }

        /// <summary>
        /// Adds a syntatic error message.
        /// </summary>
        /// <param name="message">The message.</param>
        private void AddSyntacticError(Message<T> message) {
            _messages.Add(message);
            _syntacticErrorCount++;
            _counters.SyntacticErrors++;
        }

        /// <summary>
        /// Adds a syntatic error message without position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        internal void AddSyntacticError(T source, MessageCode code) {
            AddSyntacticError(new MessageWithoutPosition<T>(source, MessageType.SyntacticError, code));
        }

        /// <summary>
        /// Adds a syntatic error message with position.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        /// <param name="position">The message position.</param>
        internal void AddSyntacticError(T source, MessageCode code, int position) {
            AddSyntacticError(new MessageWithPosition<T>(source, MessageType.SyntacticError, code, position));
        }

        /// <summary>
        /// Adds a syntatic error message with position and interrupts the translation.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        /// <param name="position">The message position.</param>
        internal void AddSyntacticFatalError(T source, MessageCode code, int position) {
            AddSyntacticError(source, code, position);
            _counters.InterruptTranslation();
        }

        /// <summary>
        /// Adds a syntatic error message without position and interrupts the translation.
        /// </summary>
        /// <param name="source">The message source.</param>
        /// <param name="code">The message code.</param>
        internal void AddSyntacticFatalError(T source, MessageCode code) {
            AddSyntacticError(source, code);
            _counters.InterruptTranslation();
        }

        /// <summary>
        /// Notifies the errors, if any, by interrupting the translation.
        /// </summary>
        internal void NotifyErrors() {
            if (TotalErrorCount > 0) {
                _counters.InterruptTranslation();
            }
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator{T}">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator{T}">enumerator</see> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<Message<T>> GetEnumerator() {
            return _messages.GetEnumerator();
        }

        /// <summary>
        /// Returns an <see cref="IEnumerator">enumerator</see> that iterates through the collection.
        /// </summary>
        /// <returns>
        /// An <see cref="IEnumerator">enumerator</see> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator() {
            return (_messages as IEnumerable).GetEnumerator();
        }

        #endregion

    }

}