/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents a message manager
    /// </summary>
    /// <remarks>A message manager is a set of message collections obtained from different sources which
    /// provides also a set of counters shared by all of them.</remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [Serializable]
    [CLSCompliant(true)]
    public sealed class MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The global counters.
        /// </summary>
        private readonly MessageCounters _counters;

        /// <summary>
        /// The messages related to the knowledge provider.
        /// </summary>
        private readonly MessageCollection<IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _knowledgeProviderMessages;

        /// <summary>
        /// The messages related to the knowledge representation.
        /// </summary>
        private readonly MessageCollection<IKnowledge<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _knowledgeMessages;

        /// <summary>
        /// The messages related to the domain.
        /// </summary>
        private readonly MessageCollection<IDomain<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _domainMessages;

        /// <summary>
        /// The messages related to the problem.
        /// </summary>
        private readonly MessageCollection<IProblem<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _problemMessages;

        /// <summary>
        /// The messages related to the metric.
        /// </summary>
        private readonly MessageCollection<IMetric<TVar>> _metricMessages;

        /// <summary>
        /// The messages related to the objects.
        /// </summary>
        private readonly MessageCollection<IObject<TType, TObj>> _objectMessages;

        /// <summary>
        /// The messages related to the operands.
        /// </summary>
        private readonly MessageCollection<IOperand<TVar>> _operandMessages;

        /// <summary>
        /// The messages related to the operations.
        /// </summary>
        private readonly MessageCollection<IOperation<TVar>> _operationMessages;

        /// <summary>
        /// The messages related to the operators.
        /// </summary>
        private readonly MessageCollection<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _operatorMessages;

        /// <summary>
        /// The messages related to the predicates.
        /// </summary>
        private readonly MessageCollection<ISignature<TType, TPred, TObj>> _predicateMessages;

        /// <summary>
        /// The messages related to the inequality predicates.
        /// </summary>
        private readonly MessageCollection<InequalityPredicate<TType, TPred>> _inequalityPredicateMessages;

        /// <summary>
        /// The messages related to the functions.
        /// </summary>
        private readonly MessageCollection<ISignature<TType, TFunc, TObj>> _functionMessages;

        /// <summary>
        /// The messages related to the facts.
        /// </summary>
        private readonly MessageCollection<IInstance<TPred, TObj>> _factMessages;

        /// <summary>
        /// The messages related to the variables.
        /// </summary>
        private readonly MessageCollection<IInstance<TFunc, TObj>> _variableMessages;

        /// <summary>
        /// The messages related to the types.
        /// </summary>
        private readonly MessageCollection<IType<TType>> _typeMessages;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MessageManager{TType, TPred, TFunc, TOp, TObj, TFact, TVar}"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        internal MessageManager(Resources resources) {
            _counters = new TranslationInterrupter<TType, TPred, TFunc, TOp, TObj, TFact, TVar>(resources, this);
            _knowledgeProviderMessages = new MessageCollection<IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(_counters);
            _knowledgeMessages = new MessageCollection<IKnowledge<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(_counters);
            _domainMessages = new MessageCollection<IDomain<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(_counters);
            _problemMessages = new MessageCollection<IProblem<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(_counters);
            _metricMessages = new MessageCollection<IMetric<TVar>>(_counters);
            _objectMessages = new MessageCollection<IObject<TType, TObj>>(_counters);
            _operandMessages = new MessageCollection<IOperand<TVar>>(_counters);
            _operationMessages = new MessageCollection<IOperation<TVar>>(_counters);
            _operatorMessages = new MessageCollection<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(_counters);
            _predicateMessages = new MessageCollection<ISignature<TType, TPred, TObj>>(_counters);
            _inequalityPredicateMessages = new MessageCollection<InequalityPredicate<TType, TPred>>(_counters);
            _functionMessages = new MessageCollection<ISignature<TType, TFunc, TObj>>(_counters);
            _factMessages = new MessageCollection<IInstance<TPred, TObj>>(_counters);
            _variableMessages = new MessageCollection<IInstance<TFunc, TObj>>(_counters);
            _typeMessages = new MessageCollection<IType<TType>>(_counters);
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticWarning">semantic warning</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticWarning">semantic warning</see> messages.</value>
        public int SemanticWarningCount {
            get {
                return _counters.SemanticWarnings;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticError">semantic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticError">semantic error</see> messages.</value>
        public int SemanticErrorCount {
            get {
                return _counters.SemanticErrors;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SyntacticError">syntactic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SyntacticError">syntactic error</see> messages.</value>
        public int SyntacticErrorCount {
            get {
                return _counters.SyntacticErrors;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="MessageType.SemanticError">semantic error</see> and
        /// <see cref="MessageType.SyntacticError">syntactic error</see> messages.
        /// </summary>
        /// <value>The number of <see cref="MessageType.SemanticError">semantic error</see> and
        /// <see cref="MessageType.SyntacticError">syntactic error</see> messages.</value>
        public int TotalErrorCount {
            get {
                return _counters.SemanticErrors + _counters.SyntacticErrors;
            }
        }

        /// <summary>
        /// Gets the messages related to knowledge provider.
        /// </summary>
        /// <value>The messages related to knowledge provider.</value>
        public MessageCollection<IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> KnowledgeProviderMessages {
            get {
                return _knowledgeProviderMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to knowledge.
        /// </summary>
        /// <value>The messages related to knowledge.</value>
        public MessageCollection<IKnowledge<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> KnowledgeMessages {
            get {
                return _knowledgeMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to domain.
        /// </summary>
        /// <value>The messages related to domain..</value>
        public MessageCollection<IDomain<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> DomainMessages {
            get {
                return _domainMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to problem.
        /// </summary>
        /// <value>The messages related to problem.</value>
        public MessageCollection<IProblem<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> ProblemMessages {
            get {
                return _problemMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to metric.
        /// </summary>
        /// <value>The messages related to metric.</value>
        public MessageCollection<IMetric<TVar>> MetricMessages {
            get {
                return _metricMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to objects.
        /// </summary>
        /// <value>The messages related to objects.</value>
        public MessageCollection<IObject<TType, TObj>> ObjectMessages {
            get {
                return _objectMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to operands.
        /// </summary>
        /// <value>The messages related to operands.</value>
        public MessageCollection<IOperand<TVar>> OperandMessages {
            get {
                return _operandMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to operations.
        /// </summary>
        /// <value>The messages related to operations.</value>
        public MessageCollection<IOperation<TVar>> OperationMessages {
            get {
                return _operationMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to operators.
        /// </summary>
        /// <value>The messages related to operators.</value>
        public MessageCollection<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> OperatorMessages {
            get {
                return _operatorMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to predicates.
        /// </summary>
        /// <value>The messages related to predicates.</value>
        public MessageCollection<ISignature<TType, TPred, TObj>> PredicateMessages {
            get {
                return _predicateMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to inequality predicates.
        /// </summary>
        /// <value>The messages related to inequality predicates.</value>
        public MessageCollection<InequalityPredicate<TType, TPred>> InequalityPredicateMessages {
            get {
                return _inequalityPredicateMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to functions.
        /// </summary>
        /// <value>The messages related to functions.</value>
        public MessageCollection<ISignature<TType, TFunc, TObj>> FunctionMessages {
            get {
                return _functionMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to facts.
        /// </summary>
        /// <value>The messages related to facts.</value>
        public MessageCollection<IInstance<TPred, TObj>> FactMessages {
            get {
                return _factMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to variables.
        /// </summary>
        /// <value>The messages related to variables.</value>
        public MessageCollection<IInstance<TFunc, TObj>> VariableMessages {
            get {
                return _variableMessages;
            }
        }

        /// <summary>
        /// Gets the messages related to types.
        /// </summary>
        /// <value>The messages related to types.</value>
        public MessageCollection<IType<TType>> TypeMessages {
            get {
                return _typeMessages;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Notifies the errors, if any, by interrupting the translation.
        /// </summary>
        internal void NotifyErrors() {
            if (TotalErrorCount > 0) {
                _counters.InterruptTranslation();
            }
        }

        #endregion

    }

}