/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Message{T}">translator message</see> without position.
    /// </summary>
    /// <typeparam name="T">The type of the source.</typeparam>
    [Serializable]
    internal sealed class MessageWithoutPosition<T> : Message<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MessageWithoutPosition{T}"/> class.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="type">The type.</param>
        /// <param name="code">The code.</param>
        internal MessageWithoutPosition(T source, MessageType type, MessageCode code)
            : base(source, type, code) {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this message has position.
        /// </summary>
        /// <value>True if this message has position; otherwise, false.</value>
        public override bool HasPosition {
            get {
                return false;
            }
        }

        /// <summary>
        /// Gets the zero-based position.
        /// </summary>
        /// <value>The zero-based position.</value>
        /// <exception cref="InvalidOperationException">The message has no position.</exception>
        public override int Position {
            get {
                throw new InvalidOperationException();
            }
        }

        #endregion

    }

}