/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents an operand.
    /// </summary>
    [Serializable]
    internal abstract class Operand {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Operand"/> class.
        /// </summary>
        internal Operand() {
            //do nothing
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the type of the operand.
        /// </summary>
        /// <value>The type of the operand.</value>
        internal abstract OperandType OperandType {
            get;
        }

        /// <summary>
        /// Gets the index of the variable.
        /// </summary>
        /// <value>The index of the variable. Zero if the operand is not a variable.</value>
        internal virtual int VariableIndex {
            get {
                return 0;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this operand is a global variable.
        /// </summary>
        /// <value>True if this operand is a global variable; otherwise, false.</value>
        internal virtual bool IsGlobalVariable {
            get {
                return false;
            }
        }

        /// <summary>
        /// Gets the value of the constant.
        /// </summary>
        /// <value>The value of the constant. Zero if the operand is not a constant.</value>
        internal virtual float ConstantValue {
            get {
                return 0;
            }
        }

        /// <summary>
        /// Gets the type of the operation.
        /// </summary>
        /// <value>The type of the operation. <see cref="F:OperationType.Assignment">assignment</see> if the
        /// operand is not an operation.</value>
        internal virtual OperationType OperationType {
            get {
                return OperationType.Assignment;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this operand is a condition.
        /// </summary>
        /// <remarks>The operand is a condition if its <see cref="OperationType">operation type</see> is a
        /// comparison, a conjunction or a disjunction.</remarks>
        /// <value>True if this operand is a condition; otherwise, false.</value>
        internal bool IsCondition {
            get {
                switch (OperationType) {
                    case OperationType.Inferiority:
                    case OperationType.NonSuperiority:
                    case OperationType.Equality:
                    case OperationType.Inequality:
                    case OperationType.NonInferiority:
                    case OperationType.Superiority:
                    case OperationType.Conjunction:
                    case OperationType.Disjunction:
                        return true;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether this operand is an effect.
        /// </summary>
        /// <remarks>The operand is an effect if its <see cref="OperationType">operation type</see> is any
        /// kind of assignment.</remarks>
        /// <value>True if this operand is an effect; otherwise, false.</value>
        internal bool IsEffect {
            get {
                switch (OperationType) {
                    case OperationType.Assignment:
                    case OperationType.SumAssignment:
                    case OperationType.SubtractionAssignment:
                    case OperationType.MultiplicationAssignment:
                    case OperationType.DivisionAssignment:
                        return true;
                    default:
                        return false;
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the <see cref="IActionCondition">action-dependant state condition</see> this operand
        /// corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IActionCondition">action-dependant state condition</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IActionCondition">action-dependant state condition</see> this operand
        /// corresponds to. A null reference on error.</returns>
        internal virtual IActionCondition GetActionCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            return null;
        }

        /// <summary>
        /// Gets the <see cref="IActionEffect">action-dependant state effect</see> this operand
        /// corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IActionEffect">action-dependant state effect</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IActionEffect">action-dependant state effect</see> this operand
        /// corresponds to. A null reference on error.</returns>
        internal virtual IActionEffect GetActionEffect(float[] globalVariables, bool[] conditionalVariables) {
            return null;
        }

        /// <summary>
        /// Gets the <see cref="IActionFunction">action-dependant state calculated value</see> this operand
        /// corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IActionFunction">action-dependant state calculated value</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IActionFunction">action-dependant state calculated value</see> this operand
        /// corresponds to. A null reference on error.</returns>
        internal virtual IActionFunction GetActionFunction(float[] globalVariables, bool[] conditionalVariables) {
            return null;
        }

        /// <summary>
        /// Gets the <see cref="IActionCondition">state condition</see> this operand corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IActionCondition">state condition</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IActionCondition">state condition</see> this operand corresponds to. A null
        /// reference on error.</returns>
        internal virtual ICondition GetCondition(float[] globalVariables, bool[] conditionalVariables) {
            return null;
        }

        /// <summary>
        /// Gets the <see cref="IEffect">state effect</see> this operand corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IEffect">state effect</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IEffect">state effect</see> this operand corresponds to. A null reference
        /// on error.</returns>
        internal virtual IEffect GetEffect(float[] globalVariables, bool[] conditionalVariables) {
            return null;
        }

        /// <summary>
        /// Gets the <see cref="IFunction">state calculated value</see> this operand corresponds to.
        /// </summary>
        /// <param name="globalVariables">The global variables which can be used by the
        /// <see cref="IFunction">state calculated value</see>.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The <see cref="IFunction">state calculated value</see> this operand corresponds to. A null
        /// reference on error.</returns>
        internal virtual IFunction GetFunction(float[] globalVariables, bool[] conditionalVariables) {
            return null;
        }

        #endregion

    }

}
