/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Core;
using Aran.Representation;
using Aran.Functional;

namespace Aran.Translation {

    /// <summary>
    /// Represents an operation <see cref="Operand">operand</see>.
    /// </summary>
    /// <typeparam name="T">The type of the variable identifiers.</typeparam>
    [Serializable]
    internal sealed class OperationOperand<T> : Operand {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        /*
         * NOTE: this class has not been documented in a detailed way due to its peculiar structure. It is, in
         * fact, an enormous case statement generated by simple combination. In it, simply again, some
         * combinations have been removed or adapted in order to use certain improvements.
         */

        /*
         * NOTE: A solution based on dynamic code generation was taken into account for solving "the functional
         * problem", but was discarded due to the lack of the needed types (from System.Reflection.Emit
         * namespace) to do so in the Compact Framework used by Xbox 360 and other target devices. Future
         * implementations may use this technique in those target devices which support it, if the estimated
         * additional translation time is less than the saved time when searching.
         */

        #region Fields

        /// <summary>
        /// The operation related translator messages.
        /// </summary>
        private readonly MessageCollection<IOperation<T>> _operationMessages;

        /// <summary>
        /// The knowledge provider source for this operation.
        /// </summary>
        private readonly IOperation<T> _source;

        /// <summary>
        /// The operation type.
        /// </summary>
        private readonly OperationType _operationType;

        /// <summary>
        /// The operand related translator messages.
        /// </summary>
        private readonly MessageCollection<IOperand<T>> _operandMessages;

        /// <summary>
        /// The left operand knowledge provider source.
        /// </summary>
        private readonly IOperand<T> _leftOperandSource;

        /// <summary>
        /// The left operand.
        /// </summary>
        private readonly Operand _leftOperand;

        /// <summary>
        /// The right operand knowledge provider source.
        /// </summary>
        private readonly IOperand<T> _rightOperandSource;

        /// <summary>
        /// The right operand.
        /// </summary>
        private readonly Operand _rightOperand;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="OperationOperand{T}"/> class.
        /// </summary>
        /// <param name="operationMessages">The operation related translator messages.</param>
        /// <param name="source">The knowledge provider source for this operation.</param>
        /// <param name="operationType">The operation type.</param>
        /// <param name="operandMessages">The operand related translator messages.</param>
        /// <param name="leftOperandSource">The left operand knowledge provider source.</param>
        /// <param name="leftOperand">The left operand.</param>
        /// <param name="rightOperandSource">The right operand knowledge provider source.</param>
        /// <param name="rightOperand">The right operand.</param>
        internal OperationOperand(MessageCollection<IOperation<T>> operationMessages,
        IOperation<T> source, OperationType operationType, MessageCollection<IOperand<T>> operandMessages,
        IOperand<T> leftOperandSource, Operand leftOperand, IOperand<T> rightOperandSource, Operand rightOperand) {
            _operationMessages = operationMessages;
            _source = source;
            _operationType = operationType;
            _operandMessages = operandMessages;
            _leftOperandSource = leftOperandSource;
            _leftOperand = leftOperand;
            _rightOperandSource = rightOperandSource;
            _rightOperand = rightOperand;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the type of the operand.
        /// </summary>
        /// <value>The type of the operand.</value>
        internal override OperandType OperandType {
            get {
                OperandType operandType = OperandType.Operation;
                if ((_leftOperand.OperandType == OperandType.Constant) &&
                (_rightOperand.OperandType == OperandType.Constant)) {
                    operandType = OperandType.Constant;
                }
                return operandType;
            }
        }

        /// <summary>
        /// Gets the type of the operation.
        /// </summary>
        /// <value>
        /// The type of the operation. <see cref="F:OperationType.Assignment">assignment</see> if the
        /// operand is not an operation.
        /// </value>
        internal override OperationType OperationType {
            get {
                return _operationType;
            }
        }

        /// <summary>
        /// Gets the value of the constant.
        /// </summary>
        /// <value>
        /// The value of the constant. Zero if the operand is not a constant.
        /// </value>
        internal override float ConstantValue {
            get {
                if (_leftOperand.OperandType != OperandType.Constant) {
                    _operandMessages.AddSemanticError(_leftOperandSource, MessageCode.LeftOperandMustBeConstant);
                    return 0;
                }
                if (_rightOperand.OperandType != OperandType.Constant) {
                    _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustBeConstant);
                    return 0;
                }
                float constantValue = _leftOperand.ConstantValue;
                float otherConstantValue = _rightOperand.ConstantValue;
                if (constantValue == 0) {
                    _operandMessages.AddSemanticWarning(_leftOperandSource, MessageCode.LeftOperandShouldNotBeZero);
                }
                switch (_operationType) {
                    case OperationType.Sum:
                        if (otherConstantValue == 0) {
                            _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                        }
                        return constantValue + otherConstantValue;
                    case OperationType.Subtraction:
                        if (otherConstantValue == 0) {
                            _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                        }
                        return constantValue - otherConstantValue;
                    case OperationType.Multiplication:
                        if (otherConstantValue == 0) {
                            _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                        }
                        if (constantValue == 1) {
                            _operandMessages.AddSemanticWarning(_leftOperandSource, MessageCode.LeftOperandShouldNotBeOne);
                        }
                        if (otherConstantValue == 1) {
                            _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                        }
                        return constantValue * otherConstantValue;
                    case OperationType.Division:
                        if (otherConstantValue == 0) {
                            _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                            return 0;
                        }
                        if (otherConstantValue == 1) {
                            _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                        }
                        return constantValue / otherConstantValue;
                    default:
                        _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                        return 0;
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionConstantVariableCondition(bool[] conditionalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionZeroVariableInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionOneVariableInferiority(variableIndex);
                    } else {
                        return new ActionConstantVariableInferiority(constantValue, variableIndex);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionZeroVariableNonSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionOneVariableNonSuperiority(variableIndex);
                    } else {
                        return new ActionConstantVariableNonSuperiority(constantValue, variableIndex);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionVariableZeroEquality(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneEquality(variableIndex);
                    } else {
                        return new ActionConstantVariableEquality(constantValue, variableIndex);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionVariableZeroInequality(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneInequality(variableIndex);
                    } else {
                        return new ActionConstantVariableInequality(constantValue, variableIndex);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionZeroVariableNonInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionOneVariableNonInferiority(variableIndex);
                    } else {
                        return new ActionConstantVariableNonInferiority(constantValue, variableIndex);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionZeroVariableSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionOneVariableSuperiority(variableIndex);
                    } else {
                        return new ActionConstantVariableSuperiority(constantValue, variableIndex);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionConstantGlobalVariableCondition(float[] globalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionZeroGlobalVariableInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionOneGlobalVariableInferiority(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableInferiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionZeroGlobalVariableNonSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionOneGlobalVariableNonSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableNonSuperiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroEquality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneEquality(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableEquality(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroInequality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneInequality(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableInequality(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionZeroGlobalVariableNonInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionOneGlobalVariableNonInferiority(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableNonInferiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionZeroGlobalVariableSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionOneGlobalVariableSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableSuperiority(constantValue, variableIndex, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionConstantResultCondition(float[] globalVariables, bool[] conditionalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionZeroResultInferiority(function);
                    } else if (constantValue == 1) {
                        return new ActionOneResultInferiority(function);
                    } else {
                        return new ActionConstantResultInferiority(constantValue, function);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionZeroResultNonSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ActionOneResultNonSuperiority(function);
                    } else {
                        return new ActionConstantResultNonSuperiority(constantValue, function);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionResultZeroEquality(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneEquality(function);
                    } else {
                        return new ActionConstantResultEquality(constantValue, function);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionResultZeroInequality(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneInequality(function);
                    } else {
                        return new ActionConstantResultInequality(constantValue, function);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionZeroResultNonInferiority(function);
                    } else if (constantValue == 1) {
                        return new ActionOneResultNonInferiority(function);
                    } else {
                        return new ActionConstantResultNonInferiority(constantValue, function);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionZeroResultSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ActionOneResultSuperiority(function);
                    } else {
                        return new ActionConstantResultSuperiority(constantValue, function);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and another variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionVariableVariableCondition(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionVariableVariableInferiority(variableIndex, otherVariableIndex);
                case OperationType.NonSuperiority:
                    return new ActionVariableVariableNonSuperiority(variableIndex, otherVariableIndex);
                case OperationType.Equality:
                    return new ActionVariableVariableEquality(variableIndex, otherVariableIndex);
                case OperationType.Inequality:
                    return new ActionVariableVariableInequality(variableIndex, otherVariableIndex);
                case OperationType.NonInferiority:
                    return new ActionVariableVariableNonInferiority(variableIndex, otherVariableIndex);
                case OperationType.Superiority:
                    return new ActionVariableVariableSuperiority(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionGlobalVariableVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionGlobalVariableVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new ActionGlobalVariableVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new ActionGlobalVariableVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new ActionGlobalVariableVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new ActionGlobalVariableVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new ActionGlobalVariableVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionVariableGlobalVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionVariableGlobalVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new ActionVariableGlobalVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new ActionVariableGlobalVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new ActionVariableGlobalVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new ActionVariableGlobalVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new ActionVariableGlobalVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionGlobalVariableGlobalVariableCondition(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionGlobalVariableGlobalVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new ActionGlobalVariableGlobalVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new ActionGlobalVariableGlobalVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new ActionGlobalVariableGlobalVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new ActionGlobalVariableGlobalVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new ActionGlobalVariableGlobalVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a constant right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionVariableConstantCondition(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionVariableZeroInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneInferiority(variableIndex);
                    } else {
                        return new ActionVariableConstantInferiority(variableIndex, constantValue);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionVariableZeroNonSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneNonSuperiority(variableIndex);
                    } else {
                        return new ActionVariableConstantNonSuperiority(variableIndex, constantValue);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionVariableZeroEquality(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneEquality(variableIndex);
                    } else {
                        return new ActionVariableConstantEquality(variableIndex, constantValue);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionVariableZeroInequality(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneInequality(variableIndex);
                    } else {
                        return new ActionVariableConstantInequality(variableIndex, constantValue);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionVariableZeroNonInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneNonInferiority(variableIndex);
                    } else {
                        return new ActionVariableConstantNonInferiority(variableIndex, constantValue);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionVariableZeroSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneSuperiority(variableIndex);
                    } else {
                        return new ActionVariableConstantSuperiority(variableIndex, constantValue);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionGlobalVariableConstantCondition(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneInferiority(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantInferiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroNonSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneNonSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantNonSuperiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroEquality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneEquality(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantEquality(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroInequality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneInequality(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantInequality(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroNonInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneNonInferiority(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantNonInferiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantSuperiority(variableIndex, constantValue, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionVariableResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionVariableResultInferiority(variableIndex, function);
                case OperationType.NonSuperiority:
                    return new ActionVariableResultNonSuperiority(variableIndex, function);
                case OperationType.Equality:
                    return new ActionVariableResultEquality(variableIndex, function);
                case OperationType.Inequality:
                    return new ActionVariableResultInequality(variableIndex, function);
                case OperationType.NonInferiority:
                    return new ActionVariableResultNonInferiority(variableIndex, function);
                case OperationType.Superiority:
                    return new ActionVariableResultSuperiority(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionGlobalVariableResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionGlobalVariableResultInferiority(variableIndex, function, globalVariables);
                case OperationType.NonSuperiority:
                    return new ActionGlobalVariableResultNonSuperiority(variableIndex, function, globalVariables);
                case OperationType.Equality:
                    return new ActionGlobalVariableResultEquality(variableIndex, function, globalVariables);
                case OperationType.Inequality:
                    return new ActionGlobalVariableResultInequality(variableIndex, function, globalVariables);
                case OperationType.NonInferiority:
                    return new ActionGlobalVariableResultNonInferiority(variableIndex, function, globalVariables);
                case OperationType.Superiority:
                    return new ActionGlobalVariableResultSuperiority(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionResultConstantCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ActionResultZeroInferiority(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneInferiority(function);
                    } else {
                        return new ActionResultConstantInferiority(function, constantValue);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ActionResultZeroNonSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneNonSuperiority(function);
                    } else {
                        return new ActionResultConstantNonSuperiority(function, constantValue);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ActionResultZeroEquality(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneEquality(function);
                    } else {
                        return new ActionResultConstantEquality(function, constantValue);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ActionResultZeroInequality(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneInequality(function);
                    } else {
                        return new ActionResultConstantInequality(function, constantValue);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ActionResultZeroNonInferiority(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneNonInferiority(function);
                    } else {
                        return new ActionResultConstantNonInferiority(function, constantValue);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ActionResultZeroSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ActionResultOneSuperiority(function);
                    } else {
                        return new ActionResultConstantSuperiority(function, constantValue);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionResultVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionResultVariableInferiority(function, variableIndex);
                case OperationType.NonSuperiority:
                    return new ActionResultVariableNonSuperiority(function, variableIndex);
                case OperationType.Equality:
                    return new ActionResultVariableEquality(function, variableIndex);
                case OperationType.Inequality:
                    return new ActionResultVariableInequality(function, variableIndex);
                case OperationType.NonInferiority:
                    return new ActionResultVariableNonInferiority(function, variableIndex);
                case OperationType.Superiority:
                    return new ActionResultVariableSuperiority(function, variableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionResultGlobalVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ActionResultGlobalVariableInferiority(function, variableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new ActionResultGlobalVariableNonSuperiority(function, variableIndex, globalVariables);
                case OperationType.Equality:
                    return new ActionResultGlobalVariableEquality(function, variableIndex, globalVariables);
                case OperationType.Inequality:
                    return new ActionResultGlobalVariableInequality(function, variableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new ActionResultGlobalVariableNonInferiority(function, variableIndex, globalVariables);
                case OperationType.Superiority:
                    return new ActionResultGlobalVariableSuperiority(function, variableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and another result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private IActionCondition GetActionResultResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            if ((_operationType == OperationType.Conjunction) ||
            (_operationType == OperationType.Disjunction)) {
                IActionCondition condition =
                    _leftOperand.GetActionCondition(globalVariables, conditionalVariables);
                IActionCondition otherCondition =
                    _rightOperand.GetActionCondition(globalVariables, conditionalVariables);
                switch (_operationType) {
                    case OperationType.Conjunction:
                        //TODO: optimice this by joining possible subconjunctions
                        return new ActionMultipleConditionConjunction(condition, otherCondition);
                    case OperationType.Disjunction:
                        //TODO: optimice this by joining possible subdisjunctions
                        return new ActionMultipleConditionDisjunction(condition, otherCondition);
                    default:
                        _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                        return null;
                }
            } else {
                IActionFunction function =
                    _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
                IActionFunction otherFunction =
                    _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
                switch (_operationType) {
                    case OperationType.Inferiority:
                        return new ActionResultResultInferiority(function, otherFunction);
                    case OperationType.NonSuperiority:
                        return new ActionResultResultNonSuperiority(function, otherFunction);
                    case OperationType.Equality:
                        return new ActionResultResultEquality(function, otherFunction);
                    case OperationType.Inequality:
                        return new ActionResultResultInequality(function, otherFunction);
                    case OperationType.NonInferiority:
                        return new ActionResultResultNonInferiority(function, otherFunction);
                    case OperationType.Superiority:
                        return new ActionResultResultSuperiority(function, otherFunction);
                    default:
                        _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                        return null;
                }
            }
        }

        /// <summary>
        /// Gets an action-dependant condition.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        internal override IActionCondition GetActionCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            switch (_leftOperand.OperandType) {
                case OperandType.Constant:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetActionConstantGlobalVariableCondition(globalVariables);
                            } else {
                                return GetActionConstantVariableCondition(conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetActionConstantResultCondition(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeConstant);
                            return null;
                    }
                case OperandType.Variable:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetActionGlobalVariableConstantCondition(globalVariables);
                            } else {
                                return GetActionVariableConstantCondition(conditionalVariables);
                            }
                        case OperandType.Variable:
                            if (_leftOperand.IsGlobalVariable) {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetActionGlobalVariableGlobalVariableCondition(globalVariables);
                                } else {
                                    return GetActionGlobalVariableVariableCondition(globalVariables,
                                        conditionalVariables);
                                }
                            } else {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetActionVariableGlobalVariableCondition(globalVariables,
                                        conditionalVariables);
                                } else {
                                    return GetActionVariableVariableCondition(conditionalVariables);
                                }
                            }
                        case OperandType.Operation:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetActionGlobalVariableResultCondition(globalVariables, conditionalVariables);
                            } else {
                                return GetActionVariableResultCondition(globalVariables, conditionalVariables);
                            }
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                case OperandType.Operation:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            return GetActionResultConstantCondition(globalVariables, conditionalVariables);
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetActionResultGlobalVariableCondition(globalVariables, conditionalVariables);
                            } else {
                                return GetActionResultVariableCondition(globalVariables, conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetActionResultResultCondition(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                default:
                    _operandMessages.AddSyntacticError(_leftOperandSource, MessageCode.InvalidLeftOperandType);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a constant right operand.
        /// </summary>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionVariableConstantEffect() {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Assignment:
                    if (constantValue == 0) {
                        return new ActionVariableZeroAssignment(variableIndex);
                    } else if (constantValue == 1) {
                        return new ActionVariableOneAssignment(variableIndex);
                    } else {
                        return new ActionVariableConstantAssignment(variableIndex, constantValue);
                    }
                case OperationType.SumAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionVariableOneSumAssignment(variableIndex);
                    } else {
                        return new ActionVariableConstantSumAssignment(variableIndex, constantValue);
                    }
                case OperationType.SubtractionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionVariableOneSubtractionAssignment(variableIndex);
                    } else {
                        return new ActionVariableConstantSubtractionAssignment(variableIndex, constantValue);
                    }
                case OperationType.MultiplicationAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionVariableConstantMultiplicationAssignment(variableIndex, constantValue);
                case OperationType.DivisionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionVariableConstantDivisionAssignment(variableIndex, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionGlobalVariableConstantEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Assignment:
                    if (constantValue == 0) {
                        return new ActionGlobalVariableZeroAssignment(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new ActionGlobalVariableOneAssignment(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.SumAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    if (constantValue == 1) {
                        return new ActionGlobalVariableOneSumAssignment(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantSumAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.SubtractionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionGlobalVariableOneSubtractionAssignment(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantSubtractionAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.MultiplicationAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionGlobalVariableConstantMultiplicationAssignment(variableIndex, constantValue, globalVariables);
                case OperationType.DivisionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionGlobalVariableConstantDivisionAssignment(variableIndex, constantValue, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a variable right operand.
        /// </summary>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionVariableVariableEffect() {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionVariableVariableAssignment(variableIndex, otherVariableIndex);
                case OperationType.SumAssignment:
                    return new ActionVariableVariableSumAssignment(variableIndex, otherVariableIndex);
                case OperationType.SubtractionAssignment:
                    return new ActionVariableVariableSubtractionAssignment(variableIndex, otherVariableIndex);
                case OperationType.MultiplicationAssignment:
                    return new ActionVariableVariableMultiplicationAssignment(variableIndex, otherVariableIndex);
                case OperationType.DivisionAssignment:
                    return new ActionVariableVariableDivisionAssignment(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionGlobalVariableVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionGlobalVariableVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new ActionGlobalVariableVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new ActionGlobalVariableVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new ActionGlobalVariableVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new ActionGlobalVariableVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionVariableGlobalVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionVariableGlobalVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new ActionVariableGlobalVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new ActionVariableGlobalVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new ActionVariableGlobalVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new ActionVariableGlobalVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionGlobalVariableGlobalVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionGlobalVariableGlobalVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new ActionGlobalVariableGlobalVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new ActionGlobalVariableGlobalVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new ActionGlobalVariableGlobalVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new ActionGlobalVariableGlobalVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionVariableResultEffect(float[] globalVariables, bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionVariableResultAssignment(variableIndex, function);
                case OperationType.SumAssignment:
                    return new ActionVariableResultSumAssignment(variableIndex, function);
                case OperationType.SubtractionAssignment:
                    return new ActionVariableResultSubtractionAssignment(variableIndex, function);
                case OperationType.MultiplicationAssignment:
                    return new ActionVariableResultMultiplicationAssignment(variableIndex, function);
                case OperationType.DivisionAssignment:
                    return new ActionVariableResultDivisionAssignment(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        private IActionEffect GetActionGlobalVariableResultEffect(float[] globalVariables, bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Assignment:
                    return new ActionGlobalVariableResultAssignment(variableIndex, function, globalVariables);
                case OperationType.SumAssignment:
                    return new ActionGlobalVariableResultSumAssignment(variableIndex, function, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new ActionGlobalVariableResultSubtractionAssignment(variableIndex, function, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new ActionGlobalVariableResultMultiplicationAssignment(variableIndex, function, globalVariables);
                case OperationType.DivisionAssignment:
                    return new ActionGlobalVariableResultDivisionAssignment(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        internal override IActionEffect GetActionEffect(float[] globalVariables, bool[] conditionalVariables) {
            if (_leftOperand.OperandType != OperandType.Variable) {
                _operandMessages.AddSemanticError(_leftOperandSource, MessageCode.LeftOperandMustBeVariable);
                return null;
            }
            switch (_rightOperand.OperandType) {
                case OperandType.Constant:
                    if (_leftOperand.IsGlobalVariable) {
                        return GetActionGlobalVariableConstantEffect(globalVariables);
                    } else {
                        return GetActionVariableConstantEffect();
                    }
                case OperandType.Variable:
                    if (_rightOperand.IsGlobalVariable) {
                        if (_leftOperand.IsGlobalVariable) {
                            return GetActionGlobalVariableGlobalVariableEffect(globalVariables);
                        } else {
                            return GetActionVariableGlobalVariableEffect(globalVariables);
                        }
                    } else {
                        if (_leftOperand.IsGlobalVariable) {
                            return GetActionGlobalVariableVariableEffect(globalVariables);
                        } else {
                            return GetActionVariableVariableEffect();
                        }
                    }
                case OperandType.Operation:
                    if (_leftOperand.IsGlobalVariable) {
                        return GetActionGlobalVariableResultEffect(globalVariables, conditionalVariables);
                    } else {
                        return GetActionVariableResultEffect(globalVariables, conditionalVariables);
                    }
                default:
                    _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionConstantVariableFunction(bool[] conditionalVariables) {
            float constantValue = _rightOperand.ConstantValue;
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionVariableOneSum(variableIndex);
                    } else {
                        return new ActionConstantVariableSum(constantValue, variableIndex);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneVariableSubtraction(variableIndex);
                    } else {
                        return new ActionConstantVariableSubtraction(constantValue, variableIndex);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionConstantVariableMultiplication(constantValue, variableIndex);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneVariableDivision(variableIndex);
                    } else {
                        return new ActionConstantVariableDivision(constantValue, variableIndex);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a global right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionConstantGlobalVariableFunction(float[] globalVariables) {
            float constantValue = _rightOperand.ConstantValue;
            int variableIndex = _leftOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionGlobalVariableOneSum(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableSum(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneGlobalVariableSubtraction(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableSubtraction(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionConstantGlobalVariableMultiplication(constantValue, variableIndex, globalVariables);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneGlobalVariableDivision(variableIndex, globalVariables);
                    } else {
                        return new ActionConstantGlobalVariableDivision(constantValue, variableIndex, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionConstantResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int constantValue = _rightOperand.VariableIndex;
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionResultOneSum(function);
                    } else {
                        return new ActionConstantResultSum(constantValue, function);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneResultSubtraction(function);
                    } else {
                        return new ActionConstantResultSubtraction(constantValue, function);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionConstantResultMultiplication(constantValue, function);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionOneResultDivision(function);
                    } else {
                        return new ActionConstantResultDivision(constantValue, function);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a constant right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionVariableConstantFunction(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionVariableOneSum(variableIndex);
                    } else {
                        return new ActionVariableConstantSum(variableIndex, constantValue);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionVariableOneSubtraction(variableIndex);
                    } else {
                        return new ActionVariableConstantSubtraction(variableIndex, constantValue);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionVariableConstantMultiplication(variableIndex, constantValue);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionVariableConstantDivision(variableIndex, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionGlobalVariableConstantFunction(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionGlobalVariableOneSum(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantSum(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionGlobalVariableOneSubtraction(variableIndex, globalVariables);
                    } else {
                        return new ActionGlobalVariableConstantSubtraction(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionGlobalVariableConstantMultiplication(variableIndex, constantValue, globalVariables);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionGlobalVariableConstantDivision(variableIndex, constantValue, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionVariableVariableFunction(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionVariableVariableSum(variableIndex, otherVariableIndex);
                case OperationType.Subtraction:
                    return new ActionVariableVariableSubtraction(variableIndex, otherVariableIndex);
                case OperationType.Multiplication:
                    return new ActionVariableVariableMultiplication(variableIndex, otherVariableIndex);
                case OperationType.Division:
                    return new ActionVariableVariableDivision(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionGlobalVariableVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionGlobalVariableVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new ActionGlobalVariableVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new ActionGlobalVariableVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new ActionGlobalVariableVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionVariableGlobalVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionVariableGlobalVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new ActionVariableGlobalVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new ActionVariableGlobalVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new ActionVariableGlobalVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionGlobalVariableGlobalVariableFunction(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionGlobalVariableGlobalVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new ActionGlobalVariableGlobalVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new ActionGlobalVariableGlobalVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new ActionGlobalVariableGlobalVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionVariableResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionVariableResultSum(variableIndex, function);
                case OperationType.Subtraction:
                    return new ActionVariableResultSubtraction(variableIndex, function);
                case OperationType.Multiplication:
                    return new ActionVariableResultMultiplication(variableIndex, function);
                case OperationType.Division:
                    return new ActionVariableResultDivision(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionGlobalVariableResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IActionFunction function = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionGlobalVariableResultSum(variableIndex, function, globalVariables);
                case OperationType.Subtraction:
                    return new ActionGlobalVariableResultSubtraction(variableIndex, function, globalVariables);
                case OperationType.Multiplication:
                    return new ActionGlobalVariableResultMultiplication(variableIndex, function, globalVariables);
                case OperationType.Division:
                    return new ActionGlobalVariableResultDivision(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionResultConstantFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionResultOneSum(function);
                    } else {
                        return new ActionResultConstantSum(function, constantValue);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ActionResultOneSubtraction(function);
                    } else {
                        return new ActionResultConstantSubtraction(function, constantValue);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionResultConstantMultiplication(function, constantValue);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ActionResultConstantDivision(function, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionResultVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionResultVariableSum(function, variableIndex);
                case OperationType.Subtraction:
                    return new ActionResultVariableSubtraction(function, variableIndex);
                case OperationType.Multiplication:
                    return new ActionResultVariableMultiplication(function, variableIndex);
                case OperationType.Division:
                    return new ActionResultVariableDivision(function, variableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionResultGlobalVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionResultGlobalVariableSum(function, variableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new ActionResultGlobalVariableSubtraction(function, variableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new ActionResultGlobalVariableMultiplication(function, variableIndex, globalVariables);
                case OperationType.Division:
                    return new ActionResultGlobalVariableDivision(function, variableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and another result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IActionFunction GetActionResultResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IActionFunction function = _leftOperand.GetActionFunction(globalVariables, conditionalVariables);
            IActionFunction otherFunction = _rightOperand.GetActionFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new ActionResultResultSum(function, otherFunction);
                case OperationType.Subtraction:
                    return new ActionResultResultSubtraction(function, otherFunction);
                case OperationType.Multiplication:
                    return new ActionResultResultMultiplication(function, otherFunction);
                case OperationType.Division:
                    return new ActionResultResultDivision(function, otherFunction);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>x
        /// <returns>The function.</returns>
        internal override IActionFunction GetActionFunction(float[] globalVariables, bool[] conditionalVariables) {
            switch (_leftOperand.OperandType) {
                case OperandType.Constant:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetActionConstantGlobalVariableFunction(globalVariables);
                            } else {
                                return GetActionConstantVariableFunction(conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetActionConstantResultFunction(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustBeConstant);
                            return null;
                    }
                case OperandType.Variable:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetActionGlobalVariableConstantFunction(globalVariables);
                            } else {
                                return GetActionVariableConstantFunction(conditionalVariables);
                            }
                        case OperandType.Variable:
                            if (_leftOperand.IsGlobalVariable) {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetActionGlobalVariableGlobalVariableFunction(globalVariables);
                                } else {
                                    return GetActionGlobalVariableVariableFunction(globalVariables,
                                        conditionalVariables);
                                }
                            } else {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetActionVariableGlobalVariableFunction(globalVariables,
                                        conditionalVariables);
                                } else {
                                    return GetActionVariableVariableFunction(conditionalVariables);
                                }
                            }
                        case OperandType.Operation:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetActionGlobalVariableResultFunction(globalVariables, conditionalVariables);
                            } else {
                                return GetActionVariableResultFunction(globalVariables, conditionalVariables);
                            }
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                case OperandType.Operation:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            return GetActionResultConstantFunction(globalVariables, conditionalVariables);
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetActionResultGlobalVariableFunction(globalVariables, conditionalVariables);
                            } else {
                                return GetActionResultVariableFunction(globalVariables, conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetActionResultResultFunction(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                default:
                    _operandMessages.AddSyntacticError(_leftOperandSource, MessageCode.InvalidLeftOperandType);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetConstantVariableCondition(bool[] conditionalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ZeroVariableInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new OneVariableInferiority(variableIndex);
                    } else {
                        return new ConstantVariableInferiority(constantValue, variableIndex);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ZeroVariableNonSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new OneVariableNonSuperiority(variableIndex);
                    } else {
                        return new ConstantVariableNonSuperiority(constantValue, variableIndex);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new VariableZeroEquality(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneEquality(variableIndex);
                    } else {
                        return new ConstantVariableEquality(constantValue, variableIndex);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new VariableZeroInequality(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneInequality(variableIndex);
                    } else {
                        return new ConstantVariableInequality(constantValue, variableIndex);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ZeroVariableNonInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new OneVariableNonInferiority(variableIndex);
                    } else {
                        return new ConstantVariableNonInferiority(constantValue, variableIndex);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ZeroVariableSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new OneVariableSuperiority(variableIndex);
                    } else {
                        return new ConstantVariableSuperiority(constantValue, variableIndex);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private ICondition GetConstantGlobalVariableCondition(float[] globalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ZeroGlobalVariableInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new OneGlobalVariableInferiority(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableInferiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ZeroGlobalVariableNonSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new OneGlobalVariableNonSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableNonSuperiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroEquality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneEquality(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableEquality(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroInequality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneInequality(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableInequality(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ZeroGlobalVariableNonInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new OneGlobalVariableNonInferiority(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableNonInferiority(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ZeroGlobalVariableSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new OneGlobalVariableSuperiority(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableSuperiority(constantValue, variableIndex, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a constant left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetConstantResultCondition(float[] globalVariables, bool[] conditionalVariables) {
            float constantValue = _leftOperand.ConstantValue;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ZeroResultInferiority(function);
                    } else if (constantValue == 1) {
                        return new OneResultInferiority(function);
                    } else {
                        return new ConstantResultInferiority(constantValue, function);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ZeroResultNonSuperiority(function);
                    } else if (constantValue == 1) {
                        return new OneResultNonSuperiority(function);
                    } else {
                        return new ConstantResultNonSuperiority(constantValue, function);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ResultZeroEquality(function);
                    } else if (constantValue == 1) {
                        return new ResultOneEquality(function);
                    } else {
                        return new ConstantResultEquality(constantValue, function);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ResultZeroInequality(function);
                    } else if (constantValue == 1) {
                        return new ResultOneInequality(function);
                    } else {
                        return new ConstantResultInequality(constantValue, function);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ZeroResultNonInferiority(function);
                    } else if (constantValue == 1) {
                        return new OneResultNonInferiority(function);
                    } else {
                        return new ConstantResultNonInferiority(constantValue, function);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ZeroResultSuperiority(function);
                    } else if (constantValue == 1) {
                        return new OneResultSuperiority(function);
                    } else {
                        return new ConstantResultSuperiority(constantValue, function);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and another variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetVariableVariableCondition(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new VariableVariableInferiority(variableIndex, otherVariableIndex);
                case OperationType.NonSuperiority:
                    return new VariableVariableNonSuperiority(variableIndex, otherVariableIndex);
                case OperationType.Equality:
                    return new VariableVariableEquality(variableIndex, otherVariableIndex);
                case OperationType.Inequality:
                    return new VariableVariableInequality(variableIndex, otherVariableIndex);
                case OperationType.NonInferiority:
                    return new VariableVariableNonInferiority(variableIndex, otherVariableIndex);
                case OperationType.Superiority:
                    return new VariableVariableSuperiority(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetGlobalVariableVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new GlobalVariableVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new GlobalVariableVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new GlobalVariableVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new GlobalVariableVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new GlobalVariableVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new GlobalVariableVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetVariableGlobalVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new VariableGlobalVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new VariableGlobalVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new VariableGlobalVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new VariableGlobalVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new VariableGlobalVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new VariableGlobalVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private ICondition GetGlobalVariableGlobalVariableCondition(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new GlobalVariableGlobalVariableInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new GlobalVariableGlobalVariableNonSuperiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Equality:
                    return new GlobalVariableGlobalVariableEquality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Inequality:
                    return new GlobalVariableGlobalVariableInequality(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new GlobalVariableGlobalVariableNonInferiority(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Superiority:
                    return new GlobalVariableGlobalVariableSuperiority(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a constant right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetVariableConstantCondition(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new VariableZeroInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneInferiority(variableIndex);
                    } else {
                        return new VariableConstantInferiority(variableIndex, constantValue);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new VariableZeroNonSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneNonSuperiority(variableIndex);
                    } else {
                        return new VariableConstantNonSuperiority(variableIndex, constantValue);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new VariableZeroEquality(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneEquality(variableIndex);
                    } else {
                        return new VariableConstantEquality(variableIndex, constantValue);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new VariableZeroInequality(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneInequality(variableIndex);
                    } else {
                        return new VariableConstantInequality(variableIndex, constantValue);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new VariableZeroNonInferiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneNonInferiority(variableIndex);
                    } else {
                        return new VariableConstantNonInferiority(variableIndex, constantValue);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new VariableZeroSuperiority(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneSuperiority(variableIndex);
                    } else {
                        return new VariableConstantSuperiority(variableIndex, constantValue);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <returns>The condition.</returns>
        private ICondition GetGlobalVariableConstantCondition(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneInferiority(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantInferiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroNonSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneNonSuperiority(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantNonSuperiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroEquality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneEquality(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantEquality(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroInequality(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneInequality(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantInequality(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroNonInferiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneNonInferiority(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantNonInferiority(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroSuperiority(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneSuperiority(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantSuperiority(variableIndex, constantValue, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetVariableResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new VariableResultInferiority(variableIndex, function);
                case OperationType.NonSuperiority:
                    return new VariableResultNonSuperiority(variableIndex, function);
                case OperationType.Equality:
                    return new VariableResultEquality(variableIndex, function);
                case OperationType.Inequality:
                    return new VariableResultInequality(variableIndex, function);
                case OperationType.NonInferiority:
                    return new VariableResultNonInferiority(variableIndex, function);
                case OperationType.Superiority:
                    return new VariableResultSuperiority(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetGlobalVariableResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new GlobalVariableResultInferiority(variableIndex, function, globalVariables);
                case OperationType.NonSuperiority:
                    return new GlobalVariableResultNonSuperiority(variableIndex, function, globalVariables);
                case OperationType.Equality:
                    return new GlobalVariableResultEquality(variableIndex, function, globalVariables);
                case OperationType.Inequality:
                    return new GlobalVariableResultInequality(variableIndex, function, globalVariables);
                case OperationType.NonInferiority:
                    return new GlobalVariableResultNonInferiority(variableIndex, function, globalVariables);
                case OperationType.Superiority:
                    return new GlobalVariableResultSuperiority(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetResultConstantCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Inferiority:
                    if (constantValue == 0) {
                        return new ResultZeroInferiority(function);
                    } else if (constantValue == 1) {
                        return new ResultOneInferiority(function);
                    } else {
                        return new ResultConstantInferiority(function, constantValue);
                    }
                case OperationType.NonSuperiority:
                    if (constantValue == 0) {
                        return new ResultZeroNonSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ResultOneNonSuperiority(function);
                    } else {
                        return new ResultConstantNonSuperiority(function, constantValue);
                    }
                case OperationType.Equality:
                    if (constantValue == 0) {
                        return new ResultZeroEquality(function);
                    } else if (constantValue == 1) {
                        return new ResultOneEquality(function);
                    } else {
                        return new ResultConstantEquality(function, constantValue);
                    }
                case OperationType.Inequality:
                    if (constantValue == 0) {
                        return new ResultZeroInequality(function);
                    } else if (constantValue == 1) {
                        return new ResultOneInequality(function);
                    } else {
                        return new ResultConstantInequality(function, constantValue);
                    }
                case OperationType.NonInferiority:
                    if (constantValue == 0) {
                        return new ResultZeroNonInferiority(function);
                    } else if (constantValue == 1) {
                        return new ResultOneNonInferiority(function);
                    } else {
                        return new ResultConstantNonInferiority(function, constantValue);
                    }
                case OperationType.Superiority:
                    if (constantValue == 0) {
                        return new ResultZeroSuperiority(function);
                    } else if (constantValue == 1) {
                        return new ResultOneSuperiority(function);
                    } else {
                        return new ResultConstantSuperiority(function, constantValue);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetResultVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ResultVariableInferiority(function, variableIndex);
                case OperationType.NonSuperiority:
                    return new ResultVariableNonSuperiority(function, variableIndex);
                case OperationType.Equality:
                    return new ResultVariableEquality(function, variableIndex);
                case OperationType.Inequality:
                    return new ResultVariableInequality(function, variableIndex);
                case OperationType.NonInferiority:
                    return new ResultVariableNonInferiority(function, variableIndex);
                case OperationType.Superiority:
                    return new ResultVariableSuperiority(function, variableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetResultGlobalVariableCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Inferiority:
                    return new ResultGlobalVariableInferiority(function, variableIndex, globalVariables);
                case OperationType.NonSuperiority:
                    return new ResultGlobalVariableNonSuperiority(function, variableIndex, globalVariables);
                case OperationType.Equality:
                    return new ResultGlobalVariableEquality(function, variableIndex, globalVariables);
                case OperationType.Inequality:
                    return new ResultGlobalVariableInequality(function, variableIndex, globalVariables);
                case OperationType.NonInferiority:
                    return new ResultGlobalVariableNonInferiority(function, variableIndex, globalVariables);
                case OperationType.Superiority:
                    return new ResultGlobalVariableSuperiority(function, variableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant condition with a result left operand and another result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        private ICondition GetResultResultCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            if ((_operationType == OperationType.Conjunction) ||
            (_operationType == OperationType.Disjunction)) {
                ICondition condition =
                    _leftOperand.GetCondition(globalVariables, conditionalVariables);
                ICondition otherCondition =
                    _rightOperand.GetCondition(globalVariables, conditionalVariables);
                switch (_operationType) {
                    case OperationType.Conjunction:
                        //TODO: optimice this by joining possible subconjunctions
                        return new MultipleConditionConjunction(condition, otherCondition);
                    case OperationType.Disjunction:
                        //TODO: optimice this by joining possible subdisjunctions
                        return new MultipleConditionDisjunction(condition, otherCondition);
                    default:
                        _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                        return null;
                }
            } else {
                IFunction function =
                    _leftOperand.GetFunction(globalVariables, conditionalVariables);
                IFunction otherFunction =
                    _rightOperand.GetFunction(globalVariables, conditionalVariables);
                switch (_operationType) {
                    case OperationType.Inferiority:
                        return new ResultResultInferiority(function, otherFunction);
                    case OperationType.NonSuperiority:
                        return new ResultResultNonSuperiority(function, otherFunction);
                    case OperationType.Equality:
                        return new ResultResultEquality(function, otherFunction);
                    case OperationType.Inequality:
                        return new ResultResultInequality(function, otherFunction);
                    case OperationType.NonInferiority:
                        return new ResultResultNonInferiority(function, otherFunction);
                    case OperationType.Superiority:
                        return new ResultResultSuperiority(function, otherFunction);
                    default:
                        _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotCondition);
                        return null;
                }
            }
        }

        /// <summary>
        /// Gets an action-dependant condition.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the condition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The condition.</returns>
        internal override ICondition GetCondition(float[] globalVariables,
        bool[] conditionalVariables) {
            switch (_leftOperand.OperandType) {
                case OperandType.Constant:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetConstantGlobalVariableCondition(globalVariables);
                            } else {
                                return GetConstantVariableCondition(conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetConstantResultCondition(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeConstant);
                            return null;
                    }
                case OperandType.Variable:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetGlobalVariableConstantCondition(globalVariables);
                            } else {
                                return GetVariableConstantCondition(conditionalVariables);
                            }
                        case OperandType.Variable:
                            if (_leftOperand.IsGlobalVariable) {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetGlobalVariableGlobalVariableCondition(globalVariables);
                                } else {
                                    return GetGlobalVariableVariableCondition(globalVariables,
                                        conditionalVariables);
                                }
                            } else {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetVariableGlobalVariableCondition(globalVariables,
                                        conditionalVariables);
                                } else {
                                    return GetVariableVariableCondition(conditionalVariables);
                                }
                            }
                        case OperandType.Operation:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetGlobalVariableResultCondition(globalVariables, conditionalVariables);
                            } else {
                                return GetVariableResultCondition(globalVariables, conditionalVariables);
                            }
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                case OperandType.Operation:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            return GetResultConstantCondition(globalVariables, conditionalVariables);
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetResultGlobalVariableCondition(globalVariables, conditionalVariables);
                            } else {
                                return GetResultVariableCondition(globalVariables, conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetResultResultCondition(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                default:
                    _operandMessages.AddSyntacticError(_leftOperandSource, MessageCode.InvalidLeftOperandType);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a constant right operand.
        /// </summary>
        /// <returns>The effect.</returns>
        private IEffect GetVariableConstantEffect() {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Assignment:
                    if (constantValue == 0) {
                        return new VariableZeroAssignment(variableIndex);
                    } else if (constantValue == 1) {
                        return new VariableOneAssignment(variableIndex);
                    } else {
                        return new VariableConstantAssignment(variableIndex, constantValue);
                    }
                case OperationType.SumAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new VariableOneSumAssignment(variableIndex);
                    } else {
                        return new VariableConstantSumAssignment(variableIndex, constantValue);
                    }
                case OperationType.SubtractionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new VariableOneSubtractionAssignment(variableIndex);
                    } else {
                        return new VariableConstantSubtractionAssignment(variableIndex, constantValue);
                    }
                case OperationType.MultiplicationAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new VariableConstantMultiplicationAssignment(variableIndex, constantValue);
                case OperationType.DivisionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new VariableConstantDivisionAssignment(variableIndex, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IEffect GetGlobalVariableConstantEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Assignment:
                    if (constantValue == 0) {
                        return new GlobalVariableZeroAssignment(variableIndex, globalVariables);
                    } else if (constantValue == 1) {
                        return new GlobalVariableOneAssignment(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.SumAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    if (constantValue == 1) {
                        return new GlobalVariableOneSumAssignment(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantSumAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.SubtractionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new GlobalVariableOneSubtractionAssignment(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantSubtractionAssignment(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.MultiplicationAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new GlobalVariableConstantMultiplicationAssignment(variableIndex, constantValue, globalVariables);
                case OperationType.DivisionAssignment:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new GlobalVariableConstantDivisionAssignment(variableIndex, constantValue, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a variable right operand.
        /// </summary>
        /// <returns>The effect.</returns>
        private IEffect GetVariableVariableEffect() {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new VariableVariableAssignment(variableIndex, otherVariableIndex);
                case OperationType.SumAssignment:
                    return new VariableVariableSumAssignment(variableIndex, otherVariableIndex);
                case OperationType.SubtractionAssignment:
                    return new VariableVariableSubtractionAssignment(variableIndex, otherVariableIndex);
                case OperationType.MultiplicationAssignment:
                    return new VariableVariableMultiplicationAssignment(variableIndex, otherVariableIndex);
                case OperationType.DivisionAssignment:
                    return new VariableVariableDivisionAssignment(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IEffect GetGlobalVariableVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new GlobalVariableVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new GlobalVariableVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new GlobalVariableVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new GlobalVariableVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new GlobalVariableVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IEffect GetVariableGlobalVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new VariableGlobalVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new VariableGlobalVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new VariableGlobalVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new VariableGlobalVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new VariableGlobalVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <returns>The effect.</returns>
        private IEffect GetGlobalVariableGlobalVariableEffect(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Assignment:
                    return new GlobalVariableGlobalVariableAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SumAssignment:
                    return new GlobalVariableGlobalVariableSumAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new GlobalVariableGlobalVariableSubtractionAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new GlobalVariableGlobalVariableMultiplicationAssignment(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.DivisionAssignment:
                    return new GlobalVariableGlobalVariableDivisionAssignment(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        private IEffect GetVariableResultEffect(float[] globalVariables, bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Assignment:
                    return new VariableResultAssignment(variableIndex, function);
                case OperationType.SumAssignment:
                    return new VariableResultSumAssignment(variableIndex, function);
                case OperationType.SubtractionAssignment:
                    return new VariableResultSubtractionAssignment(variableIndex, function);
                case OperationType.MultiplicationAssignment:
                    return new VariableResultMultiplicationAssignment(variableIndex, function);
                case OperationType.DivisionAssignment:
                    return new VariableResultDivisionAssignment(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        private IEffect GetGlobalVariableResultEffect(float[] globalVariables, bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Assignment:
                    return new GlobalVariableResultAssignment(variableIndex, function, globalVariables);
                case OperationType.SumAssignment:
                    return new GlobalVariableResultSumAssignment(variableIndex, function, globalVariables);
                case OperationType.SubtractionAssignment:
                    return new GlobalVariableResultSubtractionAssignment(variableIndex, function, globalVariables);
                case OperationType.MultiplicationAssignment:
                    return new GlobalVariableResultMultiplicationAssignment(variableIndex, function, globalVariables);
                case OperationType.DivisionAssignment:
                    return new GlobalVariableResultDivisionAssignment(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotEffect);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant effect.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The effect.</returns>
        internal override IEffect GetEffect(float[] globalVariables, bool[] conditionalVariables) {
            if (_leftOperand.OperandType != OperandType.Variable) {
                _operandMessages.AddSemanticError(_leftOperandSource, MessageCode.LeftOperandMustBeVariable);
                return null;
            }
            switch (_rightOperand.OperandType) {
                case OperandType.Constant:
                    if (_leftOperand.IsGlobalVariable) {
                        return GetGlobalVariableConstantEffect(globalVariables);
                    } else {
                        return GetVariableConstantEffect();
                    }
                case OperandType.Variable:
                    if (_rightOperand.IsGlobalVariable) {
                        if (_leftOperand.IsGlobalVariable) {
                            return GetGlobalVariableGlobalVariableEffect(globalVariables);
                        } else {
                            return GetVariableGlobalVariableEffect(globalVariables);
                        }
                    } else {
                        if (_leftOperand.IsGlobalVariable) {
                            return GetGlobalVariableVariableEffect(globalVariables);
                        } else {
                            return GetVariableVariableEffect();
                        }
                    }
                case OperandType.Operation:
                    if (_leftOperand.IsGlobalVariable) {
                        return GetGlobalVariableResultEffect(globalVariables, conditionalVariables);
                    } else {
                        return GetVariableResultEffect(globalVariables, conditionalVariables);
                    }
                default:
                    _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetConstantVariableFunction(bool[] conditionalVariables) {
            float constantValue = _rightOperand.ConstantValue;
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new VariableOneSum(variableIndex);
                    } else {
                        return new ConstantVariableSum(constantValue, variableIndex);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneVariableSubtraction(variableIndex);
                    } else {
                        return new ConstantVariableSubtraction(constantValue, variableIndex);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ConstantVariableMultiplication(constantValue, variableIndex);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneVariableDivision(variableIndex);
                    } else {
                        return new ConstantVariableDivision(constantValue, variableIndex);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a global right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IFunction GetConstantGlobalVariableFunction(float[] globalVariables) {
            float constantValue = _rightOperand.ConstantValue;
            int variableIndex = _leftOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new GlobalVariableOneSum(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableSum(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneGlobalVariableSubtraction(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableSubtraction(constantValue, variableIndex, globalVariables);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ConstantGlobalVariableMultiplication(constantValue, variableIndex, globalVariables);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneGlobalVariableDivision(variableIndex, globalVariables);
                    } else {
                        return new ConstantGlobalVariableDivision(constantValue, variableIndex, globalVariables);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a constant left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetConstantResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int constantValue = _rightOperand.VariableIndex;
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ResultOneSum(function);
                    } else {
                        return new ConstantResultSum(constantValue, function);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneResultSubtraction(function);
                    } else {
                        return new ConstantResultSubtraction(constantValue, function);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ConstantResultMultiplication(constantValue, function);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new OneResultDivision(function);
                    } else {
                        return new ConstantResultDivision(constantValue, function);
                    }
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a constant right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetVariableConstantFunction(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new VariableOneSum(variableIndex);
                    } else {
                        return new VariableConstantSum(variableIndex, constantValue);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new VariableOneSubtraction(variableIndex);
                    } else {
                        return new VariableConstantSubtraction(variableIndex, constantValue);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new VariableConstantMultiplication(variableIndex, constantValue);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new VariableConstantDivision(variableIndex, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IFunction GetGlobalVariableConstantFunction(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new GlobalVariableOneSum(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantSum(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new GlobalVariableOneSubtraction(variableIndex, globalVariables);
                    } else {
                        return new GlobalVariableConstantSubtraction(variableIndex, constantValue, globalVariables);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new GlobalVariableConstantMultiplication(variableIndex, constantValue, globalVariables);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new GlobalVariableConstantDivision(variableIndex, constantValue, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a variable right operand.
        /// </summary>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetVariableVariableFunction(bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new VariableVariableSum(variableIndex, otherVariableIndex);
                case OperationType.Subtraction:
                    return new VariableVariableSubtraction(variableIndex, otherVariableIndex);
                case OperationType.Multiplication:
                    return new VariableVariableMultiplication(variableIndex, otherVariableIndex);
                case OperationType.Division:
                    return new VariableVariableDivision(variableIndex, otherVariableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetGlobalVariableVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[otherVariableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new GlobalVariableVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new GlobalVariableVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new GlobalVariableVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new GlobalVariableVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetVariableGlobalVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new VariableGlobalVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new VariableGlobalVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new VariableGlobalVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new VariableGlobalVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and another global variable
        /// right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <returns>The function.</returns>
        private IFunction GetGlobalVariableGlobalVariableFunction(float[] globalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            int otherVariableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    return new GlobalVariableGlobalVariableSum(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new GlobalVariableGlobalVariableSubtraction(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new GlobalVariableGlobalVariableMultiplication(variableIndex, otherVariableIndex, globalVariables);
                case OperationType.Division:
                    return new GlobalVariableGlobalVariableDivision(variableIndex, otherVariableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetVariableResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new VariableResultSum(variableIndex, function);
                case OperationType.Subtraction:
                    return new VariableResultSubtraction(variableIndex, function);
                case OperationType.Multiplication:
                    return new VariableResultMultiplication(variableIndex, function);
                case OperationType.Division:
                    return new VariableResultDivision(variableIndex, function);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a global variable left operand and a result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetGlobalVariableResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            int variableIndex = _leftOperand.VariableIndex;
            IFunction function = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new GlobalVariableResultSum(variableIndex, function, globalVariables);
                case OperationType.Subtraction:
                    return new GlobalVariableResultSubtraction(variableIndex, function, globalVariables);
                case OperationType.Multiplication:
                    return new GlobalVariableResultMultiplication(variableIndex, function, globalVariables);
                case OperationType.Division:
                    return new GlobalVariableResultDivision(variableIndex, function, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a constant right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetResultConstantFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            float constantValue = _rightOperand.ConstantValue;
            switch (_operationType) {
                case OperationType.Sum:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ResultOneSum(function);
                    } else {
                        return new ResultConstantSum(function, constantValue);
                    }
                case OperationType.Subtraction:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    }
                    if (constantValue == 1) {
                        return new ResultOneSubtraction(function);
                    } else {
                        return new ResultConstantSubtraction(function, constantValue);
                    }
                case OperationType.Multiplication:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeZero);
                    } else if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ResultConstantMultiplication(function, constantValue);
                case OperationType.Division:
                    if (constantValue == 0) {
                        _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustNotBeZero);
                        return null;
                    }
                    if (constantValue == 1) {
                        _operandMessages.AddSemanticWarning(_rightOperandSource, MessageCode.RightOperandShouldNotBeOne);
                    }
                    return new ResultConstantDivision(function, constantValue);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetResultVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            conditionalVariables[variableIndex] = true;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ResultVariableSum(function, variableIndex);
                case OperationType.Subtraction:
                    return new ResultVariableSubtraction(function, variableIndex);
                case OperationType.Multiplication:
                    return new ResultVariableMultiplication(function, variableIndex);
                case OperationType.Division:
                    return new ResultVariableDivision(function, variableIndex);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and a global variable right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetResultGlobalVariableFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            int variableIndex = _rightOperand.VariableIndex;
            switch (_operationType) {
                case OperationType.Sum:
                    return new ResultGlobalVariableSum(function, variableIndex, globalVariables);
                case OperationType.Subtraction:
                    return new ResultGlobalVariableSubtraction(function, variableIndex, globalVariables);
                case OperationType.Multiplication:
                    return new ResultGlobalVariableMultiplication(function, variableIndex, globalVariables);
                case OperationType.Division:
                    return new ResultGlobalVariableDivision(function, variableIndex, globalVariables);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function with a result left operand and another result right operand.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <returns>The function.</returns>
        private IFunction GetResultResultFunction(float[] globalVariables,
        bool[] conditionalVariables) {
            IFunction function = _leftOperand.GetFunction(globalVariables, conditionalVariables);
            IFunction otherFunction = _rightOperand.GetFunction(globalVariables, conditionalVariables);
            switch (_operationType) {
                case OperationType.Sum:
                    return new ResultResultSum(function, otherFunction);
                case OperationType.Subtraction:
                    return new ResultResultSubtraction(function, otherFunction);
                case OperationType.Multiplication:
                    return new ResultResultMultiplication(function, otherFunction);
                case OperationType.Division:
                    return new ResultResultDivision(function, otherFunction);
                default:
                    _operationMessages.AddSemanticError(_source, MessageCode.OperationIsNotFunction);
                    return null;
            }
        }

        /// <summary>
        /// Gets an action-dependant function.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the function.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>x
        /// <returns>The function.</returns>
        internal override IFunction GetFunction(float[] globalVariables, bool[] conditionalVariables) {
            switch (_leftOperand.OperandType) {
                case OperandType.Constant:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetConstantGlobalVariableFunction(globalVariables);
                            } else {
                                return GetConstantVariableFunction(conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetConstantResultFunction(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSemanticError(_rightOperandSource, MessageCode.RightOperandMustBeConstant);
                            return null;
                    }
                case OperandType.Variable:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetGlobalVariableConstantFunction(globalVariables);
                            } else {
                                return GetVariableConstantFunction(conditionalVariables);
                            }
                        case OperandType.Variable:
                            if (_leftOperand.IsGlobalVariable) {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetGlobalVariableGlobalVariableFunction(globalVariables);
                                } else {
                                    return GetGlobalVariableVariableFunction(globalVariables,
                                        conditionalVariables);
                                }
                            } else {
                                if (_rightOperand.IsGlobalVariable) {
                                    return GetVariableGlobalVariableFunction(globalVariables,
                                        conditionalVariables);
                                } else {
                                    return GetVariableVariableFunction(conditionalVariables);
                                }
                            }
                        case OperandType.Operation:
                            if (_leftOperand.IsGlobalVariable) {
                                return GetGlobalVariableResultFunction(globalVariables, conditionalVariables);
                            } else {
                                return GetVariableResultFunction(globalVariables, conditionalVariables);
                            }
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                case OperandType.Operation:
                    switch (_rightOperand.OperandType) {
                        case OperandType.Constant:
                            return GetResultConstantFunction(globalVariables, conditionalVariables);
                        case OperandType.Variable:
                            if (_rightOperand.IsGlobalVariable) {
                                return GetResultGlobalVariableFunction(globalVariables, conditionalVariables);
                            } else {
                                return GetResultVariableFunction(globalVariables, conditionalVariables);
                            }
                        case OperandType.Operation:
                            return GetResultResultFunction(globalVariables, conditionalVariables);
                        default:
                            _operandMessages.AddSyntacticError(_rightOperandSource, MessageCode.InvalidRightOperandType);
                            return null;
                    }
                default:
                    _operandMessages.AddSyntacticError(_leftOperandSource, MessageCode.InvalidLeftOperandType);
                    return null;
            }
        }

        #endregion

    }

}
