/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Core;
using Aran.Representation;
using Aran.Functional;

namespace Aran.Translation {

    /// <summary>
    /// Represents an operator.
    /// </summary>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [Serializable]
    internal sealed class Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> : SignatureInfo<TObj> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constants

        /// <summary>
        /// A constant which represents no instance identifier.
        /// </summary>
        internal const int NoInstanceId = -1;

        #endregion

        #region Fields

        /// <summary>
        /// The source.
        /// </summary>
        internal readonly IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> Source;

        /// <summary>
        /// The translator messages related to operators.
        /// </summary>
        private readonly MessageCollection<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _operatorMessages;

        /// <summary>
        /// The arguments.
        /// </summary>
        /// <remarks>They are used to generate the actions.</remarks>
        private readonly Argument<TObj>[] _arguments;

        /// <summary>
        /// The action identifier offset.
        /// </summary>
        /// <remarks>The action identifier offset is the identifier from which start the count so that the
        /// first action identifier is the offset, the second is the offset plus one and so on.</remarks>
        private int _actionIdsOffset;

        /// <summary>
        /// The facts.
        /// </summary>
        internal readonly Fact<TObj>[] Facts;

        /// <summary>
        /// The variables.
        /// </summary>
        internal readonly Variable<TObj>[] Variables;

        /// <summary>
        /// The preconditions.
        /// </summary>
        internal readonly Fact<TObj>[] Preconditions;

        /// <summary>
        /// The number of dynamic preconditions.
        /// </summary>
        /// <remarks>Dynamic preconditions are those affected by the addition or deletion list of any operator.</remarks>
        private int _dynamicPreconditionCount;

        /// <summary>
        /// The sources of the precondition operations.
        /// </summary>
        /// <remarks>The source at <c>i</c> corresponds with the operation at <c>i</c> in
        /// <see cref="_preconditionOperationSources">the precondition operations</see>.</remarks>
        private readonly IOperation<TVar>[] _preconditionOperationSources;

        /// <summary>
        /// The precondition operations.
        /// </summary>
        private readonly OperationOperand<TVar>[] _preconditionOperations;

        /// <summary>
        /// The translator messages related to operations.
        /// </summary>
        private readonly MessageCollection<IOperation<TVar>> _operationMessages;

        /// <summary>
        /// The precondition obtained by translating <see cref="_preconditionOperationSources">the precondition
        /// operations</see>.
        /// </summary>
        private IActionCondition _precondition;

        /// <summary>
        /// The constant which represents no index.
        /// </summary>
        private const int NoPreconditionIndex = -1;

        /// <summary>
        /// The precondition indexes.
        /// </summary>
        /// <remarks>Each position indicates the corresponding precondition index of a precondition in the action
        /// activation table of an <see cref="ActionSelector">action selector</see>, if the predicate of the
        /// precondition is dynamic. Is the predicate is static, the corresponding position has
        /// <see cref="NoPreconditionIndex">no index</see>.</remarks>
        internal int[] _preconditionIndexes;

        /// <summary>
        /// The additions.
        /// </summary>
        internal readonly Fact<TObj>[] Additions;

        /// <summary>
        /// The deletions.
        /// </summary>
        internal readonly Fact<TObj>[] Deletions;

        /// <summary>
        /// The sources of the effect operations.
        /// </summary>
        /// <remarks>The source at <c>i</c> corresponds with the operation at <c>i</c> in
        /// <see cref="_effectOperations">the effect operations</see>.</remarks>
        private readonly IOperation<TVar>[] _effectOperationSources;

        /// <summary>
        /// The effect operations.
        /// </summary>
        private readonly OperationOperand<TVar>[] _effectOperations;

        /// <summary>
        /// The effect obtained by translating <see cref="_effectOperations">the effect operations</see>.
        /// </summary>
        private IActionEffect _effect;

        /// <summary>
        /// The operator index.
        /// </summary>
        /// <remarks>It is its internal identifier.</remarks>
        internal int Index;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="Operator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}"/> class.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="operatorMessages">The translator messages related to operators.</param>
        /// <param name="signature">The signature.</param>
        /// <param name="parameterTypeInfos">The additional information about parameter types.</param>
        /// <param name="arguments">The arguments.</param>
        /// <param name="facts">The facts.</param>
        /// <param name="variables">The variables.</param>
        /// <param name="preconditions">The preconditions.</param>
        /// <param name="preconditionOperationSources">The precondition operation sources.</param>
        /// <param name="preconditionOperations">The precondition operations.</param>
        /// <param name="additions">The additions.</param>
        /// <param name="deletions">The deletions.</param>
        /// <param name="effectOperationSources">The effect operation sources.</param>
        /// <param name="effectOperations">The effect operations.</param>
        /// <param name="operationMessages">The operation messages.</param>
        internal Operator(IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> source,
        MessageCollection<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> operatorMessages,
        Signature<TObj> signature, TypeInfo<TObj>[] parameterTypeInfos,
        Argument<TObj>[] arguments, Fact<TObj>[] facts, Variable<TObj>[] variables,
        Fact<TObj>[] preconditions, IOperation<TVar>[] preconditionOperationSources,
        OperationOperand<TVar>[] preconditionOperations, Fact<TObj>[] additions, Fact<TObj>[] deletions,
        IOperation<TVar>[] effectOperationSources, OperationOperand<TVar>[] effectOperations,
        MessageCollection<IOperation<TVar>> operationMessages)
            : base(signature, parameterTypeInfos) {
            Source = source;
            _operatorMessages = operatorMessages;
            _arguments = arguments;
            Facts = facts;
            Variables = variables;
            Preconditions = preconditions;
            _preconditionOperationSources = preconditionOperationSources;
            _preconditionOperations = preconditionOperations;
            _precondition = Action.NoPrecondition;
            Additions = additions;
            Deletions = deletions;
            _effectOperationSources = effectOperationSources;
            _effectOperations = effectOperations;
            _effect = Action.NoEffect;
            _operationMessages = operationMessages;
        }

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="Operator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}"/> class.
        /// </summary>
        internal Operator() {
            //do nothing
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this operator is executable.
        /// </summary>
        /// <remarks>It is executable if generates almost one action.</remarks>
        /// <value>true if this operator is executable; otherwise, false.</value>
        internal bool IsExecutable {
            get {
                PredicateInfo<TObj> predicateInfo;
                //for each parameter type
                foreach (TypeInfo<TObj> parameterTypeInfo in ParameterTypeInfos) {
                    //if there are no objects for the type, the operator is not executable
                    if (parameterTypeInfo.Objects.Length == 0) {
                        return false;
                    }
                }
                //for each precondition
                foreach (Fact<TObj> precondition in Preconditions) {
                    predicateInfo = precondition.PredicateInfo;
                    //if is static or its predicate way is deletion and has no initial facts, the operator is not executable
                    if ((predicateInfo.IsStatic || (predicateInfo.FactsWay == FactWay.Deletion)) &&
                    (predicateInfo.InitialFacts.Count == 0)) {
                        return false;
                    }
                }
                return true;
            }
        }

        /// <summary>
        /// Gets the number of dynamic preconditions.
        /// </summary>
        /// <value>The the number of dynamic preconditions.</value>
        internal int DynamicPreconditionCount {
            get {
                _dynamicPreconditionCount = Preconditions.Length - CountStaticPreconditions();
                return _dynamicPreconditionCount;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Counts the static preconditions.
        /// </summary>
        /// <remarks>At the same time, initialices the <see cref="_preconditionIndexes">precondition
        /// indexes</see>.</remarks>
        /// <returns>The number of static preconditions.</returns>
        private int CountStaticPreconditions() {
            int staticPreconditionCount = 0;
            int preconditionCount = Preconditions.Length;
            _preconditionIndexes = new int[preconditionCount];
            int preconditionIndex = 0;
            Fact<TObj> precondition;
            //for each precondition
            for (int i = 0; i < preconditionCount; i++) {
                precondition = Preconditions[i];
                //if is static
                if (precondition.PredicateInfo.IsStatic) {
                    //set no precondition index
                    _preconditionIndexes[i] = NoPreconditionIndex;
                    staticPreconditionCount++;
                //if is dynamic
                } else {
                    //set the precondition index
                    _preconditionIndexes[i] = preconditionIndex;
                    preconditionIndex++;
                }
            }
            return staticPreconditionCount;
        }

        /// <summary>
        /// Calculates the identifiers of some specified instances.
        /// </summary>
        /// <param name="instances">The instances.</param>
        private static void CalculateIds(Instance<TObj>[] instances) {
            int count = instances.Length;
            for (int instanceIndex = 0; instanceIndex < count; instanceIndex++) {
                instances[instanceIndex].CalculateId();
            }
        }

        /// <summary>
        /// Gets the identifiers of some specified instances.
        /// </summary>
        /// <param name="instances">The instances.</param>
        /// <returns>The identifiers of <paramref name="instances"/>.</returns>
        private static int[] GetIds(Instance<TObj>[] instances) {
            int count = instances.Length;
            int[] ids = new int[count];
            int validCount = 0;
            int instanceId = 0;
            foreach (Instance<TObj> instance in instances) {
                instanceId = instance.Id;
                if (instanceId != NoInstanceId) { //an identifier is invalid if its value is no one
                    ids[validCount] = instanceId;
                    validCount++;
                }
            }
            if (validCount != count) {
                Array.Resize(ref ids, validCount);
            }
            return ids;
        }

        /// <summary>
        /// Gets the identifiers of some specified instances.
        /// </summary>
        /// <param name="instances">The instances.</param>
        /// <param name="calculate">If set to true, calculates the identifiers.</param>
        /// <param name="instanceAlreadyPresent">The instance already present.</param>
        /// <remarks>If any instance identifier already exist, generates an error message.</remarks>
        /// <returns>The identifiers of <paramref name="instances"/>.</returns>
        private int[] GetCheckedIds(Instance<TObj>[] instances, bool calculate,
        MessageCode instanceAlreadyPresent) {
            int count = instances.Length;
            int[] ids = new int[count];
            int validCount = 0;
            Instance<TObj> instance;
            Dictionary<int, Dictionary<int, int>> instanceIdsBySignature =
                new Dictionary<int, Dictionary<int, int>>();
            Dictionary<int, int> instanceIds;
            int signatureIndex;
            int instanceId;
            //for each instance
            for (int instanceIndex = 0; instanceIndex < count; instanceIndex++) {
                instance = instances[instanceIndex];
                signatureIndex = instance.Signature.Index;
                //get the instance identifiers set for the instance signature
                if (!instanceIdsBySignature.TryGetValue(signatureIndex, out instanceIds)) {
                    instanceIds = new Dictionary<int, int>();
                    instanceIdsBySignature[signatureIndex] = instanceIds;
                }
                //calculates the instance identifier, if needed
                if (calculate) {
                    instance.CalculateId();
                }
                instanceId = instance.Id;
                try {
                    //adds the instance identifier to the set for the instance signature
                    instanceIds.Add(instanceId, instanceId);
                } catch (ArgumentException) {
                    //if identifier already exists, generates an error
                    _operatorMessages.AddSemanticError(Source, instanceAlreadyPresent, instanceIndex);
                }
                if (instanceId != NoInstanceId) { //an identifier is invalid if its value is no one
                    ids[validCount] = instanceId;
                    validCount++;
                }
            }
            if (validCount != count) {
                Array.Resize(ref ids, validCount);
            }
            return ids;
        }

        /// <summary>
        /// Checks the fact identifiers.
        /// </summary>
        /// <returns>An array of length <c>l</c> where <c>l</c> is the number of facts. Each position is set to
        /// true if the fact conflicts with other.</returns>
        private bool[] CheckFactIds() {
            int factCount = Facts.Length;
            bool[] conflictingFacts = new bool[factCount];
            Fact<TObj> fact;
            Dictionary<int, Dictionary<int, int>> factIdsByPredicate = new Dictionary<int, Dictionary<int, int>>();
            Dictionary<int, int> predicateFactIds;
            int predicateIndex;
            int factId;
            //for each fact
            for (int factIndex = 0; factIndex < factCount; factIndex++) {
                fact = Facts[factIndex];
                predicateIndex = fact.Signature.Index;
                //get the fact identifiers set for the fact predicate
                if (!factIdsByPredicate.TryGetValue(predicateIndex, out predicateFactIds)) {
                    predicateFactIds = new Dictionary<int, int>();
                    factIdsByPredicate[predicateIndex] = predicateFactIds;
                }
                factId = fact.Id;
                try {
                    //adds the fact identifier to the set for the fact predicate
                    predicateFactIds.Add(factId, factId);
                } catch (ArgumentException) {
                    //if identifier already exists, registers the error in the resulting array
                    conflictingFacts[factIndex] = true;
                }
            }
            return conflictingFacts;
        }

        /// <summary>
        /// Gets the identifiers of some specified instances.
        /// </summary>
        /// <param name="instances">The instances.</param>
        /// <returns>The identifiers of <paramref name="instances"/>.</returns>
        private static int[] CalculateAndGetIds(Instance<TObj>[] instances) {
            int count = instances.Length;
            int[] ids = new int[count];
            int validCount = 0;
            int instanceId;
            foreach (Instance<TObj> instance in instances) {
                instance.CalculateId();
                instanceId = instance.Id;
                if (instanceId != NoInstanceId) { //an identifier is invalid if its value is no one
                    ids[validCount] = instanceId;
                    validCount++;
                }
            }
            if (validCount != count) {
                Array.Resize(ref ids, validCount);
            }
            return ids;
        }

        /// <summary>
        /// Activates an action precondition into an activation table.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actionId">The action identifier.</param>
        /// <param name="preconditionIndex">The index of the precondition into the action.</param>
        /// <param name="preconditionId">The action precondition identifier.</param>
        private void ActivateActionPrecondition(int[][][][] activationTable, int actionId,
        int preconditionIndex, int preconditionId) {
            const int IntSize = 32;
            //sets to one the precondition bit
            activationTable[preconditionId][Index][preconditionIndex][actionId / IntSize] |=
                (1 << (actionId % IntSize));
        }

        /// <summary>
        /// Activates an action precondition into an activation table for all facts.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actionId">The action identifier.</param>
        /// <param name="preconditionIndex">The index of the precondition into the action.</param>
        private void ActivateActionPreconditionForAllFacts(int[][][][] activationTable, int actionId,
        int preconditionIndex) {
            const int IntSize = 32;
            int factCount = activationTable.Length;
            //for each fact
            for (int preconditionId = 0; preconditionId < factCount; preconditionId++) {
                //sets to one the precondition bit
                activationTable[preconditionId][Index][preconditionIndex][actionId / IntSize] |=
                    (1 << (actionId % IntSize));
            }
        }

        /// <summary>
        /// Deactivates all preconditions of an action into an activation table.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actionId">The action identifier.</param>
        private void DeactivateAllActionPreconditions(int[][][][] activationTable, int actionId) {
            const int IntSize = 32;
            int factCount = activationTable.Length;
            int preconditionCount = Preconditions.Length;
            Fact<TObj> precondition;
            PredicateInfo<TObj> predicateInfo;
            bool deactivate = false;
            bool deactivateForAllFacts = false;
            //for each action precondition
            for (int i = 0; i < preconditionCount; i++) {
                precondition = Preconditions[i];
                predicateInfo = precondition.PredicateInfo;
                deactivate = false;
                deactivateForAllFacts = false;
                //if precondition is static
                if (!predicateInfo.IsStatic) {
                    switch (predicateInfo.FactsWay) {
                        case FactWay.Deletion:
                            //deactivate it if is initial and predicate way is deletion
                            deactivate = precondition.IsInitial;
                            break;
                        case FactWay.Addition:
                            //deactivate for all facts if is initial and predicate way is addition
                            if (precondition.IsInitial) {
                                deactivateForAllFacts = true;
                            } else { //deactivate it if not is initial and predicate way is addition
                                deactivate = true;
                            }
                            deactivate = !precondition.IsInitial;
                            break;
                        case FactWay.None:
                            //deactivate it if predicate has no way
                            deactivate = true;
                            break;
                    }
                }
                if (deactivate) {
                    //sets to zero the precondition bit
                    activationTable[precondition.Id][Index][_preconditionIndexes[i]][actionId / IntSize] &=
                        ~(1 << (actionId % IntSize));
                } else if (deactivateForAllFacts) {
                    //for each fact
                    for (int preconditionId = 0; preconditionId < factCount; preconditionId++) {
                        //sets to zero the precondition bit
                        activationTable[preconditionId][Index][_preconditionIndexes[i]][actionId / IntSize] &=
                            ~(1 << (actionId % IntSize));
                    }
                }
            }
        }

        /// <summary>
        /// Fills the activation information aboutan action.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actionId">The action identifier.</param>
        /// <param name="preconditionIds">The action precondition identifiers.</param>
        /// <returns>True if the action is activated; false otherwise.</returns>
        private bool FillActivationInfo(int[][][][] activationTable, int actionId, out int[] preconditionIds) {
            int preconditionCount = Preconditions.Length;
            preconditionIds = new int[_dynamicPreconditionCount];
            int validPreconditionCount = 0;
            int preconditionId;
            Fact<TObj> precondition;
            PredicateInfo<TObj> predicateInfo;
            //for each action precondition
            for (int i = 0; i < preconditionCount; i++) {
                precondition = Preconditions[i];
                preconditionId = precondition.Id;
                predicateInfo = precondition.PredicateInfo;
                //if precondition predicate is static
                if (predicateInfo.IsStatic) {
                    //and precondition is not initial, deactivate action
                    if (!precondition.IsInitial) {
                        DeactivateAllActionPreconditions(activationTable, actionId);
                        return false;
                    }
                //if precondition predicate is dynamic
                } else {
                    switch (predicateInfo.FactsWay) {
                        //if precondition predicate way is deletion
                        case FactWay.Deletion:
                            if (precondition.IsInitial) {
                                //if precondition is initial, activate it
                                ActivateActionPrecondition(activationTable, actionId,
                                    _preconditionIndexes[i], preconditionId);
                            } else {
                                //if precondition is not initial, deactivate action
                                DeactivateAllActionPreconditions(activationTable, actionId);
                                return false;
                            }
                            break;
                        //if precondition predicate way is addition
                        case FactWay.Addition:
                            if (precondition.IsInitial) {
                                //if precondition is initial, activate it for all facts
                                ActivateActionPreconditionForAllFacts(activationTable, actionId,
                                    _preconditionIndexes[i]);
                            } else {
                                //if precondition is not initial, activate it
                                ActivateActionPrecondition(activationTable, actionId,
                                    _preconditionIndexes[i], preconditionId);
                            }
                            break;
                        //if precondition predicate has no way
                        case FactWay.None:
                            //activate precondition
                            ActivateActionPrecondition(activationTable, actionId,
                                _preconditionIndexes[i], preconditionId);
                            break;
                    }
                    if (preconditionId != NoInstanceId) { //an identifier is invalid if its value is no one
                        preconditionIds[validPreconditionCount] = preconditionId;
                        validPreconditionCount++;
                    }
                }
            }
            if (validPreconditionCount != _dynamicPreconditionCount) {
                Array.Resize(ref preconditionIds, validPreconditionCount);
            }
            return true;
        }

        /// <summary>
        /// Generates an action.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actions">The actions.</param>
        private void GenerateAction(int[][][][] activationTable, Action[] actions) {
            int actionId = Signature.GetInstanceId(_arguments);
            CalculateIds(Facts);
            int actionIdWithoutOffset = actionId - _actionIdsOffset;
            int[] preconditionIds;
            //if action is activated
            if (FillActivationInfo(activationTable, actionIdWithoutOffset, out preconditionIds)) {
                //action is created
                actions[actionId] = new Action(actionId, preconditionIds,
                    CalculateAndGetIds(Variables), _precondition, GetIds(Deletions), GetIds(Additions), _effect);
            }
        }

        /// <summary>
        /// Generates an action.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actions">The actions.</param>
        /// <remarks>If any fact identifier already exist, generates an error.</remarks>
        private void GenerateCheckedAction(int[][][][] activationTable, Action[] actions) {
            int actionId = Signature.GetInstanceId(_arguments);
            CalculateIds(Facts);
            int actionIdWithoutOffset = actionId - _actionIdsOffset;
            bool[] conflictingFacts = CheckFactIds();
            int[] preconditionIds;
            //if the action is activated
            if (FillActivationInfo(activationTable, actionIdWithoutOffset, out preconditionIds)) {
                int factCount = Facts.Length;
                //checks facts conflicts
                for (int factIndex = 0; factIndex < factCount; factIndex++) {
                    if (conflictingFacts[factIndex]) {
                        _operatorMessages.AddSemanticError(Source,
                            MessageCode.FactPreconditionWillBeAlreadyPresent, factIndex);
                    }
                }
                //generates the action
                actions[actionId] = new Action(actionId, preconditionIds,
                    CalculateAndGetIds(Variables), _precondition,
                    GetCheckedIds(Deletions, false, MessageCode.FactDeletionWillBeAlreadyPresent),
                    GetCheckedIds(Additions, false, MessageCode.FactAdditionWillBeAlreadyPresent), _effect);
            }
            //notifies all errors, if any
            _operatorMessages.NotifyErrors();
        }

        /// <summary>
        /// Generates the functional precondition.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the funcional precondition.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <remarks>If any operator precondition is not a condition, generates an error.</remarks>
        private void GeneratePrecondition(float[] globalVariables, bool[] conditionalVariables) {
            int preconditionCount = _preconditionOperations.Length;
            if (preconditionCount > 0) {
                IActionCondition[] conditions = new IActionCondition[preconditionCount];
                OperationOperand<TVar> operation;
                //for each knowledge provider functional precondition
                for (int conditionIndex = 0; conditionIndex < preconditionCount; conditionIndex++) {
                    operation = _preconditionOperations[conditionIndex];
                    if (operation.IsCondition) {
                        //add the precondition to the list
                        conditions[conditionIndex] =
                            operation.GetActionCondition(globalVariables, conditionalVariables);
                    } else {
                        //if precondition is not a condition, generates an error
                        _operationMessages.AddSemanticError(_preconditionOperationSources[conditionIndex],
                            MessageCode.OperationIsNotCondition);
                    }
                }
                //creates the funcional precondition by joining all
                _precondition = new ActionMultipleConditionConjunction(conditions);
            }
        }

        /// <summary>
        /// Generates the functional effect.
        /// </summary>
        /// <param name="globalVariables">The global variables to be used by the funcional effect.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        /// <remarks>If any operator effect is not an effect, generates an error.</remarks>
        private void GenerateEffect(float[] globalVariables, bool[] conditionalVariables) {
            int effectCount = _effectOperations.Length;
            if (effectCount > 0) {
                IActionEffect[] effects = new IActionEffect[effectCount];
                OperationOperand<TVar> operation;
                //for each knowledge provider functional effect
                for (int effectIndex = 0; effectIndex < effectCount; effectIndex++) {
                    operation = _effectOperations[effectIndex];
                    if (operation.IsEffect) {
                        //add the effect to the list
                        effects[effectIndex] = operation.GetActionEffect(globalVariables, conditionalVariables);
                    } else {
                        //if effect is not an effect, generates an error
                        _operationMessages.AddSemanticError(_effectOperationSources[effectIndex],
                            MessageCode.OperationIsNotEffect);
                    }
                }
                //creates the funcional effect by joining all
                _effect = new ActionMultipleEffectList(effects);
            }
        }

        /// <summary>
        /// Generates the actions.
        /// </summary>
        /// <param name="activationTable">The activation table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.</param>
        /// <param name="actions">The actions.</param>
        /// <param name="globalVariables">The global variables to be used by the actions.</param>
        /// <param name="conditionalVariables">
        /// <para>The variables which are taken into account by functional preconditions or goals.</para>
        /// <para>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.</para>
        /// </param>
        internal void GenerateActions(int[][][][] activationTable, Action[] actions, float[] globalVariables,
        bool[] conditionalVariables) {
            int argumentIndex;
            int argumentCount = ParameterTypeInfos.Length;
            int actionCountLessOne = MaxInstancesCount - 1;
            _actionIdsOffset = Signature.InstanceIdOffset;
            //generates the funcional precondition, shared by all actions
            GeneratePrecondition(globalVariables, conditionalVariables);
            //generates the funcional effect, shared by all actions
            GenerateEffect(globalVariables, new bool[conditionalVariables.Length]);
            foreach (Argument<TObj> argument in _arguments) {
                argument.InitValue();
            }
            //generates the first action
            GenerateCheckedAction(activationTable, actions);
            //generates the rest of the actions
            for (int actionIndex = 0; actionIndex < actionCountLessOne; actionIndex++) {
                argumentIndex = 0;
                //changed the arguments (increases them, like a number)
                while ((argumentIndex < argumentCount) && _arguments[argumentIndex].IncreaseValue()) {
                    argumentIndex++;
                }
                //generates the action
                GenerateAction(activationTable, actions);
            }
        }

        #endregion

    }

}
