/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Translation {

    /// <summary>
    /// Represents information about a predicate.
    /// </summary>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal sealed class PredicateInfo<T> : SignatureInfo<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The predicate initial facts.
        /// </summary>
        internal readonly IList<Fact<T>> InitialFacts;

        /// <summary>
        /// The identifiers of the predicate initial facts.
        /// </summary>
        private int[] _initialIds;

        /// <summary>
        /// The number of predicate facts added by operators.
        /// </summary>
        internal int AdditionCount;

        /// <summary>
        /// The number of predicate facts deleted by operators.
        /// </summary>
        internal int DeletionCount;

        /// <summary>
        /// <para>If the predicate is static, that is, if its facts activation state are never changed by an
        /// operator.</para>
        /// <para>In other words, no operator adds or deletes a fact of this predicate. Therefore, all predicate
        /// facts retain their initial activation state during planning time.</para>
        /// </summary>
        internal bool IsStatic;

        /// <summary>
        /// The predicate fact way.
        /// </summary>
        internal FactWay FactsWay;

        /// <summary>
        /// If the predicate is an inequality.
        /// </summary>
        internal bool IsInequality;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PredicateInfo{T}"/> class.
        /// </summary>
        /// <param name="signature">The signature.</param>
        /// <param name="parameterTypeInfos">The information about the types of the signature parameter list.</param>
        internal PredicateInfo(Signature<T> signature, params TypeInfo<T>[] parameterTypeInfos)
            : base(signature, parameterTypeInfos) {
            FactsWay = FactWay.None;
            InitialFacts = new List<Fact<T>>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PredicateInfo{T}"/> class.
        /// </summary>
        internal PredicateInfo() {
            //do nothing
        }

        #endregion

        #region Methods

        /// <summary>
        /// Determines whether the specified fact is initial.
        /// </summary>
        /// <param name="fact">The fact.</param>
        /// <returns>True if the specified fact is initial; otherwise, false.</returns>
        internal bool IsInitial(Fact<T> fact) {
            return (Array.BinarySearch<int>(_initialIds, fact.Id) >= 0);
        }

        /// <summary>
        /// Calculates the facts way.
        /// </summary>
        private void CalculateFactsWay() {
            if (AdditionCount == 0) {
                if (DeletionCount == 0) {
                    //a predicate is static if its facts are neither added nor deleted
                    IsStatic = true;
                } else {
                    //a predicate has deletion way if its facts are only deleted
                    FactsWay = FactWay.Deletion;
                }
            } else if (DeletionCount == 0) {
                //a predicate has addition way if its facts are only added
                FactsWay = FactWay.Addition;
            }
        }

        /// <summary>
        /// Calculates the max facts count.
        /// </summary>
        private void CalculateMaxFactsCount() {
            //if the predicate is static, has no facts
            if (!IsStatic) {
                if (FactsWay == FactWay.Deletion) {
                    //if predicate way is deletion, the maximum instances are the initial ones
                    MaxInstancesCount = InitialFacts.Count;
                } else {
                    //if predicate way is addition, the maximum instances are the standard maximum ones less
                    //the initial ones
                    int maxFactsCount = CalculateMaxInstancesCount();
                    if (FactsWay == FactWay.Addition) {
                        maxFactsCount -= InitialFacts.Count;
                    }
                    MaxInstancesCount = maxFactsCount;
                }
            }
        }

        /// <summary>
        /// Gets the initial facts identifiers.
        /// </summary>
        private int[] GetInitialIds() {
            int[] initialIds = new int[InitialFacts.Count];
            int initialFactIndex = 0;
            //for each initial fact
            foreach (Fact<T> fact in InitialFacts) {
                fact.CalculateId();
                //extracts its identifier
                initialIds[initialFactIndex] = fact.Id;
                initialFactIndex++;
            }
            Array.Sort(initialIds);
            return initialIds;
        }

        /// <summary>
        /// Extracts the initial facts identifiers.
        /// </summary>
        private void ExtractInitialIds() {
            _initialIds = GetInitialIds();
        }

        /// <summary>
        /// Creates the instance identifier map.
        /// </summary>
        private void CreateInstanceIdsMap() {
            switch (FactsWay) {
                case FactWay.Addition:
                    int[] initialIds = GetInitialIds();
                    Signature.SetInitialInstanceIds(initialIds, InitialInstancesBound.Minimum);
                    Signature.CreateInstanceIdMap(initialIds, MaxInstancesCount);
                    break;
                case FactWay.Deletion:
                    Signature.SetInitialInstanceIds(GetInitialIds(), InitialInstancesBound.Maximum);
                    break;
            }
        }

        /// <summary>
        /// Calculates derived values.
        /// </summary>
        /// <remarks>A derived value is calculated from previously set values and taking into account the
        /// information provided by the preceding predicate in the predicate sequence.</remarks>
        /// <param name="previousInfo">The previous predicate information.</param>
        internal void CalculateDerived(PredicateInfo<T> previousInfo) {
            CalculateFactsWay();
            CalculateInstanceIdOffset(previousInfo);
            CalculateMaxFactsCount();
            CalculateParameterSizes();
            CreateInstanceIdsMap();
            ExtractInitialIds();
        }

        #endregion

    }

}
