/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Resources;
using System.Globalization;

namespace Aran.Translation {

    /// <summary>
    /// Represents the resources for this namespace classes.
    /// </summary>
    [Serializable]
    internal sealed class Resources {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The underlying resource manager.
        /// </summary>
        private readonly ResourceManager _resources;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Resources"/> class.
        /// </summary>
        internal Resources() {
            _resources = new ResourceManager(GetType());
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <value>The culture.</value>
        private static CultureInfo Culture {
            get {
                return CultureInfo.CurrentUICulture;
            }
        }

        /// <summary>
        /// Gets the "predicate not found" message.
        /// </summary>
        /// <value>The "predicate not found" message.</value>
        internal string PredicateNotFound {
            get {
                return _resources.GetString("PredicateNotFound", Culture);
            }
        }

        /// <summary>
        /// Gets the "predicate already added" message.
        /// </summary>
        /// <value>The "predicate already added" message.</value>
        internal string PredicateAlreadyAdded {
            get {
                return _resources.GetString("PredicateAlreadyAdded", Culture);
            }
        }

        /// <summary>
        /// Gets the "invalid argument count" message.
        /// </summary>
        /// <value>The "invalid argument count" message.</value>
        internal string InvalidArgumentCount {
            get {
                return _resources.GetString("InvalidArgumentCount", Culture);
            }
        }

        /// <summary>
        /// Gets the "null arguments" message.
        /// </summary>
        /// <value>The "null arguments" message.</value>
        internal string NullArguments {
            get {
                return _resources.GetString("NullArguments", Culture);
            }
        }

        /// <summary>
        /// Gets the "incoherent argument count" message.
        /// </summary>
        /// <value>The "incoherent argument count" message.</value>
        internal string IncoherentArgumentCount {
            get {
                return _resources.GetString("IncoherentArgumentCount", Culture);
            }
        }

        /// <summary>
        /// Gets the "unknown identifier" message.
        /// </summary>
        /// <value>The "unknown identifier" message.</value>
        internal string UnknownIdentifier {
            get {
                return _resources.GetString("UnknownIdentifier", Culture);
            }
        }

        /// <summary>
        /// Gets the "unexpected error" message.
        /// </summary>
        /// <value>The "unexpected error" message.</value>
        internal string UnexpectedError {
            get {
                return _resources.GetString("UnexpectedError", Culture);
            }
        }

        /// <summary>
        /// Gets the "generation errors found" message.
        /// </summary>
        /// <value>The "generation errors found" message.</value>
        internal string GenerationErrorsFound {
            get {
                return _resources.GetString("GenerationErrorsFound", Culture);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets a parametrized message by name.
        /// </summary>
        /// <param name="name">The message name.</param>
        /// <param name="arguments">The message arguments.</param>
        /// <returns>The parametrized message.</returns>
        private string GetMessage(string name, params object[] arguments) {
            CultureInfo culture = Culture;
            return String.Format(culture, _resources.GetString(name, culture), arguments);
        }

        /// <summary>
        /// Gets an "unknown argument" message.
        /// </summary>
        /// <param name="position">The argument position.</param>
        /// <returns>An "unknown argument" message</returns>
        internal string GetUnknownArgument(int position) {
            return GetMessage("UnknownArgument", position);
        }

        /// <summary>
        /// Gets an "argument type mismatch" message.
        /// </summary>
        /// <param name="position">The argument position.</param>
        /// <returns>An "argument type mismatch" message</returns>
        internal string GetArgumentTypeMismatch(int position) {
            return GetMessage("ArgumentTypeMismatch", position);
        }

        /// <summary>
        /// Gets an "unknown signature type" message.
        /// </summary>
        /// <param name="type">The signature type.</param>
        /// <returns>An "unknown signature type" message</returns>
        internal string GetUnknownSignature(SignatureType type) {
            return GetMessage("Unknown" + type.ToString(), type);
        }

        #endregion

    }

}
