/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Translation {

    /// <summary>
    /// Represents a signature.
    /// </summary>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal sealed class Signature<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constants

        /// <summary>
        /// A constant which represents no instance identifier.
        /// </summary>
        internal const int NoInstanceId = -1;

        #endregion

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The index.
        /// </summary>
        /// <remarks>Is the internal signature identifier. Two signatures of the same
        /// <see cref="SignatureType">type</see> haven't got the same index.</remarks>
        internal readonly int Index;

        /// <summary>
        /// The parameter type identifiers.
        /// </summary>
        /// <remarks>The identifier of a type is obtained from its corresponding
        /// <see cref="TypeInfo{T}">additional information</see>.</remarks>
        internal readonly int[] ParameterTypeIds;

        /// <summary>
        /// The number of parameters.
        /// </summary>
        internal readonly int ParameterCount;

        /// <summary>
        /// The parameter section sizes.
        /// </summary>
        /// <remarks>Parameter section sizes are numbers to use in the
        /// <see cref="Instance{T}.Id">instance identifier</see> decoding task so that the identifier
        /// number can be separated in <c>n</c>, where <c>n</c> is the number of signature parameters. The
        /// number at position <c>p</c> corresponds to the identifier of the object used as argument to fill
        /// the parameter at position <c>p</c>.</remarks>
        internal int[] ParameterSectionSizes;

        /// <summary>
        /// The number of objects each parameter can have.
        /// </summary>
        internal int[] ParameterObjectCounts;

        /// <summary>
        /// The instance identifier offset.
        /// </summary>
        /// <remarks>The instance identifier offset is the identifier from which start the count so that the
        /// first instance identifier is the offset, the second is the offset plus one and so on.</remarks>
        internal int InstanceIdOffset;

        /// <summary>
        /// The identifiers of the initial instances provided by the problem.
        /// </summary>
        private int[] _initialInstanceIds;

        /// <summary>
        /// The key instance identifiers.
        /// </summary>
        /// <remarks>
        /// The key instance identifiers are the minimum instance identifiers needed to get any original
        /// instance identifier from its corresponding remapped one. Each key instance identifier is the
        /// the minimum one in a continuous used instance identifiers range.
        /// </remarks>
        private int[] _keyInstanceIds;

        /// <summary>
        /// The key remapped instance identifiers.
        /// </summary>
        /// <remarks>The key remapped instance identifiers are the minimum remapped instance identifiers
        /// needed to get any original instance identifier from its corresponding remapped one. Each key
        /// remapped instance identifier is the remapped identifier of the maximum one in a continuous
        /// used instance identifiers range.</remarks>
        private int[] _keyRemappedInstanceIds;

        /// <summary>
        /// The remapping offsets.
        /// </summary>
        /// <remarks>The remapping offsets are those applied to instance identifiers in order to get their
        /// corresponding remapped identifier. The offset applied to an identifier is the number of unused
        /// identifiers which precede it.</remarks>
        private int[] _remappingOffsets;

        /// <summary>
        /// The initial instances bound.
        /// </summary>
        private InitialInstancesBound _initialInstancesBound;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Signature{T}"/> class.
        /// </summary>
        /// <param name="resources">The resource manager.</param>
        /// <param name="index">
        /// <para>The index.</para>
        /// <para>Is the internal signature identifier. Two signatures of the same
        /// <see cref="SignatureType">type</see> haven't got the same index.</para>
        /// </param>
        /// <param name="parameterTypeIds">
        /// <para>The parameter type identifiers.</para>
        /// <para>The identifier of a type is obtained from its corresponding
        /// <see cref="TypeInfo{T}">additional information</see>.</para>
        /// </param>
        internal Signature(Resources resources, int index, params int[] parameterTypeIds) {
            _resources = resources;
            Index = index;
            ParameterTypeIds = parameterTypeIds;
            ParameterCount = parameterTypeIds.Length;
            _initialInstancesBound = InitialInstancesBound.None;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Signature{T}"/> class.
        /// </summary>
        internal Signature()
            : this(null, - 1, new int[0]) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the remapped instance identifier for an instance identifier.
        /// </summary>
        /// <param name="instanceId">The instance identifier.</param>
        /// <remarks>The remapped instance identifiers are those that have been obtained from the original ones
        /// by removing unused identifiers and reallocating used ones.</remarks>
        /// <returns>The remapped instance identifier for <paramref name="instanceId"/>.</returns>
        private int GetRemappedInstanceId(int instanceId) {
            int remappedInstanceId = instanceId;
            if (_initialInstanceIds != null) { //the remapping is only done when there are initial instances
                remappedInstanceId = NoInstanceId;
                //gets the position of the instance identifier in initial ones
                int idPosition = Array.BinarySearch<int>(_initialInstanceIds, instanceId);
                switch (_initialInstancesBound) {
                    //if the initial instances are the minimum ones
                    case InitialInstancesBound.Minimum:
                        //if the instance is not an initial one
                        if (idPosition < 0) {
                            idPosition = ~idPosition;
                            if (idPosition <= _initialInstanceIds.Length) {
                                //the remapped identifier is calculated by subtracting the position the
                                //identifier should have in initial ones, because it corresponds to the number
                                //of previous unused initial instances
                                remappedInstanceId = instanceId - idPosition;
                            }
                        }
                        //if the instance is an initial one, no remapping is done
                        break;
                    //if the initial instances are the maximum ones
                    case InitialInstancesBound.Maximum:
                        //if the instance is an initial one
                        if (idPosition >= 0) {
                            //the remapped identifier is the position of the instance identifier in initial ones
                            remappedInstanceId = InstanceIdOffset + idPosition;
                        }
                        //if the instance is not an initial one, no remapping is done
                        break;
                }
            }
            return remappedInstanceId;
        }

        /// <summary>
        /// Gets the identifier for a remapped instance identifier.
        /// </summary>
        /// <param name="remappedInstanceId">The remapped instance identifier.</param>
        /// <remarks>The remapped instance identifiers are those that have been obtained from the original ones
        /// by removing unused identifiers and reallocating used ones.</remarks>
        /// <returns>The instance identifier for <paramref name="remappedInstanceId"/>.</returns>
        private int GetInstanceId(int remappedInstanceId) {
            int instanceId = remappedInstanceId;
            if (_initialInstanceIds != null) { //the remapping is only done when there are initial instances
                int internalRemappedInstanceId = remappedInstanceId - InstanceIdOffset;
                switch (_initialInstancesBound) {
                    //if the initial instances are the minimum ones
                    case InitialInstancesBound.Minimum:
                        //gets the key remapped instance identifier for this remapped instance identifier
                        int idPosition =
                            Array.BinarySearch<int>(_keyRemappedInstanceIds, internalRemappedInstanceId);
                        if (idPosition < 0) {
                            idPosition = ~idPosition;
                        }
                        if (idPosition < _remappingOffsets.Length) {
                            //the instance identifier is the result of adding the offset to the remapped one
                            instanceId = remappedInstanceId + _remappingOffsets[idPosition];
                            //the instance identifier must be within a continuous range for being valid
                            if (instanceId < _keyInstanceIds[idPosition]) {
                                instanceId = NoInstanceId;
                            }
                        } else {
                            instanceId = NoInstanceId;
                        }
                        break;
                    //if the initial instances are the maximum ones
                    case InitialInstancesBound.Maximum:
                        //the instance identifier is the initial instance identifier which corresponds to the
                        //remapped instance identifier
                        instanceId = _initialInstanceIds[internalRemappedInstanceId];
                        break;
                }
            }
            return instanceId;
        }

        /// <summary>
        /// Sets the initial instance identifiers and their bound.
        /// </summary>
        /// <param name="initialInstanceIds">The initial instances identifiers.</param>
        /// <param name="initialInstancesBound">The initial instances bound.</param>
        internal void SetInitialInstanceIds(int[] initialInstanceIds,
        InitialInstancesBound initialInstancesBound) {
            _initialInstanceIds = initialInstanceIds;
            _initialInstancesBound = initialInstancesBound;
        }

        /// <summary>
        /// Gets the not initial instance identifiers.
        /// </summary>
        /// <param name="initialInstanceIds">The initial instance identifiers.</param>
        /// <param name="maxInstancesCount">The maximum number of instances.</param>
        /// <returns>The not initial instance identifiers.</returns>
        private static int[] GetNotInitialInstanceIds(int[] initialInstanceIds, int maxInstancesCount) {
            int initialInstanceCount = initialInstanceIds.Length;
            int[] notInitialInstanceIds = new int[maxInstancesCount - initialInstanceCount];
            if (initialInstanceCount > 0) {
                int instanceIdIndex = 0;
                int initialInstanceIdIndex = 0;
                int lastInitialInstanceIdIndex = initialInstanceIds.Length - 1;
                int initialInstanceId = initialInstanceIds[initialInstanceIdIndex];
                for (int instanceId = 0; instanceId < maxInstancesCount; instanceId++) {
                    if (instanceId != initialInstanceId) {
                        notInitialInstanceIds[instanceIdIndex] = instanceId;
                        instanceIdIndex++;
                    } else if (initialInstanceIdIndex < lastInitialInstanceIdIndex) {
                        initialInstanceIdIndex++;
                        initialInstanceId = initialInstanceIds[initialInstanceIdIndex];
                    }
                }
            } else {
                for (int instanceId = 0; instanceId < maxInstancesCount; instanceId++) {
                    notInitialInstanceIds[instanceId] = instanceId;
                }
            }
            return notInitialInstanceIds;
        }

        /// <summary>
        /// Creates the instance identifier map.
        /// </summary>
        /// <param name="initialInstanceIds">The initial instances identifiers.</param>
        /// <param name="maxInstancesCount">The maximum number of instances.</param>
        /// <remarks>Generates the <see cref="_keyInstanceIds">key instance identifiers</see>, the
        /// <see cref="_keyRemappedInstanceIds">key remapped instances identifiers</see> and the
        /// <see cref="_remappingOffsets">remapping offets</see>.
        /// </remarks>
        internal void CreateInstanceIdMap(int[] initialInstanceIds, int maxInstancesCount) {
            List<int> keyInstanceIds = new List<int>();
            List<int> keyRemappedInstanceIds = new List<int>();
            List<int> remappingOffsets = new List<int>();
            int[] notInitialInstanceIds = GetNotInitialInstanceIds(initialInstanceIds, maxInstancesCount);
            if (notInitialInstanceIds.Length > 0) {
                int firstInstanceId = notInitialInstanceIds[0];
                int lastKeyInstanceId = firstInstanceId;
                int lastInstanceId = firstInstanceId - 1;
                int unusedInstanceIds = firstInstanceId;
                int lastRemappedInstanceId = -1;
                int instanceGap;
                //for each initial instance identifier
                foreach (int instanceId in notInitialInstanceIds) {
                    instanceGap = instanceId - lastInstanceId - 1;
                    //if the identifier is start of new continuous range
                    if (instanceGap > 0) {
                        //sets the last range first identifier
                        keyInstanceIds.Add(lastKeyInstanceId);
                        //sets the last range last remapped identifier
                        keyRemappedInstanceIds.Add(lastRemappedInstanceId);
                        //sets the last range remapping offset
                        remappingOffsets.Add(unusedInstanceIds);
                        unusedInstanceIds += instanceGap;
                        //sets the identifier as the first one for the new range
                        lastKeyInstanceId = instanceId;
                    }
                    lastInstanceId = instanceId;
                    lastRemappedInstanceId++;
                }
                //adds the last continuous range information
                keyInstanceIds.Add(lastKeyInstanceId);
                keyRemappedInstanceIds.Add(lastRemappedInstanceId);
                remappingOffsets.Add(unusedInstanceIds);
            }
            _keyInstanceIds = keyInstanceIds.ToArray();
            _keyRemappedInstanceIds = keyRemappedInstanceIds.ToArray();
            _remappingOffsets = remappingOffsets.ToArray();
        }

        /// <summary>
        /// Gets the instance identifier for an argument list.
        /// </summary>
        /// <param name="argumentCount">The number of arguments.</param>
        /// <param name="argumentIds">
        /// <para>The argument identifiers.</para>
        /// <para>Each identifier corresponds to an argument.</para>
        /// </param>
        /// <param name="objects">The objects, hashed by identifier.</param>
        /// <returns>The instance identifier for <paramref name="argumentIds"/>.</returns>
        /// <exception cref="ArgumentException">The number of arguments is different from the number of
        /// parameters or some argument type is different from the corresponding parameter type.</exception>
        internal int GetInstanceId(int argumentCount, IEnumerator<T> argumentIds, IDictionary<T, Object> objects) {
            int argumentId = 0;
            int internalArgumentCount = 0;
            int argumentTypeId;
            while (argumentIds.MoveNext()) {
                try {
                    argumentTypeId = objects[argumentIds.Current].TypeDependantIds[
                        ParameterTypeIds[internalArgumentCount]];
                } catch (KeyNotFoundException) {
                    throw new ArgumentException(_resources.GetUnknownArgument(internalArgumentCount), "instance");
                }
                if (argumentTypeId == Object.NoTypeId) {
                    throw new ArgumentException(_resources.GetArgumentTypeMismatch(internalArgumentCount), "instance");
                }
                argumentId += (ParameterSectionSizes[internalArgumentCount] * argumentTypeId);
                internalArgumentCount++;
            }
            if (internalArgumentCount != argumentCount) {
                throw new ArgumentException(_resources.IncoherentArgumentCount, "instance");
            }
            return GetRemappedInstanceId(InstanceIdOffset + argumentId);
        }

        /// <summary>
        /// Gets the instance identifier for an argument list.
        /// </summary>
        /// <param name="arguments">The arguments.</param>
        /// <returns>The instance identifier for <paramref name="arguments"/>.</returns>
        internal int GetInstanceId(Argument<T>[] arguments) {
            int argumentId = 0;
            for (int parameterIndex = 0; parameterIndex < ParameterCount; parameterIndex++) {
                argumentId += (ParameterSectionSizes[parameterIndex] *
                    arguments[parameterIndex].Value.TypeDependantIds[ParameterTypeIds[parameterIndex]]);
            }
            return GetRemappedInstanceId(InstanceIdOffset + argumentId);
        }

        /// <summary>
        /// Gets the identifiers of the arguments of an the instance which corresponnd to an specified identifier.
        /// </summary>
        /// <param name="instanceId">The instance identifier.</param>
        /// <param name="types">The types.</param>
        /// <returns>The identifiers of the arguments of an the instance which corresponnd to <paramref name="instanceId"/>.</returns>
        internal T[] GetInstanceArgumentIds(int instanceId, Type<T>[] types) {
            const int FirstArgumentTypeId = 0;
            T[] argumentIds = new T[ParameterCount];
            //subtracts the offset to get the relative identifier
            int argumentsId = GetInstanceId(instanceId) - InstanceIdOffset;
            int parameterIndex = 0;
            int argumentTypeId;
            int parameterObjectCount;
            //decodes the relative identifier by using a method similar to the division-based method of
            //translating a decimal number into a binary one, but here for each "digit" (argument) there are 1
            //or more possible values (the objects of the parameter type), potentially different between "digit"
            //positions (because the type of the positions x and y may be different)
            while ((parameterIndex < ParameterCount) && (argumentsId != 0)) {
                parameterObjectCount = ParameterObjectCounts[parameterIndex];
                argumentTypeId = argumentsId % parameterObjectCount;
                argumentsId = argumentsId / parameterObjectCount;
                argumentIds[parameterIndex] = types[ParameterTypeIds[parameterIndex]].ObjectIds[argumentTypeId];
                parameterIndex++;
            }
            while (parameterIndex < ParameterCount) {
                argumentIds[parameterIndex] = types[ParameterTypeIds[parameterIndex]].ObjectIds[FirstArgumentTypeId];
                parameterIndex++;
            }
            return argumentIds;
        }

        #endregion

    }

}
