/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

namespace Aran.Translation {

    /// <summary>
    /// Represents additional information about a <see cref="Aran.Translation.Signature{T}">signature</see>.
    /// </summary>
    /// <remarks>The represented information is not included in each
    /// <see cref="Aran.Translation.Signature{T}">signature</see> because it is only used on translation time.
    /// <see cref="Aran.Translation.Signature{T}">Signature</see>s are also used for encoding/decoding tasks
    /// which do not use this information and, therefore, it must not be part of
    /// <see cref="Signature"/>. </remarks>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal class SignatureInfo<T> : Usable {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The signature.
        /// </summary>
        internal readonly Signature<T> Signature;

        /// <summary>
        /// The information about the types of the signature parameter list.
        /// </summary>
        protected internal readonly TypeInfo<T>[] ParameterTypeInfos;

        /// <summary>
        /// The maximum number of instances the signature can have.
        /// </summary>
        internal int MaxInstancesCount;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SignatureInfo{T}"/> class.
        /// </summary>
        /// <param name="signature">The signature.</param>
        /// <param name="parameterTypeInfos">The information about the types of the signature parameter list.</param>
        internal SignatureInfo(Signature<T> signature, TypeInfo<T>[] parameterTypeInfos) {
            Signature = signature;
            ParameterTypeInfos = parameterTypeInfos;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SignatureInfo{T}"/> class.
        /// </summary>
        internal SignatureInfo()
            : this(new Signature<T>(), new TypeInfo<T>[0]) {
        }

        #endregion

        #region Methods

        /// <summary>
        /// Calculates the instance identifier offset.
        /// </summary>
        /// <remarks>The instance identifier offset is the identifier from which start the count so that the
        /// first instance identifier is the offset, the second is the offset plus one and so on.</remarks>
        /// <param name="previousInfo">
        /// <para>The previous signature information.</para>
        /// <para>To obtain the offset, each signature information is visited secuentially and the signature
        /// takes into account the previous one to do the calculation of its offset.</para>
        /// </param>
        protected internal void CalculateInstanceIdOffset(SignatureInfo<T> previousInfo) {
            Signature.InstanceIdOffset =
                previousInfo.Signature.InstanceIdOffset + previousInfo.MaxInstancesCount;
        }

        /// <summary>
        /// Calculates the maximum number of instances the signature can have.
        /// </summary>
        /// <returns>The maximum number of instances the signature can have.</returns>
        protected internal int CalculateMaxInstancesCount() {
            int maxInstancesCount = 1;
            //gets the number of possible combinations of arguments
            foreach (TypeInfo<T> parameterTypeInfo in ParameterTypeInfos) {
                maxInstancesCount *= parameterTypeInfo.Objects.Length;
            }
            return maxInstancesCount;
        }

        /// <summary>
        /// Calculates the <see cref="F:Signature{T}.ParameterSectionSizes">signature parameter section
        /// sizes</see>.
        /// </summary>
        internal void CalculateParameterSizes() {
            int parameterCount = ParameterTypeInfos.Length;
            int[] parameterSectionSizes = new int[parameterCount];
            int[] parameterObjectCounts = new int[parameterCount];
            //the first parameter has a section size of 1
            int sectionSize = 1;
            int objectCount;
            TypeInfo<T> typeInfo;
            //for each parameter type
            for (int parameterIndex = 0; parameterIndex < parameterCount; parameterIndex++) {
                typeInfo = ParameterTypeInfos[parameterIndex];
                parameterSectionSizes[parameterIndex] = sectionSize;
                objectCount = typeInfo.Objects.Length;
                //the section size is the previous one multiplied by the number of objects of the previous type
                sectionSize *= objectCount;
                parameterObjectCounts[parameterIndex] = objectCount;
            }
            Signature.ParameterSectionSizes = parameterSectionSizes;
            Signature.ParameterObjectCounts = parameterObjectCounts;
        }

        /// <summary>
        /// Calculates derived values.
        /// </summary>
        /// <remarks>A derived value is calculated from previously set values and taking into account the
        /// information provided by the preceding signature in the signature sequence.</remarks>
        /// <param name="previousInfo">The previous signature information.</param>
        internal void CalculateDerived(SignatureInfo<T> previousInfo) {
            MaxInstancesCount = CalculateMaxInstancesCount();
            CalculateInstanceIdOffset(previousInfo);
            CalculateParameterSizes();
        }

        #endregion

    }

}
