/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="IInstance{TSig, TObj}">instance</see> synthetized from its components.
    /// </summary>
    /// <typeparam name="TSig">The type of the signature identifier.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    [Serializable]
    internal sealed class SyntheticInstance<TSig, TObj>
        : IInstance<TSig, TObj> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The signature identifier.
        /// </summary>
        private readonly TSig _signatureId;

        /// <summary>
        /// The argument identifiers.
        /// </summary>
        private readonly TObj[] _argumentIds;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SyntheticInstance{TSig, TObj}"/> class.
        /// </summary>
        /// <param name="signatureId">The signature identifier.</param>
        /// <param name="argumentIds">The argument identifiers.</param>
        internal SyntheticInstance(TSig signatureId, TObj[] argumentIds) {
            _signatureId = signatureId;
            _argumentIds = argumentIds;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the signature identifier.
        /// </summary>
        /// <value>The signature identifier.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public TSig SignatureId {
            get {
                return _signatureId;
            }
        }

        /// <summary>
        /// Gets the number of arguments.
        /// </summary>
        /// <value>The number of arguments.</value>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public int ArgumentCount {
            get {
                return _argumentIds.Length;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the argument identifiers.
        /// </summary>
        /// <returns>The argument identifiers.</returns>
        /// <remarks>
        /// <para>A full-retrieved result is not ensured. More on-demand retrieving tasks could be done, depending on
        /// implementation details, while managing the returned result components.</para>
        /// <para>Neither the result nor any of its elements can be a null reference.</para>
        /// </remarks>
        /// <exception cref="RepresentationException">An error occurs while retrieving.</exception>
        public IEnumerable<TObj> GetArgumentIds() {
            return _argumentIds;
        }

        #endregion

    }

}
