/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Core;

namespace Aran.Translation {

    /// <summary>
    /// Represents a <see cref="Translator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">translator</see>
    /// result.
    /// </summary>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public class TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The translator messages;
        /// </summary>
        private MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _messages;

        /// <summary>
        /// The action activation table.
        /// </summary>
        private int[][][][] _actionActivationTable;

        /// <summary>
        /// The actions.
        /// </summary>
        private Action[] _actions;

        /// <summary>
        /// The number of actions.
        /// </summary>
        private int _actionCount;

        /// <summary>
        /// The number of facts.
        /// </summary>
        private int _factCount;

        /// <summary>
        /// The number of goal facts.
        /// </summary>
        private int _goalCount;

        /// <summary>
        /// The number of variables.
        /// </summary>
        private int _variableCount;

        /// <summary>
        /// The variables which are taken into account by functional preconditions or goals.
        /// </summary>
        /// <remarks>
        /// If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.
        /// </remarks>
        private bool[] _conditionalVariables;

        /// <summary>
        /// The number of variables which are taken into account by functional preconditions or goals.
        /// </summary>
        private int _conditionalVariableCount;

        /// <summary>
        /// The number of operators.
        /// </summary>
        private int _operatorCount;

        /// <summary>
        /// The number of <see cref="Action">action</see>s by operator.
        /// </summary>
        private int[] _actionCounts;

        /// <summary>
        /// The number of preconditions by operator.
        /// </summary>
        private int[] _preconditionCounts;

        /// <summary>
        /// The initial state.
        /// </summary>
        private State _initialState;

        /// <summary>
        /// The bit array which represents the fact goal list.
        /// </summary>
        private int[] _goalFacts;

        /// <summary>
        /// The functional goal.
        /// </summary>
        private ICondition _functionalGoal;

        /// <summary>
        /// The metric.
        /// </summary>
        private IFunction _metric;

        /// <summary>
        /// The fact encoder.
        /// </summary>
        private InstanceEncoder<TPred, TObj> _factEncoder;

        /// <summary>
        /// The fact decoder.
        /// </summary>
        private InstanceDecoder<TPred, TObj> _factDecoder;

        /// <summary>
        /// The variable encoder.
        /// </summary>
        private InstanceEncoder<TFunc, TObj> _variableEncoder;

        /// <summary>
        /// The variable decoder.
        /// </summary>
        private InstanceDecoder<TFunc, TObj> _variableDecoder;

        /// <summary>
        /// The action encoder.
        /// </summary>
        private InstanceEncoder<TOp, TObj> _actionEncoder;

        /// <summary>
        /// The action decoder.
        /// </summary>
        private InstanceDecoder<TOp, TObj> _actionDecoder;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}"/> class.
        /// </summary>
        internal TranslationResult() {
            //do nothing
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the translator messages.
        /// </summary>
        /// <value>The translator messages.</value>
        public MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> Messages {
            get {
                return _messages;
            }
            internal set {
                _messages = value;
            }
        }

        /// <summary>
        /// Gets the action activation table.
        /// </summary>
        /// <value>The action activation table.</value>
        /// <remarks>
        /// <para>The table contains, for each fact index in the <see cref="State.Facts">state certainty list</see>,
        /// a bit array for each operator precondition.</para>
        /// <para>In other words, an expression with the format
        /// <c>table[factIndex][operatorIndex][preconditionIndex]</c> accesses to the bit array for the fact
        /// identified by <c>factIndex</c> and the precondition identified by <c>preconditionIndex</c> in
        /// the operator identified by <c>operatorIndex</c>. All indexes go from zero to <c>count</c> - 1,
        /// where <c>count</c> is the number of elements of the type (fact in state, operator or precondition
        /// in operator) represented by the index.</para>
        /// <para>The bit array has <c>actionCount</c> bits, where <c>actionCount</c> is the number of
        /// <see cref="Action">action</see>s for the operator identified by <c>operatorIndex</c>.</para>
        /// <para>Each bit of the array represents the activation state of the corresponding
        /// <see cref="Action">action</see>. If the bit at <c>actionIndex</c> has the one value, the
        /// <see cref="Action">action</see> identified by <c>actionIndex</c> is active when the fact
        /// at <c>factIndex</c> is true in the <see cref="State.Facts">state certainty list</see>.</para>
        /// <para> The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.</para>
        /// </remarks>
        public int[][][][] ActionActivationTable {
            get {
                return _actionActivationTable;
            }
            internal set {
                _actionActivationTable = value;
            }
        }

        /// <summary>
        /// Gets the actions.
        /// </summary>
        /// <value>The actions.</value>
        /// <remarks>Each action identified by <c>actionIndex</c> is at <c>actionIndex</c>. A null reference
        /// is there if the action is not selectable.</remarks>
        public Action[] Actions {
            get {
                return _actions;
            }
            internal set {
                _actions = value;
            }
        }

        /// <summary>
        /// Gets the number of facts.
        /// </summary>
        /// <value>The number of facts.</value>
        public int FactCount {
            get {
                return _factCount;
            }
            internal set {
                _factCount = value;
            }
        }

        /// <summary>
        /// Gets the number of fact goals.
        /// </summary>
        /// <value>The number of fact goals.</value>
        public int GoalCount {
            get {
                return _goalCount;
            }
            internal set {
                _goalCount = value;
            }
        }

        /// <summary>
        /// Gets the number of variables.
        /// </summary>
        /// <value>The number of variables.</value>
        public int VariableCount {
            get {
                return _variableCount;
            }
            internal set {
                _variableCount = value;
            }
        }

        /// <summary>
        /// Gets the variables which are taken into account by functional preconditions or goals.
        /// </summary>
        /// <remarks>If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into
        /// account.
        /// </remarks>
        /// <value>The variables which are taken into account by functional preconditions or goals.</value>
        public bool[] ConditionalVariables {
            get {
                return _conditionalVariables;
            }
            internal set {
                _conditionalVariables = value;
            }
        }

        /// <summary>
        /// Gets the number of variables which are taken into account by functional preconditions or goals.
        /// </summary>
        /// <value>The number of variables which are taken into account by functional preconditions or goals.</value>
        public int ConditionalVariableCount {
            get {
                return _conditionalVariableCount;
            }
            internal set {
                _conditionalVariableCount = value;
            }
        }

        /// <summary>
        /// Gets the number of operators.
        /// </summary>
        /// <value>The number of operators.</value>
        public int OperatorCount {
            get {
                return _operatorCount;
            }
            internal set {
                _operatorCount = value;
            }
        }

        /// <summary>
        /// Gets the number of actions.
        /// </summary>
        /// <value>The number of actions.</value>
        public int ActionCount {
            get {
                return _actionCount;
            }
            internal set {
                _actionCount = value;
            }
        }

        /// <summary>
        /// Gets the number of <see cref="Action">action</see>s by operator.
        /// </summary>
        /// <value>The number of <see cref="Action">action</see>s by operator.</value>
        /// <remarks>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</remarks>
        public int[] ActionCounts {
            get {
                return _actionCounts;
            }
            internal set {
                _actionCounts = value;
            }
        }

        /// <summary>
        /// Gets the number of preconditions by operator.
        /// </summary>
        /// <value>The precondition counts.</value>
        /// <remarks>For each index, the value represents the number of preconditions of the operator identified
        /// by that index.</remarks>
        public int[] PreconditionCounts {
            get {
                return _preconditionCounts;
            }
            internal set {
                _preconditionCounts = value;
            }
        }

        /// <summary>
        /// Gets the initial state.
        /// </summary>
        /// <value>The initial state.</value>
        public State InitialState {
            get {
                return _initialState;
            }
            internal set {
                _initialState = value;
            }
        }

        /// <summary>
        /// Gets the bit array which represents the fact goal list.
        /// </summary>
        /// <remarks>Each bit represents the goal state of a fact. If the bit has the one value the fact is a
        /// goal. The bit array format is the same used by
        /// <see cref="M:System.Collections.BitArray.#ctor(System.Int32[])">the BitArray class constructor</see>.
        /// </remarks>
        /// <value>The goal facts.</value>
        public int[] GoalFacts {
            get {
                return _goalFacts;
            }
            internal set {
                _goalFacts = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether has functional goal.
        /// </summary>
        /// <value>True if has functional goal; otherwise, false.</value>
        public bool HasFunctionalGoal {
            get {
                return (_functionalGoal != null);
            }
        }

        /// <summary>
        /// Gets the functional goal.
        /// </summary>
        /// <value>The functional goal.</value>
        public ICondition FunctionalGoal {
            get {
                return _functionalGoal;
            }
            internal set {
                _functionalGoal = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether thas metric.
        /// </summary>
        /// <value>True if has metric; otherwise, false.</value>
        public bool HasMetric {
            get {
                return (_metric != null);
            }
        }

        /// <summary>
        /// Gets the metric.
        /// </summary>
        /// <value>The metric.</value>
        public IFunction Metric {
            get {
                return _metric;
            }
            internal set {
                _metric = value;
            }
        }

        /// <summary>
        /// Gets the fact encoder.
        /// </summary>
        /// <value>The fact encoder.</value>
        public InstanceEncoder<TPred, TObj> FactEncoder {
            get {
                return _factEncoder;
            }
            internal set {
                _factEncoder = value;
            }
        }

        /// <summary>
        /// Gets fact decoder.
        /// </summary>
        /// <value>The fact decoder.</value>
        public InstanceDecoder<TPred, TObj> FactDecoder {
            get {
                return _factDecoder;
            }
            internal set {
                _factDecoder = value;
            }
        }

        /// <summary>
        /// Gets variable encoder.
        /// </summary>
        /// <value>The variable encoder.</value>
        public InstanceEncoder<TFunc, TObj> VariableEncoder {
            get {
                return _variableEncoder;
            }
            internal set {
                _variableEncoder = value;
            }
        }

        /// <summary>
        /// Gets variable decoder.
        /// </summary>
        /// <value>The variable decoder.</value>
        public InstanceDecoder<TFunc, TObj> VariableDecoder {
            get {
                return _variableDecoder;
            }
            internal set {
                _variableDecoder = value;
            }
        }

        /// <summary>
        /// Gets the action encoder.
        /// </summary>
        /// <value>The action encoder.</value>
        public InstanceEncoder<TOp, TObj> ActionEncoder {
            get {
                return _actionEncoder;
            }
            internal set {
                _actionEncoder = value;
            }
        }

        /// <summary>
        /// Gets the action decoder.
        /// </summary>
        /// <value>The action decoder.</value>
        public InstanceDecoder<TOp, TObj> ActionDecoder {
            get {
                return _actionDecoder;
            }
            internal set {
                _actionDecoder = value;
            }
        }

        #endregion

    }

}
