/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

using Aran.Core;
using Aran.Representation;
using Aran.Functional;

namespace Aran.Translation {

    /// <summary>
    /// Represents an <see cref="Object">object</see> capable of translating
    /// a <see cref="IKnowledge{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">knowledge</see> result into
    /// other data structures to be used by <see cref="N:Aran.Core">Aran.Core</see> namespace classes.
    /// </summary>
    /// <remarks>
    /// All argument types must override <see cref="M:System.Object.GetHashCode">the GetHashCode() method</see>
    /// and <see cref="M:System.Object.Equals">the Equals() method</see> because equality comparisons are
    /// intensively used.
    /// </remarks>
    /// <typeparam name="TType">The type of the type identifiers.</typeparam>
    /// <typeparam name="TPred">The type of the predicate identifiers.</typeparam>
    /// <typeparam name="TFunc">The type of the function identifiers.</typeparam>
    /// <typeparam name="TOp">The type of the operator identifiers.</typeparam>
    /// <typeparam name="TObj">The type of the object identifiers.</typeparam>
    /// <typeparam name="TFact">The type of the fact identifiers.</typeparam>
    /// <typeparam name="TVar">The type of the variable identifiers.</typeparam>
    [CLSCompliant(true)]
    [Serializable]
    public sealed class Translator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The resource manager.
        /// </summary>
        private readonly Resources _resources;

        /// <summary>
        /// The messages.
        /// </summary>
        private MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _messages;

        /// <summary>
        /// The root type identifier.
        /// </summary>
        private TType _rootTypeId;

        /// <summary>
        /// If the root type is enabled.
        /// </summary>
        private bool _rootTypeEnabled;

        /// <summary>
        /// The inequality predicates.
        /// </summary>
        private readonly InequalityPredicateSet<TType, TPred> _inequalityPredicates;

        /// <summary>
        /// If the inequality predicates are enabled.
        /// </summary>
        private bool _inequalityPredicatesEnabled;

        /// <summary>
        /// The knowledge provider.
        /// </summary>
        private IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _knowledgeProvider;

        /// <summary>
        /// The domain.
        /// </summary>
        private IDomain<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _domain;

        /// <summary>
        /// The problem.
        /// </summary>
        private IProblem<TType, TPred, TFunc, TOp, TObj, TFact, TVar> _problem;

        /// <summary>
        /// The additional information about the types.
        /// </summary>
        private IDictionary<TType, TypeInfo<TObj>> _typeInfos;

        /// <summary>
        /// The type knowledge provider sources.
        /// </summary>
        private IType<TType>[] _typeSources;

        /// <summary>
        /// The types.
        /// </summary>
        private Type<TObj>[] _types;

        /// <summary>
        /// The number of types.
        /// </summary>
        private int _typeCount;

        /// <summary>
        /// The objects, hashed by identifier.
        /// </summary>
        private IDictionary<TObj, Object> _objects;

        /// <summary>
        /// The object corresponding arguments, hashed by object odentifier.
        /// </summary>
        /// <remarks>Each value is a source-value pair. The source is the object knowledge provider source. The value is
        /// the object corresponding argument. Arguments are used in fact, variable and action generation.</remarks>
        private IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> _objectArguments;

        /// <summary>
        /// The predicate knowledge provider sources, hashed by predicate identifier.
        /// </summary>
        /// <remarks>Each value is the predicate signature knowledge provider source.</remarks>
        private IDictionary<TPred, ISignature<TType, TPred, TObj>> _predicateSources;

        /// <summary>
        /// The predicate signatures, hashed by predicate identifier.
        /// </summary>
        private IDictionary<TPred, Signature<TObj>> _predicates;

        /// <summary>
        /// The additional information about predicates, hashed by predicate identifier.
        /// </summary>
        private IDictionary<TPred, PredicateInfo<TObj>> _predicateInfos;

        /// <summary>
        /// The additional information about inequality predicates.
        /// </summary>
        private PredicateInfo<TObj>[] _inequalityPredicateInfos;

        /// <summary>
        /// The predicate entries.
        /// </summary>
        private SignatureEntry<TPred, TObj>[] _predicateEntries;

        /// <summary>
        /// The maximum fact identifiers each predicate will generate.
        /// </summary>
        private int[] _predicateMaxFactIds;

        /// <summary>
        /// The numebr of facts.
        /// </summary>
        private int _factCount;

        /// <summary>
        /// The function knowledge provider sources, hashed by function identifier.
        /// </summary>
        /// <remarks>Each value is the function signature knowledge provider source.</remarks>
        private IDictionary<TFunc, ISignature<TType, TFunc, TObj>> _functionSources;

        /// <summary>
        /// The function signatures, hashed by function identifier.
        /// </summary>
        private IDictionary<TFunc, Signature<TObj>> _functions;

        /// <summary>
        /// The additional information about functions, hashed by function identifier.
        /// </summary>
        private IDictionary<TFunc, FunctionInfo<TObj>> _functionInfos;

        /// <summary>
        /// The function entries.
        /// </summary>
        private SignatureEntry<TFunc, TObj>[] _functionEntries;

        /// <summary>
        /// The maximum variable identifiers each function will generate.
        /// </summary>
        private int[] _functionMaxVariableIds;

        /// <summary>
        /// The initial facts.
        /// </summary>
        private Fact<TObj>[] _initialFacts;

        /// <summary>
        /// The goal facts.
        /// </summary>
        private Fact<TObj>[] _goalFacts;

        /// <summary>
        /// The initial variables.
        /// </summary>
        private Variable<TObj>[] _initialVariables;

        /// <summary>
        /// The knowledge provider sources for functional initialization operations.
        /// </summary>
        private IOperation<TVar>[] _functionalInitializationSources;

        /// <summary>
        /// The functional initialization operations.
        /// </summary>
        private OperationOperand<TVar>[] _functionalInitializations;

        /// <summary>
        /// The knowledge provider sources for functional goal operations.
        /// </summary>
        private IOperation<TVar>[] _functionalGoalSources;

        /// <summary>
        /// The knowledge provider sources for functional goal operations.
        /// </summary>
        private OperationOperand<TVar>[] _functionalGoals;

        /// <summary>
        /// The variables which are taken into account by functional preconditions or goals.
        /// </summary>
        /// <remarks>
        /// If the position <c>i</c> is true, then the variable at position <c>i</c> is taken into account.
        /// </remarks>
        private bool[] _conditionalVariables;

        /// <summary>
        /// The metric target knowledge provider source.
        /// </summary>
        private IOperand<TVar> _metricTargetSource;

        /// <summary>
        /// The metric target.
        /// </summary>
        private Operand _metricTarget;

        /// <summary>
        /// The metric type.
        /// </summary>
        private MetricType _metricType;

        /// <summary>
        /// The metric target operation knowledge provider source.
        /// </summary>
        private IOperation<TVar> _metricTargetOperationSource;

        /// <summary>
        /// The number of variables.
        /// </summary>
        private int _variableCount;

        /// <summary>
        /// The variable values shared by all actions.
        /// </summary>
        private float[] _globalVariables;

        /// <summary>
        /// The operator signatures, hashed by operator identifier.
        /// </summary>
        private IDictionary<TOp, Signature<TObj>> _operatorSignatures;

        /// <summary>
        /// The operators, hashed by operator identifier.
        /// </summary>
        private IDictionary<TOp, Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> _operators;

        /// <summary>
        /// The operator signature entries.
        /// </summary>
        private SignatureEntry<TOp, TObj>[] _operatorEntries;

        /// <summary>
        /// The number of operators.
        /// </summary>
        private int _operatorCount;

        /// <summary>
        /// The maximum action identifiers each operator will generate.
        /// </summary>
        private int[] _operatorMaxActionIds;

        /// <summary>
        /// The action selection table, with the format required by an
        /// <see cref="Aran.Core.ActionSelector">action selector</see>.
        /// </summary>
        private int[][][][] _actionSelectionTable;

        /// <summary>
        /// The number of actions.
        /// </summary>
        private int _actionCount;

        /// <summary>
        /// The number of actions by operator.
        /// </summary>
        /// <remarks>For each index, the value represents the number of <see cref="Action">action</see>s of the
        /// operator identified by that index.</remarks>
        private int[] _actionCounts;

        /// <summary>
        /// The number of preconditions by operator.
        /// </summary>
        /// <remarks>For each index, the value represents the number of preconditions of the operator identified
        /// by that index.</remarks>
        private int[] _preconditionCounts;

        /// <summary>
        /// The actions.
        /// </summary>
        private Action[] _actions;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the
        /// <see cref="Translator{TType, TPred, TFunc, TOp, TObj, TFact, TVar}"/> class.
        /// </summary>
        /// <param name="knowledgeProvider">
        /// <para>The knowledge provider.</para>
        /// <para>All knowledge provider class argument types must override
        /// <see cref="M:System.Object.GetHashCode">the GetHashCode() method</see> and
        /// <see cref="M:System.Object.Equals">the Equals() method</see> because equality comparisons are
        /// intensively used.</para>
        /// </param>
        /// <exception cref="ArgumentNullException"><paramref name="knowledgeProvider"/> is a null reference.</exception>
        public Translator(IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar> knowledgeProvider) {
            if (knowledgeProvider == null) {
                throw new ArgumentNullException("knowledgeProvider");
            }
            _resources = new Resources();
            _knowledgeProvider = knowledgeProvider;
            _inequalityPredicates = new InequalityPredicateSet<TType, TPred>(_resources);
            _inequalityPredicatesEnabled = TranslatorDefaultValue.InequalitiesEnabled;
            _rootTypeEnabled = TranslatorDefaultValue.RootTypeEnabled;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the root type identifier.
        /// </summary>
        /// <value>The root type identifier.</value>
        public TType RootTypeId {
            get {
                return _rootTypeId;
            }
            set {
                _rootTypeId = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the root type is enabled.
        /// </summary>
        /// <value>True if the root type is enabled; otherwise, false.</value>
        public bool RootTypeEnabled {
            get {
                return _rootTypeEnabled;
            }
            set {
                _rootTypeEnabled = value;
            }
        }

        /// <summary>
        /// Gets the inequality predicates.
        /// </summary>
        /// <value>The inequality predicates.</value>
        public InequalityPredicateSet<TType, TPred> InequalityPredicates {
            get {
                return _inequalityPredicates;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the inequality predicates are enabled.
        /// </summary>
        /// <value>True if the inequality predicates are enabled; otherwise, false.</value>
        public bool InequalityPredicatesEnabled {
            get {
                return _inequalityPredicatesEnabled;
            }
            set {
                _inequalityPredicatesEnabled = value;
            }
        }

        /// <summary>
        /// Gets or sets the knowledge provider.
        /// </summary>
        /// <value>The knowledge provider.</value>
        public IKnowledgeProvider<TType, TPred, TFunc, TOp, TObj, TFact, TVar> KnowledgeProvider {
            get {
                return _knowledgeProvider;
            }
            set {
                if (value == null) {
                    throw new ArgumentNullException("value");
                }
                _knowledgeProvider = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets the domain and the problem by retrieving the knowledge representation.
        /// </summary>
        private void GetKnowledge() {
            IKnowledge<TType, TPred, TFunc, TOp, TObj, TFact, TVar> knowledge = _knowledgeProvider.GetKnowledge();
            if (knowledge == null) {
                _messages.KnowledgeProviderMessages.AddSyntacticFatalError(_knowledgeProvider,
                    MessageCode.NullKnowledge);
            }
            _domain = knowledge.GetDomain();
            _problem = knowledge.GetProblem();
            if (_domain == null) {
                _messages.KnowledgeMessages.AddSyntacticError(knowledge, MessageCode.NullDomain);
            }
            if (_problem == null) {
                _messages.KnowledgeMessages.AddSyntacticError(knowledge, MessageCode.NullProblem);
            }
            _messages.NotifyErrors();
        }

        /// <summary>
        /// Gets the number of knowledge provider types.
        /// </summary>
        /// <returns>The number of knowledge provider types.</returns>
        private int GetTypeCount() {
            int typeCount = _domain.TypeCount;
            if (typeCount < 0) {
                //adds an error message if type count is less than zero
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.InvalidTypeCount);
            }
            return typeCount;
        }

        /// <summary>
        /// Gets the knowledge provider types.
        /// </summary>
        /// <returns>The knowledge provider types.</returns>
        private IEnumerator<IType<TType>> GetTypes() {
            IEnumerable<IType<TType>> types = _domain.GetTypes();
            if (types == null) {
                //adds an error message if types are a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullTypes);
            }
            IEnumerator<IType<TType>> typeEnumerator = types.GetEnumerator();
            if (typeEnumerator == null) {
                //adds an error message if types enumerator is a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullTypesEnumerator);
            }
            return typeEnumerator;
        }

        /// <summary>
        /// Generates the types.
        /// </summary>
        /// <param name="typeCount">The number of knowledge provider types.</param>
        /// <param name="types">The knowledge provider types.</param>
        private void GenerateTypes(int typeCount, IEnumerator<IType<TType>> types) {
            IType<TType> type;
            TType typeId;
            TType parentId;
            TypeInfo<TObj> typeInfo = null;
            TypeInfo<TObj> parentTypeInfo;
            int typePosition = 0;
            int internalTypeId = 0;
            int adjustedTypeCount = typeCount;
            TypeInfo<TObj> rootTypeInfo = null;
            if (_rootTypeEnabled) {
                adjustedTypeCount++;
            }
            _typeSources = new IType<TType>[adjustedTypeCount];
            _typeInfos = new Dictionary<TType, TypeInfo<TObj>>(adjustedTypeCount);
            //creates the root type if enabled
            if (_rootTypeEnabled) {
                rootTypeInfo = new TypeInfo<TObj>(internalTypeId);
                _typeInfos[_rootTypeId] = rootTypeInfo;
                internalTypeId++;
            }
            bool errorFound;
            //for each knowledge provider type
            while (types.MoveNext()) {
                type = types.Current;
                errorFound = false;
                if (type == null) {
                    //adds an error message if the type is a null reference
                    _messages.DomainMessages.AddSyntacticError(_domain, MessageCode.NullType, typePosition);
                    errorFound = true;
                }
                if (!errorFound) {
                    //creates an internal representation of the type
                    typeId = type.Id;
                    if (_typeInfos.TryGetValue(typeId, out typeInfo)) {
                        if (typeInfo.HasId) {
                            //adds an error message if the type identifier is already in use
                            _messages.TypeMessages.AddSemanticError(type, MessageCode.TypeIdAlreadyUsed);
                            errorFound = true;
                        } else {
                            typeInfo.Id = internalTypeId;
                        }
                    } else {
                        typeInfo = new TypeInfo<TObj>(internalTypeId);
                        _typeInfos[typeId] = typeInfo;
                        _typeSources[internalTypeId] = type;
                    }
                    typePosition++;
                    internalTypeId++;
                }
                if (!errorFound) {
                    //adds the type as a child of its parent
                    if (type.IsRoot) {
                        //if root type is enabled then all root types are its childs
                        if (_rootTypeEnabled) {
                            rootTypeInfo.AddSubtypeInfo(typeInfo);
                        }
                    } else {
                        parentId = type.ParentId;
                        if (!_typeInfos.TryGetValue(parentId, out parentTypeInfo)) {
                            parentTypeInfo = new TypeInfo<TObj>();
                            _typeInfos[parentId] = parentTypeInfo;
                        }
                        parentTypeInfo.AddSubtypeInfo(typeInfo);
                    }
                }
            }
            //notifies errors, if any
            _messages.NotifyErrors();
            _typeCount = internalTypeId;
            if (_typeCount != adjustedTypeCount) {
                //adds an error message if the number of types is incoherent
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.IncoherentTypeCount);
            }
        }

        /// <summary>
        /// Generates the type relations.
        /// </summary>
        private void GenerateTypes() {
            int typeCount = GetTypeCount();
            IEnumerator<IType<TType>> types = GetTypes();
            GenerateTypes(typeCount, types);
            MessageCollection<IType<TType>> messages = _messages.TypeMessages;
            //for each type
            foreach (TypeInfo<TObj> typeInfo in _typeInfos.Values) {
                //sets up its subtypes
                if (!typeInfo.SetUpSubtypes()) {
                    foreach (TypeInfo<TObj> subtypeInfo in typeInfo.SubtypeInfos) {
                        //adds an error message for each type with unknown parent
                        messages.AddSemanticError(_typeSources[subtypeInfo.Id], MessageCode.InexistentParentType);
                    }
                    //notifies errors, if any
                    _messages.NotifyErrors();
                }
            }
        }

        /// <summary>
        /// Gets the number of knowledge provider objects.
        /// </summary>
        /// <returns>The number of knowledge provider objects.</returns>
        private int GetObjectCount() {
            int objectCount = _problem.ObjectCount;
            if (objectCount < 0) {
                //adds an error message if the number of objects is less than zero
                _messages.ProblemMessages.AddSyntacticFatalError(_problem, MessageCode.InvalidObjectCount);
            }
            return objectCount;
        }

        /// <summary>
        /// Gets the knowledge provider objects.
        /// </summary>
        /// <returns>The knowledge provider objects.</returns>
        private IEnumerator<IObject<TType, TObj>> GetObjects() {
            IEnumerable<IObject<TType, TObj>> objects = _problem.GetObjects();
            if (objects == null) {
                //adds an error message if the objects are a null reference
                _messages.ProblemMessages.AddSyntacticFatalError(_problem, MessageCode.NullObjects);
            }
            IEnumerator<IObject<TType, TObj>> objectEnumerator = objects.GetEnumerator();
            if (objectEnumerator == null) {
                //adds an error message if the objects enumerator is a null reference
                _messages.ProblemMessages.AddSyntacticFatalError(_problem, MessageCode.NullObjectsEnumerator);
            }
            return objectEnumerator;
        }

        /// <summary>
        /// Generates the objects.
        /// </summary>
        /// <param name="objectCount">The number of knowledge provider objects.</param>
        /// <param name="objects">The knowledge provider objects.</param>
        private void GenerateObjects(int objectCount, IEnumerator<IObject<TType, TObj>> objects) {
            IObject<TType, TObj> obj;
            Object internalObject;
            TObj objectId;
            TypeInfo<TObj> objectTypeInfo;
            ObjectInfo<TObj> objectInfo;
            int internalObjectCount = 0;
            _objects = new Dictionary<TObj, Object>(objectCount);
            _objectArguments = new Dictionary<TObj, SourceValuePair<IObject<TType, TObj>,
                Argument<TObj>>>(objectCount);
            //for each knowledge provider object
            while (objects.MoveNext()) {
                obj = objects.Current;
                if (obj == null) {
                    //adds an error message if the object is a null reference
                    _messages.ProblemMessages.AddSyntacticError(_problem, MessageCode.NullObject, internalObjectCount);
                } else {
                    //creates an internal representation of the object
                    internalObject = new Object();
                    objectId = obj.Id;
                    try {
                        _objects.Add(objectId, internalObject);
                    } catch (ArgumentException) {
                        //adds an error message if the object identifier is already in use
                        _messages.ObjectMessages.AddSemanticError(obj, MessageCode.ObjectIdAlreadyUsed);
                    }
                    //relates the object to its type
                    try {
                        objectTypeInfo = _typeInfos[obj.TypeId];
                        _objectArguments[objectId] = new SourceValuePair<IObject<TType, TObj>,
                            Argument<TObj>>(obj, new Argument<TObj>(objectTypeInfo, internalObject));
                        objectInfo = new ObjectInfo<TObj>(objectId, internalObject);
                        objectTypeInfo.AddObjectInfo(objectInfo, _typeCount);
                    } catch (KeyNotFoundException) {
                        //adds an error message if the object type does not exist
                        _messages.ObjectMessages.AddSemanticError(obj, MessageCode.InexistentObjectType);
                    }
                }
                internalObjectCount++;
            }
            //notifies errors, if any
            _messages.NotifyErrors();
            if (internalObjectCount != objectCount) {
                //adds an error message if the number of objects is incoherent
                _messages.ProblemMessages.AddSyntacticFatalError(_problem, MessageCode.IncoherentObjectCount);
            }
        }

        /// <summary>
        /// Generates the objects.
        /// </summary>
        private void GenerateObjects() {
            int objectCount = GetObjectCount();
            IEnumerator<IObject<TType, TObj>> objects = GetObjects();
            //generates the objects
            GenerateObjects(objectCount, objects);
            int typeId = 0;
            _types = new Type<TObj>[_typeCount];
            //stores the types in an array, by index
            foreach (TypeInfo<TObj> typeInfo in _typeInfos.Values) {
                _types[typeId] = new Type<TObj>(typeInfo.SetUpObjects());
                typeId++;
            }
        }

        /// <summary>
        /// Gets the number of knowledge provider predicates.
        /// </summary>
        /// <returns>The number of knowledge provider predicates.</returns>
        private int GetPredicateCount() {
            int predicateCount = _domain.PredicateCount;
            if (predicateCount < 0) {
                //adds an error message if the number of predicates is less than zero
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.InvalidPredicateCount);
            }
            return predicateCount;
        }

        /// <summary>
        /// Gets the knowledge provider predicates.
        /// </summary>
        /// <returns>The knowledge provider predicates.</returns>
        private IEnumerator<ISignature<TType, TPred, TObj>> GetPredicates() {
            IEnumerable<ISignature<TType, TPred, TObj>> predicates = _domain.GetPredicates();
            if (predicates == null) {
                //adds an error message if the predicates are a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullPredicates);
            }
            IEnumerator<ISignature<TType, TPred, TObj>> predicatesEnumerator = predicates.GetEnumerator();
            if (predicatesEnumerator == null) {
                //adds an error message if the predicates enumerator is a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullPredicatesEnumerator);
            }
            return predicatesEnumerator;
        }

        /// <summary>
        /// Generates a signature.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The signature type related messages.</param>
        /// <param name="source">The signature knowledge provider source.</param>
        /// <param name="index">The signature index.</param>
        /// <param name="parameterCount">The number of signature parameters.</param>
        /// <param name="parameters">The signature knowledge provider parameters.</param>
        /// <param name="arguments">The signature arguments.</param>
        /// <param name="indexedArguments">The indexed signature arguments.</param>
        /// <param name="parameterTypeInfos">The additional information about signature parameter types.</param>
        /// <returns>A signature.</returns>
        private Signature<TObj> GenerateSignature<S>(MessageCollection<S> messages,
        S source, int index, int parameterCount, IEnumerator<IObject<TType, TObj>> parameters,
        out IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments,
        out Argument<TObj>[] indexedArguments, out TypeInfo<TObj>[] parameterTypeInfos) {
            int[] parameterTypeIds = null;
            TypeInfo<TObj> parameterTypeInfo = null;
            IObject<TType, TObj> parameter;
            Argument<TObj> argument = null;
            int internalParameterCount = 0;
            try {
                parameterTypeIds = new int[parameterCount];
            } catch (IndexOutOfRangeException) {
                //adds an error if the numebr of parameters is invalid
                messages.AddSyntacticFatalError(source, MessageCode.InvalidParameterCount);
            }
            arguments = new Dictionary<TObj,
                SourceValuePair<IObject<TType, TObj>, Argument<TObj>>>(parameterCount);
            indexedArguments = new Argument<TObj>[parameterCount];
            parameterTypeInfos = new TypeInfo<TObj>[parameterCount];
            bool errorFound;
            //for each knowledge provider parameter
            while (parameters.MoveNext()) {
                parameter = parameters.Current;
                errorFound = false;
                if (parameter == null) {
                    //adds an error message if the parameter is a null reference
                    messages.AddSyntacticError(source, MessageCode.NullParameter, internalParameterCount);
                    errorFound = true;
                }
                if (!errorFound) {
                    try {
                        //relates the parameter to its type
                        parameterTypeInfo = _typeInfos[parameter.TypeId];
                        parameterTypeInfos[internalParameterCount] = parameterTypeInfo;
                        //creates the parameter argument
                        argument = new Argument<TObj>(parameterTypeInfo);
                    } catch (KeyNotFoundException) {
                        //adds an error message if the parameter type does not exist
                        messages.AddSemanticError(source, MessageCode.InexistentParameterType, internalParameterCount);
                        errorFound = true;
                    }
                }
                try {
                    arguments.Add(parameter.Id,
                        new SourceValuePair<IObject<TType, TObj>, Argument<TObj>>(parameter, argument));
                } catch (ArgumentException) {
                    //adds an error message if the parameter identifier is already in use
                    messages.AddSemanticError(source, MessageCode.ParameterIdAlreadyUsed, internalParameterCount);
                }
                if (!errorFound) {
                    try {
                        indexedArguments[internalParameterCount] = argument;
                        parameterTypeIds[internalParameterCount] = parameterTypeInfo.Id;
                    } catch (IndexOutOfRangeException) {
                        //adds an error message if the number of parameters if incoherent
                        messages.AddSyntacticFatalError(source, MessageCode.IncoherentParameterCount);
                    }
                }
                internalParameterCount++;
            }
            //notifies errors, if any
            messages.NotifyErrors();
            if (internalParameterCount != parameterCount) {
                //adds an error message if the number of parameters if incoherent
                messages.AddSyntacticFatalError(source, MessageCode.IncoherentParameterCount);
            }
            return new Signature<TObj>(_resources, index, parameterTypeIds);
        }

        /// <summary>
        /// Generates a signature.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The signature type related messages.</param>
        /// <param name="source">The signature knowledge provider source.</param>
        /// <param name="index">The signature index.</param>
        /// <param name="parameterCount">The number of signature parameters.</param>
        /// <param name="parameters">The signature knowledge provider parameters.</param>
        /// <param name="arguments">The signature arguments.</param>
        /// <param name="indexedArguments">The indexed signature arguments.</param>
        /// <param name="parameterTypeInfos">The additional information about signature parameter types.</param>
        /// <returns>A signature.</returns>
        private Signature<TObj> GenerateSignature<S>(MessageCollection<S> messages,
        S source, int index, int parameterCount, IEnumerable<IObject<TType, TObj>> parameters,
        out IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments,
        out Argument<TObj>[] indexedArguments, out TypeInfo<TObj>[] parameterTypeInfos) {
            if (parameters == null) {
                //adds an error if the parameters are a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullParameters);
            }
            IEnumerator<IObject<TType, TObj>> parametersEnumerator = parameters.GetEnumerator();
            if (parametersEnumerator == null) {
                //adds an error if the parameters enumerator is a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullParametersEnumerator);
            }
            //generates the signature
            return GenerateSignature(messages, source, index, parameterCount, parametersEnumerator,
                out arguments, out indexedArguments, out parameterTypeInfos);
        }

        /// <summary>
        /// Generates the signature.
        /// </summary>
        /// <param name="messages">The signature type related messages.</param>
        /// <param name="source">The signature knowledge provider source.</param>
        /// <param name="index">The signature index.</param>
        /// <param name="signature">The knowledge provider signature.</param>
        /// <param name="parameterTypeInfos">The additional information about signature parameter types.</param>
        /// <returns>A signature</returns>
        private Signature<TObj> GenerateSignature<S, TSig>(MessageCollection<S> messages,
        S source, int index, ISignature<TType, TSig, TObj> signature,
        out TypeInfo<TObj>[] parameterTypeInfos) {
            IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments;
            Argument<TObj>[] indexedArguments;
            return GenerateSignature(messages, source, index, signature.ParameterCount, signature.GetParameters(),
                out arguments, out indexedArguments, out parameterTypeInfos);
        }

        /// <summary>
        /// Adds an inequality predicate.
        /// </summary>
        /// <param name="predicateIndex">The predicate index.</param>
        /// <param name="inequalityPredicate">The inequality predicate.</param>
        /// <returns>The additional information about the inequality predicate.</returns>
        private PredicateInfo<TObj> AddInequalityPredicate(int predicateIndex,
        InequalityPredicate<TType, TPred> inequalityPredicate) {
            PredicateInfo<TObj> predicateInfo = null;
            Signature<TObj> predicate;
            TypeInfo<TObj> parameterTypeInfo = null;
            int parameterTypeId;
            TType typeId = inequalityPredicate.TypeId;
            try {
                parameterTypeInfo = _typeInfos[typeId];
                parameterTypeId = parameterTypeInfo.Id;
                //creates the inequality predicate signature
                predicate = new Signature<TObj>(_resources, predicateIndex, parameterTypeId, parameterTypeId);
                TPred predicateId = inequalityPredicate.PredicateId;
                try {
                    //adds the inequality predicate signature 
                    _predicates.Add(predicateId, predicate);
                    //creates the inequality predicate additional information
                    predicateInfo = new PredicateInfo<TObj>(predicate, parameterTypeInfo, parameterTypeInfo);
                    predicateInfo.IsInequality = true;
                    _predicateInfos[predicateId] = predicateInfo;
                } catch (ArgumentException) {
                    //adds an error message if the predicate identifier is already in use
                    _messages.InequalityPredicateMessages.AddSemanticError(inequalityPredicate, MessageCode.PredicateIdAlreadyUsed);
                }
            } catch (KeyNotFoundException) {
                //adds an error message if the predicate parameter type does not exist
                _messages.InequalityPredicateMessages.AddSemanticError(inequalityPredicate, MessageCode.InexistentParameterType);
            }
            //returns the inequality predicate additional information
            return predicateInfo;
        }

        /// <summary>
        /// Initializes the additional information about predicates.
        /// </summary>
        /// <param name="predicateCount">The number of predicates.</param>
        /// <returns>The number of predicates, after adding the inequality ones.</returns>
        private int InitializePredicateInfos(int predicateCount) {
            int adjustedPredicateCount = predicateCount;
            //adds the number of predicate inequalities to the total count
            if (_inequalityPredicatesEnabled) {
                adjustedPredicateCount += _inequalityPredicates.Count;
            }
            _predicates = new Dictionary<TPred, Signature<TObj>>(adjustedPredicateCount);
            _predicateSources = new Dictionary<TPred, ISignature<TType, TPred, TObj>>(adjustedPredicateCount);
            _predicateInfos = new Dictionary<TPred, PredicateInfo<TObj>>(adjustedPredicateCount);
            //adds the inequality predicates additional information
            if (_inequalityPredicatesEnabled) {
                int predicateIndex = 0;
                _inequalityPredicateInfos = new PredicateInfo<TObj>[_inequalityPredicates.Count];
                foreach (InequalityPredicate<TType, TPred> inequalityPredicate in _inequalityPredicates) {
                    _inequalityPredicateInfos[predicateIndex] = AddInequalityPredicate(predicateIndex, inequalityPredicate);
                    predicateIndex++;
                }
                _messages.NotifyErrors();
            }
            return adjustedPredicateCount;
        }

        /// <summary>
        /// Generates the predicates.
        /// </summary>
        /// <param name="predicateCount">The number of knowledge provider predicates.</param>
        /// <param name="predicates">The knowledge provider predicates.</param>
        private void GeneratePredicates(int predicateCount,
        IEnumerator<ISignature<TType, TPred, TObj>> predicates) {
            ISignature<TType, TPred, TObj> predicate;
            Signature<TObj> internalPredicate;
            TPred predicateId;
            TypeInfo<TObj>[] parameterTypeInfos;
            int adjustedPredicateCount = InitializePredicateInfos(predicateCount);
            int internalPredicateCount = adjustedPredicateCount - predicateCount;
            int predicatePosition = 0;
            //for each knowledge provider predicate
            while (predicates.MoveNext()) {
                predicate = predicates.Current;
                if (predicate == null) {
                    //adds an error message if the predicate is a null reference
                    _messages.DomainMessages.AddSyntacticError(_domain, MessageCode.NullPredicate, predicatePosition);
                } else {
                    predicateId = predicate.Id;
                    //generates the predicate signature
                    internalPredicate = GenerateSignature(_messages.PredicateMessages, predicate,
                        internalPredicateCount, predicate, out parameterTypeInfos);
                    try {
                        //adds the predicate signature
                        _predicates.Add(predicateId, internalPredicate);
                        _predicateSources[predicateId] = predicate;
                        _predicateInfos[predicateId] = new PredicateInfo<TObj>(internalPredicate, parameterTypeInfos);
                    } catch (ArgumentException) {
                        //adds an error message if the predicate identifier is already in use
                        _messages.PredicateMessages.AddSemanticError(predicate, MessageCode.PredicateIdAlreadyUsed);
                    }
                }
                internalPredicateCount++;
                predicatePosition++;
            }
            //notifies errors, if any
            _messages.NotifyErrors();
            if (internalPredicateCount != adjustedPredicateCount) {
                //adds an error message if the number of predicates is incoherent
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.IncoherentPredicateCount);
            }
        }

        /// <summary>
        /// Generates the predicates.
        /// </summary>
        private void GeneratePredicates() {
            int predicateCount = GetPredicateCount();
            IEnumerator<ISignature<TType, TPred, TObj>> predicates = GetPredicates();
            GeneratePredicates(predicateCount, predicates);
        }

        /// <summary>
        /// Gets the number of knowledge provider functions.
        /// </summary>
        /// <returns>The number of knowledge provider functions.</returns>
        private int GetFunctionCount() {
            int functionCount = _domain.FunctionCount;
            if (functionCount < 0) {
                //adds an error if the number of functions is less than zero
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.InvalidFunctionCount);
            }
            return functionCount;
        }

        /// <summary>
        /// Gets the knowledge provider functions.
        /// </summary>
        /// <returns>The knowledge provider functions.</returns>
        private IEnumerator<ISignature<TType, TFunc, TObj>> GetFunctions() {
            IEnumerable<ISignature<TType, TFunc, TObj>> functions = _domain.GetFunctions();
            if (functions == null) {
                //adds an error message if the functions are a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullFunctions);
            }
            IEnumerator<ISignature<TType, TFunc, TObj>> functionsEnumerator = functions.GetEnumerator();
            if (functionsEnumerator == null) {
                //adds an error message if the functions enumerator is a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullFunctionsEnumerator);
            }
            return functionsEnumerator;
        }

        /// <summary>
        /// Generates the functions.
        /// </summary>
        /// <param name="functionCount">The number of knowledge provider functions.</param>
        /// <param name="functions">The knowledge provider functions.</param>
        private void GenerateFunctions(int functionCount, IEnumerator<ISignature<TType, TFunc, TObj>> functions) {
            ISignature<TType, TFunc, TObj> function;
            Signature<TObj> internalFunction;
            TFunc functionId;
            TypeInfo<TObj>[] parameterTypeInfos;
            int internalFunctionCount = 0;
            _functions = new Dictionary<TFunc, Signature<TObj>>(functionCount);
            _functionSources = new Dictionary<TFunc, ISignature<TType, TFunc, TObj>>(functionCount);
            _functionInfos = new Dictionary<TFunc, FunctionInfo<TObj>>(functionCount);
            //for each knowledge provider function
            while (functions.MoveNext()) {
                function = functions.Current;
                if (function == null) {
                    //adds an error message if the function is a null reference
                    _messages.DomainMessages.AddSyntacticError(_domain, MessageCode.NullFunction, internalFunctionCount);
                } else {
                    functionId = function.Id;
                    //generates function signature
                    internalFunction = GenerateSignature<ISignature<TType, TFunc, TObj>, TFunc>(
                        _messages.FunctionMessages, function, internalFunctionCount, function,
                        out parameterTypeInfos);
                    try {
                        //adds the function
                        _functions.Add(functionId, internalFunction);
                        _functionSources[functionId] = function;
                        _functionInfos[functionId] = new FunctionInfo<TObj>(internalFunction, parameterTypeInfos);
                    } catch (ArgumentException) {
                        //adds an error message if the function identifier is already in use
                        _messages.FunctionMessages.AddSemanticError(function, MessageCode.FunctionIdAlreadyUsed);
                    }
                }
                internalFunctionCount++;
            }
            //notifies errors, if any
            _messages.NotifyErrors();
            if (internalFunctionCount != functionCount) {
                //adds an error message if the number of functions is incoherent
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.IncoherentFunctionCount);
            }
        }

        /// <summary>
        /// Generates the functions.
        /// </summary>
        private void GenerateFunctions() {
            int functionCount = GetFunctionCount();
            IEnumerator<ISignature<TType, TFunc, TObj>> functions = GetFunctions();
            GenerateFunctions(functionCount, functions);
        }

        /// <summary>
        /// Gets the number of knowledge provider operators.
        /// </summary>
        /// <returns>The number of knowledge provider operators.</returns>
        private int GetOperatorCount() {
            int operatorCount = _domain.OperatorCount;
            if (operatorCount < 0) {
                //adds an error message if the number of operators is less than zero
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.InvalidOperatorCount);
            }
            return operatorCount;
        }

        /// <summary>
        /// Gets the knowledge provider operators.
        /// </summary>
        /// <returns>The knowledge provider operators.</returns>
        private IEnumerator<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> GetOperators() {
            IEnumerable<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>
                operators = _domain.GetOperators();
            if (operators == null) {
                //adds an error mesage if the operators are a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullOperators);
            }
            IEnumerator<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>
                operatorsEnumerator = operators.GetEnumerator();
            if (operatorsEnumerator == null) {
                //adds an error mesage if the operators enumerator is a null reference
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullOperatorsEnumerator);
            }
            return operatorsEnumerator;
        }

        /// <summary>
        /// Generates a list of arguments for a signature.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The argument related translator messages.</param>
        /// <param name="source">The signature knowledge provider source.</param>
        /// <param name="signature">The signature.</param>
        /// <param name="argumentCount">The number of arguments.</param>
        /// <param name="argumentIdsEnumerator">The argument identifiers.</param>
        /// <param name="arguments">All the arguments.</param>
        /// <returns>The list of arguments.</returns>
        private static Argument<TObj>[] GenerateArguments<S>(MessageCollection<S> messages,
        S source, Signature<TObj> signature, int argumentCount, IEnumerator<TObj> argumentIdsEnumerator,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments) {
            Argument<TObj>[] internalArguments = new Argument<TObj>[argumentCount];
            Argument<TObj> argument;
            TObj argumentId;
            int[] parameterTypeIds = signature.ParameterTypeIds;
            int internalArgumentCount = 0;
            //for each argument
            while (argumentIdsEnumerator.MoveNext()) {
                argumentId = argumentIdsEnumerator.Current;
                try {
                    argument = arguments[argumentId].Value;
                    if (!argument.TypeInfo.IsSubtypeOf(parameterTypeIds[internalArgumentCount])) {
                        //adds an error message if the type of the argument is not a subtype of the parameter's one
                        messages.AddSemanticError(source, MessageCode.ArgumentTypeMismatch, internalArgumentCount);
                    } else {
                        try {
                            internalArguments[internalArgumentCount] = argument;
                            //marks argument as used
                            argument.Used = true;
                        } catch (IndexOutOfRangeException) {
                            //adds an error message if the number of arguments is incoherent
                            messages.AddSyntacticFatalError(source, MessageCode.IncoherentArgumentCount);
                        }
                    }
                } catch (KeyNotFoundException) {
                    //adds an error message if an argument object does not exist
                    messages.AddSemanticError(source, MessageCode.InexistentObject, internalArgumentCount);
                }
                internalArgumentCount++;
            }
            //notifies errors, if any
            messages.NotifyErrors();
            if (internalArgumentCount != argumentCount) {
                //adds an error message if the number of arguments is incoherent
                messages.AddSyntacticFatalError(source, MessageCode.IncoherentArgumentCount);
            }
            return internalArguments;
        }

        /// <summary>
        /// Generates the arguments.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The argument related translator messages.</param>
        /// <param name="source">The signature knowledge provider source.</param>
        /// <param name="signature">The signature.</param>
        /// <param name="argumentCount">The number of arguments.</param>
        /// <param name="argumentIds">The argument identifiers.</param>
        /// <param name="arguments">All the arguments.</param>
        /// <returns>The list of arguments.</returns>
        private static Argument<TObj>[] GenerateArguments<S>(MessageCollection<S> messages,
        S source, Signature<TObj> signature, int argumentCount, IEnumerable<TObj> argumentIds,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments) {
            if (argumentCount != signature.ParameterTypeIds.Length) {
                //adds an error message if the number of arguments is different from the number of parameters
                messages.AddSemanticFatalError(source, MessageCode.ArgumentCountMismatch);
            }
            if (argumentIds == null) {
                //adds an error message if the argument identifiers are a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullArgumentIds);
            }
            IEnumerator<TObj> argumentIdsEnumerator = argumentIds.GetEnumerator();
            if (argumentIdsEnumerator == null) {
                //adds an error message if the argument identifiers enumerator is a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullArgumentIdsEnumerator);
            }
            return GenerateArguments(messages, source, signature, argumentCount, argumentIdsEnumerator, arguments);
        }

        /// <summary>
        /// Generates a fact.
        /// </summary>
        /// <param name="fact">The knowledge provider fact.</param>
        /// <param name="arguments">All the arguments.</param>
        /// <returns>The fact.</returns>
        private Fact<TObj> GenerateFact(IInstance<TPred, TObj> fact,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments) {
            PredicateInfo<TObj> predicateInfo = null;
            try {
                predicateInfo = _predicateInfos[fact.SignatureId];
            } catch (KeyNotFoundException) {
                //adds an error message if the predicate does not exist
                _messages.FactMessages.AddSemanticFatalError(fact, MessageCode.InexistentPredicate);
            }
            int argumentCount = fact.ArgumentCount;
            IEnumerable<TObj> argumentIds = fact.GetArgumentIds();
            //generates fact arguments
            Argument<TObj>[] factArguments = GenerateArguments(_messages.FactMessages, fact,
                predicateInfo.Signature, argumentCount, argumentIds, arguments);
            //generates and returns the fact
            return new Fact<TObj>(predicateInfo, factArguments);
        }

        /// <summary>
        /// Gets a fact list from the knowledge provider.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The fact related translator messages.</param>
        /// <param name="source">The facts knowledge provider source.</param>
        /// <param name="factCount">The number of facts.</param>
        /// <param name="facts">The knowledge provider facts.</param>
        /// <returns>The facts list.</returns>
        private static IEnumerator<IIdentifiedInstance<TPred, TObj, TFact>>
        GetAndCheckFacts<S>(MessageCollection<S> messages, S source, int factCount,
        IEnumerable<IIdentifiedInstance<TPred, TObj, TFact>> facts) {
            if (factCount < 0) {
                //adds an error message if the number of facts is less than zero
                messages.AddSyntacticFatalError(source, MessageCode.InvalidFactCount);
            }
            if (facts == null) {
                //adds an error message if the facts are a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullFacts);
            }
            IEnumerator<IIdentifiedInstance<TPred, TObj, TFact>> factsEnumerator = facts.GetEnumerator();
            if (factsEnumerator == null) {
                //adds an error message if the facts enumerator is a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullFactsEnumerator);
            }
            return factsEnumerator;
        }

        /// <summary>
        /// Gets a variable list from the knowledge provider.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The variable related translator messages.</param>
        /// <param name="source">The variables knowledge provider source.</param>
        /// <param name="variableCount">The number of variables.</param>
        /// <param name="variables">The knowledge provider variables.</param>
        /// <returns>The variables list.</returns>
        private static IEnumerator<IIdentifiedInstance<TFunc, TObj, TVar>>
        GetAndCheckVariables<S>(MessageCollection<S> messages, S source, int variableCount,
        IEnumerable<IIdentifiedInstance<TFunc, TObj, TVar>> variables) {
            if (variableCount < 0) {
                //adds an error message if the number of variables is less than zero
                messages.AddSyntacticFatalError(source, MessageCode.InvalidVariableCount);
            }
            if (variables == null) {
                //adds an error message if the variables are a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullVariables);
            }
            IEnumerator<IIdentifiedInstance<TFunc, TObj, TVar>> instancesEnumerator = variables.GetEnumerator();
            if (instancesEnumerator == null) {
                //adds an error message if the variables enumerator is a null reference
                messages.AddSyntacticFatalError(source, MessageCode.NullVariablesEnumerator);
            }
            return instancesEnumerator;
        }

        /// <summary>
        /// Generates the facts.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The fact related translator messages.</param>
        /// <param name="source">The facts knowledge provider source.</param>
        /// <param name="factCount">The number of facts.</param>
        /// <param name="facts">The knowledge provider facts.</param>
        /// <param name="factsById">The generated facts, by identifier.</param>
        /// <param name="arguments">The object arguments to be used by the facts.</param>
        /// <returns>The generated facts, by index.</returns>
        private Fact<TObj>[] GenerateFacts<S>(MessageCollection<S> messages,
        S source, int factCount, IEnumerator<IIdentifiedInstance<TPred, TObj, TFact>> facts,
        out IDictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>, Fact<TObj>>> factsById,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments) {
            IIdentifiedInstance<TPred, TObj, TFact> fact;
            Fact<TObj>[] internalFacts = new Fact<TObj>[factCount];
            factsById = new Dictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>,
                Fact<TObj>>>(factCount);
            Fact<TObj> internalFact;
            int internalFactCount = 0;
            //for each knowledge provider fact
            while (facts.MoveNext()) {
                fact = facts.Current;
                if (fact == null) {
                    //adds an error message if the fact is a null reference
                    messages.AddSyntacticFatalError(source, MessageCode.NullFact, internalFactCount);
                }
                //generates the internal fact representation
                internalFact = GenerateFact(fact, arguments);
                try {
                    factsById.Add(fact.Id, new SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>,
                        Fact<TObj>>(fact, internalFact));
                } catch (ArgumentException) {
                    //adds an error message if the fact identifier is already in use
                    _messages.FactMessages.AddSemanticFatalError(fact, MessageCode.FactIdAlreadyUsed);
                }
                try {
                    internalFacts[internalFactCount] = internalFact;
                } catch (IndexOutOfRangeException) {
                    //adds an error message if the number of facts is incoherent
                    messages.AddSyntacticFatalError(source, MessageCode.IncoherentFactCount);
                }
                internalFactCount++;
            }
            if (internalFactCount != factCount) {
                //adds an error message if the number of facts is incoherent
                messages.AddSyntacticFatalError(source, MessageCode.IncoherentFactCount);
            }
            return internalFacts;
        }

        /// <summary>
        /// Gets a fact action message code.
        /// </summary>
        /// <param name="action">The action.</param>
        /// <param name="code">The message code.</param>
        /// <returns>A more accurate message code.</returns>
        private static MessageCode GetActionMessageCode(FactAction action, MessageCode code) {
            switch (action) {
                case FactAction.Initialization:
                    switch (code) {
                        case MessageCode.InvalidFactCount:
                            return MessageCode.InvalidInitialFactCount;
                        case MessageCode.NullFacts:
                            return MessageCode.NullInitialFacts;
                        case MessageCode.NullFactsEnumerator:
                            return MessageCode.NullInitialFactsEnumerator;
                        case MessageCode.IncoherentFactCount:
                            return MessageCode.IncoherentInitialFactCount;
                        case MessageCode.InexistentFact:
                            return MessageCode.InexistentInitialFact;
                    }
                    break;
                case FactAction.Goal:
                    switch (code) {
                        case MessageCode.InvalidFactCount:
                            return MessageCode.InvalidGoalFactCount;
                        case MessageCode.NullFacts:
                            return MessageCode.NullGoalFacts;
                        case MessageCode.NullFactsEnumerator:
                            return MessageCode.NullGoalFactsEnumerator;
                        case MessageCode.IncoherentFactCount:
                            return MessageCode.IncoherentGoalFactCount;
                        case MessageCode.InexistentFact:
                            return MessageCode.InexistentGoalFact;
                    }
                    break;
                case FactAction.Precondition:
                    switch (code) {
                        case MessageCode.InvalidFactCount:
                            return MessageCode.InvalidPreconditionFactCount;
                        case MessageCode.NullFacts:
                            return MessageCode.NullPreconditionFacts;
                        case MessageCode.NullFactsEnumerator:
                            return MessageCode.NullPreconditionFactsEnumerator;
                        case MessageCode.IncoherentFactCount:
                            return MessageCode.IncoherentPreconditionFactCount;
                        case MessageCode.InexistentFact:
                            return MessageCode.InexistentPreconditionFact;
                    }
                    break;
                case FactAction.Addition:
                    switch (code) {
                        case MessageCode.InvalidFactCount:
                            return MessageCode.InvalidAdditionFactCount;
                        case MessageCode.NullFacts:
                            return MessageCode.NullAdditionFacts;
                        case MessageCode.NullFactsEnumerator:
                            return MessageCode.NullAdditionFactsEnumerator;
                        case MessageCode.IncoherentFactCount:
                            return MessageCode.IncoherentAdditionFactCount;
                        case MessageCode.InexistentFact:
                            return MessageCode.InexistentAdditionFact;
                    }
                    break;
                case FactAction.Deletion:
                    switch (code) {
                        case MessageCode.InvalidFactCount:
                            return MessageCode.InvalidDeletionFactCount;
                        case MessageCode.NullFacts:
                            return MessageCode.NullDeletionFacts;
                        case MessageCode.NullFactsEnumerator:
                            return MessageCode.NullDeletionFactsEnumerator;
                        case MessageCode.IncoherentFactCount:
                            return MessageCode.IncoherentDeletionFactCount;
                        case MessageCode.InexistentFact:
                            return MessageCode.InexistentDeletionFact;
                    }
                    break;
            }
            return code;
        }

        /// <summary>
        /// Gets a knowledge provider selected fact list.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The fact related translator messages.</param>
        /// <param name="source">The selected fact list knowledge provider source.</param>
        /// <param name="selectedFactCount">The number of knowledge provider selected facts.</param>
        /// <param name="selectedFacts">The knowledge provider selected facts.</param>
        /// <param name="selectedFactsAction">The selected facts action.</param>
        /// <returns>The selected facts.</returns>
        private static IEnumerator<TFact> GetAndCheckFacts<S>(MessageCollection<S> messages,
        S source, int selectedFactCount, IEnumerable<TFact> selectedFacts, FactAction selectedFactsAction) {
            if (selectedFactCount < 0) {
                //adds an error message if the number of facts is less than zero
                messages.AddSyntacticFatalError(source, GetActionMessageCode(selectedFactsAction, MessageCode.InvalidFactCount));
            }
            if (selectedFacts == null) {
                //adds an error message if the facts are a null reference
                messages.AddSyntacticFatalError(source, GetActionMessageCode(selectedFactsAction, MessageCode.NullFacts));
            }
            IEnumerator<TFact> selectedFactsEnumerator = selectedFacts.GetEnumerator();
            if (selectedFactsEnumerator == null) {
                //adds an error message if the facts enumerator is a null reference
                messages.AddSyntacticFatalError(source, GetActionMessageCode(selectedFactsAction, MessageCode.NullFactsEnumerator));
            }
            return selectedFactsEnumerator;
        }

        /// <summary>
        /// Selects facts from a total list.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The fact related translator messages.</param>
        /// <param name="source">The facts selection list knowledge provider source.</param>
        /// <param name="factsById">The facts by identifier.</param>
        /// <param name="selectedFactCount">The number of selected facts.</param>
        /// <param name="selectedFacts">The knowledge provider selected facts.</param>
        /// <param name="selectedFactsAction">The knowledge provider selected facts action.</param>
        /// <returns>A list of facts selected from <paramref name="factsById"/> which corresponds to
        /// <paramref name="selectedFacts"/>.</returns>
        private static Fact<TObj>[] SelectFacts<S>(MessageCollection<S> messages, S source,
        IDictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>, Fact<TObj>>> factsById,
        int selectedFactCount, IEnumerable<TFact> selectedFacts, FactAction selectedFactsAction) {
            IEnumerator<TFact> selectedFactsEnumerator =
                GetAndCheckFacts(messages, source, selectedFactCount, selectedFacts, selectedFactsAction);
            Fact<TObj>[] internalSelectedFacts = new Fact<TObj>[selectedFactCount];
            Fact<TObj> internalSelectedFact;
            TFact factId;
            int internalSelectedFactCount = 0;
            //for each knowledge provider selected fact
            while (selectedFactsEnumerator.MoveNext()) {
                factId = selectedFactsEnumerator.Current;
                try {
                    //selects the fact
                    internalSelectedFact = factsById[factId].Value;
                    try {
                        //adds the fact
                        internalSelectedFacts[internalSelectedFactCount] = internalSelectedFact;
                    } catch (IndexOutOfRangeException) {
                        //adds an error message if the number of facts is incoherent
                        messages.AddSyntacticFatalError(source, GetActionMessageCode(selectedFactsAction, MessageCode.IncoherentFactCount));
                    }
                    internalSelectedFact.AddAction(messages, source, internalSelectedFactCount, selectedFactsAction);
                } catch (KeyNotFoundException) {
                    //adds an error message if the selected fact does not exist
                    messages.AddSemanticError(source, GetActionMessageCode(selectedFactsAction, MessageCode.InexistentFact), internalSelectedFactCount);
                }
                internalSelectedFactCount++;
            }
            //notifies errors, if any
            messages.NotifyErrors();
            if (internalSelectedFactCount != selectedFactCount) {
                //adds an error message if the number of facts is incoherent
                messages.AddSyntacticFatalError(source, GetActionMessageCode(selectedFactsAction, MessageCode.IncoherentFactCount));
            }
            return internalSelectedFacts;
        }

        /// <summary>
        /// Generates a variable.
        /// </summary>
        /// <param name="variable">The knowledge provider variable.</param>
        /// <param name="arguments">The total arguments.</param>
        /// <param name="index">The variable index.</param>
        /// <returns>A variable.</returns>
        private Variable<TObj> GenerateVariable(IInstance<TFunc, TObj> variable,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments,
        int index) {
            FunctionInfo<TObj> functionInfo = null;
            try {
                functionInfo = _functionInfos[variable.SignatureId];
            } catch (KeyNotFoundException) {
                //adds a knowledge provider error message if the variable function does not exist
                _messages.VariableMessages.AddSemanticFatalError(variable, MessageCode.InexistentFunction);
            }
            int argumentCount = variable.ArgumentCount;
            IEnumerable<TObj> argumentIds = variable.GetArgumentIds();
            //generates function arguments
            Argument<TObj>[] variableArguments = GenerateArguments(_messages.VariableMessages, variable,
                functionInfo.Signature, argumentCount, argumentIds, arguments);
            //generates and returns the variable
            return new Variable<TObj>(functionInfo, variableArguments, index);
        }

        /// <summary>
        /// Generates the variables.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The variable related translator messages.</param>
        /// <param name="source">The variables knowledge provider source.</param>
        /// <param name="variableCount">The number of variables.</param>
        /// <param name="variables">The knowledge provider variables.</param>
        /// <param name="variablesById">The generated variables, by identifier.</param>
        /// <param name="arguments">The object arguments to be used by the variables.</param>
        /// <returns>The generated variables, by index.</returns>
        private Variable<TObj>[] GenerateVariables<S>(MessageCollection<S> messages,
        S source, int variableCount, IEnumerator<IIdentifiedInstance<TFunc, TObj, TVar>> variables,
        out IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variablesById,
        IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments) {
            IIdentifiedInstance<TFunc, TObj, TVar> variable;
            Variable<TObj>[] internalVariables = new Variable<TObj>[variableCount];
            variablesById = new Dictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>,
                Variable<TObj>>>(variableCount);
            Variable<TObj> internalVariable;
            int internalVariableCount = 0;
            //for each knowledge provider variable
            while (variables.MoveNext()) {
                variable = variables.Current;
                if (variable == null) {
                    //adds an error message if the variable is a null reference
                    messages.AddSyntacticFatalError(source, MessageCode.NullVariable, internalVariableCount);
                }
                //generates the variable
                internalVariable = GenerateVariable(variable, arguments, internalVariableCount);
                try {
                    variablesById.Add(variable.Id, new SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>,
                        Variable<TObj>>(variable, internalVariable));
                } catch (ArgumentException) {
                    //adds an error message if the variable identifier is already in use
                    _messages.VariableMessages.AddSemanticFatalError(variable, MessageCode.VariableIdAlreadyUsed);
                }
                try {
                    internalVariables[internalVariableCount] = internalVariable;
                } catch (IndexOutOfRangeException) {
                    //adds an error message if the number of variables is incoherent
                    messages.AddSyntacticFatalError(source, MessageCode.IncoherentVariableCount);
                }
                internalVariableCount++;
            }
            if (internalVariableCount != variableCount) {
                //adds an error message if the number of variables is incoherent
                messages.AddSyntacticFatalError(source, MessageCode.IncoherentVariableCount);
            }
            return internalVariables;
        }

        /// <summary>
        /// Checks if all facts have been used.
        /// </summary>
        /// <param name="facts">The facts.</param>
        private void CheckFactUse(IDictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>,
        Fact<TObj>>> facts) {
            //for each fact
            foreach (SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>, Fact<TObj>> fact in facts.Values) {
                if (!fact.Value.Used) {
                    //adds an error message if the fact has not been used
                    _messages.FactMessages.AddSemanticWarning(fact.Source, MessageCode.FactNotUsed);
                }
            }
        }

        /// <summary>
        /// Checks if all objects have been used.
        /// </summary>
        /// <param name="objects">The objects.</param>
        /// <param name="objectNotUsed">The "object not used" translator message code.</param>
        private void CheckObjectUse(IDictionary<TObj, SourceValuePair<IObject<TType, TObj>,
        Argument<TObj>>> objects, MessageCode objectNotUsed) {
            //for each object
            foreach (SourceValuePair<IObject<TType, TObj>, Argument<TObj>> obj in objects.Values) {
                if (!obj.Value.Used) {
                    //adds an error message if the object has not been used
                    _messages.ObjectMessages.AddSemanticWarning(obj.Source, objectNotUsed);
                }
            }
        }

        /// <summary>
        /// Checks if all variables have been used.
        /// </summary>
        /// <param name="variables">The variables.</param>
        private void CheckVariableUse(IDictionary<TVar,
        SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variables) {
            //for each variable
            foreach (SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>,
            Variable<TObj>> variable in variables.Values) {
                if (!variable.Value.Used) {
                    //adds an error message if the variable has not been used
                    _messages.VariableMessages.AddSemanticWarning(variable.Source, MessageCode.VariableNotUsed);
                }
            }
        }

        /// <summary>
        /// Generates an operand.
        /// </summary>
        /// <param name="operand">The knowledge provider operand.</param>
        /// <param name="variables">The variables to be used by the operand.</param>
        /// <param name="directVariableAccess">If set to true, direct variable access must be used.</param>
        /// <returns>The generated operand.</returns>
        private Operand GenerateOperand(IOperand<TVar> operand,
        IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variables,
        bool directVariableAccess) {
            switch (operand.Type) {
                //if operand is a variable
                case OperandType.Variable:
                    SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>,
                        Variable<TObj>> sourceValuePair;
                    Variable<TObj> variable;
                    if (variables.TryGetValue(operand.VariableId, out sourceValuePair)) {
                        variable = sourceValuePair.Value;
                        //marks function and variable as used
                        variable.FunctionInfo.Used = true;
                        variable.Used = true;
                        return new VariableOperand<TObj>(variable, directVariableAccess);
                    } else {
                        //adds an error message if the variable is unknown
                        _messages.OperandMessages.AddSemanticFatalError(operand, MessageCode.InexistentVariable);
                    }
                    break;
                //if operand is a constant
                case OperandType.Constant:
                    return new ConstantOperand(operand.ConstantValue);
                //if operand is an operation
                case OperandType.Operation:
                    IOperation<TVar> operation = operand.Operation;
                    if (operation == null) {
                        //adds an error message if the operation is a null reference
                        _messages.OperandMessages.AddSyntacticFatalError(operand, MessageCode.NullOperation);
                    }
                    return GenerateOperation(operation, variables, directVariableAccess);
                default:
                    //adds an error message if the operand type is unknown
                    _messages.OperandMessages.AddSyntacticFatalError(operand, MessageCode.InvalidOperandType);
                    break;
            }
            return null;
        }

        /// <summary>
        /// Generates an operation.
        /// </summary>
        /// <param name="operation">The knowledge provider operation.</param>
        /// <param name="variables">The variables to be used by the operation.</param>
        /// <param name="directVariableAccess">If set to true, direct variable access must be used.</param>
        /// <returns>The generated operation</returns>
        private OperationOperand<TVar> GenerateOperation(IOperation<TVar> operation,
        IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variables,
        bool directVariableAccess) {
            int operandCount = operation.OperandCount;
            if (operandCount != 2) {
                //adds an error message if the number of operands is different from two
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.InvalidOperandCount);
            }
            IEnumerable<IOperand<TVar>> operands = operation.GetOperands();
            if (operands == null) {
                //adds an error message if the operands are a null reference
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.NullOperands);
            }
            IEnumerator<IOperand<TVar>> operandsEnumerator = operands.GetEnumerator();
            if (operandsEnumerator == null) {
                //adds an error message if the operands are a null reference
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.NullOperandsEnumerator);
            }
            if (!operandsEnumerator.MoveNext()) {
                //adds an error message if the number of operands is incoherent
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.IncoherentOperandCount);
            }
            IOperand<TVar> leftOperand = operandsEnumerator.Current;
            if (leftOperand == null) {
                //adds an error message if the left operand is a null reference
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.NullOperand, 0);
            }
            if (!directVariableAccess && IsEffect(operation.Type) && (leftOperand.Type == OperandType.Variable)) {
                SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>> variable;
                if (variables.TryGetValue(leftOperand.VariableId, out variable)) {
                    //marks written variables
                    variable.Value.FunctionInfo.IsReadOnly = false;
                }
            }
            if (!operandsEnumerator.MoveNext()) {
                //adds an error message if the number of operands is incoherent
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.IncoherentOperandCount);
            }
            IOperand<TVar> rightOperand = operandsEnumerator.Current;
            if (rightOperand == null) {
                //adds an error message if the right operand is a null reference
                _messages.OperationMessages.AddSyntacticFatalError(operation, MessageCode.NullOperand, 1);
            }
            return new OperationOperand<TVar>(_messages.OperationMessages, operation,
                operation.Type, _messages.OperandMessages, leftOperand,
                GenerateOperand(leftOperand, variables, directVariableAccess), rightOperand,
                GenerateOperand(rightOperand, variables, directVariableAccess));
        }

        /// <summary>
        /// Gets an operation section message code.
        /// </summary>
        /// <param name="section">The section.</param>
        /// <param name="code">The message code.</param>
        /// <returns>A more accurate message code.</returns>
        private static MessageCode GetSectionMessageCode(OperationSection section, MessageCode code) {
            switch (section) {
                case OperationSection.Initialization:
                    switch (code) {
                        case MessageCode.InvalidOperationCount:
                            return MessageCode.InvalidInitializationOperationCount;
                        case MessageCode.NullOperation:
                            return MessageCode.NullInitializationOperation;
                        case MessageCode.NullOperations:
                            return MessageCode.NullInitializationOperations;
                        case MessageCode.NullOperationsEnumerator:
                            return MessageCode.NullInitializationOperationsEnumerator;
                        case MessageCode.IncoherentOperationCount:
                            return MessageCode.IncoherentInitializationOperationCount;
                    }
                    break;
                case OperationSection.Goal:
                    switch (code) {
                        case MessageCode.InvalidOperationCount:
                            return MessageCode.InvalidGoalOperationCount;
                        case MessageCode.NullOperation:
                            return MessageCode.NullGoalOperation;
                        case MessageCode.NullOperations:
                            return MessageCode.NullGoalOperations;
                        case MessageCode.NullOperationsEnumerator:
                            return MessageCode.NullGoalOperationsEnumerator;
                        case MessageCode.IncoherentOperationCount:
                            return MessageCode.IncoherentGoalOperationCount;
                    }
                    break;
                case OperationSection.Precondition:
                    switch (code) {
                        case MessageCode.InvalidOperationCount:
                            return MessageCode.InvalidPreconditionOperationCount;
                        case MessageCode.NullOperation:
                            return MessageCode.NullPreconditionOperation;
                        case MessageCode.NullOperations:
                            return MessageCode.NullPreconditionOperations;
                        case MessageCode.NullOperationsEnumerator:
                            return MessageCode.NullPreconditionOperationsEnumerator;
                        case MessageCode.IncoherentOperationCount:
                            return MessageCode.IncoherentPreconditionOperationCount;
                    }
                    break;
                case OperationSection.Effect:
                    switch (code) {
                        case MessageCode.InvalidOperationCount:
                            return MessageCode.InvalidEffectOperationCount;
                        case MessageCode.NullOperation:
                            return MessageCode.NullEffectOperation;
                        case MessageCode.NullOperations:
                            return MessageCode.NullEffectOperations;
                        case MessageCode.NullOperationsEnumerator:
                            return MessageCode.NullEffectOperationsEnumerator;
                        case MessageCode.IncoherentOperationCount:
                            return MessageCode.IncoherentEffectOperationCount;
                    }
                    break;
            }
            return code;
        }

        /// <summary>
        /// Generates the operations.
        /// </summary>
        /// <typeparam name="S">The message source type.</typeparam>
        /// <param name="messages">The operation related translator messages.</param>
        /// <param name="source">The operations knowledge provider source.</param>
        /// <param name="operationCount">The operation count.</param>
        /// <param name="operations">The knowledge provider operations.</param>
        /// <param name="variables">The variables to be used by the operations.</param>
        /// <param name="directVariableAccess">If set to true, direct variable access must be used.</param>
        /// <param name="operationSources">The operation knowledge provider sources.</param>
        /// <param name="section">The operations section.</param>
        /// <returns>The generated operations.</returns>
        private OperationOperand<TVar>[] GenerateOperations<S>(MessageCollection<S> messages,
        S source, int operationCount, IEnumerable<IOperation<TVar>> operations,
        IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variables,
        bool directVariableAccess, out IOperation<TVar>[] operationSources, OperationSection section) {
            if (operationCount < 0) {
                //adds an error message if the number of operations is less than zero
                messages.AddSyntacticFatalError(source, GetSectionMessageCode(section, MessageCode.InvalidOperationCount));
            }
            if (operations == null) {
                //adds an error message if the operations are a null reference
                messages.AddSyntacticFatalError(source, GetSectionMessageCode(section, MessageCode.NullOperations));
            }
            IEnumerator<IOperation<TVar>> operationsEnumerator = operations.GetEnumerator();
            if (operationsEnumerator == null) {
                //adds an error message if the operations enumerator is a null reference
                messages.AddSyntacticFatalError(source, GetSectionMessageCode(section, MessageCode.NullOperationsEnumerator));
            }
            operationSources = null;
            try {
                operationSources = new IOperation<TVar>[operationCount];
            } catch (IndexOutOfRangeException) {
                //adds an error message if the number of operations is invalid
                messages.AddSyntacticFatalError(source, GetSectionMessageCode(section, MessageCode.InvalidOperationCount));
            }
            OperationOperand<TVar>[] internalOperations = new OperationOperand<TVar>[operationCount];
            int internalOperationCount = 0;
            IOperation<TVar> operation;
            //for each knowledge provider operation
            while (operationsEnumerator.MoveNext()) {
                operation = operationsEnumerator.Current;
                if (operation == null) {
                    //adds an error message if the operation is a null reference
                    messages.AddSyntacticFatalError(source, GetSectionMessageCode(section, MessageCode.NullOperation), internalOperationCount);
                }
                try {
                    //generates the operation and stores its source
                    operationSources[internalOperationCount] = operation;
                    internalOperations[internalOperationCount] =
                        GenerateOperation(operation, variables, directVariableAccess);
                } catch (IndexOutOfRangeException) {
                    //adds an error message if the number of operations is incoherent
                    messages.AddSyntacticFatalError(source, MessageCode.IncoherentOperationCount);
                }
                internalOperationCount++;
            }
            if (operationCount != internalOperationCount) {
                //adds an error message if the number of operations is incoherent
                messages.AddSyntacticFatalError(source, MessageCode.IncoherentOperationCount);
            }
            return internalOperations;
        }

        /// <summary>
        /// Determines whether the specified operation type is a condition.
        /// </summary>
        /// <param name="operationType">The operation type.</param>
        /// <returns>True if the specified operation type is a condition; otherwise, false.</returns>
        private static bool IsCondition(OperationType operationType) {
            switch (operationType) {
                case OperationType.Inferiority:
                case OperationType.NonSuperiority:
                case OperationType.Equality:
                case OperationType.Inequality:
                case OperationType.NonInferiority:
                case OperationType.Superiority:
                case OperationType.Conjunction:
                case OperationType.Disjunction:
                    return true;
                default:
                    return false;
            }
        }

        /// <summary>
        /// Determines whether the specified operation type is an effect.
        /// </summary>
        /// <param name="operationType">The operation type.</param>
        /// <returns>True if the specified operation type is an effect; otherwise, false.</returns>
        private static bool IsEffect(OperationType operationType) {
            switch (operationType) {
                case OperationType.Assignment:
                case OperationType.SumAssignment:
                case OperationType.SubtractionAssignment:
                case OperationType.MultiplicationAssignment:
                case OperationType.DivisionAssignment:
                    return true;
                default:
                    return false;
            }
        }

        /// <summary>
        /// Determines whether the specified operation type is a function.
        /// </summary>
        /// <param name="operationType">The operation type.</param>
        /// <returns>True if the specified operation type is a function; otherwise, false.</returns>
        private static bool IsFunction(OperationType operationType) {
            switch (operationType) {
                case OperationType.Sum:
                case OperationType.Subtraction:
                case OperationType.Multiplication:
                case OperationType.Division:
                    return true;
                default:
                    return false;
            }
        }

        /// <summary>
        /// Generates the initial state.
        /// </summary>
        /// <returns>The initial state.</returns>
        private State GenerateInitialState() {
            //creates the state with the initial facts
            int[] initialFacts = GetInitialFactsBitArray();
            State initialState = new State(initialFacts, _variableCount);
            int initializationCount = _functionalInitializations.Length;
            bool[] conditionalVariables = new bool[_conditionalVariables.Length];
            if (initializationCount > 0) {
                IEffect[] initializations = new IEffect[initializationCount];
                OperationOperand<TVar> operation;
                //for each initialization
                for (int initializationIndex = 0; initializationIndex < initializationCount; initializationIndex++) {
                    operation = _functionalInitializations[initializationIndex];
                    if (!IsEffect(operation.OperationType)) {
                        //adds an error if the initialization is not an effect
                        _messages.OperationMessages.AddSemanticFatalError(
                            _functionalInitializationSources[initializationIndex], MessageCode.OperationIsNotEffect);
                    }
                    //adds the initialization to the list
                    initializations[initializationIndex] =
                        operation.GetEffect(_globalVariables, conditionalVariables);
                }
                //creates a unique initialization by joining all
                IEffect initialization = new MultipleEffectList(initializations);
                initialization.ApplyTo(initialState);
            }
            return initialState;
        }

        /// <summary>
        /// Generates the functional goal.
        /// </summary>
        /// <returns>The functional goal.</returns>
        private ICondition GenerateFunctionalGoal() {
            ICondition goal = null;
            int goalCount = _functionalGoals.Length;
            if (goalCount > 0) {
                ICondition[] goals = new ICondition[goalCount];
                OperationOperand<TVar> operation;
                //for each functional goal
                for (int goalIndex = 0; goalIndex < goalCount; goalIndex++) {
                    operation = _functionalGoals[goalIndex];
                    if (!IsCondition(operation.OperationType)) {
                        //adds an error if the initialization is not a condition
                        _messages.OperationMessages.AddSemanticFatalError(_functionalGoalSources[goalIndex], MessageCode.OperationIsNotCondition);
                    }
                    //adds the goal to the list
                    goals[goalIndex] = operation.GetCondition(_globalVariables, _conditionalVariables);
                }
                //creates a unique goal by joining all
                goal = new MultipleConditionConjunction(goals);
            }
            return goal;
        }

        /// <summary>
        /// Generates the metric.
        /// </summary>
        /// <returns>The metric.</returns>
        private IFunction GenerateMetric() {
            IFunction metric = null;
            if (_problem.HasMetric) {
                switch (_metricTarget.OperandType) {
                    //if metric is a variable
                    case OperandType.Variable:
                        switch (_metricType) {
                            case MetricType.Minimization:
                                //if metric is going to be minimized, generates the access function
                                metric = new VariableAccess(_metricTarget.VariableIndex);
                                break;
                            case MetricType.Maximization:
                                //if metric is going to be maximized, generates the negated access function
                                metric = new NegatedVariableAccess(_metricTarget.VariableIndex);
                                break;
                        }
                        break;
                    //if metric is an operation
                    case OperandType.Operation:
                        //generates the operation
                        metric = _metricTarget.GetFunction(_globalVariables, new bool[_variableCount]);
                        if (!IsFunction(_metricTarget.OperationType)) {
                            //adds an error message if the operation is not a function
                            _messages.OperationMessages.AddSemanticFatalError(_metricTargetOperationSource, MessageCode.OperationIsNotFunction);
                        }
                        if (_metricType == MetricType.Maximization) {
                            //if metric is going to be maximized, negates the function
                            metric = new ResultNegation(metric);
                        }
                        break;
                    case OperandType.Constant:
                    default:
                        //adds an error message if metric is not a variable or operation
                        _messages.OperandMessages.AddSemanticFatalError(_metricTargetSource, MessageCode.InvalidMetricTargetType);
                        break;
                }
            } else {
                //if no metric is defined, plan length is used
                metric = new PlanLengthGetter();
            }
            return metric;
        }

        /// <summary>
        /// Generates an operator.
        /// </summary>
        /// <param name="index">The operator index.</param>
        /// <param name="op">The knowledge provider operator.</param>
        /// <returns>The generated operator.</returns>
        private Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>
        GenerateOperator(int index, IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> op) {
            IDictionary<TObj, SourceValuePair<IObject<TType, TObj>, Argument<TObj>>> arguments;
            Argument<TObj>[] indexedArguments;
            TypeInfo<TObj>[] parameterTypeInfos;
            //generates the signature
            Signature<TObj> signature = GenerateSignature(_messages.OperatorMessages, op, index,
                op.ParameterCount, op.GetParameters(), out arguments, out indexedArguments,
                out parameterTypeInfos);
            IDictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>, Fact<TObj>>> factsById;
            int factCount = op.FactCount;
            //generates the facts
            Fact<TObj>[] facts = GenerateFacts(_messages.OperatorMessages, op, factCount,
                GetAndCheckFacts(_messages.OperatorMessages, op, factCount, op.GetFacts()), out factsById,
                arguments);
            IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variablesById;
            int variableCount = op.VariableCount;
            //generates the variables
            Variable<TObj>[] variables = GenerateVariables(_messages.OperatorMessages, op, variableCount,
                GetAndCheckVariables(_messages.OperatorMessages, op, variableCount, op.GetVariables()),
                out variablesById, arguments);
            //selects precondition facts
            Fact<TObj>[] preconditions = SelectFacts(_messages.OperatorMessages, op, factsById,
                op.PreconditionCount, op.GetPreconditionIds(), FactAction.Precondition);
            //selects deletion facts
            Fact<TObj>[] deletions = SelectFacts(_messages.OperatorMessages, op, factsById,
                op.DeletionCount, op.GetDeletionIds(), FactAction.Deletion);
            //selects addition facts
            Fact<TObj>[] additions = SelectFacts(_messages.OperatorMessages, op, factsById,
                op.AdditionCount, op.GetAdditionIds(), FactAction.Addition);
            CheckFactUse(factsById);
            IOperation<TVar>[] preconditionOperationSources;
            //generates precondition operations
            OperationOperand<TVar>[] preconditionOperations = GenerateOperations(_messages.OperatorMessages, op,
                op.FunctionalPreconditionCount, op.GetFunctionalPreconditions(), variablesById, false,
                out preconditionOperationSources, OperationSection.Precondition);
            IOperation<TVar>[] effectOperationSources;
            //generates effect operations
            OperationOperand<TVar>[] effectOperations = GenerateOperations(_messages.OperatorMessages, op,
                op.FunctionalEffectCount, op.GetFunctionalEffects(), variablesById, false,
                out effectOperationSources, OperationSection.Effect);
            //checks if all objects and variables have been used
            CheckObjectUse(arguments, MessageCode.ArgumentNotUsed);
            CheckVariableUse(variablesById);
            if ((additions.Length == 0) && (deletions.Length == 0) && (effectOperations.Length == 0)) {
                //adds an error message if the operator has no effects
                _messages.OperatorMessages.AddSemanticFatalError(op, MessageCode.OperatorWithNoEffects);
            }
            //generates the operator
            return new Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>(op,
                _messages.OperatorMessages, signature, parameterTypeInfos, indexedArguments, facts, variables,
                preconditions, preconditionOperationSources, preconditionOperations, additions, deletions,
                effectOperationSources, effectOperations, _messages.OperationMessages);
        }

        /// <summary>
        /// Generates the operators.
        /// </summary>
        /// <param name="operatorCount">The number of operators.</param>
        /// <param name="operators">The knowledge provider operators.</param>
        private void GenerateOperators(int operatorCount,
        IEnumerator<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> operators) {
            IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> op;
            Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> internalOperator;
            TOp operatorId;
            int internalOperatorCount = 0;
            _operatorSignatures = new Dictionary<TOp, Signature<TObj>>(operatorCount);
            _operators = new Dictionary<TOp, Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>(operatorCount);
            //for each knowledge provider operator
            while (operators.MoveNext()) {
                op = operators.Current;
                if (op == null) {
                    //adds an error message if the operator is null
                    _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.NullOperator, internalOperatorCount);
                }
                operatorId = op.Id;
                //generates the operator
                internalOperator = GenerateOperator(internalOperatorCount, op);
                try {
                    _operatorSignatures.Add(operatorId, internalOperator.Signature);
                } catch (ArgumentException) {
                    //adds an error message if the operator identified is already in use
                    _messages.OperatorMessages.AddSemanticFatalError(op, MessageCode.OperatorIdAlreadyUsed);
                }
                _operators.Add(operatorId, internalOperator);
                internalOperatorCount++;
            }
            if (internalOperatorCount != operatorCount) {
                //adds an error message if the  number of operators is incoherent
                _messages.DomainMessages.AddSyntacticFatalError(_domain, MessageCode.IncoherentOperatorCount);
            }
        }

        /// <summary>
        /// Generates the operators.
        /// </summary>
        private void GenerateOperators() {
            int operatorCount = GetOperatorCount();
            IEnumerator<IOperator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>>
                operators = GetOperators();
            GenerateOperators(operatorCount, operators);
            _operatorCount = _operators.Count;
        }

        /// <summary>
        /// Gets the inequality facts.
        /// </summary>
        /// <returns>The inequality facts.</returns>
        private ICollection<Fact<TObj>> GetInequalityFacts() {
            const int AnyParameter = 0;
            ICollection<Fact<TObj>> inequalityFacts = new List<Fact<TObj>>();
            TypeInfo<TObj> typeInfo;
            Object[] objects;
            Fact<TObj> fact;
            if (_inequalityPredicatesEnabled) {
                //for each inequality predicate
                foreach (PredicateInfo<TObj> predicateInfo in _inequalityPredicateInfos) {
                    typeInfo = predicateInfo.ParameterTypeInfos[AnyParameter];
                    objects = typeInfo.Objects;
                    //for each object
                    foreach (Object oneObject in objects) {
                        //ror each different object
                        foreach (Object otherObject in objects) {
                            if (oneObject != otherObject) {
                                //adds an inequality fact wich relates the two objects
                                fact = new Fact<TObj>(predicateInfo,
                                    new Argument<TObj>(typeInfo, oneObject),
                                    new Argument<TObj>(typeInfo, otherObject));
                                fact.AddAction<object>(null, null, 0, FactAction.Initialization);
                                inequalityFacts.Add(fact);
                            }
                        }
                    }
                }
            }
            return inequalityFacts;
        }

        /// <summary>
        /// Generates the initial and goal facts.
        /// </summary>
        private void GenerateInitialAndGoalFacts() {
            IDictionary<TFact, SourceValuePair<IIdentifiedInstance<TPred, TObj, TFact>, Fact<TObj>>> factsById;
            int factCount = _problem.FactCount;
            //generates the facts
            GenerateFacts(_messages.ProblemMessages, _problem, factCount,
                GetAndCheckFacts(_messages.ProblemMessages, _problem, factCount, _problem.GetFacts()),
                out factsById, _objectArguments);
            //gets inequality predicates
            ICollection<Fact<TObj>> inequalityFacts = GetInequalityFacts();
            //selects initial facts
            Fact<TObj>[] initialFacts = SelectFacts(_messages.ProblemMessages, _problem,
                factsById, _problem.InitialFactCount, _problem.GetInitialFactIds(), FactAction.Initialization);
            int inequalityFactsCount = inequalityFacts.Count;
            int initialFactsCount = initialFacts.Length;
            //adds inequality facts to initial ones
            _initialFacts = new Fact<TObj>[inequalityFactsCount + initialFactsCount];
            inequalityFacts.CopyTo(_initialFacts, 0);
            Array.Copy(initialFacts, 0, _initialFacts, inequalityFactsCount, initialFactsCount);
            //selects goal facts
            _goalFacts = SelectFacts(_messages.ProblemMessages, _problem, factsById, _problem.GoalFactCount,
                _problem.GetGoalFactIds(), FactAction.Goal);
            //checks if any fact has not been used
            CheckFactUse(factsById);
        }

        /// <summary>
        /// Generates the initial and goal functions.
        /// </summary>
        private void GenerateInitialAndGoalFunctions() {
            IDictionary<TVar, SourceValuePair<IIdentifiedInstance<TFunc, TObj, TVar>, Variable<TObj>>> variablesById;
            int variableCount = _problem.VariableCount;
            //generates initial variables
            _initialVariables = GenerateVariables(_messages.ProblemMessages, _problem, variableCount,
                GetAndCheckVariables(_messages.ProblemMessages, _problem, variableCount,
                _problem.GetVariables()), out variablesById, _objectArguments);
            //generates functional initializations
            _functionalInitializations = GenerateOperations(_messages.ProblemMessages, _problem,
                _problem.FunctionalInitializationCount, _problem.GetFunctionalInitializations(), variablesById,
                true, out _functionalInitializationSources, OperationSection.Initialization);
            //generates functional goals
            _functionalGoals = GenerateOperations(_messages.ProblemMessages, _problem,
                _problem.FunctionalGoalCount, _problem.GetFunctionalGoals(), variablesById, true,
                out _functionalGoalSources, OperationSection.Goal);
            //if the problem has metric
            if (_problem.HasMetric) {
                IMetric<TVar> metric = _problem.GetMetric();
                if (metric == null) {
                    //adds an error message if the metric is a null reference
                    _messages.ProblemMessages.AddSyntacticFatalError(_problem, MessageCode.NullMetric);
                }
                _metricTargetSource = metric.GetTarget();
                if (_metricTargetSource == null) {
                    //adds an error message if the metric target is a null reference
                    _messages.MetricMessages.AddSyntacticFatalError(metric, MessageCode.NullMetricTarget);
                }
                //generates metric target
                _metricTarget = GenerateOperand(_metricTargetSource, variablesById, true);
                if (_metricTarget.OperandType == OperandType.Operation) {
                    _metricTargetOperationSource = _metricTargetSource.Operation;
                }
                _metricType = metric.Type;
                switch (_metricType) {
                    case MetricType.Minimization:
                    case MetricType.Maximization:
                        break;
                    default:
                        //adds an error message if the metric type is unknown
                        _messages.MetricMessages.AddSyntacticFatalError(metric, MessageCode.InvalidMetricType);
                        break;
                }
            }
            //checks if any variable has not been used
            CheckVariableUse(variablesById);
        }

        /// <summary>
        /// Checks if any predicate has not been used.
        /// </summary>
        private void CheckPredicateUse() {
            PredicateInfo<TObj> predicateInfo;
            TPred predicateId;
            //for each predicate
            foreach (KeyValuePair<TPred, PredicateInfo<TObj>> predicateInfoPair in _predicateInfos) {
                predicateInfo = predicateInfoPair.Value;
                //if predicate is not in use
                if (!predicateInfo.Used) {
                    predicateId = predicateInfoPair.Key;
                    //adds an error message
                    if (!predicateInfo.IsInequality) {
                        _messages.PredicateMessages.AddSemanticWarning(_predicateSources[predicateId], MessageCode.PredicateNotUsed);
                    }
                }
            }
        }

        /// <summary>
        /// Checks if any function has not been used.
        /// </summary>
        private void CheckFunctionUse() {
            //for each function
            foreach (KeyValuePair<TFunc, FunctionInfo<TObj>> functionInfoPair in _functionInfos) {
                if (!functionInfoPair.Value.Used) {
                    //adds an error message if has not been used
                    _messages.FunctionMessages.AddSemanticWarning(_functionSources[functionInfoPair.Key], MessageCode.FunctionNotUsed);
                }
            }
        }

        /// <summary>
        /// Checks if any object, predicate or function has not been used.
        /// </summary>
        private void CheckUses() {
            CheckObjectUse(_objectArguments, MessageCode.ObjectNotUsed);
            CheckPredicateUse();
            CheckFunctionUse();
        }

        /// <summary>
        /// Completes the information about predicates.
        /// </summary>
        private void CompletePredicateInfos() {
            PredicateInfo<TObj> lastPredicateInfo = new PredicateInfo<TObj>();
            int predicateCount = _predicateInfos.Count;
            _predicateEntries = new SignatureEntry<TPred, TObj>[predicateCount];
            _predicateMaxFactIds = new int[predicateCount];
            int predicateIndex = 0;
            PredicateInfo<TObj> predicateInfo;
            //for each predicate
            foreach (KeyValuePair<TPred, PredicateInfo<TObj>> entry in _predicateInfos) {
                predicateInfo = entry.Value;
                predicateInfo.CalculateDerived(lastPredicateInfo);
                if (!predicateInfo.IsStatic && ((predicateInfo.FactsWay != FactWay.Deletion) ||
                (predicateInfo.InitialFacts.Count > 0))) {
                    //creates the predicate entry
                    _predicateEntries[predicateIndex] =
                        new SignatureEntry<TPred, TObj>(entry.Key, entry.Value.Signature);
                    //calculates the maximum fact identifier for the predicate
                    _predicateMaxFactIds[predicateIndex] = predicateInfo.Signature.InstanceIdOffset +
                        predicateInfo.MaxInstancesCount - 1;
                    predicateIndex++;
                } else {
                    //removes static predicates and those only which can be only be deleted and has no inital facts
                    _predicates.Remove(entry.Key);
                }
                lastPredicateInfo = predicateInfo;
            }
            //calculates the number of facts
            _factCount = lastPredicateInfo.Signature.InstanceIdOffset +
                lastPredicateInfo.MaxInstancesCount;
            Array.Resize(ref _predicateEntries, predicateIndex);
            Array.Resize(ref _predicateMaxFactIds, predicateIndex);
        }

        /// <summary>
        /// Completes the information about operators.
        /// </summary>
        private void CompleteOperators() {
            Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> lastOperator =
                new Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>();
            _operatorEntries = new SignatureEntry<TOp, TObj>[_operatorCount];
            _operatorMaxActionIds = new int[_operatorCount];
            _actionCounts = new int[_operatorCount];
            _preconditionCounts = new int[_operatorCount];
            int operatorIndex = 0;
            Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> op;
            TOp operatorId;
            Signature<TObj> operatorSignature;
            IList<TOp> unexecutableOperatorIds = new List<TOp>();
            //for each operator
            foreach (KeyValuePair<TOp, Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar>> entry in _operators) {
                op = entry.Value;
                operatorId = entry.Key;
                if (op.IsExecutable) {
                    op.Index = operatorIndex;
                    op.CalculateDerived(lastOperator);
                    //calculates operator signature
                    operatorSignature = op.Signature;
                    //generates operator entry
                    _operatorEntries[operatorIndex] =
                        new SignatureEntry<TOp, TObj>(operatorId, operatorSignature);
                    //calculates the maximum operator action identifier
                    _operatorMaxActionIds[operatorIndex] = operatorSignature.InstanceIdOffset +
                        op.MaxInstancesCount - 1;
                    //stores the number of actions for this operator
                    _actionCounts[operatorIndex] = op.MaxInstancesCount;
                    //stores the number of preconditions for this operator
                    _preconditionCounts[operatorIndex] = op.DynamicPreconditionCount;
                    lastOperator = op;
                    operatorIndex++;
                } else {
                    _operatorSignatures.Remove(operatorId);
                    unexecutableOperatorIds.Add(operatorId);
                    _operatorCount--;
                    _messages.OperatorMessages.AddSemanticWarning(op.Source, MessageCode.UnexecutableOperator);
                }
            }
            //removes all unexecutable operators
            foreach (TOp id in unexecutableOperatorIds) {
                _operators.Remove(id);
            }
            Array.Resize(ref _operatorEntries, _operatorCount);
            Array.Resize(ref _operatorMaxActionIds, _operatorCount);
            Array.Resize(ref _actionCounts, _operatorCount);
            Array.Resize(ref _preconditionCounts, _operatorCount);
            //calculates the numebr of actions
            _actionCount = lastOperator.Signature.InstanceIdOffset + lastOperator.MaxInstancesCount;
        }

        /// <summary>
        /// Completes the information about functions.
        /// </summary>
        private void CompleteFunctionInfos() {
            FunctionInfo<TObj> lastFunctionInfo = new FunctionInfo<TObj>();
            FunctionInfo<TObj> lastReadOnlyFunctionInfo = new FunctionInfo<TObj>();
            int functionCount = _functionInfos.Count;
            _functionEntries = new SignatureEntry<TFunc, TObj>[functionCount];
            _functionMaxVariableIds = new int[functionCount];
            int functionIndex = 0;
            FunctionInfo<TObj> functionInfo;
            //for each function
            foreach (KeyValuePair<TFunc, FunctionInfo<TObj>> entry in _functionInfos) {
                functionInfo = entry.Value;
                if (functionInfo.IsReadOnly) {
                    functionInfo.CalculateDerived(lastReadOnlyFunctionInfo);
                    //removes read-only functions
                    _functions.Remove(entry.Key);
                    lastReadOnlyFunctionInfo = functionInfo;
                } else {
                    functionInfo.CalculateDerived(lastFunctionInfo);
                    //creates function entry
                    _functionEntries[functionIndex] =
                        new SignatureEntry<TFunc, TObj>(entry.Key, entry.Value.Signature);
                    //calculates the maximum variable identifier
                    _functionMaxVariableIds[functionIndex] = functionInfo.Signature.InstanceIdOffset +
                        functionInfo.MaxInstancesCount - 1;
                    lastFunctionInfo = functionInfo;
                    functionIndex++;
                }
            }
            //calculates the number of variables
            _variableCount = lastFunctionInfo.Signature.InstanceIdOffset + lastFunctionInfo.MaxInstancesCount;
            _conditionalVariables = new bool[_variableCount];
            //the number of global variables is the total one less the number of read-only variables
            int globalVariableCount = lastReadOnlyFunctionInfo.Signature.InstanceIdOffset +
                lastReadOnlyFunctionInfo.MaxInstancesCount;
            //creates the global variables values array
            _globalVariables = new float[globalVariableCount];
            Array.Resize(ref _functionEntries, functionIndex);
            Array.Resize(ref _functionMaxVariableIds, functionIndex);
        }

        /// <summary>
        /// Initializes the action selection table.
        /// </summary>
        private void InitializeActionSelectionTable() {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            int[][][] factActionSelectionTable;
            int[][] operatorActionSelectionTable;
            int preconditionCount;
            int actionBitArrayLength;
            _actionSelectionTable = new int[_factCount][][][];
            //for each fact
            for (int factId = 0; factId < _factCount; factId++) {
                factActionSelectionTable = new int[_operatorCount][][];
                _actionSelectionTable[factId] = factActionSelectionTable;
                //for each operator
                for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                    preconditionCount = _preconditionCounts[operatorIndex];
                    operatorActionSelectionTable = new int[preconditionCount][];
                    factActionSelectionTable[operatorIndex] = operatorActionSelectionTable;
                    actionBitArrayLength = (_actionCounts[operatorIndex] + IntSizeLessOne) / IntSize;
                    //and for each precondition
                    for (int preconditionIndex = 0; preconditionIndex < preconditionCount; preconditionIndex++) {
                        //stores an action bit array with the same length for each precondition of the same operator
                        operatorActionSelectionTable[preconditionIndex] = new int[actionBitArrayLength];
                    }
			    }
            }
        }
        /// <summary>
        /// Filters an action activation list.
        /// </summary>
        /// <param name="actionList">The action activation list.</param>
        /// <param name="actionIds">The identifiers of the actions to preserve after filtering.</param>
        /// <returns>The action activation list, only with the actions specified in
        /// <paramref name="actionIds"/>.</returns>
        private static int[] FilterActionActivationList(int[] actionList, int[] actionIds) {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            int actionCount = actionIds.Length;
            int actionId;
            //creates the new action activation list
            int[] filteredActionList = new int[(actionCount + IntSizeLessOne) / IntSize];
            //for each action to preserve
            for (int actionIndex = 0; actionIndex < actionCount; actionIndex++) {
                actionId = actionIds[actionIndex];
                //adds the action activation value to the new action activation list
                if ((actionList[actionId / IntSize] & (1 << (actionId % IntSize))) != 0) {
                    filteredActionList[actionIndex / IntSize] |= (1 << (actionIndex % IntSize));
                }
            }
            return filteredActionList;
        }

        /// <summary>
        /// Filters the action activation table of an specified operator.
        /// </summary>
        /// <param name="operatorIndex">The operator index.</param>
        /// <param name="actionIds">The identifiers of the actions to preserve after filtering.</param>
        private void FilterActionActivationTable(int operatorIndex, int[] actionIds) {
            int[][] op;
            int preconditionCount;
            foreach (int[][][] fact in _actionSelectionTable) {
                op = fact[operatorIndex];
                preconditionCount = op.Length;
                for (int precondition = 0; precondition < preconditionCount; precondition++) {
                    op[precondition] = FilterActionActivationList(op[precondition], actionIds);
                }
            }
        }

        /// <summary>
        /// Removes the unreachable actions.
        /// </summary>
        private void RemoveUnreachableActions() {
            const Action UnreachableAction = null;
            //creates the new action list
            Action[] reachableActions = new Action[_actions.Length];
            Action action;
            int actionIndex = 0;
            int maxActionIndex;
            int reachableActionCount = 0;
            int operatorReachableActionCount;
            int oldInstanceIdOffset;
            int newInstanceIdOffset;
            Signature<TObj> signature;
            List<int> actionIdList = new List<int>();
            List<int> oldAtionIdListWithoutOffset = new List<int>();
            //for each operator
            for (int operatorIndex = 0; operatorIndex < _operatorCount; operatorIndex++) {
                maxActionIndex = actionIndex + _actionCounts[operatorIndex];
                signature = _operatorEntries[operatorIndex].Signature;
                oldInstanceIdOffset = signature.InstanceIdOffset;
                newInstanceIdOffset = reachableActionCount;
                operatorReachableActionCount = 0;
                actionIdList.Clear();
                oldAtionIdListWithoutOffset.Clear();
                //for each action
                for (; actionIndex < maxActionIndex; actionIndex++) {
                    action = _actions[actionIndex];
                    //if the action is reachable
                    if (action != UnreachableAction) {
                        //adds the action to the new action list
                        reachableActions[reachableActionCount] = action;
                        actionIdList.Add(action.Id - oldInstanceIdOffset + newInstanceIdOffset);
                        //stores the action old identifier for encoding/decoding purposes
                        oldAtionIdListWithoutOffset.Add(action.Id - oldInstanceIdOffset);
                        //assigns the action new identifier
                        action.Id = reachableActionCount;
                        reachableActionCount++;
                        operatorReachableActionCount++;
                    }
                }
                //assigns the new action count 
                _actionCounts[operatorIndex] = operatorReachableActionCount;
                //assigns the new action identifier offset
                signature.InstanceIdOffset = newInstanceIdOffset;
                //assigns the new maximum action identifier count
                _operatorMaxActionIds[operatorIndex] = reachableActionCount - 1;
                //activates instance identifier remapping
                signature.SetInitialInstanceIds(actionIdList.ToArray(), InitialInstancesBound.Maximum);
                FilterActionActivationTable(operatorIndex, oldAtionIdListWithoutOffset.ToArray());
            }
            Array.Resize(ref reachableActions, reachableActionCount);
            //assigns the new action list
            _actions = reachableActions;
            //assigns the new action count
            _actionCount = reachableActionCount;
        }

        /// <summary>
        /// Generates the actions.
        /// </summary>
        private void GenerateActions() {
            InitializeActionSelectionTable();
            _actions = new Action[_actionCount];
            //for each operator
            foreach (Operator<TType, TPred, TFunc, TOp, TObj, TFact, TVar> op in _operators.Values) {
                //generates the operator actions
                op.GenerateActions(_actionSelectionTable, _actions, _globalVariables, _conditionalVariables);
            }
            //the following call is an optimization of the action-related result data
            //there is no need to call it, but it reduces significantly the amount of action-related data
            RemoveUnreachableActions();
        }

        /// <summary>
        /// Completes the information about predicates, functions and operators.
        /// </summary>
        private void CompleteGeneratedInfo() {
            CompletePredicateInfos();
            CompleteFunctionInfos();
            CompleteOperators();
        }

        /// <summary>
        /// Gets the initial facts bit array.
        /// </summary>
        /// <returns>The initial facts bit array.</returns>
        private int[] GetInitialFactsBitArray() {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            int[] initialFactsBitArray = new int[(_factCount + IntSizeLessOne) / IntSize];
            Dictionary<int, Dictionary<int, int>> factIdsByPredicate =
                new Dictionary<int, Dictionary<int, int>>(_predicates.Count);
            Dictionary<int, int> predicateFactIds;
            int predicateIndex;
            PredicateInfo<TObj> predicateInfo;
            int factId;
            int initialFactsCount = _initialFacts.Length;
            Fact<TObj> fact;
            //for each fact
            for (int factIndex = 0; factIndex < initialFactsCount; factIndex++) {
                fact = _initialFacts[factIndex];
                predicateIndex = fact.Signature.Index;
                if (!factIdsByPredicate.TryGetValue(predicateIndex, out predicateFactIds)) {
                    predicateFactIds = new Dictionary<int, int>();
                    factIdsByPredicate[predicateIndex] = predicateFactIds;
                }
                factId = fact.Id;
                try {
                    predicateFactIds.Add(factId, factId);
                } catch (ArgumentException) {
                    //adds an error message if the fact is already present in the list
                    _messages.ProblemMessages.AddSemanticFatalError(_problem, MessageCode.InitialFactAlreadyPresent, factIndex);
                }
                predicateInfo = fact.PredicateInfo;
                //no static and addition predicates are represented because their facts are innmutable
                if (!predicateInfo.IsStatic && (predicateInfo.FactsWay != FactWay.Addition)) {
                    //adds the fact to the bit array
                    initialFactsBitArray[factId / IntSize] |= (1 << (factId % IntSize));
                }
            }
            return initialFactsBitArray;
        }

        /// <summary>
        /// Gets the goal facts bit array.
        /// </summary>
        /// <returns>The goal facts bit array.</returns>
        private int[] GetGoalFactsBitArray() {
            const int IntSize = 32;
            const int IntSizeLessOne = IntSize - 1;
            int[] goalFactsBitArray = new int[(_factCount + IntSizeLessOne) / IntSize];
            Dictionary<int, Dictionary<int, int>> factIdsByPredicate =
                new Dictionary<int, Dictionary<int, int>>(_predicates.Count);
            Dictionary<int, int> predicateFactIds;
            int predicateIndex;
            PredicateInfo<TObj> predicateInfo;
            int factId;
            int goalCount = _goalFacts.Length; ;
            Fact<TObj> fact;
            //for each goal
            for (int goalIndex = 0; goalIndex < goalCount; goalIndex++) {
                fact = _goalFacts[goalIndex];
                predicateInfo = fact.PredicateInfo;
                if ((predicateInfo.IsStatic || (predicateInfo.FactsWay == FactWay.Deletion))
                && !fact.IsInitial) {
                    //adds an error message if the goal is unreachable, that is, its predicate is static or
                    //its facts can only be deleted and the fact is not initial
                    _messages.ProblemMessages.AddSemanticFatalError(_problem, MessageCode.UnreachableGoal, goalIndex);
                }
                predicateIndex = fact.Signature.Index;
                if (!factIdsByPredicate.TryGetValue(predicateIndex, out predicateFactIds)) {
                    predicateFactIds = new Dictionary<int, int>();
                    factIdsByPredicate[predicateIndex] = predicateFactIds;
                }
                fact.CalculateId();
                factId = fact.Id;
                try {
                    predicateFactIds.Add(factId, factId);
                } catch (ArgumentException) {
                    //adds an error message if the fact is already present in the list
                    _messages.ProblemMessages.AddSemanticFatalError(_problem, MessageCode.GoalFactAlreadyPresent, goalIndex);
                }
                //only initial facts and non-addition predicates are represented because their are mutable
                if ((predicateInfo.FactsWay != FactWay.Addition) || !fact.IsInitial) {
                    //adds the fact to the bit array
                    goalFactsBitArray[factId / IntSize] |= (1 << (factId % IntSize));
                }
            }
            return goalFactsBitArray;
        }

        /// <summary>
        /// Calculates the initial variable identifiers.
        /// </summary>
        private void CalculateInitialVariableIds() {
            Dictionary<int, Dictionary<int, int>> variableIdsByFunction =
                new Dictionary<int, Dictionary<int, int>>(_functions.Count);
            Dictionary<int, int> functionVariableIds;
            int functionIndex;
            int variableId;
            int variableCount = _initialVariables.Length;
            Variable<TObj> variable;
            //for each variable
            for (int variableIndex = 0; variableIndex < variableCount; variableIndex++) {
                variable = _initialVariables[variableIndex];
                functionIndex = variable.Signature.Index;
                if (!variableIdsByFunction.TryGetValue(functionIndex, out functionVariableIds)) {
                    functionVariableIds = new Dictionary<int, int>();
                    variableIdsByFunction[functionIndex] = functionVariableIds;
                }
                //calculates the variable identifier
                variable.CalculateId();
                variableId = variable.Id;
                try {
                    functionVariableIds.Add(variableId, variableId);
                } catch (ArgumentException) {
                    //adds an error message if the variable identifier is already in use
                    _messages.ProblemMessages.AddSemanticFatalError(_problem, MessageCode.VariableAlreadyDefined, variableIndex);
                }
            }
        }

        /// <summary>
        /// Resets this translator.
        /// </summary>
        private void Reset() {
            _messages = null;
            _domain = null;
            _problem = null;
            _typeInfos = null;
            _typeSources = null;
            _types = null;
            _objects = null;
            _objectArguments = null;
            _predicateSources = null;
            _predicates = null;
            _predicateInfos = null;
            _inequalityPredicateInfos = null;
            _predicateEntries = null;
            _predicateMaxFactIds = null;
            _functionSources = null;
            _functions = null;
            _functionInfos = null;
            _functionEntries = null;
            _functionMaxVariableIds = null;
            _initialFacts = null;
            _goalFacts = null;
            _initialVariables = null;
            _functionalInitializationSources = null;
            _functionalInitializations = null;
            _functionalGoalSources = null;
            _functionalGoals = null;
            _metricTargetSource = null;
            _metricTarget = null;
            _metricTargetOperationSource = null;
            _globalVariables = null;
            _operatorSignatures = null;
            _operators = null;
            _operatorEntries = null;
            _operatorMaxActionIds = null;
            _actionSelectionTable = null;
            _actionCounts = null;
            _preconditionCounts = null;
            _actions = null;
        }

        /// <summary>
        /// Creates the result.
        /// </summary>
        /// <returns>The result</returns>
        private TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar> CreateResult() {
            TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar> result =
                new TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar>();
            result.Messages = _messages;
            result.ActionCount = _actionCount;
            result.ActionCounts = _actionCounts;
            result.ActionDecoder = new InstanceDecoder<TOp, TObj>(_resources,
                _operatorEntries, _operatorMaxActionIds, _actionCount, _types);
            result.ActionEncoder = new InstanceEncoder<TOp, TObj>(_resources,
                SignatureType.Operator, _operatorSignatures, _objects);
            result.Actions = _actions;
            result.ActionActivationTable = _actionSelectionTable;
            result.FactCount = _factCount;
            result.FactDecoder = new InstanceDecoder<TPred, TObj>(_resources,
                _predicateEntries, _predicateMaxFactIds, _factCount, _types);
            result.FactEncoder = new InstanceEncoder<TPred, TObj>(_resources,
                SignatureType.Predicate, _predicates, _objects);
            CalculateInitialVariableIds();
            result.FunctionalGoal = GenerateFunctionalGoal();
            result.ConditionalVariables = _conditionalVariables;
            int conditionalVariableCount = 0;
            foreach (bool variable in _conditionalVariables) {
                if (variable) {
                    conditionalVariableCount++;
                }
            }
            result.ConditionalVariableCount = conditionalVariableCount;
            result.GoalCount = _goalFacts.Length;
            result.GoalFacts = GetGoalFactsBitArray();
            result.InitialState = GenerateInitialState();
            result.Metric = GenerateMetric();
            result.OperatorCount = _operatorCount;
            result.PreconditionCounts = _preconditionCounts;
            result.VariableCount = _variableCount;
            result.VariableDecoder = new InstanceDecoder<TFunc, TObj>(_resources,
                _functionEntries, _functionMaxVariableIds, _variableCount, _types);
            result.VariableEncoder = new InstanceEncoder<TFunc, TObj>(_resources,
                SignatureType.Function, _functions, _objects);
            return result;
        }

        /// <summary>
        /// Executes the translation.
        /// </summary>
        /// <returns>The <see cref="TranslationResult{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">
        /// translation result</see>.</returns>
        /// <exception cref="GenerationException{TType, TPred, TFunc, TOp, TObj, TFact, TVar}">An error
        /// occurred while generating the translation result.</exception>
        /// <exception cref="RepresentationException">An error occurred while managing knowledge representation.</exception>
        public TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar> Translate() {
            try {
                _messages = new MessageManager<TType, TPred, TFunc, TOp, TObj, TFact, TVar>(_resources);
                GetKnowledge();
                GenerateTypes();
                GenerateObjects();
                GeneratePredicates();
                GenerateFunctions();
                GenerateOperators();
                GenerateInitialAndGoalFacts();
                GenerateInitialAndGoalFunctions();
                CheckUses();
                CompleteGeneratedInfo();
                GenerateActions();
                if (_messages.TotalErrorCount > 0) {
                    Reset();
                    _messages.NotifyErrors();
                }
                TranslationResult<TType, TPred, TFunc, TOp, TObj, TFact, TVar> result = CreateResult();
                Reset();
                return result;
            } catch (RepresentationException) {
                throw;
            } catch (GenerationException<TType, TPred, TFunc, TOp, TObj, TFact, TVar> ex) {
                throw ex; //HACK: the exception is rethrown in order to delete internal stack trace
            } catch (Exception exception) {
                throw new GenerationException<TType, TPred, TFunc, TOp, TObj, TFact, TVar>(
                    _resources.UnexpectedError, exception, _messages);
            } finally {
                Reset();
            }
        }

        #endregion

    }

}