/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;
using System.Collections.Generic;

namespace Aran.Translation {

    /// <summary>
    /// Represents additional information about an <see cref="Aran.Translation.Type{T}">type</see>.
    /// </summary>
    /// <remarks>The represented information is not included in each
    /// <see cref="Aran.Translation.Type{T}">object</see> because it is only used on translation time.
    /// <see cref="Aran.Translation.Type{T}">object</see>s are also used for encoding/decoding tasks
    /// which do not use this information and, therefore, it must not be part of
    /// <see cref="Aran.Translation.Type{T}"/>. </remarks>
    /// <typeparam name="T">The type of the object identifiers.</typeparam>
    [Serializable]
    internal sealed class TypeInfo<T> {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Constants

        /// <summary>
        /// A constant which represents no identifier.
        /// </summary>
        private const int NoId = -1;

        #endregion

        #region Fields

        /// <summary>
        /// The identifier.
        /// </summary>
        internal int Id;

        /// <summary>
        /// The parent type additional information.
        /// </summary>
        internal TypeInfo<T> ParentInfo;

        /// <summary>
        /// The subtypes additional information.
        /// </summary>
        internal TypeInfo<T>[] SubtypeInfos;

        /// <summary>
        /// The objects of this type.
        /// </summary>
        internal Object[] Objects;

        /// <summary>
        /// A temporal collection of subtypes information.
        /// </summary>
        private ICollection<TypeInfo<T>> _temporalSubtypeInfos;

        /// <summary>
        /// A temporal collection of objects information.
        /// </summary>
        private ICollection<ObjectInfo<T>> _temporalObjectInfos;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TypeInfo{T}"/> class.
        /// </summary>
        /// <param name="id">The identifier.</param>
        internal TypeInfo(int id) {
            Id = id;
            _temporalSubtypeInfos = new List<TypeInfo<T>>();
            _temporalObjectInfos = new List<ObjectInfo<T>>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TypeInfo{T}"/> class.
        /// </summary>
        internal TypeInfo()
            : this(NoId) {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this type information has identifier.
        /// </summary>
        /// <value>true if this type information has identifier; otherwise, false.</value>
        internal bool HasId {
            get {
                return (Id != NoId);
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Adds a subtype information.
        /// </summary>
        /// <param name="subtypeInfo">The subtype information.</param>
        internal void AddSubtypeInfo(TypeInfo<T> subtypeInfo) {
            subtypeInfo.ParentInfo = this;
            _temporalSubtypeInfos.Add(subtypeInfo);
        }

        /// <summary>
        /// Sets the up the subtypes.
        /// </summary>
        /// <returns>True if the type is now completely defined; false otherwise.</returns>
        internal bool SetUpSubtypes() {
            TypeInfo<T>[] subtypeInfos = new TypeInfo<T>[_temporalSubtypeInfos.Count];
            int i = 0;
            foreach (TypeInfo<T> subtypeInfo in _temporalSubtypeInfos) {
                subtypeInfos[i] = subtypeInfo;
                i++;
            }
            SubtypeInfos = subtypeInfos;
            _temporalSubtypeInfos = null; //this data structure will never be used again, so it can be garbage-collected
            return HasId; //if at this time it does not have identifier, there is an error.
        }

        /// <summary>
        /// Adds an object information.
        /// </summary>
        /// <param name="objectInfo">The object information.</param>
        /// <param name="typeCount">The number of types.</param>
        internal void AddObjectInfo(ObjectInfo<T> objectInfo, int typeCount) {
            const TypeInfo<T> NoTypeInfo = null;
            TypeInfo<T> currentTypeInfo = this;
            ICollection<ObjectInfo<T>> currentTypeObjectInfos;
            int currentTypeObjectId;
            int[] objectTypeDependantIds = new int[typeCount];
            //initializes the object type-dependant identifiers
            for (int i = 0; i < typeCount; i++) {
                objectTypeDependantIds[i] = Object.NoTypeId;
            }
            //for each type of the object
            do {
                currentTypeObjectInfos = currentTypeInfo._temporalObjectInfos;
                currentTypeObjectId = currentTypeObjectInfos.Count;
                //adds the object to the type
                currentTypeObjectInfos.Add(objectInfo);
                //sets the identifier of the object which depends on this type
                objectTypeDependantIds[currentTypeInfo.Id] = currentTypeObjectId;
                currentTypeInfo = currentTypeInfo.ParentInfo;
            } while (currentTypeInfo != NoTypeInfo);
            objectInfo.Object.TypeDependantIds = objectTypeDependantIds;
        }

        /// <summary>
        /// Sets the up the objects.
        /// </summary>
        /// <returns>The object identifiers.</returns>
        internal T[] SetUpObjects() {
            int objectCount = _temporalObjectInfos.Count;
            Object[] objects = new Object[objectCount];
            T[] objectIds = new T[objectCount];
            int i = 0;
            //for each temporal object
            foreach (ObjectInfo<T> objectInfo in _temporalObjectInfos) {
                //stores the object
                objects[i] = objectInfo.Object;
                //stores the object identifier
                objectIds[i] = objectInfo.Id;
                i++;
            }
            Objects = objects;
            _temporalObjectInfos = null; //this data structure will never be used again, so it can be garbage-collected
            return objectIds;
        }

        /// <summary>
        /// Determines whether this type is subtype of the specified one.
        /// </summary>
        /// <param name="typeId">The type identifier.</param>
        /// <returns>
        /// True if this type is subtype of the specified one; otherwise, false.
        /// </returns>
        internal bool IsSubtypeOf(int typeId) {
            const TypeInfo<T> NoTypeInfo = null;
            bool isSubtype = (Id == typeId);
            TypeInfo<T> currentTypeInfo = ParentInfo;
            //climbs through the type hierarchy in order to get the result
            while (!isSubtype && (currentTypeInfo != NoTypeInfo)) {
                isSubtype = (currentTypeInfo.Id == typeId);
                currentTypeInfo = currentTypeInfo.ParentInfo;
            }
            return isSubtype;
        }

        #endregion

    }

}