/* Aran, an XNA-compliant .NET automated planning framework
 * Copyright (C) 2008 Abel Garca Plaza
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

using System;

using Aran.Representation;

namespace Aran.Translation {

    /// <summary>
    /// Represents a variable <see cref="Operand">operand</see>.
    /// </summary>
    /// <typeparam name="T">The type of the objects.</typeparam>
    [Serializable]
    internal sealed class VariableOperand<T> : Operand {

        /*
         * NOTE: The current implementation of this translator is functional but highly improvable in all ways:
         * design, clarity, readability, performance... Its current state derives from an incomplete requirement
         * discovering phase so that the final desing and implementation was done by adding multiple
         * modifications during the development in order to fullfil all the final requirements discovered. This
         * desing relies on an enormous main type (Translator) instead of divide its funcionality into many other
         * helper types. In addition, the desing of the main type and all others does not make a correct use of
         * Object Oriented techniques in order to reduce the amount of repeated or similar code. Future
         * implementations may correct these errors.
         */

        #region Fields

        /// <summary>
        /// The variable.
        /// </summary>
        private readonly Variable<T> _variable;

        /// <summary>
        /// If variable must be directly accessed.
        /// </summary>
        /// <remarks><para>A directly accessed variable is accessed using no
        /// <see cref="Aran.Core.Action">action</see> information. In that case, the
        /// <see cref="VariableIndex">variable index</see> points to a position into the
        /// <see cref="Aran.Core.State.Variables">state variable list</see> or a global one.</para>
        /// <para>An indirectly accessed variable is accessed
        /// using <see cref="Aran.Core.Action.Variables">action variable index list</see> information. In that
        /// case, the <see cref="VariableIndex">variable index</see> points to a position into the
        /// <see cref="Aran.Core.Action.Variables">action variable list</see>, which, at the same time, points to
        /// a position into the <see cref="Aran.Core.State.Variables">state variable list</see> or a global one</para>
        /// </remarks>
        private readonly bool _directAccess;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="VariableOperand{T}"/> class.
        /// </summary>
        /// <param name="variable">The variable.</param>
        /// <param name="directAccess">If variable must be directly accessed.</param>
        internal VariableOperand(Variable<T> variable, bool directAccess) {
            _variable = variable;
            _directAccess = directAccess;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets the type of the operand.
        /// </summary>
        /// <value>The type of the operand.</value>
        internal override OperandType OperandType {
            get {
                return OperandType.Variable;
            }
        }

        /// <summary>
        /// Gets the index of the variable.
        /// </summary>
        /// <value>
        /// The index of the variable. Zero if the operand is not a variable.
        /// </value>
        internal override int VariableIndex {
            get {
                if (_directAccess) {
                    return _variable.Id;
                } else {
                    return _variable.Index;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether this operand is a global variable.
        /// </summary>
        /// <value>True if this operand is a global variable; otherwise, false.</value>
        internal override bool IsGlobalVariable {
            get {
                return _variable.FunctionInfo.IsReadOnly;
            }
        }

        #endregion

    }

}
